#include "syntx.h"

#define        NumofBeDgns      33
#define        NumofBeNext      19

extern struct Syntx_Term *sentence; 
extern int                sentence_len;
extern struct SPOC_Pos    sentence_spoc;
extern boolean            syntx_error;
extern char               fatalerr_msg[];
extern boolean            intrnl_dsply;




DIAGNOSIS_ARRAY be_dgns[NumofBeDgns + 1] = { 
                                             "err_be_objpron",
                                             "next(find_obj)",
                                             "next(find_sub)",
                                             "next(pv_phrs)",
                                             "next(hv_phrs)",
                                             "next(fnd_adv1)",
                                             "next(fnd_adv2)",
                                             "next(pbe_adv1)",
                                             "next(pbe_adv2)",
                                             "next(hbe_adv1)",
                                             "next(hbe_adv2)",
                                             "next(X_H_B_H)",
                                             "next(X_H_B_V1)",
                                             "next(X_H_B_V2)",
                                             "next(X_B_X)",
                                             "next(X_B_V1)",
                                             "next(X_B_V2)",
                                             "next(PV_H_B_H)",
                                             "next(PVHBV1)",
                                             "next(PVHBV2)",
                                             "passive_phrase",
                                             "vt_ing_phrase",
                                             "vi_ing_phrase",
                                             "cmb_rht_one",
                                             "cmb_rht_two",
                                             "be_phrs(am)",
                                             "be_phrs(is)",
                                             "be_phrs(are)",
                                             "be_phrs(was)",
                                             "be_phrs(were)",
                                             "be_phrs(be)",
                                             "be_phrs(been)",
                                             "be_phrs(being)",
                                              "" 
                                           };   

static int        symptoms_idx;
SYMPTOM_ARRAY     symptoms[mmax(2*MAXPTRNS+2,MAXSYMP)];
  
boolean be_phrase();
int     do_beP();
boolean beP_cntwd();
boolean beP_rht1wd();
boolean beP_rht2wd();
boolean beP_rht1ptrn();
boolean beP_rht2ptrn();
boolean beP_rht1phrs();
void    beP_cmb_and_next();
void    beP_other_next();


/*----------------------------------------------------------------------

  be_phrase  --  control the action of do_beP

    input  -- pos:  the position where the action to be taken
              lklhd: the likelyhood at which the action will take
              who: indicate who invoke the function

    output -- function returns FALSE if some fatal errors occured, 
                               TRUE otherwise
 
-----------------------------------------------------------------------*/
boolean be_phrase(pos, lklhd, who)
int pos;
int lklhd;
int who;
{
  int beP_code, be_pos;

  if (intrnl_dsply) {
    begin_title("be_phrase", pos, lklhd);
  }

  load_prb("BePhrase");

  if ((beP_code = do_beP(pos, lklhd, who, &be_pos)) == 0) {
    if (intrnl_dsply) {
      printf("no be phrases can be formed\n");
    }
    goto end;
  }

  switch (beP_code) {
    case -1:                      /* case error "err_be_objpron" occures */
            clean_stack();
            if (intrnl_dsply) {
              end_title("be_phrase");
            }
            return FALSE;

    case 1:                 /* am, is, are, was, were, be, been or being */
            sentence_spoc.prd_pos = be_pos;
            break;
         
    case 2:           /* no phrases are formed, but suggests some action */
            break;

    default: stx_err("unexpected beP_code", __LINE__, __FILE__);
  }

end:
  if (intrnl_dsply) {
    end_title("be_phrase");
  }
  return TRUE;
}


/*---------------------------------------------------------------------

  do_beP  --  execute be_phrase.prg  

    input  -- pos: the position where the do_beP works on
              lklhd: the likelihood the do_beP works at
              who:   who issued be_phrase
    output -- be_pos: the position of the formed phrase

  function returns -1: case error "err_be_objpron" occures
                    0: no be phrase can be formed
                    1: am, is, are, was, were, be, been or being is formed
                    2: no phrases are formed, but suggests some action

----------------------------------------------------------------------*/
int do_beP(pos, lklhd, who, be_pos)
int pos, lklhd, who, *be_pos;
{
  DIAGNOSIS_ARRAY   dgns[NumofBeDgns + 1];
  char              dgns_var[VAR_MAX + 1],  ptrn[PTRN_PHRS_MAX+1],
                    ptrn_names[MAXPTRNS+1][PTRN_PHRS_MAX+1],
                    word[WORD_MAX+1], phrs[9],
                    used_ptrn[3][PTRN_PHRS_MAX+1] = {"", "", ""};
  int               i, cnt, rht1, rht2, lh, lklhds[MAXPTRNS], return_val=0;
  boolean           next[NumofBeNext];


  symptoms_idx = 0;
  cnt = pos;
  rht1 = pos + 1;
  rht2 = pos + 2;
  for (i = 0; i < NumofBeNext; i++) next[i] = FALSE;

  switch(who) {
    case FindPredicate: 
                        strcpy(symptoms[symptoms_idx++], "who(find_prd)");
                        break;
  
    case PronvbPhrase: 
                        strcpy(symptoms[symptoms_idx++], "who(pv_phrs)");
                        break;

    case AdvPhrase: 
                        strcpy(symptoms[symptoms_idx++], "who(adv_phrs)");
                        break;

    case PronvbBeAdv: 
                        strcpy(symptoms[symptoms_idx++], "who(pbe_adv)");
                        break;

    case HavePhrase: 
                        strcpy(symptoms[symptoms_idx++], "who(hv_phrs)");
                        break;

    case HaveBeAdv: 
                        strcpy(symptoms[symptoms_idx++], "who(hbe_adv)");
                        break;

    case AuxHave: 
                        strcpy(symptoms[symptoms_idx++], "who(aux_hv)");
                        break;

    case AuxHaveBeAdv: 
                        strcpy(symptoms[symptoms_idx++], "who(X_H_B_V)");
                        break;

    case PvHave: 
                        strcpy(symptoms[symptoms_idx++], "who(pv_hv)");
                        break;

    case PvHaveBeAdv: 
                        strcpy(symptoms[symptoms_idx++], "who(PV_H_B_V)");
                        break;

    case AuxPhrase: 
                        strcpy(symptoms[symptoms_idx++], "who(aux_phrs)");
                        break;

    case AuxBeAdv: 
                        strcpy(symptoms[symptoms_idx++], "who(X_B_V)");
                        break;

    default: stx_err("unexpected problem code", __LINE__, __FILE__);
  }

  get_word(sentence, sentence_len, cnt, word);
  if (!EQUAL(word,"")) {
    if (beP_cntwd(word, used_ptrn[0])) {
      if (word_check(word)==0) {
        sprintf(symptoms[symptoms_idx++], "cnt_wd(%s)", word);
      } else {
        sprintf(symptoms[symptoms_idx++], "cnt_wd(%s_)", word);
      }
    }
  }

  get_word(sentence, sentence_len, rht1, word);
  if (!EQUAL(word,"")) {
    if (beP_rht1wd(word)) {
      if (word_check(word)==0) {
        sprintf(symptoms[symptoms_idx++], "rht1_wd(%s)", word);
      } else {
        sprintf(symptoms[symptoms_idx++], "rht1_wd(%s_)", word);
      }
    }
  }

  get_word(sentence, sentence_len, rht2, word);
  if (!EQUAL(word,"")) {
    if (beP_rht2wd(word)) {
      if (word_check(word)==0) {
        sprintf(symptoms[symptoms_idx++], "rht2_wd(%s)", word);
      } else {
        sprintf(symptoms[symptoms_idx++], "rht2_wd(%s_)", word);
      }
    }
  }


  get_ptrn_names(sentence, sentence_len, rht1, ptrn_names, lklhds);
  i = 0;
  strcpy(ptrn, ptrn_names[i++]);
  while (!EQUAL(ptrn,"")) {
    if (beP_rht1ptrn(ptrn)) {
      sprintf(symptoms[symptoms_idx++], "rht1_ptn(%s)", ptrn);
      if (EQUAL(used_ptrn[1],"")) strcpy(used_ptrn[1], ptrn);
    }
    strcpy(ptrn, ptrn_names[i++]);
  }

  get_ptrn_names(sentence, sentence_len, rht2, ptrn_names, lklhds);
  i = 0;
  strcpy(ptrn, ptrn_names[i++]);
  while (!EQUAL(ptrn,"")) {
    if (beP_rht2ptrn(ptrn)) {
      sprintf(symptoms[symptoms_idx++], "rht2_ptn(%s)", ptrn);
      if (EQUAL(used_ptrn[2],"")) strcpy(used_ptrn[2], ptrn);
    }
    strcpy(ptrn, ptrn_names[i++]);
  }


  if (beP_rht1phrs(sentence, sentence_len, rht1, phrs)) 
    sprintf(symptoms[symptoms_idx++], "rht1_ph(%s)", phrs);

  strcpy(symptoms[symptoms_idx],"");

  mst_smp(symptoms, name, state, type, value, errcde);
  sol_prb(name, state, type, value, errcde);
  if (errcde[0] != 0) {
    stx_err("failed to solve problem", __LINE__, __FILE__);
  }

  if EQUAL(state, "S") { 
    copy_dgns(dgns, be_dgns);
    get_dgn(dgns,name,state,type,value,errcde);
    if (value[0]) {                      /* has at least one true value */
      boolean cmb_rht_one = FALSE, cmb_rht_two = FALSE, 
              passive = FALSE, vt_ing = FALSE, vi_ing = FALSE;
      char    clss[MAXPHRASE][PTRN_PHRS_MAX+1], 
              phrase[PTRN_PHRS_MAX+1] = ""; 
      struct  Syntx_Term *stc; 
      int     stc_len;
      struct  SPOC_Pos stc_spoc;


      strcpy(clss[0], used_ptrn[0]);
      strcpy(clss[1], used_ptrn[1]);
      strcpy(clss[2], used_ptrn[2]);

      i = 0;
      while (! EQUAL(dgns[i],"")) {
        strcpy(dgns_var, dgns[i++]);
      
        if (EQUAL(dgns_var,"err_be_objpron")) {
          syntx_error = TRUE;
          strcpy(fatalerr_msg, "err_be_objpron");
          return -1;
        }

        if (EQUAL(dgns_var,"next(find_sub)")) { 
          next[0] = TRUE; 
          continue; 
        }

        if (EQUAL(dgns_var,"next(find_obj)")) { 
          next[1] = TRUE;
          continue; 
        }

        if (EQUAL(dgns_var,"next(pv_phrs)")) { 
          next[2] = TRUE; 
          continue; 
        }

        if (EQUAL(dgns_var,"next(hv_phrs)")) { 
          next[7] = TRUE; 
          continue; 
        }

        if (EQUAL(dgns_var,"next(fnd_adv1)")) { 
          next[3] = TRUE;
          beP_other_next(sentence, &sentence_len, &sentence_spoc, 
                         cnt, next, lklhd);
          return_val = 2;
          continue; 
        }

        if (EQUAL(dgns_var,"next(fnd_adv2)")) { 
          next[4] = TRUE;
          beP_other_next(sentence, &sentence_len, &sentence_spoc, 
                         cnt, next, lklhd);
          return_val = 2; 
          continue; 
        }

        if (EQUAL(dgns_var,"next(pbe_adv1)")) { 
          next[5]  = TRUE;
          beP_other_next(sentence, &sentence_len, &sentence_spoc, 
                         cnt, next, lklhd);
          return_val = 2;
          continue; 
        } 

        if (EQUAL(dgns_var,"next(hbe_adv1)")) { 
          next[8] = TRUE;
          beP_other_next(sentence, &sentence_len, &sentence_spoc, 
                         cnt, next, lklhd);
          return_val = 2;  
          continue; 
        } 

        if (EQUAL(dgns_var,"next(pbe_adv2)")) { 
          next[6] = TRUE;
          beP_other_next(sentence, &sentence_len, &sentence_spoc, 
                         cnt, next, lklhd);
          return_val = 2; 
          continue; 
        } 

        if (EQUAL(dgns_var,"next(hbe_adv2)")) { 
          next[9] = TRUE;
          beP_other_next(sentence, &sentence_len, &sentence_spoc, 
                         cnt, next, lklhd);
          return_val = 2; 
          continue; 
        } 

        if (EQUAL(dgns_var,"next(X_H_B_H)")) { 
          next[10] = TRUE;
          continue; 
        }

        if (EQUAL(dgns_var,"next(X_H_B_V1)")) { 
          next[11] = TRUE;
          beP_other_next(sentence, &sentence_len, &sentence_spoc, 
                         cnt, next, lklhd);
          return_val = 2;
          continue; 
        }

        if (EQUAL(dgns_var,"next(X_H_B_V2)")) { 
          next[12] = TRUE;
          beP_other_next(sentence, &sentence_len, &sentence_spoc, 
                         cnt, next, lklhd);
          return_val = 2;  
          continue; 
        }

        if (EQUAL(dgns_var,"next(X_B_X)")) { 
          next[13] = TRUE;
          continue; 
        }

        if (EQUAL(dgns_var,"next(X_B_V1)")) { 
          next[14] = TRUE;
          beP_other_next(sentence, &sentence_len, &sentence_spoc, 
                         cnt, next, lklhd);
          return_val = 2; 
          continue; 
        }

        if (EQUAL(dgns_var,"next(X_B_V2)")) { 
          next[15] = TRUE;
          beP_other_next(sentence, &sentence_len, &sentence_spoc, 
                         cnt, next, lklhd);
          return_val = 2;
          continue; 
        }

        if (EQUAL(dgns_var,"next(PV_H_B_H)")) { 
          next[16] = TRUE;
          continue; 
        }

        if (EQUAL(dgns_var,"next(PVHBV1)")) { 
          next[17] = TRUE;
          beP_other_next(sentence, &sentence_len, &sentence_spoc, 
                         cnt, next, lklhd);
          return_val = 2;
          continue; 
        }

        if (EQUAL(dgns_var,"next(PVHBV2)")) { 
          next[18] = TRUE;
          beP_other_next(sentence, &sentence_len, &sentence_spoc, 
                         cnt, next, lklhd);
          return_val = 2;  
          continue; 
        }

        if (EQUAL(dgns_var,"passive_phrase")) { 
          passive = TRUE; 
          continue; 
        }

        if (EQUAL(dgns_var,"vt_ing_phrase")) { 
          vt_ing = TRUE; 
          continue; 
        }

        if (EQUAL(dgns_var,"vi_ing_phrase")) { 
          vi_ing = TRUE; 
          continue; 
        }

        if (EQUAL(dgns_var,"cmb_rht_one")) { 
          cmb_rht_one = TRUE; 
          continue; 
        }

        if (EQUAL(dgns_var,"cmb_rht_two")) { 
          cmb_rht_two = TRUE; 
          continue; 
        }


        if (EQUAL(dgns_var,"be_phrs(am)")) { 
          stc = (struct Syntx_Term *)stc_cpy(&stc_len, &stc_spoc,
                                   sentence, sentence_len, &sentence_spoc);
          strcpy(phrase, "am");
          beP_cmb_and_next(stc,&stc_len,&stc_spoc,cnt,clss,phrase,passive, 
            vt_ing, vi_ing, lklhd, cmb_rht_one, cmb_rht_two, next);
          stc_dsp(stc);
          return_val = 1;
          continue;
        }
        if (EQUAL(dgns_var,"be_phrs(is)")) {
          stc = (struct Syntx_Term *)stc_cpy(&stc_len, &stc_spoc,
                                   sentence, sentence_len, &sentence_spoc);
          strcpy(phrase, "is");
          beP_cmb_and_next(stc,&stc_len,&stc_spoc,cnt,clss,phrase,passive,
            vt_ing, vi_ing, lklhd, cmb_rht_one, cmb_rht_two, next);
          stc_dsp(stc); 
          return_val = 1;
          continue;
        }
        if (EQUAL(dgns_var,"be_phrs(are)")) { 
          stc = (struct Syntx_Term *)stc_cpy(&stc_len, &stc_spoc,
                                   sentence, sentence_len, &sentence_spoc);
          strcpy(phrase, "are");
          beP_cmb_and_next(stc,&stc_len,&stc_spoc,cnt,clss,phrase,passive,
            vt_ing, vi_ing, lklhd, cmb_rht_one, cmb_rht_two, next);
          stc_dsp(stc); 
          return_val = 1;
          continue;
        }
        if (EQUAL(dgns_var,"be_phrs(was)")) { 
          stc = (struct Syntx_Term *)stc_cpy(&stc_len, &stc_spoc,
                                   sentence, sentence_len, &sentence_spoc);
          strcpy(phrase, "was");
          beP_cmb_and_next(stc,&stc_len,&stc_spoc,cnt,clss,phrase,passive,
            vt_ing, vi_ing, lklhd, cmb_rht_one, cmb_rht_two, next);
          stc_dsp(stc);
          return_val = 1;
          continue;
        }
        if (EQUAL(dgns_var,"be_phrs(were)")) { 
          stc = (struct Syntx_Term *)stc_cpy(&stc_len, &stc_spoc,
                                   sentence, sentence_len, &sentence_spoc);
          strcpy(phrase, "were");
          beP_cmb_and_next(stc,&stc_len,&stc_spoc,cnt,clss,phrase,passive,
            vt_ing, vi_ing, lklhd, cmb_rht_one, cmb_rht_two, next);
          stc_dsp(stc);
          return_val = 1;
          continue;
        }
        if (EQUAL(dgns_var,"be_phrs(be)")) { 
          stc = (struct Syntx_Term *)stc_cpy(&stc_len, &stc_spoc,
                                   sentence, sentence_len, &sentence_spoc);
          strcpy(phrase, "be"); 
          beP_cmb_and_next(stc,&stc_len,&stc_spoc,cnt,clss,phrase,passive,
            vt_ing, vi_ing, lklhd, cmb_rht_one, cmb_rht_two, next);
          stc_dsp(stc);
          return_val = 1;
          continue;
        }
        if (EQUAL(dgns_var,"be_phrs(been)")) { 
          stc = (struct Syntx_Term *)stc_cpy(&stc_len, &stc_spoc,
                                   sentence, sentence_len, &sentence_spoc);
          strcpy(phrase, "been");
          beP_cmb_and_next(stc,&stc_len,&stc_spoc,cnt,clss,phrase,passive,
            vt_ing, vi_ing, lklhd, cmb_rht_one, cmb_rht_two, next);
          stc_dsp(stc);
          return_val = 1;
          continue;
        }
        if (EQUAL(dgns_var,"be_phrs(being)")) { 
          stc = (struct Syntx_Term *)stc_cpy(&stc_len, &stc_spoc,
                                   sentence, sentence_len, &sentence_spoc);
          strcpy(phrase, "being"); 
          beP_cmb_and_next(stc,&stc_len,&stc_spoc,cnt,clss,phrase,passive,
            vt_ing, vi_ing, lklhd, cmb_rht_one, cmb_rht_two, next);
          stc_dsp(stc);
          return_val = 1;
          continue;
        }
      }

      *be_pos = pos;
    }
  }
  else stx_err("problem is unsatisfied", __LINE__, __FILE__);
  

  strcpy(state, "A");
  rst_prb(name, state, type, value, errcde);
  return return_val;
}


/*---------------------------------------------------------------------

  beP_cntwd  --  check if current word is considered in the log file

    input  -- word: current word to be checked
    output -- ptrn: the pattern of the word

  function returns TRUE if word is considered in log file, FALSE otherwise

------------------------------------------------------------------------*/
boolean beP_cntwd(word, ptrn)
char *word, *ptrn;
{
  int  i, len;
  char ch;

  len = strlen(word);
  for (i = 0; i < len; i++) {
    if ((ch = word[i]) == '\'') {
      word[i] = '_';
    }
    else word[i] = tolower(ch);
  }

  if (EQUAL(word,"am")     OR
      EQUAL(word,"is")     OR
      EQUAL(word,"are")    OR
      EQUAL(word,"isn_t")  OR
      EQUAL(word,"aren_t"))   { strcpy(ptrn, "be_prst"); return TRUE; }

  if (EQUAL(word,"was")     OR
      EQUAL(word,"were")    OR
      EQUAL(word,"wasn_t")  OR
      EQUAL(word,"weren_t"))   { strcpy(ptrn, "be_past"); return TRUE; }

  if (EQUAL(word,"be")   OR
      EQUAL(word,"been") OR
      EQUAL(word,"being"))  { strcpy(ptrn, word); return TRUE; }

  if (EQUAL(word,"she_s")  OR
      EQUAL(word,"he_s")   OR
      EQUAL(word,"it_s")   OR
      EQUAL(word,"i_m")    OR
      EQUAL(word,"you_re"))   { strcpy(ptrn, "sgl_be"); return TRUE; }

  if (EQUAL(word,"we_re")  OR
      EQUAL(word,"they_re"))  { strcpy(ptrn, "pl_be"); return TRUE; }

  return FALSE;
}


/*---------------------------------------------------------------------

  beP_rht1wd  --  check if right1 word is considered in the log file

    input  -- word: right1 word to be checked
    output -- none

  function returns TRUE if word is considered in log file, FALSE otherwise

------------------------------------------------------------------------*/
boolean beP_rht1wd(word)
char *word;
{
  int  i, len;

  len = strlen(word);
  for (i = 0; i < len; i++) word[i] = tolower(word[i]);

  if (EQUAL(word,"us")   OR
      EQUAL(word,"him")  OR
      EQUAL(word,"her")  OR
      EQUAL(word,"them"))   return TRUE;

  return FALSE;
}


/*---------------------------------------------------------------------

  beP_rht2wd  --  check if right2 word is considered in the log file

    input  -- word: right2 word to be checked
    output -- none

  function returns TRUE if word is considered in log file, FALSE otherwise

------------------------------------------------------------------------*/
boolean beP_rht2wd(word)
char *word;
{
  int  i, len;

  len = strlen(word);
  for (i = 0; i < len; i++) word[i] = tolower(word[i]);

  if (EQUAL(word,"us")   OR
      EQUAL(word,"him")  OR
      EQUAL(word,"her")  OR
      EQUAL(word,"them"))   return TRUE;

  return FALSE;
}


/*---------------------------------------------------------------------

  beP_rht1ptrn  --  check if right1 pattern is considered in the log file

    input  -- ptrn: right1 pattern name to be checked
    output -- none

  function returns TRUE if ptrn is considered in log file, FALSE otherwise

------------------------------------------------------------------------*/
boolean beP_rht1ptrn(ptrn)
char *ptrn;
{
  if (EQUAL(ptrn,"vt_pprt")  OR
      EQUAL(ptrn,"vi_pprt")  OR
      EQUAL(ptrn,"vt_ing")   OR
      EQUAL(ptrn,"vi_ing")   OR
      EQUAL(ptrn,"artcl_df") OR
      EQUAL(ptrn,"adv")      OR
      EQUAL(ptrn,"adv_er"))     return TRUE;

  return FALSE;
}

/*---------------------------------------------------------------------

  beP_rht2ptrn  --  check if right2 pattern is considered in the log file

    input  -- ptrn: right2 pattern name to be checked
    output -- none

  function returns TRUE if ptrn is considered in log file, FALSE otherwise

------------------------------------------------------------------------*/
boolean beP_rht2ptrn(ptrn)
char *ptrn;
{
  if (EQUAL(ptrn,"vt_pprt") OR
      EQUAL(ptrn,"vi_pprt") OR
      EQUAL(ptrn,"vt_ing")  OR
      EQUAL(ptrn,"vi_ing")  OR
      EQUAL(ptrn,"adv_est"))   return TRUE;

  return FALSE;
}

/*----------------------------------------------------------------------

  beP_rht1phrs  --  check if right1 phrase is considered in the log file

    input  -- stc:  a sentence
              slen: sentence length
              idx:  token index
    output -- rht1_phrs: name of right1 phrase

   function returns TRUE if it is considered in log file, FALSE otherwise

-----------------------------------------------------------------------*/
boolean beP_rht1phrs(stc, slen, idx, rht1_phrs)
struct Syntx_Term *stc;
int    slen;
int    idx;
char   *rht1_phrs;
{
  int  lh;

  if (has_phrase(stc, slen, idx, "adv", &lh)) {
    strcpy(rht1_phrs, "adv");
    return TRUE;
  }
  if (has_phrase(stc, slen, idx, "adv_er", &lh)) {
    strcpy(rht1_phrs, "adv_er");
    return TRUE;
  }
  if (has_phrase(stc, slen, idx, "adv_est", &lh)) {
    strcpy(rht1_phrs, "adv_est");
    return TRUE;
  }

  return FALSE;
}



/*---------------------------------------------------------------------

  beP_cmb_and_next  --  combine tokens and issue next command

    input  -- stc:    sentence
              slen:   sentence length
              spoc:   positions of SPOC
              cnt:    current position
              clss:   the classifications of the tokens to be combined
              phrase: the name of the combined phrase
              passive: indicate the phrase is a passive phrase
              vt_ing:  indicate the phrase is a vt + ing
              vi_ing:  indicate the phrase is a vi + ing
              lh:     likelihood
              cmb1:   indicate combining cnt and right1
              cmb2:   indicate combining cnt, right1 and right2
              next:   indicate the next command to be executed

    output -- none
 

------------------------------------------------------------------------*/
void beP_cmb_and_next(stc,slen,spoc,cnt,clss,phrase,passive,vt_ing,vi_ing,
                      lh, cmb1, cmb2, next)
struct  Syntx_Term *stc; 
int     *slen;
struct  SPOC_Pos *spoc;
int     cnt;
char    clss[MAXPHRASE][PTRN_PHRS_MAX+1]; 
char    *phrase;
boolean passive;
boolean vt_ing;
boolean vi_ing;
int     lh;
boolean cmb1;
boolean cmb2;
boolean next[NumofBeNext];
{
  boolean vt = FALSE, vti = FALSE;
  int     l;

  l = strlen(phrase);
  if (passive) { phrase[l] = 'P'; phrase[l+1] = '\0'; }
  if (vt_ing AND vi_ing) {
    phrase[l] = 'B'; phrase[l+1] = '\0';
    vti = TRUE;
  }
  else {
    if (vt_ing) { phrase[l] = 'T'; phrase[l+1] = '\0'; vt = TRUE; }
    if (vi_ing) { phrase[l] = 'I'; phrase[l+1] = '\0'; }
  }


  if ((!cmb1) AND (!cmb2)) {
    if (intrnl_dsply) {
      printf("%d  ==>  phrase '%s'\n\n", cnt, phrase);
    }    
    cmb_token(stc, slen, spoc, cnt, cnt, clss, phrase, lh);
  }
 
  if (cmb1) { 
    if (intrnl_dsply) {
      printf("%d + %d  ==>  phrase '%s'\n\n", cnt, cnt+1, phrase);
    }     
    cmb_token(stc, slen, spoc, cnt, cnt+1, clss, phrase, lh);
  }

  if (cmb2) { 
    if (intrnl_dsply) {
      printf("%d + %d + %d  ==>  phrase '%s'\n\n",cnt,cnt+1,cnt+2, phrase);
    }
    cmb_token(stc, slen, spoc, cnt, cnt+2, clss, phrase, lh);
  }


  if ((next[0]) AND !vt) {                         /* next[0] = find_sub */
    spoc->prd_pos = cnt;
    stc_push(stc, *slen, spoc, FindSubject, cnt-1, lh, BePhrase);
  }

  if ((next[1]) AND (vt OR vti)) {                 /* next[1] = find_obj */
    spoc->prd_pos = cnt;
    stc_push(stc, *slen, spoc, FindObject, cnt+1, lh, BePhrase);
  }
  
  if (next[2]) {                                    /* next[2] = pv_phrs */
    if (is_word(sentence, sentence_len, cnt-1)) { 
      stc_push(stc, *slen, spoc, PronvbPhrase, cnt-1, lh, BePhrase);
    }
    else stc_push(stc, *slen, spoc, PronvbPhrase, cnt-2, lh, BePhrase);
  }
  
  if (next[7]) {                                    /* next[7] = hv_phrs */
    if (is_word(sentence, sentence_len, cnt-1)) { 
      stc_push(stc, *slen, spoc, HavePhrase, cnt-1, lh, BePhrase);
    }
    else stc_push(stc, *slen, spoc, HavePhrase, cnt-2, lh, BePhrase);
  }

  if (next[10]) {                                  /* next[10] = X_H_B_H */
    if (is_word(sentence, sentence_len, cnt-1)) { 
      stc_push(stc, *slen, spoc, HavePhrase, cnt-1, lh, AuxHaveBe);
    }
    else stc_push(stc, *slen, spoc, HavePhrase, cnt-2, lh, AuxHaveBe);
  }
  
  if (next[13]) {                                    /* next[13] = X_B_X */
    if (is_word(sentence, sentence_len, cnt-1)) { 
      stc_push(stc, *slen, spoc, AuxPhrase, cnt-1, lh, AuxBe);
    }
    else stc_push(stc, *slen, spoc, AuxPhrase, cnt-2, lh, AuxBe);
  }
  
  if (next[16]) {                                 /* next[16] = PV_H_B_H */
    if (is_word(sentence, sentence_len, cnt-1)) { 
      stc_push(stc, *slen, spoc, HavePhrase, cnt-1, lh, PvHaveBe);
    }
    else stc_push(stc, *slen, spoc, HavePhrase, cnt-2, lh, PvHaveBe);
  }

}


/*---------------------------------------------------------------------

  beP_other_next  --  issue the next other command

    input  -- stc:    sentence
              slen:   sentence length
              spoc:   positions of SPOC
              cnt:    current position
              next:   indicate the next command to be executed
              lh:     likelihood

    output -- none

------------------------------------------------------------------------*/
void beP_other_next(stc, slen, spoc, cnt, next, lh)
struct  Syntx_Term *stc; 
int     *slen;
struct  SPOC_Pos *spoc;
int     cnt;
boolean next[NumofBeNext];
int     lh;
{

  if (next[3]) {                                             /* fnd_adv1 */
    stc_push(stc, *slen, spoc, AdvPhrase, cnt+1, lh, BePhrase);
    next[3] = FALSE;
  }
  if (next[4]) {                                             /* fnd_adv2 */
    stc_push(stc, *slen, spoc, AdvPhrase, cnt+2, lh, BePhrase);
    next[4] = FALSE;
  }
  if (next[5]) {                                             /* pbe_adv1 */
    stc_push(stc, *slen, spoc, AdvPhrase, cnt+1, lh, PronvbBe);
    next[5] = FALSE;
  }
  if (next[6]) {                                             /* pbe_adv2 */
    stc_push(stc, *slen, spoc, AdvPhrase, cnt+2, lh, PronvbBe);
    next[6] = FALSE;
  }
  if (next[8]) {                                             /* hbe_adv1 */
    stc_push(stc, *slen, spoc, AdvPhrase, cnt+1, lh, HaveBe);
    next[8] = FALSE;
  }
  if (next[9]) {                                             /* hbe_adv2 */
    stc_push(stc, *slen, spoc, AdvPhrase, cnt+2, lh, HaveBe);
    next[9] = FALSE;
  }
  if (next[11]) {                                            /* X_H_B_V1 */
    stc_push(stc, *slen, spoc, AdvPhrase, cnt+1, lh, AuxHaveBe);
    next[11] = FALSE;
  }
  if (next[12]) {                                            /* X_H_B_V2 */
    stc_push(stc, *slen, spoc, AdvPhrase, cnt+2, lh, AuxHaveBe);
    next[12] = FALSE;
  }
  if (next[14]) {                                              /* X_B_V1 */
    stc_push(stc, *slen, spoc, AdvPhrase, cnt+1, lh, AuxBe);
    next[14] = FALSE;
  }
  if (next[15]) {                                              /* X_B_V2 */
    stc_push(stc, *slen, spoc, AdvPhrase, cnt+2, lh, AuxBe);
    next[15] = FALSE;
  }
  if (next[17]) {                                              /* PVHBV1 */
    stc_push(stc, *slen, spoc, AdvPhrase, cnt+1, lh, PvHaveBe);
    next[17] = FALSE;
  }
  if (next[18]) {                                              /* PVHBV2 */
    stc_push(stc, *slen, spoc, AdvPhrase, cnt+2, lh, PvHaveBe);
    next[18] = FALSE;
  }

}
