 
/*************************************************************************/
/*                                                                       */
/*                             create_bname.c                            */
/*                                                                       */
/*                             (Nov. 28, 1992)                           */
/*                                                                       */
/*   This is the program to scan the original dictionary, then produces  */
/*   files bcktname.dat and class.dat.                                   */
/*                                                                       */
/*   bcktname.dat: records all bucket names defined by the prefix of a   */
/*                 word  of PREFIXLEN long together with the number of   */
/*                 words in the bucket.                                  */
/*             It has following format:                                  */
/*                                nnnn  (total number of buckets in file)*/
/*                                PPP nnn                                */
/*                                ppp nnn                                */
/*                                 . . .                                 */
/*                                ppp nnn                                */
/*                                                                       */
/*   class.dat: records all class codes appeared in original dictionary. */
/*                                                                       */
/*************************************************************************/
#include <stdlib.h>
#include <string.h>
#include "create.h"



#define      ORGDIC     "word.dat.org"
#define      BNAME      "bcktname.dat"
#define      CLASS      "class.dat"


struct       bucketname {
                          char  prefix[PREFIXLEN + 1]; /* bucket name */
                          long  freq;/* number of words in this bucket */
                        };
             
struct       class      {
                          char speech;  /* classification of a word */
                          long freq;/* number of words in this class */
                        };



char         Word[MAXWORDLEN + 1];   /* used to get a word from dict */

long int  codeindex;
struct class  Code[NUMOFCLASS];

FILE   *dict, *bname, *cls;


boolean has_class(ch)
char ch;
{
  int  i;
  
  for (i = 0; i < codeindex; i ++) {
    if (ch == Code[i].speech) {
      Code[i].freq++;
      return TRUE;
    }
  }
  return FALSE;
}

main()

{ 
  long int           j, bkindex;
  unsigned long int  total, i;
  struct bucketname  bkname[NUMOFBUCKET];
  char               prefix[PREFIXLEN + 1];
  char               buffer[PREFIXLEN + 1];
  char               pattern[10];
  unsigned           WriteSucc;
  boolean            new;
  char               ch;
  int  len;
  
  total = 0;
  bkindex = 0;
  codeindex = 0;
  buffer[0] = '\0';

  
  
  if ((dict = fopen(ORGDIC, READ)) == NULL)
  {
    fprintf(stderr, "fail to open the %s \n", ORGDIC);
    exit(0);
  };


  if ((bname = fopen(BNAME, WRITE)) == NULL)
  {
    fprintf(stderr, "fail to open the %s \n", BNAME);
    exit(0);
  };


  if ((cls = fopen(CLASS, WRITE)) == NULL)
  {
    fprintf(stderr, "fail to open the %s \n", CLASS);
    exit(0);
  };



  for (i = 0; i < NUMOFWORDS; i ++)
  {
    fscanf(dict, "%s %s\n", Word, pattern);
    total ++;
  
    if ((total % 1000) == 0)
    {
      printf(" %ld WORDS HAVE BEEN PROCESSED \n\n", total);
    }

    strncpy(prefix, Word, PREFIXLEN);
    while ((j = strlen(prefix)) < PREFIXLEN)
    {
      prefix[j] = ' ';
      prefix[j + 1] = '\0';
    };

    
/*printf("WORD:  %s,  PREFIX:  %s, PATTERN:  %s\n", Word, prefix, pattern);*/
/*printf("BKINDEX %d, CODEINDEX %d\n", bkindex, codeindex);*/

    if (strcmp(buffer, prefix) == 0)
    {
      j = bkindex - 1;
      bkname[j].freq = bkname[j].freq + 1;
    }
    else
    {
      new = TRUE;
      j = bkindex - 1;

      while ((j >= 0) && (new))
      {
        if (strcmp(prefix, bkname[j].prefix) == 0)
        {
          bkname[j].freq ++;
          new = FALSE;
        };

        j = j - 1;
      };

    
      if (new)
      {
        strcpy(bkname[bkindex].prefix, prefix);
        strcpy(buffer, prefix);
        bkname[bkindex++].freq = 1;

/*printf("bckt index %d Word %s pattern %s\n", bkindex, Word, pattern);*/
      };   

    }; 
    
    
    len = strlen(pattern);
    for (j = 0; j < len; j++) {
      if (!has_class(pattern[j])) {
        Code[codeindex].speech = pattern[j];
        Code[codeindex++].freq = 1;
      }
    }
     
 };



  WriteSucc = fprintf(bname, "%d\n", bkindex);
  if (! WriteSucc)
  {
    fprintf(stderr, "fail to write into %s\n", BNAME);
    exit(0);
  };

  WriteSucc = fprintf(cls, "%d\n", codeindex);
  if (! WriteSucc)
  {
    fprintf(stderr, "fail to write into %s\n", CLASS);
    exit(0);
  };


 
  for (i = 0; i < bkindex; i++)
  {
    WriteSucc = fprintf(bname, "%s %d\n", bkname[i].prefix, bkname[i].freq);
/*printf("  %s  \n", bkname[i].prefix);*/
    if (! WriteSucc)
    {
      fprintf(stderr, "fail to write into %s\n", BNAME);
      exit(0);
    };

  };

  printf("\nThere are %ld words in total\n", total);
  printf("\nThere are %d buckets described as follows\n", bkindex);



  printf("\nThere are %d codes to classify words\n", codeindex);
/* printf("\n\n  CODE        DENOTATION    NUMBER OF WORDS IN THIS CLASS\n\n")*/
  for (i = 0; i < codeindex; i++)
  {
    WriteSucc = fprintf(cls, "%c %d\n", Code[i].speech, Code[i].freq);
    if (! WriteSucc)
    {
      fprintf(stderr, "fail to write into %s\n", CLASS);
      exit(0);
    };
  };

 
  fclose(dict);
  fclose(bname);
  fclose(cls);

}
