/**************************************************************************/
/*                                                                        */
/*                              create_bsize.c                            */
/*                                                                        */
/*                              (April, 8 1993)                           */
/*                                                                        */
/*    This is the program to fix the hash function for every bucket and   */
/*    fix the bucket size.                                                */
/*                                                                        */
/*    input data file: word.dat                                           */
/*                     bckt.idx (old bucket info)                         */
/*                                                                        */
/*    output data file: bckt.idx(updated bucket info)                     */
/*                                                                        */
/*    procedure:                                                          */
/*               1. open input data files;                                */
/*               2. scanning word.dat word by word;                       */
/*               3. find the corresponding bucket in bckt.idx;            */
/*               4. fix hash function and bucket size based on following, */
/*                  4.1 bucket size > 0,                                  */
/*                      this bucket has already been fixed, do nothing;   */
/*                  4.2 bucket size < 0,                                  */
/*                      this bucket has been modified by inserting or     */
/*                      deleting                                          */
/*                      some words into(from) the bucket. try to fix this */
/*                      bucket again(start from the original size);       */
/*                  4.3 bucket size = 0,                                  */
/*                      this bucket is not fixed yet. try to fix it (start*/
/*                      from some proper size).                           */
/*                                                                        */
/**************************************************************************/
#include <stdlib.h>
#include <string.h>
                                                             
#include     "create.h"
#include     "HashFnct.h"
 
#define  BIDX     "bckt.idx"
#define  WORDDAT  "word.dat"
#define  STOPBCKT ""

struct         cft {
                     int     fnt;
                     int     tcft;
                     int     mcft;
                   };


struct   BcktIdxEnt  Bindex[SIZEOFWORDBK];
char                 prefix[PREFIXLEN + 1], Word[MAXWORDLEN + 1], pattern[10];
unsigned long int    idx;
int                  max, conflict;

                     
FILE *wddat, *bidx; 






int  GetBcktIdx()

{

  unsigned   i, val;
  



  val = BcktHash(prefix);
  i = 0;
/*printf("hashvalue1 %d\n", val);*/
/*printf("prefix %s\n", wordbk[val].bcktnm);*/
  while ((strcmp(Bindex[val].bcktnm, prefix) != 0) && (i <= MAXBKCONFLICT))
  {
    if (Bindex[val].bcktnm[0] == '\0')
    {
      i = MAXBKCONFLICT;
    };

    val = (val + 1) % SIZEOFWORDBK;
    i++;
  };

  if (i > MAXBKCONFLICT)
  {
    return(-1);
  }
  else
  {
    return(val);
  };
}







void  CheckBckt(fnct, num, size, buffer)

int       fnct;
unsigned  num;
unsigned  size;
int       buffer[];




{
  int       i, c;
  unsigned  val; 
  char      ch;

  


  for (i = 0; i < size; i ++)
  {
    buffer[i] = 0;
  };

  max = 0;
  conflict = 0;

  /*printf("Using HashFnct%d   ", fnct);*/

  for (i = 0; i < num; i ++)
  {
    fscanf(wddat, "%s %s\n", Word, pattern);
    idx = idx + strlen(Word) + strlen(pattern) + 2;
 /* printf("word %s  idx %ld\n", Word, idx);*/

    switch(fnct)
    {
      case  1:  val = WordHash1(Word, size);
                break;
      case  2:  val = WordHash2(Word, size);
                break;
      case  3:  val = WordHash3(Word, size);
                break;
      case  4:  val = WordHash4(Word, size);
                break;
      case  5:  val = WordHash5(Word, size);
                break;

    };


    c = 0;

    if (buffer[val] != 0)
    {
      conflict ++;
    };

    while (buffer[val] != 0)
    {
      val = (val + 1) % size;
      c++;
    };
    if (max < c)
    {  
      max = c;
    };

    buffer[val] = strlen(Word);

  };

  /*printf("total conflicts are %d    max conflict is %d\n", conflict, max);*/


}

                 


void  HashCheck(fnct, num, size)

int       fnct;
unsigned  num;
unsigned  size;

{
  int       *buffer;

  


  if (! (buffer = (int *)malloc(size * sizeof(int))))
  {
    printf("fail to allocate buffer\n");
    exit(0);
  };


  CheckBckt(fnct, num, size, buffer);

  free(buffer);

}



main()

{ 
  int                i, bcktidx, minp, diff;
  unsigned long int  start, bcktloc, total, tmp;
  char               ch;
  int                size, oldsize, sztmp;
  unsigned           freq;
  char               ans[2];
  struct  cft        minfnt, min;
  boolean            prnt, updatebckt;



 
  bcktloc = 0;  
  tmp = 0;
  idx = 0;
  diff = 0;

 

  if ((wddat = fopen(WORDDAT, READ)) == NULL)
  {
    fprintf(stderr, "fail to open the %s \n", WORDDAT);
    exit(0);
  };



  if ((bidx = fopen(BIDX, READWRITE)) == NULL)
  {
    fprintf(stderr, "fail to open the %s \n", BIDX);
    exit(0);
  };

  ReadSucc = fread(Bindex, sizeof(Bindex), 1, bidx);
  if (! ReadSucc)
  {
    fprintf(stderr, "fail to read from %s\n", BIDX);
    exit(0);
  };

  

  printf("Do you need to print the procedure on screen?(y OR n)\n");
  scanf("%s", ans);
  if (ans[0] == 'y')
  {
    prnt = TRUE;
  }
  else
  {
    prnt = FALSE;
  };


  total = 0;
  while (total < NUMOFWORDS)
  {
    sztmp = 0;
   
    fscanf(wddat, "%s %s\n", Word, pattern);
    idx =  idx + strlen(Word) + strlen(pattern) + 2;
/*printf("word %s  idx %ld\n", Word, idx);*/
    total ++;
 
  
    strncpy(prefix, Word, PREFIXLEN);
    while ((i = strlen(prefix)) < PREFIXLEN)
    {
      prefix[i] = ' ';
      prefix[i + 1] = '\0';
    };


    if ((bcktidx = GetBcktIdx()) == -1)
    {
      printf("there is no prefix %s in %s for word %s\n", prefix, BIDX,Word);
      printf("total %ld\n", total);
      exit(0);
    };

    if (strcmp(prefix, STOPBCKT) == 0) {
      printf("save bucketidx  total %ld\n", total);
      fseek(bidx, 0, 0);
  
      WriteSucc = fwrite(Bindex, sizeof(Bindex), 1, bidx);
      if (! WriteSucc)
      {
        fprintf(stderr, "fail to write into %s\n", BIDX);
        exit(0);
      };
      exit(1);
    }
      
    freq = Bindex[bcktidx].freq;
    Bindex[bcktidx].bkloc = Bindex[bcktidx].bkloc + diff;

    updatebckt = FALSE;

    if (Bindex[bcktidx].bksize > 0) 
    {
      if (prnt)
      {
      printf("bucket %s has been fixed; freq %d,  size %d,  fnct %c, loc%ld\n", 
              prefix, freq, Bindex[bcktidx].bksize, Bindex[bcktidx].fnct,
              Bindex[bcktidx].bkloc);
      };

      for (i = 1; i < freq; i ++)
      {
        fscanf(wddat, "%s %s\n", Word, pattern);
        idx = idx + strlen(Word) + strlen(pattern) + 2;
        total ++;
      };
  
      bcktloc = bcktloc + Bindex[bcktidx].bksize;

      continue;
    };




    if (freq == 1)
    {
      Bindex[bcktidx].fnct = '1';
      Bindex[bcktidx].bksize = freq;
      Bindex[bcktidx].bkloc = bcktloc;
      Bindex[bcktidx].maxcnft = 0;

      bcktloc = bcktloc + Bindex[bcktidx].bksize;

      if (prnt)
      {
    printf("bucket %s has been fixed.\n", prefix);
    printf("freq %d,  size %d,  fnct %c, total conflict %d,  max conflict %d\n",
            freq, Bindex[bcktidx].bksize, Bindex[bcktidx].fnct, 0,
            Bindex[bcktidx].maxcnft);
    printf("loc %ld\n", Bindex[bcktidx].bkloc);
      };

      continue;
    };

    

          
    if (prnt)
    {
    printf("\n********************************************************\n");
    printf("the freq is %d\n\n", freq);
    };
    printf("now fix bucket %s\n", prefix);


    start = idx - strlen(Word) - strlen(pattern) - 2;
    total --;
    
 
    
    min.tcft = freq;
    min.mcft = 0;

    if (Bindex[bcktidx].bksize < 0)
    {
      size = - Bindex[bcktidx].bksize;
      oldsize = size;
      updatebckt = TRUE;
      Bindex[bcktidx].bksize = 0;
    }
    else
    {
      size = freq;
      if ((size >= 50) && (size < 100))
      {
        size = size * 3 / 2;
      }
      else
      {
        if ((size >= 100) && (size < 500))
        {
          size = size * 2;
        }
        else
        {
          if ((size >= 500) && (size < 1000))
          {
            size = size * 5 / 2;
          }
          else
          {
            if ((size >= 1000) && (size < 3000))
            {
              size = size * 3;
            }
            else
            {
              if (size >= 3000)
              {
                size = size * 7 / 2;
              };
            };
          };
        };
      };
    };


    while ((min.tcft > (CNFLTPRCNT * freq)) || (min.mcft > 10))
    {
      minfnt.tcft = freq;

      for (i = 1; i < 5; i++)
      {
        idx = start;
        fseek(wddat, idx, 0);

        HashCheck(i, freq, size);

        if (minfnt.tcft > conflict)  
        {
          minfnt.fnt = i;
          minfnt.tcft = conflict;
          minfnt.mcft = max;
        };

      };

      if (minfnt.tcft < min.tcft)
      {
        min.fnt = minfnt.fnt;
        min.tcft = minfnt.tcft;
        min.mcft = minfnt.mcft;
        minp = size;
      };
    
      if ((freq > 100) && (((min.tcft * 1.0 / freq) - CNFLTPRCNT) > 0.07))
      {
        size = size + freq / 2;
      }
      else
      {
        size ++;
      };

      if ((size - sztmp) > 100)
      {
        printf("size is %d now\n", size);
       printf("prefix %s, freq %d fnct %d,  total conflict %d,  max conflict %d, minsize %d\n\n", 
        prefix, freq,  min.fnt, min.tcft, min.mcft, minp);
        sztmp = size;
      };


    /*#################################*/
    
      if( size > (30*freq) ) {  break;  }
    
    /*#################################*/


    }; /* end of while (min.tcft... */


 /*printf("with size %d,  fnct %d,  total conflict %d,  max conflict %d\n\n",*/ 
/*            minp, min.fnt, min.tcft, min.mcft);*/
      
    switch(min.fnt)
    {
      case  1:  Bindex[bcktidx].fnct = '1';
                break;
      case  2:  Bindex[bcktidx].fnct = '2';
                break;
      case  3:  Bindex[bcktidx].fnct = '3';
                break;
      case  4:  Bindex[bcktidx].fnct = '4';
                break;
      case  5:  Bindex[bcktidx].fnct = '5';
                break;


    };
  
    Bindex[bcktidx].bksize = minp;
    Bindex[bcktidx].bkloc = bcktloc;
    Bindex[bcktidx].maxcnft = min.mcft;

    bcktloc = bcktloc + Bindex[bcktidx].bksize;

    if (updatebckt)
    {
      diff = diff + Bindex[bcktidx].bksize - oldsize;
    };

    total = total + freq;

    if (prnt)
    {
    printf("bucket %s has been fixed.\n", prefix);
    printf("freq %d,  size %d,  fnct %c, total conflict %d,  max conflict %d\n",
            freq, Bindex[bcktidx].bksize, Bindex[bcktidx].fnct, min.tcft,
            min.mcft);
    printf("loc %ld\n", Bindex[bcktidx].bkloc);

    printf("********************************************************\n\n\n");
    };

    if ((total - tmp) > 1000)
    {
      printf("save bucketidx  total %ld\n", total);
      tmp = total;
      fseek(bidx, 0, 0);
  
      WriteSucc = fwrite(Bindex, sizeof(Bindex), 1, bidx);
      if (! WriteSucc)
      {
        fprintf(stderr, "fail to write into %s\n", BIDX);
        exit(0);
      };
    };


   /* printf("\nwould you try to fix another bucket(y or n)?\n\n");*/
   /* scanf("%s",ans );*/
  
   /* if (strcmp(ans, "n") == 0)  break;*/


  }; /* end of while (total..  */

  printf("%ld  words have been processed\n\n", total);
  printf("%ld  memories are needed for word.idx\n\n", bcktloc);


  fseek(bidx, 0, 0);
  
  WriteSucc = fwrite(Bindex, sizeof(Bindex), 1, bidx);
  if (! WriteSucc)
  {
    fprintf(stderr, "fail to write into %s\n", BIDX);
    exit(0);
  };



  fclose(wddat);
  fclose(bidx);
}
  
