#include "syntx.h"

#define        NumofFmdfDgns    10


extern struct Syntx_Term *sentence; 
extern int                sentence_len;
extern struct SPOC_Pos    sentence_spoc;
extern int                system_status;
extern boolean            intrnl_dsply;

DIAGNOSIS_ARRAY fmdf_dgns[NumofFmdfDgns + 1] = {
                                                  "do_cnt_noun",
                                                  "do_cnt_pron",
                                                  "do_cnt_adj",
                                                  "do_cnt_adv",
                                                  "do_cnt_prep",
                                                  "do_cnt_to",
                                                  "cmb_lft_nounsgl",
                                                  "cmb_lft_nounpl",
                                                  "cnt_noun_sgl",
                                                  "cnt_noun_pl",
                                                  ""
                                                }; 

int               symptoms_idx;
SYMPTOM_ARRAY     symptoms[mmax(MAXPTRNS+1,MAXSYMP)];
  

void    find_modifier();
boolean findmodifier();
boolean fmdfr_cntptrn();
boolean fmdfr_cntwd();
boolean fmdfr_rhtptrn();
boolean fmdfr_lftphrs();


/*----------------------------------------------------------------------

  find_modifier  --  control the action of findmodifier

    input  -- pos:   the position where the action to be started
              lklhd: the likelyhood at which the action will take
              who:   the code of the block which issued this action
    output -- none
 
-----------------------------------------------------------------------*/
void find_modifier(pos, lklhd, who)
int pos, lklhd, who;
{
  int  lh;

  if ((pos == 0) AND (system_status == FindError)) {
    stc_push(sentence, sentence_len, &sentence_spoc, SPOChecker, 
             0, lklhd, FindModifier);
    return;
  }

  if (intrnl_dsply) {
    begin_title("find_modifier", -1, -1);
  }

  if (pos < 0) goto endfmdf;

  load_prb("FindModifier");
  while (pos < sentence_len) {
    if (is_word(sentence, sentence_len, pos))
      if (findmodifier(pos, lklhd)) break;
    pos++;
  }

  if (pos >= sentence_len) {
    stc_push(sentence, sentence_len, &sentence_spoc, SPOChecker, 
             0, lklhd, FindModifier);
  }

endfmdf:  ;
  if (intrnl_dsply) {
    end_title("find_modifier");
  }
}

/*---------------------------------------------------------------------

  findmodifier  --  execute find_modifier.prg  

    input  -- pos: the position where the findmodifier works on
              lklhd: the likelihood the findmodifier works at
    output -- function returns TRUE if at least on action suggested,
                               FALSE otherwise

----------------------------------------------------------------------*/
boolean findmodifier(pos, lklhd)
int pos, lklhd;
{
  DIAGNOSIS_ARRAY   dgns[NumofFmdfDgns + 1];
  char              dgns_var[VAR_MAX + 1],  ptrn[PTRN_PHRS_MAX+1],
                    ptrn_names[MAXPTRNS+1][PTRN_PHRS_MAX+1],
                    phrs_name[PTRN_PHRS_MAX+1], word[WORD_MAX+1];
  int               i, cnt, lft, rht, lh, lklhds[MAXPTRNS], len;
  boolean           atleast_one = FALSE;
  struct Syntx_Term *stc;
  struct  SPOC_Pos  stc_spoc;
  
  symptoms_idx = 0;
  cnt = pos;
  lft = pos - 1;
  rht = pos + 1;

  get_phrs_name(sentence, sentence_len, lft, phrs_name, &lh);
  if (fmdfr_lftphrs(phrs_name)) {
    sprintf(symptoms[symptoms_idx++], "lft_phrs(%s)", phrs_name);
  }

  get_ptrn_names(sentence, sentence_len, cnt, ptrn_names, lklhds);
  i = 0;
  strcpy(ptrn, ptrn_names[i++]);
  while (!EQUAL(ptrn,"")) {
    if (fmdfr_cntptrn(ptrn)) {
      sprintf(symptoms[symptoms_idx++], "cnt_ptrn(%s)", ptrn);
    }
    strcpy(ptrn, ptrn_names[i++]);
  }
  if (has_adjpron(sentence, sentence_len, cnt, &lh)) {
    sprintf(symptoms[symptoms_idx++], "cnt_ptrn(pron_adj)");
  }

  get_word(sentence, sentence_len, cnt, word);
  if (!EQUAL(word,"")) {
    if (fmdfr_cntwd(word)) {
      if (word_check(word)==0) {
        sprintf(symptoms[symptoms_idx++], "cnt_wd(%s)", word);
      } else {
        sprintf(symptoms[symptoms_idx++], "cnt_wd(%s_)", word);
      }
    }
  }

  get_ptrn_names(sentence, sentence_len, rht, ptrn_names, lklhds);
  i = 0;
  strcpy(ptrn, ptrn_names[i++]);
  while (!EQUAL(ptrn,"")) {
    if (fmdfr_rhtptrn(ptrn)) {
      sprintf(symptoms[symptoms_idx++], "rht_ptrn(%s)", ptrn);
    }
    strcpy(ptrn, ptrn_names[i++]);
  }
  
  if (has_verb_only(sentence, sentence_len, rht)) 
    sprintf(symptoms[symptoms_idx++], "rht_verb_only");

  strcpy(symptoms[symptoms_idx],"");

  mst_smp(symptoms, name, state, type, value, errcde);
  sol_prb(name, state, type, value, errcde);
  if (errcde[0] != 0) {
    stx_err("failed to solve problem", __LINE__, __FILE__);
  }

  if EQUAL(state, "S") { 
    copy_dgns(dgns, fmdf_dgns);
    get_dgn(dgns,name,state,type,value,errcde);
    if (value[0]) {                      /* has at least one true value */
      i = 0;
      while (! EQUAL(dgns[i],"")) {
        strcpy(dgns_var, dgns[i++]);

        if (EQUAL(dgns_var,"do_cnt_noun")) {
          if (intrnl_dsply) {
            printf("suggest noun_phrase on position %d\n\n", cnt);
          }

          stc_push(sentence, sentence_len, &sentence_spoc, NounPhrase, 
                   cnt, lklhd, FindModifier);
          atleast_one = TRUE;
          continue;
        }

        if (EQUAL(dgns_var,"do_cnt_pron")) {
          if (intrnl_dsply) {
            printf("suggest pronoun_phrase on position %d\n\n", cnt);
          }

          stc_push(sentence, sentence_len, &sentence_spoc, PronounPhrase,
                   cnt, lklhd, FindModifier);
          atleast_one = TRUE;
          continue;
        }

        if (EQUAL(dgns_var,"do_cnt_adj")) {
          stc = (struct Syntx_Term *)stc_cpy(&len, &stc_spoc, sentence, 
                                             sentence_len, &sentence_spoc);
          if (keep_adjlike(stc, len, cnt, &lh) != -1) {
            if (intrnl_dsply) {
              printf("suggest adj_phrase on position %d\n\n", cnt);
            }

            stc_push(stc, len, &stc_spoc, AdjPhrase, cnt, lh,FindModifier);
            atleast_one = TRUE;
          }
          stc_dsp(stc);
          continue;
        }

        if (EQUAL(dgns_var,"do_cnt_adv")) {
          stc = (struct Syntx_Term *)stc_cpy(&len, &stc_spoc, sentence, 
                                             sentence_len, &sentence_spoc);
          if (keep_adv(stc, len, cnt, &lh) != -1) {

            if (intrnl_dsply) {
              printf("suggest adv_phrase on position %d\n\n", cnt);
            }

            stc_push(stc , len, &stc_spoc, AdvPhrase, cnt,lh,FindModifier);
            atleast_one = TRUE;
          }
          stc_dsp(stc);
          continue;
        }

        if (EQUAL(dgns_var,"do_cnt_prep")) {
          stc = (struct Syntx_Term *)stc_cpy(&len, &stc_spoc, sentence, 
                                             sentence_len, &sentence_spoc);
          if (keep_prep(stc, len, cnt, &lh) != -1) {
            if (intrnl_dsply) {
              printf("suggest prep_phrase on position %d\n\n", cnt);
            }

            stc_push(stc, len, &stc_spoc, PrepPhrase, cnt,lh,FindModifier);
            atleast_one = TRUE;
          }
          stc_dsp(stc);
          continue;
        }

        if (EQUAL(dgns_var,"do_cnt_to")) {
          stc = (struct Syntx_Term *)stc_cpy(&len, &stc_spoc, sentence, 
                                             sentence_len, &sentence_spoc);
          if (keep_prep(stc, len, cnt, &lh) != -1) {
            if (intrnl_dsply) {
              printf("suggest to_phrase on position %d\n\n", cnt);
            }

            stc_push(stc, len, &stc_spoc, ToPhrase, cnt, lh, FindModifier);
            atleast_one = TRUE;
          }
          stc_dsp(stc);
          continue;
        }

        if (EQUAL(dgns_var,"cmb_lft_nounsgl")) {
          char  pt[2], clss[MAXPHRASE][PTRN_PHRS_MAX+1];

          if (intrnl_dsply) {
            printf("%d + %d  ==>  phrase noun_sgl\n\n", lft, cnt);
          }
          stc = (struct Syntx_Term *)stc_cpy(&len, &stc_spoc, sentence, 
                                             sentence_len, &sentence_spoc);
          strcpy(clss[0], "");
          strcpy(clss[1], "noun_sgl");
          name_class_lhd(stc, len, cnt, "noun_sgl", pt, &lh);
          cmb_token(stc, &len, &stc_spoc, lft, cnt, clss, "noun_sgl", lh);
          stc_push(stc, len,&stc_spoc,FindModifier,cnt,lklhd,FindModifier);
          atleast_one = TRUE;
          stc_dsp(stc);
          continue;
        }

        if (EQUAL(dgns_var,"cmb_lft_nounpl")) {
          char  pt[2], clss[MAXPHRASE][PTRN_PHRS_MAX+1];

          if (intrnl_dsply) {
            printf("%d + %d  ==>  phrase noun_pl\n\n", lft, cnt);
          }
          stc = (struct Syntx_Term *)stc_cpy(&len, &stc_spoc, sentence, 
                                             sentence_len, &sentence_spoc);
          strcpy(clss[0], "");
          strcpy(clss[1], "noun_pl");
          name_class_lhd(stc, len, cnt, "noun_pl", pt, &lh);
          cmb_token(stc, &len, &stc_spoc, lft, cnt, clss, "noun_pl", lh);
          stc_push(stc, len,&stc_spoc,FindModifier,cnt,lklhd,FindModifier);
          atleast_one = TRUE;
          stc_dsp(stc);
          continue;
        }
      
        if (EQUAL(dgns_var,"cnt_noun_sgl")) {
          char  pt[2], clss[MAXPHRASE][PTRN_PHRS_MAX+1];

          if (intrnl_dsply) {
            printf("%d  ==>  phrase noun_sgl\n\n", cnt);
          }
          stc = (struct Syntx_Term *)stc_cpy(&len, &stc_spoc, sentence, 
                                             sentence_len, &sentence_spoc);
          strcpy(clss[0], "noun_sgl");
          name_class_lhd(stc, len, cnt, "noun_sgl", pt, &lh);
          cmb_token(stc, &len, &stc_spoc, cnt, cnt, clss, "noun_sgl", lh);
          stc_push(stc, len,&stc_spoc,FindModifier,cnt,lklhd,FindModifier);
          atleast_one = TRUE;
          stc_dsp(stc);
          continue;
        }

        if (EQUAL(dgns_var,"cnt_noun_pl")) {
          char  pt[2], clss[MAXPHRASE][PTRN_PHRS_MAX+1];

          if (intrnl_dsply) {
            printf("%d  ==>  phrase noun_pl\n\n", cnt);
          }
          stc = (struct Syntx_Term *)stc_cpy(&len, &stc_spoc, sentence, 
                                             sentence_len, &sentence_spoc);
          strcpy(clss[0], "noun_pl");
          name_class_lhd(stc, len, cnt, "noun_pl", pt, &lh);
          cmb_token(stc, &len, &stc_spoc, cnt, cnt, clss, "noun_pl", lh);
          stc_push(stc, len,&stc_spoc,FindModifier,cnt,lklhd,FindModifier);
          atleast_one = TRUE;
          stc_dsp(stc);
          continue;
        }

      } /* end of while */

      stc = (struct Syntx_Term *)stc_cpy(&len, &stc_spoc, sentence, 
                                         sentence_len, &sentence_spoc);
      if (drop_mdfr(stc, len, cnt, &lh) != -1) {
        stc_push(stc, len,&stc_spoc,FindModifier,cnt+1,lklhd,FindModifier);
      }
      stc_dsp(stc);
    }
  }
  else stx_err("problem is unsatisfied", __LINE__, __FILE__);

  strcpy(state, "A");
  rst_prb(name, state, type, value, errcde);
  return atleast_one;
}


/*---------------------------------------------------------------------

  fmdfr_cntptrn  --  check if the pattern is considered in the log file

    input  -- ptrn: current pattern name to be checked
    output -- none

  function returns TRUE if ptrn is considered in log file, FALSE otherwise

------------------------------------------------------------------------*/
boolean fmdfr_cntptrn(ptrn)
char *ptrn;
{
  char str[4];

  str[0] = ptrn[0];
  str[1] = ptrn[1];
  str[2] = ptrn[2];
  str[3] = '\0';

  if ((EQUAL(str,"adj")) OR
      (EQUAL(str,"adv")) OR
      (EQUAL(str,"nou")) OR
      (EQUAL(str,"art")) OR
      (EQUAL(str,"pro")) OR
      (EQUAL(str,"pre")))   return TRUE;

  if ((EQUAL(ptrn,"vt_ing"))   OR
      (EQUAL(ptrn,"vi_ing"))   OR
      (EQUAL(ptrn,"vt_pprt"))  OR
      (EQUAL(ptrn,"vi_pprt")))    return TRUE;

  return FALSE;
}


/*---------------------------------------------------------------------

  fmdfr_rhtptrn  --  check if the right pattern is considered in the 
                     log file

    input  -- ptrn: right pattern name to be checked
    output -- none

  function returns TRUE if ptrn is considered in log file, FALSE otherwise

------------------------------------------------------------------------*/
boolean fmdfr_rhtptrn(ptrn)
char *ptrn;
{
  if (EQUAL(ptrn,"adj_est") OR
      EQUAL(ptrn,"adv_est") OR
      EQUAL(ptrn,"vt_pl")   OR
      EQUAL(ptrn,"vi_pl")   OR
      EQUAL(ptrn,"be")      OR
      EQUAL(ptrn,"lvb_pl"))    return TRUE;

  return FALSE;
}

/*---------------------------------------------------------------------

  fmdfr_cntwd  --  check if current word is considered in the log file

    input  -- word: current word to be checked

  function returns TRUE if word is considered in log file, FALSE otherwise

------------------------------------------------------------------------*/
boolean fmdfr_cntwd(word)
char *word;
{
  int  i, len;
  char ch;

  len = strlen(word);
  for (i = 0; i < len; i++) {
    if ((ch = word[i]) == '\'') {
      word[i] = '_';
    }
    else word[i] = tolower(ch);
  }

  if (EQUAL(word,"to"))   return TRUE; 

  return FALSE;
}
/*---------------------------------------------------------------------

  fmdfr_lftphrs  --  check if left phrase is considered in the log file

    input  -- phrs: left phrase name to be checked
    output -- none

  function returns TRUE if phrs is considered in log file, FALSE otherwise

------------------------------------------------------------------------*/
boolean fmdfr_lftphrs(phrs)
char *phrs;
{
  char str[4];

  str[0] = phrs[0];
  str[1] = phrs[1];
  str[2] = phrs[2];
  str[3] = '\0';

  if (EQUAL(str,"adj")) return TRUE;

  if ((EQUAL(phrs,"vt_ing"))   OR
      (EQUAL(phrs,"vi_ing"))   OR
      (EQUAL(phrs,"vt_pprt"))  OR
      (EQUAL(phrs,"vi_pprt"))  OR
      (EQUAL(phrs,"noun_sgl")))    return TRUE;

  return FALSE;
}
