#include "syntx.h"
#include "stdlib.h"
#include "string.h"

#define STCLEN  1000

extern int     *original_sentence; 
extern int     original_sentence_len; 
extern int     sentence_len;         
extern char    *sentence_intprt;   
extern int     UNDEF_code;
extern char    text_name[];
extern struct  Syntx_Term *sentence; 

extern struct  User_Word  *user_dict;
  
extern boolean syntx_intprt;
extern struct  SPOC_Pos    sentence_spoc;

extern boolean             intrnl_dsply;
/*----- added on 04/06/97 -----*/
/*                             */ 
extern boolean  first_time;
extern boolean  display_syntx_msg;
extern FILE    *syntx_msg;
/*                             */
/*-----------------------------*/

extern char    triple_usr[];
extern char    triple_bak[];



struct  inner_stc { /* structure to keep the inner formate of a sentence */
          int      *stc;         /* an array to store the inner sentence */
          char     *itpr;        /* a string to store the intepretation  */
          boolean  occured;     /* the sentence has occured in this text */
          boolean  red;                              /* red / black flag */
          struct inner_stc *left;           /* pointer to the left child */
          struct inner_stc *right;         /* pointer to the right child */
        };

struct  stc_queue {    /* structure to keep a stc node pointer in queue */
          struct inner_stc *ptr;
          struct stc_queue *next;
        };


struct  syntax_err {/*structure to keep a syntax error and relative info */
          char   errmsg[100];                           /* error message */
          int    lh;                  /* likelihood of the error message */
          struct Syntx_Term *stc;     /* pointer to the sentence when 
                                         error occures */
          int    stc_len;                             /* sentence length */
          struct SPOC_Pos   stc_spoc;               /* positions of SPOC */
          struct syntax_err *next;                /* pointer to the next */
        };

struct  inner_triple { /* structure to keep the inner formate of a triple*/
          int      trp[4];         /* an array to store the inner triple */
          int      freq;                      /* frequency of the triple */
          boolean  red;                              /* red / black flag */
          struct inner_triple *left;        /* pointer to the left child */
          struct inner_triple *right;      /* pointer to the right child */
        };
struct  trp_queue {  /* structure to keep a triple node pointer in queue */
          struct inner_triple *ptr;
          struct trp_queue *next;
        };


struct inner_stc *stcdct_head;    /* head pointer to sentence dictionary */
struct inner_stc *stcdct_end;                     /* pointer to end node */
long   stc_total; /* total number of sentence in the sentence dictionary */
struct syntax_err *stxerr_list = NULL;

struct inner_triple *triple_head;   /* head pointer to triple dictionary */
struct inner_triple *triple_end;                  /* pointer to end node */
long   triple_total; /* total number of triples in the triple dictionary */
boolean new_triple_usr; /*indicate the triple dictionary was just created*/


void stcdct_init();
void free_stcnode();
struct inner_stc * stc_new();
void stcdct_insert();
int  stc_cmp();
struct inner_stc *stc_split();
struct inner_stc *stc_rotate();
boolean stcdct_search();
void save_stc();
void save_intprt();
void read_text_stc();
void charseq_to_intseq();
void write_text_stc();
boolean find_stc();

void find_errtype();

void init_stxerrlist();
void save_stxerr();
void get_errmsg();


void triple_init();
void free_trpnode();
struct inner_triple * triple_new();
void triple_insert();
int  triple_cmp();
struct inner_triple *trp_split();
struct inner_triple *trp_rotate();
boolean triple_search();
void get_numoftriples();

void save_triple();
void read_triple_usr();
void write_triple_usr();
boolean find_triple();

void stx_err();
boolean get_stc_ptrns();
struct Syntx_Term *stc_cpy();
int  next_move();
boolean syntax_analysis();
void sentence_check();
void  stc_push();



/*---------------------------------------------------------------------

  stcdct_init  --  initialize the sentence dictionary

    input  -- none
    output -- none

----------------------------------------------------------------------*/
void stcdct_init()
{
  stcdct_end = stc_new(0);
  stcdct_end->left = stcdct_end;
  stcdct_end->right = stcdct_end;
  stcdct_end->red = FALSE;
  stcdct_head = stc_new(0);
  stcdct_head->stc[0] = STC_END;
  stc_total = 0;
}


/*----------------------------------------------------------------------

  free_stcnode  --  free a sentence node in sentence tree.

    input  -- ptr: a pointer to the node
    output -- none

------------------------------------------------------------------------*/
void free_stcnode(ptr)
struct inner_stc *ptr;
{
  free(ptr->stc);
  free(ptr->itpr);
  free(ptr);
}


/*----------------------------------------------------------------------

  stc_new  --  allocate memory for a new sentence

    input  -- len: sentence length
    output -- none

  function returns a pointer to the memory

-----------------------------------------------------------------------*/
struct inner_stc *stc_new(len)
int len;
{
  struct inner_stc *ptr;

  if (!(ptr = (struct inner_stc *) malloc(sizeof(*ptr))))
    stx_err("memory request failed", __LINE__, __FILE__);
  if (!(ptr->stc = (int *) malloc((len+1) * sizeof(int))))
    stx_err("memory request failed", __LINE__, __FILE__);
  if (!(ptr->itpr = (char *) malloc((len+1) * sizeof(char))))
    stx_err("memory request failed", __LINE__, __FILE__);
  ptr->occured = FALSE;
  ptr->red = FALSE;
  ptr->left = stcdct_end;
  ptr->right = stcdct_end;
  return ptr;
}

/*---------------------------------------------------------------------

  stcdct_insert  --  insert a sentence into the sentence dictionary

    input  -- stc: pointer to a sentence (in inner form)
              itpr: a string pointing to the interpretation
              len: the length of the sentence
              ptr: a pointer to the start node 
              occured: indicate if the inserted node should be marked 
    output -- none

-----------------------------------------------------------------------*/
void stcdct_insert(stc, itpr, len, ptr, occured)
int  *stc;
char *itpr;
int  len;
struct inner_stc *ptr;
boolean occured;
{
  struct inner_stc *ggp, *gp, *p;
  int    cmp, i, *stc1, code;

  if (!(stc1 = (int *) malloc((len+1) * sizeof(int))))
    stx_err("memory request failed", __LINE__, __FILE__);
  for (i = 0; i < len; i++) { 
    if ((code = stc[i]) == UNDEF_code) { stc1[i] = -999; }
    else stc1[i] = code;
  }
  stc1[i] = STC_END;

  p = ptr;
  gp = ptr;

  do {
    ggp = gp;
    gp = p;
    p = ptr;
    
    if ((cmp = stc_cmp(stc1, ptr->stc)) == 0) {         /* same sentence */
      free(stc1);
      return; 
    }
    if (cmp == -1) {                                  /* stc1 < ptr->stc */
      ptr = ptr->left;
    }
    else {                                            /* stc1 > ptr->stc */
      ptr = ptr->right;
    }

    if (((ptr->left)->red) AND ((ptr->right)->red)) 
      ptr = stc_split(stc1, ggp, gp, p, ptr);
  } while (ptr != stcdct_end);

  ptr = stc_new(len);
  for (i = 0; i <= len; i++) {
    ptr->stc[i] = stc1[i];
  }
  strcpy(ptr->itpr, itpr);
  if (occured) ptr->occured = TRUE;
  
  if (stc_cmp(stc1, p->stc) == 1) {                     /* stc1 > p->stc */
    p->right = ptr; }
  else { p->left = ptr; }

  ptr = stc_split(stc1, ggp, gp, p, ptr);
  stc_total++;
  free(stc1);
}


/*---------------------------------------------------------------------

  stc_cmp  --  compare two sentences

    input  -- stc1: pointer to the 1st sentence
              stc2: pointer to the 2nd sentence
    output -- none

  function returns  1,   stc1 > stc2
                    0,   stc1 = stc2
                   -1,   stc1 < stc2

-----------------------------------------------------------------------*/
int stc_cmp(stc1, stc2)
int *stc1, *stc2;
{
  int i = 0;
  boolean stc1_end, stc2_end;

  stc1_end = (stc1[0] == STC_END);
  stc2_end = (stc2[0] == STC_END);

  while ((!stc1_end) AND (!stc2_end)) {
    int code1, code2;

    code1 = stc1[i];
    code2 = stc2[i++];

    if (code1 < code2) return -1;
    if (code1 > code2) return 1;

    stc1_end = (stc1[i] == STC_END);
    stc2_end = (stc2[i] == STC_END);
  }

  if (stc1_end AND stc2_end) return 0;

  if (stc1_end) return -1;
  return 1;
}
  

/*--------------------------------------------------------------------

  stc_split  --  split a 4-node in constructing a red-black stc tree

    input  -- stc: pointer to a sentence (in inner form)
              ggp: pointer to the great-grandparent
              gp:  pointer to the grandparent
              p:   pointer to the parent
              ptr: pointer to the node considered 
    output -- none

  function returns the pointer after spliting.

-----------------------------------------------------------------------*/
struct inner_stc *stc_split(stc, ggp, gp, p, ptr)
int *stc;
struct inner_stc *ggp, *gp, *p, *ptr;
{
  
  ptr->red = TRUE;
  (ptr->left)->red = FALSE;
  (ptr->right)->red = FALSE;

  if (p->red) {

    gp->red = TRUE;
    if ((stc_cmp(stc, gp->stc) * stc_cmp(stc, p->stc)) < 0)
      p = stc_rotate(stc, gp);

    ptr = stc_rotate(stc, ggp);
    ptr->red = FALSE;
  }
  
  (stcdct_head->right)->red = FALSE;
  return ptr;
}



/*--------------------------------------------------------------------

  stc_rotate  --  rotate nodes in constructing a red-black stc tree

    input  -- stc: a pointer to a sentence
              ptr: a pointer to a node
    output -- none

  function returns a pointer to a node after rotation

----------------------------------------------------------------------*/
struct inner_stc *stc_rotate(stc, ptr)
int *stc;
struct inner_stc *ptr;
{
  struct inner_stc *c, *gc;

  if (stc_cmp(stc, ptr->stc) == -1) { c = ptr->left; }
  else c = ptr->right;

  if (stc_cmp(stc, c->stc) == -1) {
    gc = c->left; c->left = gc->right; gc->right = c; }
  else {
    gc = c->right; c->right = gc->left; gc->left = c; }

  if (stc_cmp(stc, ptr->stc) == -1) { ptr->left = gc; }
  else ptr->right = gc;

  return gc;
}


/*--------------------------------------------------------------------
  
  stcdct_search  --  search a sentence in the sentence dictionary

    input  -- stc: stc: pointer to a sentence (in inner form)
              len: the length of the sentence
    output -- none

  function returns TRUE if the stc is found, FALSE otherwise

---------------------------------------------------------------------*/
boolean stcdct_search(stc, len)
int *stc, len;
{
  int i, *stc1, code;
  struct inner_stc *ptr;

  if (!(stc1 = (int *) malloc((len+1) * sizeof(int))))
    stx_err("memory request failed", __LINE__, __FILE__);
  for (i = 0; i < len; i++) { 
    if ((code = stc[i]) == UNDEF_code) { stc1[i] = -999; }
    else stc1[i] = code;
  }
  stc1[i] = STC_END;

  ptr = stcdct_head;

  while (ptr != stcdct_end) {
    int  cmp;

    cmp = stc_cmp(stc1, ptr->stc);
    if (cmp == 0) { ptr->occured = TRUE; free(stc1); return TRUE; }
    if (cmp == -1) { ptr = ptr->left; continue; }
    ptr = ptr->right;
  }
  free(stc1);
  return FALSE;
}
  

/*---------------------------------------------------------------------

  save_stc  --  save the sentence after a good interpretation was found

    input  -- none
    output -- none

----------------------------------------------------------------------*/
void save_stc()
{
  stcdct_insert(original_sentence, sentence_intprt, original_sentence_len, 
                stcdct_head, TRUE);
}



/*---------------------------------------------------------------------

  save_intprt  --  save the interpretation in sentence_intprt

    input  -- stc:  a sentence
              slen:  sentence length
    output -- none

-----------------------------------------------------------------------*/
void save_intprt(stc, slen)
struct Syntx_Term *stc;
int    slen;
{
  int  i;

  for (i = 0; i < slen; i++) {
    int  begin, end, j, k;

    if (EQUAL(stc[i].phrase,"")) continue;
    begin = stc[i].begin;
    end = stc[i].end;
    k = 0;
    for (j = begin; j <= end; j++) 
      sentence_intprt[j] = stc[i].usedptrns[k++];
  }
}


/*----------------------------------------------------------------------

  read_text_stc -- copy user sentence file from disk to memory, create 
                   the sentence dictionary
       
    input  -- none
    output -- none

----------------------------------------------------------------------*/   
void read_text_stc()
{
  char   line[STCLEN], ch, text_stc[124];

  FILE   *stx;

  stcdct_init();

  sprintf(text_stc, "%s.stc", text_name);
  if ((stx = fopen(text_stc, READ)) == NULL) {
/*----- added on 04/06/97 -----*/
/*                             */ 
    if (display_syntx_msg) {
      char text_msg[120];
  
      first_time = TRUE;
      display_syntx_msg = FALSE;

      sprintf(text_msg, "%s.err", text_name);
      syntx_msg = fopen(text_msg, WRITE);
    }
/*                             */
/*-----------------------------*/
    return;
  }

  fscanf(stx, "%[^\n]", line);
  
  if( strlen(line) > (STCLEN -1) )
    {  printf("\n ERROR: line exceeds STCLEN");
       printf("\n         Increase STCLEN\n");
       exit(0);
     }
     
  ch = getc(stx);
  while (!feof(stx)) {
    int  stc_len, stc[100];

    charseq_to_intseq(line, stc, &stc_len);

    fscanf(stx, "%[^\n]", line);
    if( strlen(line) > (STCLEN -1) )
    {  printf("\n ERROR: line exceeds STCLEN");
       printf("\n         Increase STCLEN");
       exit(0);
     }
     
    ch = getc(stx);

    line[stc_len] = '\0';

    stcdct_insert(stc, line, stc_len, stcdct_head, FALSE);

    fscanf(stx, "%[^\n]", line);
    if( strlen(line) > (STCLEN -1) )
    {  printf("\n ERROR: line exceeds STCLEN");
       printf("\n         Increase STCLEN");
       exit(0);
     }
     
    ch = getc(stx);
  }

  fclose(stx);
}


/*--------------------------------------------------------------------

  charseq_to_intseq  --  transfer a sequence of number in character 
                         formate into a sequence of number in integer
                         formate.

    input  -- char_seq: a string contains the number sequence seperated 
                        by space
    output -- int_seq:  an integer array contains the transfered numbers
              size:     the size of int_seq

----------------------------------------------------------------------*/
void charseq_to_intseq(char_seq, int_seq, size)
char *char_seq;
int  *int_seq, *size;
{
  int  len, num = 0, i, j;
  char str[10], ch;

  len = strlen(char_seq);
  j = 0;
  for (i = 0; i <= len; i++) {
    ch = char_seq[i];
    if ((ch != ' ') AND (ch != '\0')) { str[j++] = ch; continue; }
    if (j > 0) {
      str[j] = '\0';
      int_seq[num++] = atoi(str);
      j = 0;
    }
  }
  *size = num;
}


/*--------------------------------------------------------------------

  write_text_stc  --  copy the updated sentence dictionary back to the
                      disk,

    input  -- none
    output -- none

----------------------------------------------------------------------*/
void write_text_stc()
{
  struct inner_stc *stc_ptr;
  struct stc_queue *head, *end;
  char   text_stc[124];
  FILE  *textstc;

  
  sprintf(text_stc, "%s.stc", text_name);
  if ((textstc = fopen(text_stc, WRITE)) == NULL) {
    sprintf(err_msg, "failed to open %s", text_stc);
    stx_err(err_msg, __LINE__, __FILE__);
  }

  if (!(head = (struct stc_queue *) malloc(sizeof(*head))))
    stx_err("memory request failed", __LINE__, __FILE__);
  head->ptr = stcdct_head;
  head->next = NULL;
  end = head;

  while (head != NULL) {
    int i;
    struct stc_queue *tmp;

    stc_ptr = head->ptr;
    if (stc_ptr->occured) {         /* the sentence occured in this text */
      if (stc_ptr->stc[0] != STC_END) {
        fprintf(textstc, "%d", stc_ptr->stc[0]);
        i = 1;
      }
      else { i = 0; }

      while (stc_ptr->stc[i] != STC_END) 
        fprintf(textstc, " %d", stc_ptr->stc[i++]);
      if (i > 0) { 
        fprintf(textstc, "\n");
        fprintf(textstc, "%s\n", stc_ptr->itpr);
      }
    }

    if (stc_ptr->left != stcdct_end) {
      tmp = end;
      if (!(end = (struct stc_queue *) malloc(sizeof(*end))))
        stx_err("memory request failed", __LINE__, __FILE__);
      end->ptr = stc_ptr->left;
      end->next = NULL;
      tmp->next = end;
    }
    if (stc_ptr->right != stcdct_end) {
      tmp = end;
      if (!(end = (struct stc_queue *) malloc(sizeof(*end))))
        stx_err("memory request failed", __LINE__, __FILE__);
      end->ptr = stc_ptr->right;
      end->next = NULL;
      tmp->next = end;
    }
    tmp = head;
    head = head->next;
    free(tmp);
    free_stcnode(stc_ptr);
  }
  free_stcnode(stcdct_end);
  fclose(textstc);
}
   
  
  
/*---------------------------------------------------------------------

  find_stc  --  check if the sentence dictionary contains the 
                current sentence.

    input  -- none
    output -- none

  function returns TRUE if the dictionary contains the one, 
  FALSE otherwise.

----------------------------------------------------------------------*/
boolean find_stc()
{
  return(stcdct_search(original_sentence, original_sentence_len));
}





/*****************************************************************/

void print_node(ptr)
struct inner_stc *ptr;
{
  int i = 0;

  printf("stc: ");
  while (ptr->stc[i] != STC_END) printf("%d ", ptr->stc[i++]);
  printf("\n");

  printf("occured: %d\n", ptr->occured);
  printf("red: %d\n\n", ptr->red);
}

void print_stcdct()
{
  struct inner_stc *ptr, *que[100];
  int s = 0, e = 0, level[100], l = 0;

  printf("\ntotal sentences: %ld\n", stc_total);
  que[0] = stcdct_head;
  level[0] = l;
  e = 1;
  while (s < e) {
    ptr = que[s];
    l = level[s] + 1;
    printf("at level %d\n", level[s++]);
    print_node(ptr);
    if (ptr->left != stcdct_end) { que[e] = ptr->left; level[e++] = l; }
    if (ptr->right != stcdct_end) { que[e] = ptr->right; level[e++] = l; }
  }
}
  


/*---------------------------------------------------------------------

  find_errtype  --  find error type of a sentence by doing syntax analysis
                    starting at "find_predicate" (skip "simplify")

    input  -- none
    output -- none

----------------------------------------------------------------------*/
void find_errtype()
{
  int cmd_code, pos, lklhd, who, stc_len, i;
  struct Syntx_Term *stc; 
  struct SPOC_Pos    spoc;

  syntx_intprt = FALSE;

  for (i = VERYFREQLY; i <= RARELY; i++) {

    if (syntx_intprt) break;

    if (get_stc_ptrns(i)) {

      sentence_spoc.sub_pos = -1;
      sentence_spoc.prd_pos = -1;
      sentence_spoc.obj_pos = -1;
      sentence_spoc.cpl_pos = -1;

      stc_push(sentence, sentence_len, &sentence_spoc, FindPredicate, 0, 
               class_likelihood[i], Simplify);

      stc = (struct Syntx_Term *)stc_cpy(&stc_len, &spoc, sentence,
                                         sentence_len, &sentence_spoc);

      while (cmd_code = next_move(&pos, &lklhd, &who)) {

        syntax_analysis(cmd_code, pos, lklhd, who);
      }
    
      sentence = (struct Syntx_Term *)stc_cpy(&sentence_len, 
                 &sentence_spoc, stc, stc_len, &spoc);    
    }
  }
}


/*----------------------------------------------------------------------

  init_stxerrlist  --  initialize stxerr_list

    input  -- none
    output -- none

------------------------------------------------------------------------*/
void init_stxerrlist()
{
  struct syntax_err *ptr, *tmp;

  if (stxerr_list == NULL) goto creat;

  ptr = stxerr_list->next;
  while (ptr != NULL) {
    tmp = ptr->next;
    free(ptr->stc);
    free(ptr);
    ptr = tmp;
  }
  /*  free(stxerr_list); */ /* deleted Sept 9, 2005*/
  stxerr_list->lh = 1000;   /* added Sept 9, 2005*/
  stxerr_list->next = NULL; /* added Sept 9, 2005*/
  return;                   /* added Sept 9, 2005*/

creat:
  if (!(stxerr_list = (struct syntax_err *) malloc(sizeof(*ptr))))
    stx_err("memory request failed", __LINE__, __FILE__);
  stxerr_list->lh = 1000;
  stxerr_list->next = NULL;
}

      
/*----------------------------------------------------------------------

  save_stxerr -- put a syntax error into stxerr_list, keep the decreasing 
                 order. 
    
    input  -- msg:   error message
              lh:    lilkelihood of the error message 
              stc:   sentence to be saved
              slen:  sentence length
              spoc:  positions of SPOC
    output -- none

----------------------------------------------------------------------*/
void save_stxerr(msg, lh, stc, slen, spoc)
char *msg;
int  lh;
struct Syntx_Term *stc;
int  slen;
struct SPOC_Pos *spoc;
{
  struct syntax_err *ptr, *old, *tmp;

  ptr = stxerr_list;
  while (ptr != NULL) {
    if (lh > (ptr->lh)) break;
    old = ptr;
    ptr = ptr->next;
  }
  if (!(old->next = tmp = (struct syntax_err *) malloc(sizeof(*tmp))))
      stx_err("memory request failed", __LINE__, __FILE__); 
  strcpy(tmp->errmsg, msg);
  tmp->lh = lh;
  tmp->stc = (struct Syntx_Term *)stc_cpy(&(tmp->stc_len), 
                                        &(tmp->stc_spoc), stc, slen, spoc);
  tmp->next = ptr;
}


/*----------------------------------------------------------------------

  get_errmsg  --  get the most likely error message

    input  -- none
    output -- msg: return the message

-----------------------------------------------------------------------*/
void get_errmsg(msg)
char *msg;
{
  struct syntax_err *ptr;

  ptr = stxerr_list->next;
  if (ptr == NULL) { strcpy(msg, ""); return; }
  else {
    strcpy(msg, ptr->errmsg);
  }
  
  if (intrnl_dsply) {
    sentence_check(ptr->stc, ptr->stc_len, &(ptr->stc_spoc));
  }
}




/*---------------------------------------------------------------------

  triple_init  --  initialize the triple dictionary

    input  -- none
    output -- none

----------------------------------------------------------------------*/
void triple_init()
{
  triple_end = NULL;
  triple_end = triple_new();
  triple_end->left = triple_end;
  triple_end->right = triple_end;
  triple_end->red = FALSE;
  triple_head = triple_new();
  triple_head->trp[0] = TRP_END;
  triple_total = 0;
}


/*----------------------------------------------------------------------

  free_trpnode  --  free a triple node in triple tree.

    input  -- ptr: a pointer to the node
    output -- none

------------------------------------------------------------------------*/
void free_trpnode(ptr)
struct inner_triple *ptr;
{
  free(ptr);
}


/*----------------------------------------------------------------------

  triple_new  --  allocate memory for a new triple

    input  -- none
    output -- none

  function returns a pointer to the memory

-----------------------------------------------------------------------*/
struct inner_triple *triple_new()
{
  struct inner_triple *ptr;

  if (!(ptr = (struct inner_triple *) malloc(sizeof(*ptr))))
    stx_err("memory request failed", __LINE__, __FILE__);
  ptr->freq = 0;
  ptr->red = FALSE;
  ptr->left = triple_end;
  ptr->right = triple_end;
  return ptr;
}


/*---------------------------------------------------------------------

  triple_insert  --  insert a triple into the triple dictionary

    input  -- trp: pointer to a triple (in inner form)
              freq: the frequency of the triple
              ptr: a pointer to the start node 
    output -- none

-----------------------------------------------------------------------*/
void triple_insert(trp, freq, ptr)
int  *trp;
int  freq;
struct inner_triple *ptr;
{
  struct inner_triple *ggp, *gp, *p;
  int    cmp, i, trp1[4], code;

  for (i = 0; i < 3; i++) { 
    if ((code = trp[i]) == UNDEF_code) { trp1[i] = -999; }
    else trp1[i] = code;
  }
  trp1[i] = TRP_END;

  p = ptr;
  gp = ptr;

  do {
    ggp = gp;
    gp = p;
    p = ptr;
    
    if ((cmp = triple_cmp(trp1, ptr->trp)) == 0) {        /* same triple */
      return; 
    }
    if (cmp == -1) {                                  /* trp1 < ptr->trp */
      ptr = ptr->left;
    }
    else {                                            /* trp1 > ptr->trp */
      ptr = ptr->right;
    }

    if (((ptr->left)->red) AND ((ptr->right)->red)) 
      ptr = trp_split(trp1, ggp, gp, p, ptr);
  } while (ptr != triple_end);

  ptr = triple_new();
  for (i = 0; i < 4; i++) {
    ptr->trp[i] = trp1[i];
  }
  ptr->freq = freq;
  
  if (triple_cmp(trp1, p->trp) == 1) {                  /* trp1 > p->trp */
    p->right = ptr; }
  else { p->left = ptr; }

  ptr = trp_split(trp1, ggp, gp, p, ptr);
  triple_total++;
}


/*---------------------------------------------------------------------

  triple_cmp  --  compare two triples

    input  -- trp1: pointer to the 1st triple
              trp2: pointer to the 2nd triple
    output -- none

  function returns  1,   trp1 > trp2
                    0,   trp1 = trp2
                   -1,   trp1 < trp2

-----------------------------------------------------------------------*/
int triple_cmp(trp1, trp2)
int *trp1, *trp2;
{
  int i = 0;
  boolean trp1_end, trp2_end;

  trp1_end = (trp1[0] == TRP_END);
  trp2_end = (trp2[0] == TRP_END);

  while ((!trp1_end) AND (!trp2_end)) {
    int code1, code2;

    code1 = trp1[i];
    code2 = trp2[i++];

    if (code1 < code2) return -1;
    if (code1 > code2) return 1;

    trp1_end = (trp1[i] == TRP_END);
    trp2_end = (trp2[i] == TRP_END);
  }

  if (trp1_end AND trp2_end) return 0;

  if (trp1_end) return -1;
  return 1;
}
  

/*--------------------------------------------------------------------

  trp_split  --  split a 4-node in constructing a red-black trp tree

    input  -- trp: pointer to a triple (in inner form)
              ggp: pointer to the great-grandparent
              gp:  pointer to the grandparent
              p:   pointer to the parent
              ptr: pointer to the node considered 
    output -- none

  function returns the pointer after spliting.

-----------------------------------------------------------------------*/
struct inner_triple *trp_split(trp, ggp, gp, p, ptr)
int *trp;
struct inner_triple *ggp, *gp, *p, *ptr;
{
  
  ptr->red = TRUE;
  (ptr->left)->red = FALSE;
  (ptr->right)->red = FALSE;

  if (p->red) {

    gp->red = TRUE;
    if ((triple_cmp(trp, gp->trp) * triple_cmp(trp, p->trp)) < 0)
      p = trp_rotate(trp, gp);

    ptr = trp_rotate(trp, ggp);
    ptr->red = FALSE;
  }
  
  (triple_head->right)->red = FALSE;
  return ptr;
}

/*--------------------------------------------------------------------

  trp_rotate  --  rotate nodes in constructing a red-black trp tree

    input  -- trp: a pointer to a triple
              ptr: a pointer to a node
    output -- none

  function returns a pointer to a node after rotation

----------------------------------------------------------------------*/
struct inner_triple *trp_rotate(trp, ptr)
int *trp;
struct inner_triple *ptr;
{
  struct inner_triple *c, *gc;

  if (triple_cmp(trp, ptr->trp) == -1) { c = ptr->left; }
  else c = ptr->right;

  if (triple_cmp(trp, c->trp) == -1) {
    gc = c->left; c->left = gc->right; gc->right = c; }
  else {
    gc = c->right; c->right = gc->left; gc->left = c; }

  if (triple_cmp(trp, ptr->trp) == -1) { ptr->left = gc; }
  else ptr->right = gc;

  return gc;
}


/*--------------------------------------------------------------------
  
  triple_search  --  search a triple in the triple dictionary

    input  -- trp: pointer to a triple (in inner form)
    output -- none

  function returns TRUE if the trp is found, FALSE otherwise

---------------------------------------------------------------------*/
boolean triple_search(trp)
int *trp;
{
  int i, trp1[4], code;
  struct inner_triple *ptr;

  for (i = 0; i < 3; i++) { 
    if ((code = trp[i]) == UNDEF_code) { trp1[i] = -999; }
    else trp1[i] = code;
  }
  trp1[i] = TRP_END;

  ptr = triple_head;

  while (ptr != triple_end) {
    int  cmp;

    cmp = triple_cmp(trp1, ptr->trp);
    if (cmp == 0) { (ptr->freq)++; return TRUE; }
    if (cmp == -1) { ptr = ptr->left; continue; }
    ptr = ptr->right;
  }
  return FALSE;
}
  

/*----------------------------------------------------------------------

  read_triple_usr -- copy user triple file from disk to memory, create 
                     the triple dictionary.
       
    input  -- none
    output -- none

----------------------------------------------------------------------*/   
void read_triple_usr()
{
  char   line[200], ch;
  int    num, i;

  FILE   *trp_usr;

  triple_init();
  new_triple_usr = FALSE;

open_triple:

  if ((trp_usr = fopen(triple_usr, READ)) == NULL) {
    char str[120];

    printf("\nCannot find file = %s\n", triple_usr);
    printf("Press return to confirm creation of new file, or type another file name.\n");
    printf("\nCONFIRM >> ");
    
    gets(str);
    if (strcmp(str, "") != 0)
    {
      strcpy(triple_usr, str);
      goto open_triple;
    }
    new_triple_usr = TRUE;
    return;
  }

  fscanf(trp_usr, "%[^\n]", line);
  ch = getc(trp_usr);
  get_numoftriples(line, &num);
  
  for (i = 0; i < num; i++) {
    int  trp[4], len;

    fscanf(trp_usr, "%[^\n]", line);
    ch = getc(trp_usr);
    charseq_to_intseq(line, trp, &len);

    triple_insert(trp, trp[3], triple_head);
  }
  fclose(trp_usr);
}


/*---------------------------------------------------------------------
  
  get_numoftriples  --  return the number of triples in triple dictionary.

    input  -- title: the first line in the triple dictionary
    output -- num: the number of triples in user dictionary

-----------------------------------------------------------------------*/
void get_numoftriples(title, num)
char *title;
int  *num;
{
  char  number[120];
  int   len, i, k = 0;

  len = strlen(title);
  for (i = 0; i < len; i++) 
    if (title[i] == '=') break;
  i++;
  while (i < len) number[k++] = title[i++];
  number[k] = '\0';
  *num = atoi(number);
} 


/*---------------------------------------------------------------------

  save_triple  --  save the triple

    input  -- trp: the triple to be saved
    output -- none

----------------------------------------------------------------------*/
void save_triple(trp)
int *trp;
{
  triple_insert(trp, 2, triple_head);
}


/*--------------------------------------------------------------------

  write_triple_usr  --  copy the updated triple dictionary back to the 
                        disk

    input  -- none
    output -- none

----------------------------------------------------------------------*/
void write_triple_usr()
{
  struct inner_triple *trp_ptr;
  struct trp_queue *head, *end;
  char   title[80], cmd[130];
  FILE  *trp_usr;

/*--------- 042797 ----------
  sprintf(title, "%s Number of Triples = %ld\n", VERSION21, triple_total);
---------------------------*/

/*--------- 110398 ----------
  sprintf(title, "%s Number of Triples = %ld\n", VERSION22, triple_total);
---------------------------*/

/*-----------ver3.0------------
  sprintf(title, "%s Number of Triples = %ld\n", VERSION30, triple_total);
---------end-ver3.0----------*/

/*-----------ver7.0-122004----------*/
  sprintf(title, "%s Number of Triples = %ld\n", VERSION70, triple_total);
/*---------end-ver7.0----------*/



  if (triple_total == 0) {
    if ((trp_usr = fopen(triple_usr, WRITE)) == NULL) {
      sprintf(err_msg, "failed to open %s", triple_usr);
      stx_err(err_msg, __LINE__, __FILE__);
    }
    fprintf(trp_usr, "%s", title);
    fclose(trp_usr);
    return;
  }

  if (!new_triple_usr) {
    sprintf(cmd, "mv -f %s %s", triple_usr, triple_bak);
    system(cmd);
  }

  if ((trp_usr = fopen(triple_usr, WRITE)) == NULL) {
    sprintf(err_msg, "failed to open %s", triple_usr);
    stx_err(err_msg, __LINE__, __FILE__);
  }

  fprintf(trp_usr, "%s", title);
  if (!(head = (struct trp_queue *) malloc(sizeof(*head))))
    stx_err("memory request failed", __LINE__, __FILE__);
  head->ptr = triple_head;
  head->next = NULL;
  end = head;

  while (head != NULL) {
    int i, freq;
    struct trp_queue *tmp;

    trp_ptr = head->ptr;
    if (trp_ptr->trp[0] != TRP_END) {
      for (i = 0; i < 3; i++) fprintf(trp_usr, "%d ", trp_ptr->trp[i]);
      freq = trp_ptr->freq - 1;
      if (freq > 1000) freq = 1000;
      if (freq < -1000) freq = -1000;
      fprintf(trp_usr, "%d\n", freq);
    }

    if (trp_ptr->left != triple_end) {
      tmp = end;
      if (!(end = (struct trp_queue *) malloc(sizeof(*end))))
        stx_err("memory request failed", __LINE__, __FILE__);
      end->ptr = trp_ptr->left;
      end->next = NULL;
      tmp->next = end;
    }
    if (trp_ptr->right != triple_end) {
      tmp = end;
      if (!(end = (struct trp_queue *) malloc(sizeof(*end))))
        stx_err("memory request failed", __LINE__, __FILE__);
      end->ptr = trp_ptr->right;
      end->next = NULL;
      tmp->next = end;
    }
    tmp = head;
    head = head->next;
    free(tmp);
    free_trpnode(trp_ptr);
  }
  free_trpnode(triple_end);
  fclose(trp_usr);
}
   
  
/*---------------------------------------------------------------------

  find_triple  --  check if the triple dictionary contains the triple.

    input  -- trp: pointer to a triple
    output -- none

  function returns TRUE if the dictionary contains the one, 
  FALSE otherwise.

----------------------------------------------------------------------*/
boolean find_triple(trp)
int *trp;
{
  return(triple_search(trp));
}

