#include "syntx.h"

#define        NumofMdfChkDgns    14

extern struct Syntx_Term *sentence; 
extern int                sentence_len;
extern struct SPOC_Pos    sentence_spoc;
extern boolean            syntx_intprt;
extern int                system_status;
extern boolean            intrnl_dsply;



DIAGNOSIS_ARRAY mdfchk_dgns[NumofMdfChkDgns + 1] = { 
                                                     "err_double_sub",
                                                     "err_double_compl",
                                                     "err_compl_missing",
                                                     "err_compl_disagree",
                                                     "psblerr_incompl_to",
                                                     "have_compl",
                                                     "cmb_lft",
                                                     "cmb_rht",
                                                     "is_phrs(noun_sgl)",
                                                     "is_phrs(noun_pl)",
                                                     "is_phrs(vt_ing)",
                                                     "is_phrs(vi_ing)",
                                                     "is_phrs(to_phrs)",
                                                     "is_phrs(let_cmpl)",
                                                     ""
                                                   };

static int        symptoms_idx;
SYMPTOM_ARRAY     symptoms[mmax(MAXPTRNS+9,MAXSYMP)];
boolean           conj_occured = FALSE;

void    modifier_checker();
int     do_mdfcheck();
  
boolean legal_modify_phrs();
boolean mdfchk_lftptrn();
boolean mdfchk_lftphrs();
boolean mdfchk_cntphrs();
boolean mdfchk_rhtphrs();
boolean mdfchk_subphrs();
boolean mdfchk_prdctphrs();
boolean conj_phrs();


/*----------------------------------------------------------------------

  modifier_checker  --  control the action of do_mdfcheck

    input  -- pos:  the position where the action to be taken
              lklhd: the likelyhood at which the action will take
              who: indicate who invoke the function    
    output -- none
 
-----------------------------------------------------------------------*/
void modifier_checker(pos, lklhd, who)
int pos, lklhd, who;
{
  int  check_code, next_pos, i;
  char syntx_err[100];

  if (who == SPOChecker) {
    sentence_spoc.cpl_pos = -1;
    if (system_status == FindError) { 
      save_stc();
      clean_stack();
      return; 
    }
  }

  if (intrnl_dsply) {
    begin_title("modifier_checker", pos, lklhd);
  }
  
  if (pos < 0) goto end_modifier_checker;

  load_prb("ModifierChecker");
  while (pos <= sentence_len) {
    check_code = do_mdfcheck(pos, lklhd, &next_pos);
    switch(check_code) {
      case 0:                                    /* nothing is concluded */
              pos++;
              break;

      case 1:                                       /* is_phrs(noun_sgl) */
      case 2:                                        /* is_phrs(noun_pl) */
      case 3:                                         /* is_phrs(vt_ing) */
      case 4:                                         /* is_phrs(vi_ing) */
      case 5:                                        /* is_phrs(to_phrs) */
      case 11:                                       /* is_phrs(let_cmpl)*/
              pos = next_pos;
              break;

      case 6:                                          /* err_double_sub */
              strcpy(syntx_err, "err_double_sub");
              save_stxerr(syntx_err, lklhd, sentence, sentence_len,
                          &sentence_spoc);
              goto end_modifier_checker;

      case 7:                                        /* err_double_compl */
              strcpy(syntx_err, "err_double_compl");
              save_stxerr(syntx_err, lklhd, sentence, sentence_len,
                          &sentence_spoc);
              goto end_modifier_checker;

      case 8:                                      /* err_compl_disagree */
              strcpy(syntx_err, "err_compl_disagree");
              save_stxerr(syntx_err, lklhd, sentence, sentence_len,
                          &sentence_spoc);
              goto end_modifier_checker;

      case 9:                                       /* err_compl_missing */
              strcpy(syntx_err, "err_compl_missing");
              save_stxerr(syntx_err, lklhd, sentence, sentence_len,
                          &sentence_spoc);
              goto end_modifier_checker;

      case 10:                                     /* psblerr_incompl_to */
              strcpy(syntx_err, "psblerr_incompl_to");
              save_stxerr(syntx_err, lklhd, sentence, sentence_len,
                          &sentence_spoc);
              goto end_modifier_checker;


      default: stx_err("unexpected code by do_mdfcheck",__LINE__,__FILE__);
    }
  }

  for (i = 0; i < sentence_len; i++) {
    int  lh;
    char word[WORD_MAX+1];

    if ((i == sentence_spoc.sub_pos) OR (i == sentence_spoc.prd_pos) OR 
        (i == sentence_spoc.obj_pos) OR (i == sentence_spoc.cpl_pos))  
      continue;   
    if (is_word(sentence, sentence_len, i)) {

      get_word(sentence, sentence_len, i, word);
      if (clause_conj(word)) continue;
      if (EQUAL(word,"that")) break;

      sprintf(syntx_err, "err_word_left(%s)", word);
      save_stxerr(syntx_err, lklhd, sentence, sentence_len,&sentence_spoc);
      goto end_modifier_checker;
    }
    if (conj_phrs(sentence, sentence_len, i)) break;
    if (!legal_modify_phrs(sentence, sentence_len, i)) {
      strcpy(syntx_err, "err_wrong_phrs");
      save_stxerr(syntx_err, lklhd, sentence, sentence_len,&sentence_spoc);
      goto end_modifier_checker;
    }
  }

update_classlh(sentence, sentence_len);
save_intprt(sentence, sentence_len);
save_stc();
clean_stack();
syntx_intprt = TRUE;

  if (intrnl_dsply) {
    printf("\n\nThis is a GOOD sentence\n");
    sentence_check(sentence, sentence_len);
    printf("subject at %d, predicate at %d, object at %d, complement at %d\n\n", sentence_spoc.sub_pos, sentence_spoc.prd_pos, 
         sentence_spoc.obj_pos, sentence_spoc.cpl_pos);
  } 
/*print_dict();
print_stcdct();*/

end_modifier_checker:  
  if (intrnl_dsply) {
    end_title("modifier_checker");
  }
}



/*---------------------------------------------------------------------

  do_mdfcheck  --  execute modifier_checker.prg  

    input  -- pos: the position to be checked
              lklhd: the likelihood at which do_mdfcheck works
    output -- next_pos: the next position to be checked in case of 
                        is_phrs(XXX)

  function returns 0: nothing is concluded
                   1: is_phrs(noun_sgl) 
                   2: is_phrs(noun_pl)
                   3: is_phrs(vt_ing)
                   4: is_phrs(vi_ing)
                   5: is_phrs(to_phrs)
                   6: err_double_sub
                   7: err_double_compl
                   8: err_compl_disagree
                   9: err_compl_missing
                  10: psblerr_incompl_to
                  11: is_phrs(let_cmpl)

----------------------------------------------------------------------*/
int do_mdfcheck(pos, lklhd, next_pos)
int pos, lklhd, *next_pos;
{
  DIAGNOSIS_ARRAY   dgns[NumofMdfChkDgns + 1];
  char              dgns_var[VAR_MAX + 1], phrase[PTRN_PHRS_MAX+1],
                    ptrn_names[MAXPTRNS+1][PTRN_PHRS_MAX+1],
                    ptrn[PTRN_PHRS_MAX+1],
                    used_ptrn[3][PTRN_PHRS_MAX+1] = {"", "", ""};
  int               sub, prdct, obj, compl, cnt, lft, rht, return_val,
                    i, lh, lklhds[MAXPTRNS];
  boolean           lftstop;

  symptoms_idx = 0;
  sub = sentence_spoc.sub_pos;
  prdct = sentence_spoc.prd_pos;
  obj = sentence_spoc.obj_pos;
  compl = sentence_spoc.cpl_pos;
  lft = pos - 1;
  cnt = pos;
  rht = pos + 1;

  if (((cnt == sub) AND (sub != prdct))  OR 
      (cnt == obj)                       OR
      (cnt == compl))  return 0;

  if (is_word(sentence, sentence_len, cnt)) return 0;

  if (is_word(sentence, sentence_len, lft)) {
    get_ptrn_names(sentence, sentence_len, lft, ptrn_names, lklhds);
    i = 0;
    strcpy(ptrn, ptrn_names[i++]);
    while (!EQUAL(ptrn,"")) {
      if (mdfchk_lftptrn(ptrn)) {
        sprintf(symptoms[symptoms_idx++], "lft_ptrn(%s)", ptrn);
        if (EQUAL(used_ptrn[0],"")) strcpy(used_ptrn[0], ptrn);
      }
      strcpy(ptrn, ptrn_names[i++]);
    }
    if (has_adjpron(sentence, sentence_len, lft, &lh)) {
      sprintf(symptoms[symptoms_idx++], "lft_ptrn(pron_adj)");
      strcpy(used_ptrn[0], "pron_adj");
    }
    if (conj_occured) { 
      sprintf(symptoms[symptoms_idx++], "conj_occured");
    }
    else {
      char word[WORD_MAX+1];
  
      get_word(sentence, sentence_len, lft, word);
      if (clause_conj(word) OR EQUAL(word,"that")) {
        sprintf(symptoms[symptoms_idx++], "conj_occured");
        conj_occured = TRUE;
      }
    }
  }
  else {
    if (mdfchk_lftphrs(sentence, sentence_len, lft, phrase))  
      sprintf(symptoms[symptoms_idx++], "lft_phrs(%s)", phrase);
    if (conj_occured) { 
      sprintf(symptoms[symptoms_idx++], "conj_occured");
    }
    else {
      if (conj_phrs(sentence, sentence_len, lft)) {
        sprintf(symptoms[symptoms_idx++], "conj_occured");
        conj_occured = TRUE;
      }
    }
  }

  
  if (cnt == sentence_spoc.prd_pos) { 
    sprintf(symptoms[symptoms_idx++], "cnt_is_prdct");
  }
  else {
    if (mdfchk_cntphrs(sentence, sentence_len, cnt, phrase))  
      sprintf(symptoms[symptoms_idx++], "cnt_phrs(%s)", phrase);
  }
  if (conj_phrs(sentence, sentence_len, cnt)) 
    sprintf(symptoms[symptoms_idx++], "cnt_conj_phrs");


  if (mdfchk_rhtphrs(sentence, sentence_len, rht, phrase, &lftstop)) { 
  /* if statement is added on Nov-10-98 to exclude case "rht_phrs(pron_thr)" */
  if( strcmp(phrase,"pron_thr") != 0 )
   {
      sprintf(symptoms[symptoms_idx++], "rht_phrs(%s)", phrase);
      if (lftstop) sprintf(symptoms[symptoms_idx++], "rht_lftstop");
    }
  }
  
  if ((cnt > prdct) AND (obj == -1) AND (cnt < sentence_len))
    sprintf(symptoms[symptoms_idx++], "prdct_cnt");

  if ((cnt > obj) AND (obj != -1) AND (cnt < sentence_len))
    sprintf(symptoms[symptoms_idx++], "obj_cnt");

  if ((cnt == obj+1) AND (obj != -1) AND (cnt < sentence_len))
    sprintf(symptoms[symptoms_idx++], "cnt_next_to_obj");

  if (cnt == (sentence_len - 1)) 
    sprintf(symptoms[symptoms_idx++], "last_one");

  if (compl != -1) 
    sprintf(symptoms[symptoms_idx++], "prdct_compl_occured");

  if (sub == lft) {
    sprintf(symptoms[symptoms_idx++], "sub_at_lft");
  }

  if (sub == rht) {
    sprintf(symptoms[symptoms_idx++], "sub_at_rht");
  }

  if (mdfchk_subphrs(sentence, sentence_len, sub, phrase))
    sprintf(symptoms[symptoms_idx++], "subject(%s)", phrase);

  if (mdfchk_prdctphrs(sentence, sentence_len, prdct, phrase))
    sprintf(symptoms[symptoms_idx++], "prdct(%s)", phrase);


  strcpy(symptoms[symptoms_idx],"");


  mst_smp(symptoms, name, state, type, value, errcde);
  sol_prb(name, state, type, value, errcde);
  if (errcde[0] != 0) {
    stx_err("failed to solve problem", __LINE__, __FILE__);
  }

  if EQUAL(state, "S") { 
    copy_dgns(dgns, mdfchk_dgns);
    get_dgn(dgns,name,state,type,value,errcde);
    if (value[0]) {                      /* has at least one true value */
      boolean cmb_lft = FALSE, cmb_rht = FALSE;
      char    phrase_name[PTRN_PHRS_MAX+1] = "";
      char    clss[MAXPHRASE][PTRN_PHRS_MAX+1]; 

      i = 0;
      while (! EQUAL(dgns[i],"")) {
        strcpy(dgns_var, dgns[i++]);

        if (EQUAL(dgns_var,"err_double_sub")) {
          return_val = 6; goto end_do_mdfcheck;
        }

        if (EQUAL(dgns_var,"err_double_compl")) {
          return_val = 7; goto end_do_mdfcheck;
        }

        if (EQUAL(dgns_var,"err_compl_disagree")) {
          return_val = 8; goto end_do_mdfcheck;
        }

        if (EQUAL(dgns_var,"err_compl_missing")) {
          return_val = 9; goto end_do_mdfcheck;
        }

        if (EQUAL(dgns_var,"psblerr_incompl_to")) {
          return_val = 10; goto end_do_mdfcheck;
        }

        if (EQUAL(dgns_var,"cmb_lft")) { cmb_lft = TRUE; continue; }

        if (EQUAL(dgns_var,"cmb_rht")) { cmb_rht = TRUE; continue; }

        if (EQUAL(dgns_var,"have_compl")) {
          sentence_spoc.cpl_pos = cnt;
          return_val = 0;
          goto end_do_mdfcheck;
         }

         if (EQUAL(dgns_var,"is_phrs(noun_sgl)")) { 
          strcpy(phrase_name, "noun_sgl");
          return_val = 1;
          continue;
        }
        if (EQUAL(dgns_var,"is_phrs(noun_pl)")) { 
          strcpy(phrase_name, "noun_pl");
          return_val = 2;
          continue;
        }
        if (EQUAL(dgns_var,"is_phrs(vt_ing)")) { 
          strcpy(phrase_name, "vt_ing");
          return_val = 3;
          continue;
        }
        if (EQUAL(dgns_var,"is_phrs(vi_ing)")) { 
          strcpy(phrase_name, "vi_ing");
          return_val = 4;
          continue;
        }
        if (EQUAL(dgns_var,"is_phrs(to_phrs)")) { 
          strcpy(phrase_name, "to_phrs");
          return_val = 5;
          continue;
        }
        if (EQUAL(dgns_var,"is_phrs(let_cmpl)")) { 
          strcpy(phrase_name, "let_cmpl");
          return_val = 11;
          sentence_spoc.cpl_pos = lft;
          continue;
        }
      }
      if (EQUAL(phrase_name,"")) { return_val = 0; goto end_do_mdfcheck; }
  
      if (EQUAL(phrase_name,"to_phrs")) { *next_pos = pos-1; }
      else { *next_pos = pos; }

      if ((!cmb_lft) AND (!cmb_rht)) {
        if (intrnl_dsply) {
          printf("%d  ==>  phrase '%s'\n\n", lft, phrase_name);
        }    
        strcpy(clss[0], used_ptrn[0]);
        cmb_token(sentence, &sentence_len, &sentence_spoc, lft, lft, clss, 
                  phrase_name, lklhd);
        goto end_do_mdfcheck;
      }

      if (cmb_lft) { 
        if (intrnl_dsply) {
          printf("%d + %d  ==>  phrase '%s'\n\n", lft, cnt, phrase_name);
        }     
        strcpy(clss[0], used_ptrn[0]);
        strcpy(clss[1], used_ptrn[1]);
        cmb_token(sentence, &sentence_len, &sentence_spoc, lft, cnt, clss, 
                  phrase_name, lklhd);
        *next_pos = *next_pos - 1;
        cnt = cnt - 1;
        rht = rht - 1;
      }
      if (cmb_rht) { 
        if (intrnl_dsply) {
          printf("%d + %d  ==>  phrase '%s'\n\n", cnt, rht, phrase_name);
        }    
        strcpy(clss[0], used_ptrn[1]);
        strcpy(clss[1], used_ptrn[2]);
        cmb_token(sentence, &sentence_len, &sentence_spoc, cnt, rht, clss, 
                  phrase_name, lklhd);
      }
      goto end_do_mdfcheck;

    }
  }
  else stx_err("problem is unsatisfied", __LINE__, __FILE__);
  
  return_val = 0;

end_do_mdfcheck:

  strcpy(state, "A");
  rst_prb(name, state, type, value, errcde);
  return return_val;
}



  

/*---------------------------------------------------------------------

  legal_modify_phrs  --  check if a token is a legal modify phrase

    input  -- stc:  a sentence
              len:  sentence length
              idx:  token index
    output -- none

  function return TRUE if the token has such phrase, 
                  FALSE otherwise

-----------------------------------------------------------------------*/
boolean legal_modify_phrs(stc, len, idx)
struct Syntx_Term *stc;
int    len;
int    idx;
{
  int  lh;
  char str[4], phrs[PTRN_PHRS_MAX+1], lft_phrs[PTRN_PHRS_MAX+1],
       rht_phrs[PTRN_PHRS_MAX+1];

  get_phrs_name(stc, len, idx, phrs, &lh);

  str[0] = phrs[0];
  str[1] = phrs[1];
  str[2] = phrs[2];
  str[3] = '\0';

  if (EQUAL(str,"adv") OR
      EQUAL(str,"pre") OR
      EQUAL(str,"let"))   return TRUE;

  if (EQUAL(str,"nou")) {
    char word[WORD_MAX+1];

    if (is_word(stc, len, idx-1)) {
      get_word(stc, len, idx-1, word);
      if (clause_conj(word)) return TRUE;
      return FALSE;
    }

    get_phrs_name(stc, len, idx-1, lft_phrs, &lh);
    str[0] = lft_phrs[0];
    str[1] = lft_phrs[1];
    str[2] = lft_phrs[2];
    str[3] = '\0';
    if (EQUAL(str,"pre") OR EQUAL(str,"nou")) return TRUE;
  
    return FALSE;
  }

  if (EQUAL(phrs,"to_phrs")) return TRUE;

  if (EQUAL(phrs,"adj")) {
    get_phrs_name(stc, len, idx-1, lft_phrs, &lh);
    if (EQUAL(lft_phrs,"noun_sgl") OR
        EQUAL(lft_phrs,"noun_pl"))    {
      get_phrs_name(stc, len, idx+1, rht_phrs, &lh);
      if (EQUAL(rht_phrs,"")) return TRUE;
      str[0] = rht_phrs[0];
      str[1] = rht_phrs[1];
      str[2] = rht_phrs[2];
      str[3] = '\0';
      if (EQUAL(str,"pre") OR EQUAL(str,"to_")) return TRUE;
    }
  }

  return FALSE;
}


/*---------------------------------------------------------------------

  conj_phrs  --  check if a token is a conjuction phrase

    input  -- stc:  a sentence
              len:  sentence length
              idx:  token index
    output -- none

  function return TRUE if the token has such phrase, 
                  FALSE otherwise

-----------------------------------------------------------------------*/
boolean conj_phrs(stc, len, idx)
struct Syntx_Term *stc;
int    len;
int    idx;
{
  int  lh;
  char str[4], phrs[PTRN_PHRS_MAX+1], rht_phrs[PTRN_PHRS_MAX+1],
       word1[WORD_MAX+1], word2[WORD_MAX+1];

  get_phrs_name(stc, len, idx, phrs, &lh);
  get_1stwd_of_phrs(stc, len, idx, word1);
  get_lastwd_of_phrs(stc, len, idx, word2);

  if (EQUAL(word1,word2) AND 
      (EQUAL(word1,"that") OR EQUAL(word1,"which"))) {
    get_phrs_name(stc, len, idx+1, rht_phrs, &lh);
    if (EQUAL(rht_phrs,"")) return FALSE;
    str[0] = rht_phrs[0];
    str[1] = rht_phrs[1];
    str[2] = rht_phrs[2];
    str[3] = '\0';
    if (EQUAL(str,"nou") OR
        EQUAL(str,"adj") OR
        EQUAL(str,"vt_") OR
        EQUAL(str,"vi_") OR
        EQUAL(str,"am")  OR
        EQUAL(str,"is")  OR
        EQUAL(str,"are") OR
        EQUAL(str,"was") OR
        EQUAL(str,"wer") OR
        EQUAL(str,"pro") OR
        EQUAL(str,"hs_") OR
        EQUAL(str,"hv_") OR
        EQUAL(str,"hd_") OR
        EQUAL(str,"doe") OR
        EQUAL(str,"do_") OR
        EQUAL(str,"did") OR
        EQUAL(str,"aux") OR
        EQUAL(str,"auX") OR
        EQUAL(str,"pv_"))   return TRUE;
  }
  return FALSE;
}


/*---------------------------------------------------------------------

  mdfchk_lftptrn  --  check if left pattern is considered in the log file

    input  -- ptrn: left pattern name to be checked
    output -- none

  function returns TRUE if ptrn is considered in log file, FALSE otherwise

------------------------------------------------------------------------*/
boolean mdfchk_lftptrn(ptrn)
char *ptrn;
{
  if ((EQUAL(ptrn,"artcl_df")) OR
      (EQUAL(ptrn,"artcl_id")) OR
      (EQUAL(ptrn,"be"))       OR
      (EQUAL(ptrn,"vt_pl"))    OR
      (EQUAL(ptrn,"vi_pl")))   return TRUE;

  return FALSE;
}

/*----------------------------------------------------------------------

  mdfchk_lftphrs  --  check if left phrase is considered in the log file

    input  -- stc:  a sentence
              slen: sentence length
              idx:  token index
    output -- phrs: phrase name

   function returns TRUE if it is considered in log file, FALSE otherwise

-----------------------------------------------------------------------*/
boolean mdfchk_lftphrs(stc, slen, idx, phrs)
struct Syntx_Term *stc;
int    slen;
int    idx;
char   *phrs;
{
  int  lh;
  char str[4];

  get_phrs_name(stc, slen, idx, phrs, &lh);

  str[0] = phrs[0];
  str[1] = phrs[1];
  str[2] = phrs[2];
  str[3] = '\0';

  if (EQUAL(str,"nou")) return TRUE;

  if (EQUAL(str,"to_")) {
    if (EQUAL(phrs,"to_phrs")) return FALSE;
    return TRUE;
  }

  return FALSE;
}


/*----------------------------------------------------------------------

  mdfchk_cntphrs  --  check if current phrase is considered in the log file

    input  -- stc:  a sentence
              slen: sentence length
              idx:  token index
    output -- phrs: phrase name

   function returns TRUE if it is considered in log file, FALSE otherwise

-----------------------------------------------------------------------*/
boolean mdfchk_cntphrs(stc, slen, idx, phrs)
struct Syntx_Term *stc;
int    slen;
int    idx;
char   *phrs;
{
  int  lh;
  char str[4], word[WORD_MAX+1];

  get_phrs_name(stc, slen, idx, phrs, &lh);

  if (EQUAL(phrs,"")) return FALSE;

  str[0] = phrs[0];
  str[1] = phrs[1];
  str[2] = phrs[2];
  str[3] = '\0';

  if (EQUAL(str,"nou") OR                           /* noun_sgl, noun_pl */
      EQUAL(str,"adj") OR                        /* adj, adj_er, adj_est */
      EQUAL(str,"bei") OR                                       /* being */
      EQUAL(str,"pre") OR                                    /* prep ... */
      EQUAL(str,"adv"))                          /* adv, adv_er, adv_est */
    return TRUE;

  if (EQUAL(str,"pro")) {
    if (EQUAL(phrs,"pron_thr")) return FALSE;
    if (EQUAL(phrs,"pron_obj")) {
      get_1stwd_of_phrs(stc, slen, idx, word);
      if (EQUAL(word,"me")) {
        strcpy(phrs, "pron_me");
        return TRUE;
      }
      return TRUE;
    }
    else return TRUE;
  }

  if (EQUAL(phrs,"vt_ing")  OR
      EQUAL(phrs,"vi_ing")  OR
      EQUAL(phrs,"vt_pprt") OR
      EQUAL(phrs,"vi_pprt"))   return TRUE;

  return FALSE;
}


/*----------------------------------------------------------------------

  mdfchk_rhtphrs  --  check if right phrase is considered in the log file

    input  -- stc:  a sentence
              slen: sentence length
              idx:  token index
    output -- phrs: phrase name
              lftstop: indicate if the phrase is left stop

   function returns TRUE if it is considered in log file, FALSE otherwise

-----------------------------------------------------------------------*/
boolean mdfchk_rhtphrs(stc, slen, idx, phrs, lftstop)
struct Syntx_Term *stc;
int    slen;
int    idx;
char   *phrs;
boolean *lftstop;
{
  int  lh;
  char str[4], word[WORD_MAX+1];

  *lftstop = left_stop_noun(stc, slen, idx);

  get_phrs_name(stc, slen, idx, phrs, &lh);

  if (EQUAL(phrs,"")) return FALSE;

  str[0] = phrs[0];
  str[1] = phrs[1];
  str[2] = phrs[2];
  str[3] = '\0';

  if (EQUAL(str,"nou") OR                           /* noun_sgl, noun_pl */
      EQUAL(str,"adj") OR                        /* adj, adj_er, adj_est */
      EQUAL(str,"pre") OR                                    /* prep ... */
      EQUAL(str,"adv"))                          /* adv, adv_er, adv_est */
    return TRUE;

  if (EQUAL(str,"pro")) {
    if (EQUAL(phrs,"pron_obj")) {
      get_1stwd_of_phrs(stc, slen, idx, word);
      if (EQUAL(word,"me")) {
        strcpy(phrs, "pron_me");
        return TRUE;
      }
      return TRUE;
    }
    else return TRUE;
  }

  if (EQUAL(phrs,"vt_ing")  OR
      EQUAL(phrs,"vi_ing")  OR
      EQUAL(phrs,"vt_pprt") OR
      EQUAL(phrs,"vi_pprt"))   return TRUE;


  return FALSE;
}


/*----------------------------------------------------------------------

  mdfchk_subphrs  --  check if the subject phrase is considered in the 
                      log file

    input  -- stc:  a sentence
              slen: sentence length
              idx:  token index
    output -- phrs: phrase name

   function returns TRUE if it is considered in log file, FALSE otherwise

-----------------------------------------------------------------------*/
boolean mdfchk_subphrs(stc, slen, idx, phrs)
struct Syntx_Term *stc;
int    slen;
int    idx;
char   *phrs;
{
  int  lh;
  char str[4];

  get_phrs_name(stc, slen, idx, phrs, &lh);

  if (EQUAL(phrs,"")) return FALSE;

  str[0] = phrs[0];
  str[1] = phrs[1];
  str[2] = phrs[2];
  str[3] = '\0';

  if (EQUAL(str,"nou")) { strcpy(phrs, "noun"); return TRUE; }

  if (EQUAL(str,"pro")) {
    if (EQUAL(phrs,"pron_slf")) return FALSE;
    if (EQUAL(phrs,"pron_thr")) return TRUE;
    strcpy(phrs, "pron");
    return TRUE;
  }

  return FALSE;
}


/*----------------------------------------------------------------------

  mdfchk_prdctphrs  --  check if the predicate phrase is considered in
                        the log file

    input  -- stc:  a sentence
              slen: sentence length
              idx:  token index
    output -- phrs: phrase name

   function returns TRUE if it is considered in log file, FALSE otherwise

-----------------------------------------------------------------------*/
boolean mdfchk_prdctphrs(stc, slen, idx, phrs)
struct Syntx_Term *stc;
int    slen;
int    idx;
char   *phrs;
{
  int  lh;
  char word[WORD_MAX+1];

  get_phrs_name(stc, slen, idx, phrs, &lh);

  if (EQUAL(phrs,"")) return FALSE;

  if (EQUAL(phrs,"pv_be")   OR
      EQUAL(phrs,"am")      OR
      EQUAL(phrs,"is")      OR
      EQUAL(phrs,"are")     OR
      EQUAL(phrs,"was")     OR
      EQUAL(phrs,"were")    OR
      EQUAL(phrs,"be")      OR
      EQUAL(phrs,"hs_be")   OR
      EQUAL(phrs,"hv_be")   OR
      EQUAL(phrs,"hd_be")   OR
      EQUAL(phrs,"aux_be")  OR
      EQUAL(phrs,"auX_be"))    { return TRUE; }

  if (EQUAL(phrs,"lvb_sgl") OR
      EQUAL(phrs,"lvb_pl")  OR
      EQUAL(phrs,"lvb_pst"))   { strcpy(phrs, "lvb_cmp"); return TRUE; }

  get_lastwd_of_phrs(stc, slen, idx, word);
  if (strlen(word) == 3) {
    word[0] = tolower(word[0]);
    word[1] = tolower(word[1]);
    word[2] = tolower(word[2]);
    if (EQUAL(word,"let")) { strcpy(phrs, "let"); return TRUE; }
    return FALSE;
  }
  return FALSE;
}
