#include "syntx.h"
#include "string.h"


#define        NumObverrDgns    27


extern struct  Syntx_Term *sentence; 
extern int                 sentence_len;
extern int                *original_sentence;
extern int                 original_sentence_len; 
extern boolean obvious_error;
extern int     sentence_type;
extern char    sentence_punc;
extern boolean has_front_undef;
extern boolean intrnl_dsply;

DIAGNOSIS_ARRAY obverr_dgns[NumObverrDgns + 1] = { 
                                                   "prdct_found",
                                                   "err_dupl_word",
                                                   "err_dupl_pattern",
                                                   "err_dupl_aux",
                                                   "err_double_comp",
/*                                                   "err_omit_word", */
                                                 "err_becauseof_nomnpron",
                                                   "err_no_prdct",
                                                   "err_adjpron_artcl",
                                                   "err_artcl_pron",
                                                   "err_artcl_nonnoun",
                                                   "err_either_neither",
                                                   "err_whom_who",
                                                   "err_its",
                                                   "err_prep",
                                                   "err_there_is",
                                                   "err_quantity",
                                                   "err_a_an",
                                                   "err_vi_obj",
                                                   "err_tophrase",
                                                   "err_be_complement",
                                                   "err_have_tense",
                                                   "err_aux_tense",
                                                   "err_do_tense",
                                                  "err_subprdct_disagree",
                                                   "err_verb_tense",
                                                   "err_prep_as_conj",
                                                   "err_cmb_sub",
                                                   "" 
                                                 };   




char              obverr_msg[100];

/*##################*/
char              qnterr_msg[100];
int               qnterr_start, qnterr_end;
extern boolean    quant_error;

char              prnerr_msg[100];
int               prnerr_start, prnerr_end;
extern boolean    sucprn_error;
/*##################*/

/**@@@@@@@@@7/99@@@@@@@@**/
char              pronoun_noun_msg[100];
int               pronoun_noun_start, pronoun_noun_end;
extern boolean    pronoun_noun_error;
/**@@@@@@end7/99@@@@@@@@**/

int               obverr_start, obverr_end;
boolean           no_prdct_sofar, has_either, has_neither;
static int        symptoms_idx;
SYMPTOM_ARRAY     symptoms[mmax(3*(MAXPTRNS+1)+16,MAXSYMP)];



boolean find_obvious_err();
boolean obvious_err();
boolean obverr_cntptrn();
boolean obverr_rht1ptrn();
boolean obverr_rht2ptrn();
boolean obverr_cntwd();
boolean obverr_rht1wd();
boolean obverr_rht2wd();
boolean find_triple();
void load_prb();
boolean has_prdct();
void end_title();
void get_ptrn_names();
boolean has_adjpron();
void get_word();
boolean has_verb_only();
void begin_title();


int     Triple[4];



/*----------------------------------------------------------------------

  find_obvious_err  --  control the action of obvious_error.prg

    input  -- none
    output -- none

  function returns TRUE if some obvious error is found, FALSE otherwise.

------------------------------------------------------------------------*/
boolean find_obvious_err()
{
  int   i, len;

  if (intrnl_dsply) {
    begin_title("obvious_error", -1, -1);
  }

  load_prb("FindObviousErr");
  len = sentence_len - 1;
  no_prdct_sofar = (!has_prdct(sentence, sentence_len, 0));
  has_either = has_neither = FALSE;
  for (i = 0; i < len; i++) {
    obverr_start = i;
    if (obvious_error=obvious_err(i, obverr_msg)){
      if (intrnl_dsply) {
        end_title("obvious_error");
      }
      return obvious_error;
    }
  }
  if (intrnl_dsply) {
    end_title("obvious_error");
  }
  return obvious_error;
}



/*---------------------------------------------------------------------

  obvious_err  --  execute the obvious_error.prg to check if there is 
                   an obvious error around the current, right1 and 
                   right2 words

    input  -- idx:   current token index

    output -- msg:   error message if an obvious error occures

  function returns TRUE if there is an obvious error, FALSE otherwise.

----------------------------------------------------------------------*/
boolean obvious_err(idx, msg)
int    idx;
char   *msg;
{
  int   cnt, rht1, rht2, lklhds[MAXPTRNS], i, lh;
  char  dgns_var[VAR_MAX + 1], ptrn_names[MAXPTRNS+1][PTRN_PHRS_MAX+1],
        ptrn[PTRN_PHRS_MAX+1], word1[WORD_MAX+1], word2[WORD_MAX+1],
        word3[WORD_MAX+1];
  DIAGNOSIS_ARRAY dgns[NumObverrDgns + 1];
  boolean has_error = FALSE;


  symptoms_idx = 0;
  cnt = idx;
  rht1 = idx + 1;
  rht2 = idx + 2;

  Triple[0] = original_sentence[cnt];
  Triple[1] = original_sentence[rht1];
  Triple[2] = original_sentence[rht2];

  if (find_triple(Triple)) return FALSE;

  if (sentence_type == SimpleSentence) 
    sprintf(symptoms[symptoms_idx++], "simple_sentence");
  if (sentence_type == TitleSentence)
    sprintf(symptoms[symptoms_idx++], "title_sentence");
  if (sentence_punc == '.')
    sprintf(symptoms[symptoms_idx++], "punc_period");
  if (sentence_punc == '?')
    sprintf(symptoms[symptoms_idx++], "punc_question");
  if (no_prdct_sofar) 
    sprintf(symptoms[symptoms_idx++], "no_prdct_sofar");
  if (has_front_undef)
    sprintf(symptoms[symptoms_idx++], "lft_to_first_undef");


  if (cnt == 0) sprintf(symptoms[symptoms_idx++], "cnt_first_word");
  get_ptrn_names(sentence, sentence_len, cnt, ptrn_names, lklhds);
  i = 0;
  strcpy(ptrn, ptrn_names[i++]);
  while (!EQUAL(ptrn,"")) {
    if (EQUAL(ptrn,"undef")) {
      sprintf(symptoms[symptoms_idx++], "cnt_ptn(noun_sgl)");
      sprintf(symptoms[symptoms_idx++], "cnt_ptn(noun_pl)");
      i = 3;
      break;
    }
    if (obverr_cntptrn(ptrn)) {
      sprintf(symptoms[symptoms_idx++], "cnt_ptn(%s)", ptrn);
    }
    strcpy(ptrn, ptrn_names[i++]);
  }
  if (i == 2) sprintf(symptoms[symptoms_idx++], "cnt_one_ptn_only");
  if (i == 3) sprintf(symptoms[symptoms_idx++], "cnt_two_ptns_only");
  if (has_adjpron(sentence, sentence_len, cnt, &lh))   
    strcpy(symptoms[symptoms_idx++], "cnt_ptn(pron_adj)");
  if (has_verb_only(sentence, sentence_len, cnt)) 
    sprintf(symptoms[symptoms_idx++], "cnt_prdctverb_only");


  if (rht1 == (sentence_len - 1)) 
    sprintf(symptoms[symptoms_idx++], "rht1_last_word");
  get_ptrn_names(sentence, sentence_len, rht1, ptrn_names, lklhds);
  i = 0;
  strcpy(ptrn, ptrn_names[i++]);
  while (!EQUAL(ptrn,"")) {
    if (EQUAL(ptrn,"undef")) {
      sprintf(symptoms[symptoms_idx++], "rht1_ptn(noun_sgl)");
      sprintf(symptoms[symptoms_idx++], "rht1_ptn(noun_pl)");
      i = 3;
      break;
    }
    if (obverr_rht1ptrn(ptrn)) {
      sprintf(symptoms[symptoms_idx++], "rht1_ptn(%s)", ptrn);
    }
    strcpy(ptrn, ptrn_names[i++]);
  }
  if (i == 2) sprintf(symptoms[symptoms_idx++], "rht1_one_ptn_only");
  if (i == 3) sprintf(symptoms[symptoms_idx++], "rht1_two_ptns_only");
  if (has_adjpron(sentence, sentence_len, rht1, &lh))   
    strcpy(symptoms[symptoms_idx++], "rht1_ptn(pron_adj)");
  if (has_verb_only(sentence, sentence_len, rht1)) 
    sprintf(symptoms[symptoms_idx++], "rht1_prdctverb_only");


  get_ptrn_names(sentence, sentence_len, rht2, ptrn_names, lklhds);
  i = 0;
  strcpy(ptrn, ptrn_names[i++]);
  while (!EQUAL(ptrn,"")) {
    if (EQUAL(ptrn,"undef")) {
      sprintf(symptoms[symptoms_idx++], "rht2_ptn(noun_pl)");
      i = 3;
      break;
    }
    if (obverr_rht2ptrn(ptrn)) {
      sprintf(symptoms[symptoms_idx++], "rht2_ptn(%s)", ptrn);
    }
    strcpy(ptrn, ptrn_names[i++]);
  }
  if (i == 2) sprintf(symptoms[symptoms_idx++], "rht2_one_ptn_only");
  if (i == 3) sprintf(symptoms[symptoms_idx++], "rht2_two_ptns_only");


  
  get_word(sentence, sentence_len, cnt, word1);
  get_word(sentence, sentence_len, rht1, word2);
  get_word(sentence, sentence_len, rht2, word3);
  
  if (EQUAL(word1,word2))
    sprintf(symptoms[symptoms_idx++], "cnt_rht1_sameword");
    

  /*###############4/16/99#######################*/
  /** check if the current word is the same    ***/
  /** as the right word except first letter    ***/
  /** of current word is upper case            ***/
  /***********************************************/
  if( ( word1[0] >= 'A') && ( word1[0] <= 'Z') )
     {  
        word1[0] += 32;
        if (EQUAL(word1,word2))
        sprintf(symptoms[symptoms_idx++], "cnt_rht1_sameword");
        word1[0] -= 32;
      }
  /*###########end of #4/16/99###################*/

  if ((!EQUAL(word1,""))      AND
      (!EQUAL(word1,"UNDEF")))    {
    if (obverr_cntwd(word1)) {
      if (word_check(word1)==0) {
        sprintf(symptoms[symptoms_idx++], "cnt_wd(%s)", word1);
      } else {
        sprintf(symptoms[symptoms_idx++], "cnt_wd(%s_)", word1);
      }
    }
  }
  if (has_either) { sprintf(symptoms[symptoms_idx++], "either_occured"); }
  else { has_either = EQUAL(word1,"either"); }
  if (has_neither) { sprintf(symptoms[symptoms_idx++],"neither_occured"); }
  else { has_neither = EQUAL(word1,"neither"); }


  if (!EQUAL(word2,"")) {
    if (obverr_rht1wd(word2)) {
      if (word_check(word2)==0) {
        sprintf(symptoms[symptoms_idx++], "rht1_wd(%s)", word2);
      } else {
        sprintf(symptoms[symptoms_idx++], "rht1_wd(%s_)", word2);
      }
    }
  }


  if ((!EQUAL(word3,""))      AND
      (!EQUAL(word3,"UNDEF")))    {
    if (obverr_rht2wd(word3)) {
      if (word_check(word3)==0) {
        sprintf(symptoms[symptoms_idx++], "rht2_wd(%s)", word3);
      } else {
        sprintf(symptoms[symptoms_idx++], "rht2_wd(%s_)", word3);
      }
    }
  }

  strcpy(symptoms[symptoms_idx],"");
  if (symptoms_idx == 0) return FALSE;

  mst_smp(symptoms, name, state, type, value, errcde);
  sol_prb(name, state, type, value, errcde);
  if (errcde[0] != 0) {
    stx_err("failed to solve problem", __LINE__, __FILE__);
  }

  if EQUAL(state, "S") { 
    copy_dgns(dgns, obverr_dgns);
    get_dgn(dgns,name,state,type,value,errcde);
    if (value[0]) {                      /* has at least one true value */
      i = 0;
      while (! EQUAL(dgns[i],"")) {
        strcpy(dgns_var, dgns[i++]);
        
        if (EQUAL(dgns_var,"prdct_found")) {
          no_prdct_sofar = FALSE;
          continue;
        }

        has_error = TRUE;

        if (EQUAL(dgns_var,"err_dupl_word")) {
          strcpy(msg, "err_dupl_word");
          obverr_end = rht1;
          break;
        }
        if (EQUAL(dgns_var,"err_dupl_pattern")) {
          strcpy(msg, "err_dupl_pattern");
          obverr_end = rht1;
          break;
        }
        if (EQUAL(dgns_var,"err_dupl_aux")) {
          strcpy(msg, "err_dupl_aux");
          obverr_end = rht1;
          break;
        }
        if (EQUAL(dgns_var,"err_double_comp")) {
          strcpy(msg, "err_double_comp");
          obverr_end = rht1;
          break;
        }
/*        if (EQUAL(dgns_var,"err_omit_word")) {
          strcpy(msg, "err_omit_word");
          if (rht2 < original_sentence_len) { obverr_end = rht2; }
          else obverr_end = original_sentence_len - 1;
          break;
        } */
        if (EQUAL(dgns_var,"err_becauseof_nomnpron")) {
          strcpy(msg, "err_becauseof_nomnpron");
          if (rht2 < original_sentence_len) { obverr_end = rht2; }
          else obverr_end = original_sentence_len - 1;
          break;
        }
        if (EQUAL(dgns_var,"err_no_prdct")) {
          strcpy(msg, "err_no_prdct");
          obverr_start = -1;
          obverr_end = -1;
          break;
        }
        if (EQUAL(dgns_var,"err_adjpron_artcl")) {
          strcpy(msg, "err_adjpron_artcl");
          obverr_end = rht1;
          break;
        }
        if (EQUAL(dgns_var,"err_artcl_pron")) {
          strcpy(msg, "err_artcl_pron");
          obverr_end = rht1;
          break;
        }
        if (EQUAL(dgns_var,"err_artcl_nonnoun")) {
          strcpy(msg, "err_artcl_nonnoun");
          obverr_end = rht1;
          break;
        }
        if (EQUAL(dgns_var,"err_either_neither")) {
          strcpy(msg, "err_either_neither");
          obverr_end = cnt;
          break;
        }
        if (EQUAL(dgns_var,"err_whom_who")) {
          strcpy(msg, "err_whom_who");
          if (rht2 < original_sentence_len) { obverr_end = rht2; }
          else obverr_end = original_sentence_len - 1;
          break;
        }
        if (EQUAL(dgns_var,"err_its")) {
          strcpy(msg, "err_its");
          obverr_end = rht1;
          break;
        }
        if (EQUAL(dgns_var,"err_prep")) {
          strcpy(msg, "err_prep");
          obverr_end = rht1;
          break;
        }
        if (EQUAL(dgns_var,"err_there_is")) {
          strcpy(msg, "err_there_is");
          if (rht2 < original_sentence_len) { obverr_end = rht2; }
          else obverr_end = original_sentence_len - 1;
          break;
        }
        if (EQUAL(dgns_var,"err_quantity")) {
          strcpy(msg, "err_quantity");
          obverr_end = rht1;
          break;
        }
        if (EQUAL(dgns_var,"err_a_an")) {
          strcpy(msg, "err_a_an");
          obverr_end = rht1;
          break;
        }
        if (EQUAL(dgns_var,"err_vi_obj")) {
          strcpy(msg, "err_vi_obj");
          obverr_end = rht1;
          break;
        }
        if (EQUAL(dgns_var,"err_tophrase")) {
          strcpy(msg, "err_tophrase");
          obverr_end = rht1;
          break;
        }
        if (EQUAL(dgns_var,"err_be_complement")) {
          strcpy(msg, "err_be_complement");
          obverr_end = rht1;
          break;
        }
        if (EQUAL(dgns_var,"err_have_tense")) {
          strcpy(msg, "err_have_tense");
          obverr_end = rht1;
          break;
        }
        if (EQUAL(dgns_var,"err_aux_tense")) {
          strcpy(msg, "err_aux_tense");
          obverr_end = rht1;
          break;
        }
        if (EQUAL(dgns_var,"err_do_tense")) {
          strcpy(msg, "err_do_tense");
          obverr_end = rht1;
          break;
        }
        if (EQUAL(dgns_var,"err_subprdct_disagree")) {
          strcpy(msg, "err_subprdct_disagree");
          if (rht2 < original_sentence_len) { obverr_end = rht2; }
          else obverr_end = original_sentence_len - 1;
          break;
        }
        if (EQUAL(dgns_var,"err_verb_tense")) {
          strcpy(msg, "err_verb_tense");
          if (rht2 < original_sentence_len) { obverr_end = rht2; }
          else obverr_end = original_sentence_len - 1;
          break;
        }
        if (EQUAL(dgns_var,"err_prep_as_conj")) {
          strcpy(msg, "err_prep_as_conj");
          obverr_end = rht1;
          break;
        }
        if (EQUAL(dgns_var,"err_cmb_sub")) {
          strcpy(msg, "err_cmb_sub");
          if (rht2 < original_sentence_len) { obverr_end = rht2; }
          else obverr_end = original_sentence_len - 1;
          break;
        }
      }
    }
  }
  else stx_err("problem is unsatisfied", __LINE__, __FILE__);

  strcpy(state, "A");
  rst_prb(name, state, type, value, errcde);
  if (has_error) return TRUE;
  return FALSE;
}


/*---------------------------------------------------------------------

  obverr_cntptrn  --  check if current pattern is considered in the 
                      log file

    input  -- ptrn: current pattern name to be checked
    output -- none

  function returns TRUE if ptrn is considered in log file, FALSE otherwise

------------------------------------------------------------------------*/
boolean obverr_cntptrn(ptrn)
char *ptrn;
{
  char str[4];

  str[0] = ptrn[0];
  str[1] = ptrn[1];
  str[2] = ptrn[2];
  str[3] = '\0';

  if (EQUAL(str,"pro") OR   /* pron_sgl, pron_pl, pron */
      EQUAL(str,"con") OR   /* conj */
      EQUAL(str,"int") OR   /* intj */
      EQUAL(str,"abr") OR   /* abrv */
      EQUAL(str,"lvb") OR   /* lvb_pl, lvb_sgl ... */
      EQUAL(str,"bee") OR   /* been */
      EQUAL(str,"bei"))     /* being */   
    return FALSE;

  if (EQUAL(str,"vt_")) {
    if (EQUAL(ptrn,"vt_pprt") OR EQUAL(ptrn,"vt_pl")) { return FALSE; }
    else { return TRUE; }
  }

  if (EQUAL(str,"vi_")) {
    if (EQUAL(ptrn,"vi_pprt") OR EQUAL(ptrn,"vi_pl")) { return FALSE; }
    else { return TRUE; }
  }

  if (EQUAL(ptrn,"adj") OR
      EQUAL(ptrn,"adv"))    return FALSE;
    
  return TRUE;
}

/*---------------------------------------------------------------------

  obverr_rht1ptrn  --  check if right1 pattern is considered in the 
                       log file

    input  -- ptrn: right1 pattern name to be checked
    output -- none

  function returns TRUE if ptrn is considered in log file, FALSE otherwise

------------------------------------------------------------------------*/
boolean obverr_rht1ptrn(ptrn)
char *ptrn;
{
  char str[4];

  str[0] = ptrn[0];
  str[1] = ptrn[1];
  str[2] = ptrn[2];
  str[3] = '\0';

  if (EQUAL(str,"int") OR    /* intj */
      EQUAL(str,"bei") OR    /* being */
      EQUAL(str,"abr") OR    /* abrv */
      EQUAL(str,"lvb"))      /* lvb_pl, .... */ 
    return FALSE;

  return TRUE;
}


/*---------------------------------------------------------------------

  obverr_rht2ptrn  --  check if right2 pattern is considered in the 
                       log file

    input  -- ptrn: right2 pattern name to be checked
    output -- none

  function returns TRUE if ptrn is considered in log file, FALSE otherwise

------------------------------------------------------------------------*/
boolean obverr_rht2ptrn(ptrn)
char *ptrn;
{
  char str[4];

  str[0] = ptrn[0];
  str[1] = ptrn[1];
  str[2] = ptrn[2];
  str[3] = '\0';

  if (EQUAL(str,"con") OR    /* conj */
      EQUAL(str,"pro") OR    /* pron_sgl, pron_pl, pron */
      EQUAL(str,"art") OR    /* artcl_df, artcl_id */
      EQUAL(str,"int") OR    /* intj */
      EQUAL(str,"sgl") OR    /* sgl_hv ... */
      EQUAL(str,"pl_") OR    /* pl_hv ... */
      EQUAL(str,"dgt") OR    /* dgt_sgl, dgt_pl */
      EQUAL(str,"adv") OR    /* adv, adv_er, adv_est */
      EQUAL(str,"be")  OR    /* be */
      EQUAL(str,"bee") OR    /* been */
      EQUAL(str,"pre"))      /* prep */ 
    return TRUE;

  if (EQUAL(str,"vt_")) {
    if (EQUAL(ptrn,"vt_ing") OR EQUAL(ptrn,"vt_pst")) { return FALSE; }
    else { return TRUE; }
  }

  if (EQUAL(str,"vi_")) {
    if (EQUAL(ptrn,"vi_ing") OR EQUAL(ptrn,"vi_pst")) { return FALSE; }
    else { return TRUE; }
  }

  if (EQUAL(ptrn,"noun_pl") OR 
      EQUAL(ptrn,"adj"))        return TRUE;

  return FALSE;
}


/*---------------------------------------------------------------------

  obverr_cntwd  --  check if current word is considered in the log file

    input  -- word: current word to be checked
    output -- none

  function returns TRUE if word is considered in log file, FALSE otherwise

------------------------------------------------------------------------*/
boolean obverr_cntwd(word)
char *word;
{
  int  i, len;
  char ch;

  len = strlen(word);
  if ((len == 1)        AND
      (word[0] == 'i'))     return FALSE;

  for (i = 0; i < len; i++) {
    ch = word[i];
    if (ch == '\'') { word[i] = '_'; }
    else { word[i] = tolower(ch); }
  }

  if (len <= 2) {
    if (EQUAL(word,"i")   OR
        EQUAL(word,"he")  OR
        EQUAL(word,"it")  OR
        EQUAL(word,"we")  OR
        EQUAL(word,"a")   OR
        EQUAL(word,"an")  OR
        EQUAL(word,"or")  OR
        EQUAL(word,"to")  OR
        EQUAL(word,"do"))     return TRUE;

    return FALSE;
  }

  if (len <= 4) {
    if (EQUAL(word,"she")   OR
        EQUAL(word,"they")  OR
        EQUAL(word,"you")   OR
        EQUAL(word,"its")   OR
        EQUAL(word,"he_s")  OR
        EQUAL(word,"it_s")  OR
        EQUAL(word,"the")   OR
        EQUAL(word,"that")  OR
        EQUAL(word,"this")  OR
        EQUAL(word,"each")  OR
        EQUAL(word,"both")  OR
        EQUAL(word,"many")  OR
        EQUAL(word,"less")  OR
        EQUAL(word,"one")   OR
        EQUAL(word,"few")   OR
        EQUAL(word,"more")  OR
        EQUAL(word,"most")  OR
        EQUAL(word,"and")   OR
        EQUAL(word,"nor")   OR
        EQUAL(word,"here")  OR
        EQUAL(word,"had")   OR
        EQUAL(word,"very")  OR
        EQUAL(word,"whom")  OR
        EQUAL(word,"not")   OR
        EQUAL(word,"like"))     return TRUE;

    return FALSE;
  }

  if (len <= 8) {
    if (EQUAL(word,"she_s")    OR
        EQUAL(word,"anyone")   OR
        EQUAL(word,"anybody")  OR
        EQUAL(word,"anything") OR
        EQUAL(word,"nobody")   OR
        EQUAL(word,"nothing")  OR
        EQUAL(word,"someone")  OR
        EQUAL(word,"somebody") OR
        EQUAL(word,"either")   OR
        EQUAL(word,"neither")  OR
        EQUAL(word,"everyone") OR
        EQUAL(word,"several")  OR
        EQUAL(word,"these")    OR
        EQUAL(word,"those")    OR
        EQUAL(word,"little")   OR
        EQUAL(word,"fewer")    OR
        EQUAL(word,"ought")    OR
        EQUAL(word,"there")    OR
        EQUAL(word,"because")  OR
        EQUAL(word,"except")   OR
        EQUAL(word,"without"))    return TRUE; 

    return FALSE;
  }

  if (EQUAL(word,"something")   OR
      EQUAL(word,"everybody")   OR
      EQUAL(word,"everything"))    { word[8] = '\0'; return TRUE; }

  return FALSE;
}

/*---------------------------------------------------------------------

  obverr_rht1wd  --  check if right1 word is considered in the log file

    input  -- word: right1 word to be checked
    output -- none

  function returns TRUE if word is considered in log file, FALSE otherwise

------------------------------------------------------------------------*/
boolean obverr_rht1wd(word)
char *word;
{
  int  i, len;
  char ch, ch1, ch2, wd[WORD_MAX+1];

  len = strlen(word);
  if ((len == 1)        AND
      (word[0] == 'i'))     { strcpy(word,"vwl_wd"); return TRUE; }

  for (i = 0; i < len; i++) {
    ch = word[i];
    if (ch == '\'') { word[i] = '_'; }
    else { word[i] = tolower(ch); }
  }

  if (len <= 2) {
    if (EQUAL(word,"i")   OR
        EQUAL(word,"he")  OR
        EQUAL(word,"it")  OR
        EQUAL(word,"we")  OR
        EQUAL(word,"am")  OR
        EQUAL(word,"is")  OR
        EQUAL(word,"do")  OR
        EQUAL(word,"or")  OR
        EQUAL(word,"to")  OR
        EQUAL(word,"of"))     return TRUE;
  }

  if ((len == 3) OR (len == 4)) {
    if (EQUAL(word,"she")   OR
        EQUAL(word,"they")  OR
        EQUAL(word,"you")   OR
        EQUAL(word,"that")  OR
        EQUAL(word,"this")  OR
        EQUAL(word,"one")   OR
        EQUAL(word,"each")  OR
        EQUAL(word,"both")  OR
        EQUAL(word,"many")  OR
        EQUAL(word,"few")   OR
        EQUAL(word,"have")  OR
        EQUAL(word,"has")   OR
        EQUAL(word,"was")   OR
        EQUAL(word,"are")   OR
        EQUAL(word,"were")  OR
        EQUAL(word,"will")  OR
        EQUAL(word,"does")  OR
        EQUAL(word,"and")   OR
        EQUAL(word,"not"))      return TRUE;
  }

  if ((len > 4) AND (len < 9)) {
    if (EQUAL(word,"anyone")   OR
        EQUAL(word,"anybody")  OR
        EQUAL(word,"anything") OR
        EQUAL(word,"nobody")   OR
        EQUAL(word,"nothing")  OR
        EQUAL(word,"someone")  OR
        EQUAL(word,"somebody") OR
        EQUAL(word,"either")   OR
        EQUAL(word,"neither")  OR
        EQUAL(word,"everyone") OR
        EQUAL(word,"several")  OR
        EQUAL(word,"these")    OR
        EQUAL(word,"those")    OR
        EQUAL(word,"shall")    OR
        EQUAL(word,"don_t")    OR
        EQUAL(word,"doesn_t")  OR
        EQUAL(word,"which")    OR
        EQUAL(word,"UNDEF"))     return TRUE;
  }

  if (EQUAL(word,"something")   OR
      EQUAL(word,"everybody")   OR
      EQUAL(word,"everything"))    { word[8] = '\0'; return TRUE; }

  ch1 = word[0];
  ch2 = word[1];

  if (ch1 == 'u') { strcpy(word,"u_wd"); return TRUE; }

  if ((ch1 == 'a') OR (ch1 == 'i')) {
    strcpy(word,"vwl_wd");
    return TRUE; 
  }

  if (ch1 == 'e') {
    if ((ch2 == 'u') OR (ch2 == 'w')) return FALSE;
    strcpy(word,"vwl_wd");
    return TRUE; 
  }

  if (ch1 == 'o') {
    char ch3, ch4;

    ch3 = word[2];
    ch4 = word[3];
    if ((ch2 == 'n') AND (ch3 == 'e')) return FALSE;
    if ((ch2 == 'n') AND (ch3 == 'c') AND (ch4 == 'e')) return FALSE;
    strcpy(word,"vwl_wd");
    return TRUE; 
  }

  if (EQUAL(word,"hour")    OR
      EQUAL(word,"honest"))    { strcpy(word,"vwl_wd"); return TRUE; }

  if (EQUAL(word,"herb")      OR
      EQUAL(word,"herbicide"))   { strcpy(word,"u_wd"); return TRUE; }

  strcpy(wd, word);
  wd[5] = '\0';
  if (EQUAL(wd,"herba")) { strcpy(word,"u_wd"); return TRUE; }
  if (EQUAL(wd,"honor")) { strcpy(word,"vwl_wd"); return TRUE; }
  wd[4] = '\0';
  if (EQUAL(wd,"heir")) { strcpy(word,"u_wd"); return TRUE; }

  return FALSE;
}


/*---------------------------------------------------------------------

  obverr_rht2wd  --  check if right2 word is considered in the log file

    input  -- word: right2 word to be checked
    output -- none

  function returns TRUE if word is considered in log file, FALSE otherwise

------------------------------------------------------------------------*/
boolean obverr_rht2wd(word)
char *word;
{
  int  i, len;
  char ch;

  len = strlen(word);
  if ((len == 1)        AND
      (word[0] == 'i'))     return FALSE; 

  for (i = 0; i < len; i++) {
    ch = word[i];
    if (ch == '\'') { word[i] = '_'; }
    else { word[i] = tolower(ch); }
  }

  if (len <= 2) {
    if (EQUAL(word,"i")  OR
        EQUAL(word,"he") OR
        EQUAL(word,"we") OR
        EQUAL(word,"am") OR
        EQUAL(word,"is") OR
        EQUAL(word,"me") OR
        EQUAL(word,"us") OR
        EQUAL(word,"do"))    return TRUE;

    return FALSE;
  }

  if (len <= 4) {
    if (EQUAL(word,"she")  OR
        EQUAL(word,"they") OR
        EQUAL(word,"him")  OR
        EQUAL(word,"them") OR
        EQUAL(word,"was")  OR
        EQUAL(word,"are")  OR
        EQUAL(word,"were") OR
        EQUAL(word,"one")  OR
        EQUAL(word,"few")  OR
        EQUAL(word,"have") OR
        EQUAL(word,"has")  OR
        EQUAL(word,"does"))    return TRUE;

    return FALSE;
  }

  if (len <= 8) {
    if (EQUAL(word,"anyone")   OR
        EQUAL(word,"anybody")  OR
        EQUAL(word,"anything") OR
        EQUAL(word,"nobody")   OR
        EQUAL(word,"nothing")  OR
        EQUAL(word,"someone")  OR
        EQUAL(word,"somebody") OR
        EQUAL(word,"these")    OR
        EQUAL(word,"those")    OR
        EQUAL(word,"little")   OR
        EQUAL(word,"don_t")    OR
        EQUAL(word,"doesn_t"))     return TRUE;

    return FALSE;
  }

  if (EQUAL(word,"something")) { word[8] = '\0'; return TRUE; }

  return FALSE;
}


/*----------------------------------------------------------------------

  find_succ_quant  --  checks for two successive quantitative pronouns 

    input  -- none
    output -- none

  function returns TRUE if some error is found, FALSE otherwise.

------------------------------------------------------------------------*/
boolean find_succ_quant()
{
  char      word1[WORD_MAX+1], word2[WORD_MAX+1];
  int       i, len;
  
  quant_error = FALSE;
  len = sentence_len - 1;
  
 for (i = 0; i < len; i++)
  {
     qnterr_start = i;
     get_word(sentence, sentence_len, i, word1);
     get_word(sentence, sentence_len, i+1, word2);

    if( ( EQUAL(word1,"a")     && EQUAL(word2,"the")   ) ||
        ( EQUAL(word1,"an")    && EQUAL(word2,"any")   ) ||
        ( EQUAL(word1,"an")    && EQUAL(word2,"every") ) ||
        ( EQUAL(word1,"any")   && EQUAL(word2,"all")   ) ||
        ( EQUAL(word1,"any")   && EQUAL(word2,"an")    ) ||
        ( EQUAL(word1,"any")   && EQUAL(word2,"each")  ) ||
        ( EQUAL(word1,"any")   && EQUAL(word2,"every") ) ||
        ( EQUAL(word1,"any")   && EQUAL(word2,"no")    ) ||
        ( EQUAL(word1,"any")   && EQUAL(word2,"the")   ) ||
        ( EQUAL(word1,"every") && EQUAL(word2,"all")   ) ||
        ( EQUAL(word1,"every") && EQUAL(word2,"an")    ) ||
        ( EQUAL(word1,"every") && EQUAL(word2,"any")   ) ||
        ( EQUAL(word1,"every") && EQUAL(word2,"each")  ) ||
        ( EQUAL(word1,"every") && EQUAL(word2,"no")    ) ||
        ( EQUAL(word1,"every") && EQUAL(word2,"the")   ) ||
        ( EQUAL(word1,"no")    && EQUAL(word2,"all")   ) ||
        ( EQUAL(word1,"no")    && EQUAL(word2,"an")    ) ||
        ( EQUAL(word1,"no")    && EQUAL(word2,"any")   ) ||
        ( EQUAL(word1,"no")    && EQUAL(word2,"each")  ) ||
        ( EQUAL(word1,"no")    && EQUAL(word2,"every") ) ||
        ( EQUAL(word1,"no")    && EQUAL(word2,"the")   ) ||
        ( EQUAL(word1,"the")   && EQUAL(word2,"all")   ) ||
        ( EQUAL(word1,"the")   && EQUAL(word2,"any")   ) ||
        ( EQUAL(word1,"the")   && EQUAL(word2,"each")  ) ||
        ( EQUAL(word1,"the")   && EQUAL(word2,"every") ) ||
        ( EQUAL(word1,"the")   && EQUAL(word2,"no")    )    )
         
                                   {
                                     strcpy(qnterr_msg, "qntfr_dtrmnr");
                                     qnterr_end = i+1;
                                     
                                     Triple[0] = original_sentence[i];
                                     Triple[1] = original_sentence[i+1];
                                     Triple[2] = original_sentence[i+2];

                                     quant_error = TRUE; 
                                     return quant_error;
                                   }
   }


  return quant_error;
}


/*----------------------------------------------------------------------

  find_succ_pronoun  --  checks for two successive pronouns 

    input  -- none
    output -- none

  function returns TRUE if some error is found, FALSE otherwise.

------------------------------------------------------------------------*/
boolean find_succ_pronoun()
{
  char      word1[WORD_MAX+1], word2[WORD_MAX+1];
  int       i, len;
  
  sucprn_error = FALSE;
  len = sentence_len - 1;
  
 for (i = 0; i < len; i++)
  {
     prnerr_start = i;
     get_word(sentence, sentence_len, i, word1);
     get_word(sentence, sentence_len, i+1, word2);

    if( ( EQUAL(word1,"I")  && EQUAL(word2,"he")   ) ||
        ( EQUAL(word1,"I")  && EQUAL(word2,"she")  ) ||
        ( EQUAL(word1,"I")  && EQUAL(word2,"it")   ) ||
        ( EQUAL(word1,"I")  && EQUAL(word2,"they") ) ||
        ( EQUAL(word1,"I")  && EQUAL(word2,"we")   ) ||
        ( EQUAL(word1,"I")  && EQUAL(word2,"you")  ) ||
         
        ( EQUAL(word1,"he")  && EQUAL(word2,"I")    ) ||
        ( EQUAL(word1,"he")  && EQUAL(word2,"she")  ) ||
        ( EQUAL(word1,"he")  && EQUAL(word2,"it")   ) ||
        ( EQUAL(word1,"he")  && EQUAL(word2,"they") ) ||
        ( EQUAL(word1,"he")  && EQUAL(word2,"we")   ) ||
        ( EQUAL(word1,"he")  && EQUAL(word2,"you")  ) ||  
        
        ( EQUAL(word1,"she")  && EQUAL(word2,"I")    ) ||
        ( EQUAL(word1,"she")  && EQUAL(word2,"he")   ) ||
        ( EQUAL(word1,"she")  && EQUAL(word2,"it")   ) ||
        ( EQUAL(word1,"she")  && EQUAL(word2,"they") ) ||
        ( EQUAL(word1,"she")  && EQUAL(word2,"we")   ) ||
        ( EQUAL(word1,"she")  && EQUAL(word2,"you")  ) ||
        
        ( EQUAL(word1,"we")  && EQUAL(word2,"I")    ) ||
        ( EQUAL(word1,"we")  && EQUAL(word2,"he")   ) ||
        ( EQUAL(word1,"we")  && EQUAL(word2,"she")  ) ||
        ( EQUAL(word1,"we")  && EQUAL(word2,"it")   ) ||
        ( EQUAL(word1,"we")  && EQUAL(word2,"they") ) ||
        ( EQUAL(word1,"we")  && EQUAL(word2,"you")  ) ||
        
        ( EQUAL(word1,"they")  && EQUAL(word2,"I")   ) ||
        ( EQUAL(word1,"they")  && EQUAL(word2,"he")  ) ||
        ( EQUAL(word1,"they")  && EQUAL(word2,"she") ) ||
        ( EQUAL(word1,"they")  && EQUAL(word2,"it")  ) ||
        ( EQUAL(word1,"they")  && EQUAL(word2,"we")  ) ||
        ( EQUAL(word1,"they")  && EQUAL(word2,"you") )    )
      
                                  {
                                     strcpy(prnerr_msg, "succ_pronoun");
                                     prnerr_end = i+1;
                                     
                                     Triple[0] = original_sentence[i];
                                     Triple[1] = original_sentence[i+1];
                                     Triple[2] = original_sentence[i+2];

                                     sucprn_error = TRUE; 
                                     return sucprn_error;
                                   }
   }


  return sucprn_error;
}


/**@@@@@@@@@7/99@@@@@@@@**/
/*----------------------------------------------------------------------

  find_pron_noun  --  checks for pronoun followed by [adjective,adverb] noun 

    input  -- none
    output -- none

  function returns TRUE if some error is found, FALSE otherwise.

------------------------------------------------------------------------*/
boolean find_pron_noun()
{
  char      word1[WORD_MAX+1], word2pos[3], word3pos[3];
  int       i, len;

  pronoun_noun_error = FALSE;
  len = sentence_len - 1;
  
 /*** 1. pronoun followed by noun ***/
 for (i = 0; i < len; i++)
  {
     pronoun_noun_start = i;
     get_word(sentence, sentence_len, i, word1);
     /**get_word(sentence, sentence_len, i+1, word2);***/
     get_word_pos(sentence, sentence_len, i+1, word2pos);
     word2pos[1] = '\0';
     
if(  
     ( EQUAL(word1,"he")   || EQUAL(word1,"He")   ||
       EQUAL(word1,"she")  || EQUAL(word1,"She")  ||
       EQUAL(word1,"they") || EQUAL(word1,"They") ||
       EQUAL(word1,"we")   || EQUAL(word1,"We")   ||
       EQUAL(word1,"you")  || EQUAL(word1,"You")  ||
       EQUAL(word1,"It")   || EQUAL(word1,"I")  
     
      ) &&
      ( EQUAL(word2pos,"n")  || EQUAL(word2pos,"N")   )
    )
                                  {
                                     strcpy(pronoun_noun_msg, "err_pronoun_noun");
                                     pronoun_noun_end = i+1;
                                     
                                     Triple[0] = original_sentence[i];
                                     Triple[1] = original_sentence[i+1];
                                     Triple[2] = original_sentence[i+2];

                                     pronoun_noun_error = TRUE; 
                                     return pronoun_noun_error;
                                     
                                   }
   }

/*** 2. pronoun followed by adjective or adverb then noun ***/

 len = sentence_len - 2;
 for (i = 0; i < len; i++)
  {
     pronoun_noun_start = i;
     get_word(sentence, sentence_len, i, word1);
     /**get_word(sentence, sentence_len, i+1, word2);***/
     get_word_pos(sentence, sentence_len, i+1, word2pos);
     get_word_pos(sentence, sentence_len, i+2, word3pos);
     word2pos[1] = '\0';
     word3pos[1] = '\0';

if(  
     ( EQUAL(word1,"he")   || EQUAL(word1,"He")   ||
       EQUAL(word1,"she")  || EQUAL(word1,"She")  ||
       EQUAL(word1,"they") || EQUAL(word1,"They") ||
       EQUAL(word1,"we")   || EQUAL(word1,"We")   ||
       EQUAL(word1,"you")  || EQUAL(word1,"You")  ||
       EQUAL(word1,"It")   || EQUAL(word1,"I")  
     
      ) &&
      ( EQUAL(word2pos,"a")  || EQUAL(word2pos,"y")   ) &&
      ( EQUAL(word3pos,"n")  || EQUAL(word3pos,"N")   )
    )
                          {
                              strcpy(pronoun_noun_msg, "err_pronoun_noun");
                              pronoun_noun_end = i+2;
                                     
                              Triple[0] = original_sentence[i];
                              Triple[1] = original_sentence[i+1];
                              Triple[2] = original_sentence[i+2];
                              pronoun_noun_error = TRUE; 
                              return pronoun_noun_error;
                                     
                           }
   }

/*** 3. pronoun followed by adjective/adverb/noun then noun ***/

 len = sentence_len - 2;
 for (i = 0; i < len; i++)
  {
     pronoun_noun_start = i;
     get_word(sentence, sentence_len, i, word1);
     /**get_word(sentence, sentence_len, i+1, word2);***/
     get_word_pos(sentence, sentence_len, i+2, word3pos);
     word3pos[1] = '\0';
if(  
     ( EQUAL(word1,"he")   || EQUAL(word1,"He")   ||
       EQUAL(word1,"she")  || EQUAL(word1,"She")  ||
       EQUAL(word1,"they") || EQUAL(word1,"They") ||
       EQUAL(word1,"we")   || EQUAL(word1,"We")   ||
       EQUAL(word1,"you")  || EQUAL(word1,"You")  ||
       EQUAL(word1,"It")   || EQUAL(word1,"I")  
     
      ) &&
      ( adj_adv_noun(sentence,sentence_len,i+1)       ) &&
      ( EQUAL(word3pos,"n")  || EQUAL(word3pos,"N")   )
    )
                          {
                              strcpy(pronoun_noun_msg, "err_pronoun_noun");
                              pronoun_noun_end = i+2;
                                     
                              Triple[0] = original_sentence[i];
                              Triple[1] = original_sentence[i+1];
                              Triple[2] = original_sentence[i+2];
                              pronoun_noun_error = TRUE; 
                              return pronoun_noun_error;
                                     
                           }
   }


  return pronoun_noun_error;
}


/**@@@@@@end7/99@@@@@@@@**/

/*---------------------------------------------------------------------*/

