#include "syntx.h"

#define        NumofPrepDgns    7
#define        NumofPrepNext    4

extern struct Syntx_Term *sentence; 
extern int                sentence_len;
extern struct SPOC_Pos    sentence_spoc;
extern boolean            intrnl_dsply;





DIAGNOSIS_ARRAY prep_dgns[NumofPrepDgns + 1] = { 
                                                  "next(find_mdf)",
                                                  "next(P_N)",
                                                  "next(P_A)",
                                                  "next(P_A_N)",
                                                  "is_phrs(prep_sgl)",
                                                  "is_phrs(prep_pl)",
                                                  "is_phrs(prep_obj)",
                                                  "" 
                                               };   


static int        symptoms_idx;
SYMPTOM_ARRAY     symptoms[mmax(MAXPTRNS+2,MAXSYMP)];
  


void    prep_phrase();
int     do_prepP();
boolean prepP_rhtptrn();
boolean prepP_rhtphrs();
boolean prepP_cmb_and_next();



/*----------------------------------------------------------------------

  prep_phrase  --  control the action of do_prepP

    input  -- pos:  the position where the action to be taken
              lklhd: the likelyhood at which the action will take
              who: indicate who invoke the function
    output -- none
 
-----------------------------------------------------------------------*/
void prep_phrase(pos, lklhd, who)
int pos, lklhd, who;
{
  int prepP_code;

  if (intrnl_dsply) {
    begin_title("prep_phrase", pos, lklhd);
  }

  load_prb("PrepPhrase");
  if ((prepP_code= do_prepP(pos, lklhd, who)) == 0) {
    if (intrnl_dsply) {
      printf("no preposition phrases can be formed\n");
    }
    goto end_prep_phrase;
  }

  switch (prepP_code) {
    case 1:                                               /* prep phrase */
    case 2:                                       /* noun_phrase at rht1 */
    case 3:                                        /* adj_phrase at rht1 */
    case 4:                                       /* noun_phrase at rht2 */
            break;

    default: stx_err("unexpected code by do_prepP", __LINE__, __FILE__);
  }

end_prep_phrase: 
  if (intrnl_dsply) {
    end_title("prep_phrase");
  }
}



/*---------------------------------------------------------------------

  do_prepP  --  execute prep_phrase.prg  

    input  -- pos: the position where the prep_phrase works on
              lklhd: the likelihood the prep_phrase works at
              who:   who issued adv_phrase
    output -- none

  function returns  0: no preposition phrase can be formed
                    1: preposition phrase is formed
                    2: noun_phrase at rht1
                    3: adj_phrase at rht1
                    4: noun_phrase at rht2

----------------------------------------------------------------------*/
int do_prepP(pos, lklhd, who)
int pos, lklhd, who;
{
  DIAGNOSIS_ARRAY   dgns[NumofPrepDgns + 1];
  char              dgns_var[VAR_MAX + 1],  ptrn[PTRN_PHRS_MAX+1],
                    ptrn_names[MAXPTRNS+1][PTRN_PHRS_MAX+1],
                    phrs_name[PTRN_PHRS_MAX+1];
  int               i, rht, rht2, cnt, prep_lh, lh, lklhds[MAXPTRNS],
                    return_val = 0;
  boolean           find_mdf = FALSE, rht2_adj_est = FALSE;

  symptoms_idx = 0;
  cnt = pos;
  rht = pos + 1;
  rht2 = rht + 1;

  switch(who) {
    case FindModifier: 
                       strcpy(symptoms[symptoms_idx++], "who(find_mdf)");
                       break;
    case NounPhrase:
                       strcpy(symptoms[symptoms_idx++], "who(P_N)");
                       break;
  
    default: stx_err("unexpected problem code", __LINE__, __FILE__);
  }

  if (!has_prep(sentence, sentence_len, cnt, &prep_lh)) return 0;
  strcpy(symptoms[symptoms_idx++], "cnt_ptrn(prep)");
  
  get_ptrn_names(sentence, sentence_len, rht, ptrn_names, lklhds);
  i = 0;
  strcpy(ptrn, ptrn_names[i++]);
  while (!EQUAL(ptrn,"")) {
    if (prepP_rhtptrn(ptrn)) {
      sprintf(symptoms[symptoms_idx++], "rht_ptrn(%s)", ptrn);
    }
    strcpy(ptrn, ptrn_names[i++]);
  }
  if (has_adjpron(sentence, sentence_len, rht, &lh)) 
    sprintf(symptoms[symptoms_idx++], "rht_ptrn(pron_adj)");
  if (has_objpron(sentence, sentence_len, rht)) 
    sprintf(symptoms[symptoms_idx++], "rht_ptrn(pron_obj)");

  if ((rht != sentence_spoc.sub_pos) AND (rht != sentence_spoc.obj_pos)) {
    get_phrs_name(sentence, sentence_len, rht, phrs_name, &lh);
    if (prepP_rhtphrs(phrs_name)) 
      sprintf(symptoms[symptoms_idx++], "rht_phrs(%s)", phrs_name);
  }

  if (has_pattern(sentence, sentence_len, rht2, "adj_est", &lh)) { 
    sprintf(symptoms[symptoms_idx++], "rht2_adj_est");
    rht2_adj_est = TRUE;
  }
  strcpy(symptoms[symptoms_idx],"");

  mst_smp(symptoms, name, state, type, value, errcde);
  sol_prb(name, state, type, value, errcde);
  if (errcde[0] != 0) {
    stx_err("failed to solve problem", __LINE__, __FILE__);
  }

  if EQUAL(state, "S") { 
    copy_dgns(dgns, prep_dgns);
    get_dgn(dgns,name,state,type,value,errcde);
    if (value[0]) {                      /* has at least one true value */
      char    clss[MAXPHRASE][PTRN_PHRS_MAX+1]; 
      struct  Syntx_Term *stc; 
      int     stc_len;
      struct  SPOC_Pos stc_spoc;

      i = 0;
      while (! EQUAL(dgns[i],"")) {
        strcpy(dgns_var, dgns[i++]);

        if (EQUAL(dgns_var,"next(find_mdf)")) { 
          find_mdf = TRUE;
          continue;
        }
        if (EQUAL(dgns_var,"next(P_A)")) { 
          if (rht2_adj_est) {
            if (intrnl_dsply) {
              printf("suggests adj_phrase at %d first\n\n", rht2);
            }

            stc_push(sentence, sentence_len, &sentence_spoc, AdjPhrase,
                     rht2,lklhd,PrepPhrase);
          }
          else {
            if (intrnl_dsply) {
              printf("suggests adj_phrase at %d first\n\n", rht);
            }

            stc_push(sentence, sentence_len, &sentence_spoc,AdjPhrase,rht,
                     lklhd,PrepPhrase);
          }
          return_val = 3;
          continue;
        }
        if (EQUAL(dgns_var,"next(P_N)")) { 
          if (intrnl_dsply) {
            printf("suggests noun_phrase at %d first\n\n", rht);
          }
          stc_push(sentence,sentence_len,&sentence_spoc,NounPhrase,rht, 
                   lklhd, PrepPhrase);
          return_val = 2;
          continue;
        }
        if (EQUAL(dgns_var,"next(P_A_N)")) { 
          if (intrnl_dsply) {
            printf("suggests noun_phrase at %d first\n\n", rht+1);
          }

          stc_push(sentence,sentence_len,&sentence_spoc, NounPhrase,
                   rht+1,lklhd,PrepPhrase);
          return_val = 4;
          continue;
        }


        if (EQUAL(dgns_var,"is_phrs(prep_sgl)")) {
          strcpy(clss[0], "prep");
          strcpy(clss[1], "noun_sgl");
          stc = (struct Syntx_Term *)stc_cpy(&stc_len, &stc_spoc,
                                   sentence, sentence_len, &sentence_spoc);
          if (prepP_cmb_and_next(stc,&stc_len,&stc_spoc,cnt,clss, 
                                 "prep_sgl", prep_lh, lklhd, find_mdf)) 
            return_val = 1;
          stc_dsp(stc);
          continue;
        }

        if (EQUAL(dgns_var,"is_phrs(prep_pl)")) { 
          strcpy(clss[0], "prep");
          strcpy(clss[1], "noun_pl");
          stc = (struct Syntx_Term *)stc_cpy(&stc_len, &stc_spoc,
                                   sentence, sentence_len, &sentence_spoc);
          if (prepP_cmb_and_next(stc,&stc_len,&stc_spoc,cnt,clss, 
                                 "prep_pl", prep_lh, lklhd, find_mdf)) 
            return_val = 1;
          stc_dsp(stc);
          continue;
        }

        if (EQUAL(dgns_var,"is_phrs(prep_obj)")) { 
          char  pron_ptrn[PTRN_PHRS_MAX+1];

          pronoun_info(sentence, sentence_len, rht, pron_ptrn, &lh);
          strcpy(clss[0], "prep");
          strcpy(clss[1], pron_ptrn);
          stc = (struct Syntx_Term *)stc_cpy(&stc_len, &stc_spoc,
                                   sentence, sentence_len, &sentence_spoc);
          if (prepP_cmb_and_next(stc, &stc_len, &stc_spoc, cnt, clss, 
                                 "prep_obj", prep_lh, lklhd, find_mdf)) 
            return_val = 1;
          stc_dsp(stc);
          continue;
        }
      }
    }
  }
  else stx_err("problem is unsatisfied", __LINE__, __FILE__);


  strcpy(state, "A");
  rst_prb(name, state, type, value, errcde);
  return return_val;
}


/*---------------------------------------------------------------------

  prepP_rhtptrn  --  check if right pattern is considered in the log file

    input  -- ptrn: right pattern name to be checked
    output -- none

  function returns TRUE if ptrn is considered in log file, FALSE otherwise

------------------------------------------------------------------------*/
boolean prepP_rhtptrn(ptrn)
char *ptrn;
{
  char str[4];

  str[0] = ptrn[0];
  str[1] = ptrn[1];
  str[2] = ptrn[2];
  str[3] = '\0';

  if ((EQUAL(str,"nou")) OR
      (EQUAL(str,"art")))   return TRUE;

  if ((EQUAL(ptrn,"vt_ing"))  OR
      (EQUAL(ptrn,"vi_ing"))  OR
      (EQUAL(ptrn,"vt_pprt")) OR
      (EQUAL(ptrn,"vi_pprt")) OR
      (EQUAL(ptrn,"being"))   OR
      (EQUAL(ptrn,"adj"))     OR
      (EQUAL(ptrn,"adj_er"))  OR
      (EQUAL(ptrn,"adv")))       return TRUE;

  return FALSE;
}



/*---------------------------------------------------------------------

  prepP_rhtphrs  --  check if right phrase is considered in the log file

    input  -- phrs: right phrase name to be checked
    output -- none

  function returns TRUE if phrs is considered in log file, FALSE otherwise

------------------------------------------------------------------------*/
boolean prepP_rhtphrs(phrs)
char *phrs;
{
  char str[4];

  str[0] = phrs[0];
  str[1] = phrs[1];
  str[2] = phrs[2];
  str[3] = '\0';

  if ((EQUAL(str,"nou")) OR
      (EQUAL(str,"adj")))   return TRUE;

  if ((EQUAL(phrs,"vt_pprt")) OR
      (EQUAL(phrs,"vi_pprt")))   return TRUE;

  return FALSE;
}

/*---------------------------------------------------------------------

  prepP_cmb_and_next  --  combine tokens and issue next command

    input  -- stc:    sentence
              slen:   sentence length
              spoc:   positions of SPOC;
              cnt:    current position
              clss:   the classifications of the tokens to be combined
              phrase: the name of the combined phrase
              lh1:    likelihood at which the tokens are combined
              lh2:    likelihood at which the the next command is excuted
              find_mdf: indicate the next command is find_mdf

    output -- function return TRUE if some to phrase can be formed,
                              FALSE otherwise.
------------------------------------------------------------------------*/
boolean prepP_cmb_and_next(stc,slen,spoc,cnt,clss,phrase,lh1,lh2,find_mdf)
struct  Syntx_Term *stc; 
int     *slen;
struct  SPOC_Pos *spoc;
int     cnt;
char    clss[MAXPHRASE][PTRN_PHRS_MAX+1]; 
char    *phrase;
int     lh1, lh2;
boolean find_mdf;
{
  if (find_mdf) {
    if (intrnl_dsply) {
      printf("%d + %d  ==>  phrase '%s'\n\n", cnt, cnt+1, phrase);
    }     
   
    cmb_token(stc, slen, spoc, cnt, cnt+1, clss, phrase, lh1);
    stc_push(stc, *slen, spoc, FindModifier, cnt+1, lh2, PrepPhrase);
    return TRUE;
  }

  return FALSE;
}

