#include "syntx.h"

extern struct Syntx_Term *sentence; 
extern int                sentence_len;
extern struct SPOC_Pos    sentence_spoc;
extern boolean            intrnl_dsply;



  
void    pronoun_phrase();
int     do_pronounP();
boolean nounP_lftptrn();
boolean nounP_cntptrn();
boolean nounP_rhtptrn();


/*----------------------------------------------------------------------

  pronoun_phrase  --  control the action of do_pronounP

    input  -- pos:  the position where the action to be taken
              lklhd: the likelyhood at which the action will take
              who: indicate who invoke the function    
    output -- none
 
-----------------------------------------------------------------------*/
void pronoun_phrase(pos, lklhd, who)
int pos;
int lklhd;
int who;
{
  int  pronounP_code;

  if (intrnl_dsply) {
    begin_title("pronoun_phrase", pos, lklhd);
  }

  if ((pronounP_code = do_pronounP(pos, lklhd)) == 0) {
    if (intrnl_dsply) {
      printf("no pronoun phrases can be formed\n");
    }
    goto end;
  }

  switch(who) {
    case FindObject:
                     switch(pronounP_code) {
                       case 0:        /* no pronoun phrase can be formed */
                       case 1:              /* pron_sub phrase is formed */
                       case 8:              /* pron_thr phrase is formed */
                               return;
 
                       case 2:              /* pron_obj phrase is formed */
                       case 3:              /* pron_sgl phrase is formed */
                       case 4:               /* pron_pl phrase is formed */
                       case 5:              /* pron_you phrase is formed */
                       case 6:              /* pron_pss phrase is formed */
                       case 7:              /* pron_slf phrase is formed */

                               sentence_spoc.obj_pos = pos;
                               stc_push(sentence, sentence_len, 
                                        &sentence_spoc, FindSubject, 
                                        sentence_spoc.prd_pos-1,lklhd, 
                                        PronounPhrase);
                               break;

                       default: return;
                               /*stx_err("unexpected code by do_pronounP", 
                                        __LINE__, __FILE__); */
                     }  /* end of switch(pronounP_code) */
                     break;

    case FindSubject: 
                     switch(pronounP_code) {
                       case 0:        /* no pronoun phrase can be formed */
                       case 2:              /* pron_obj phrase is formed */
                               return;

                       case 1:              /* pron_sub phrase is formed */
                       case 3:              /* pron_sgl phrase is formed */
                       case 4:               /* pron_pl phrase is formed */
                       case 5:              /* pron_you phrase is formed */
                       case 6:              /* pron_pss phrase is formed */
                       case 7:              /* pron_slf phrase is formed */
                       case 8:              /* pron_thr phrase is formed */

                               sentence_spoc.sub_pos = pos;
                               stc_push(sentence, sentence_len,
                                        &sentence_spoc, FindModifier, 0, 
                                        lklhd, PronounPhrase);
                               break;
 
                       default: return;
                               /*stx_err("unexpected code by do_pronounP", 
                                        __LINE__, __FILE__); */
                     }  /* end of switch(pronounP_code) */
                     break;

    case FindModifier: 
                     switch(pronounP_code) {
                       case 0:        /* no pronoun phrase can be formed */
                       case 2:              /* pron_obj phrase is formed */
                       case 1:              /* pron_sub phrase is formed */
                       case 3:              /* pron_sgl phrase is formed */
                       case 4:               /* pron_pl phrase is formed */
                       case 5:              /* pron_you phrase is formed */
                       case 6:              /* pron_pss phrase is formed */
                       case 7:              /* pron_slf phrase is formed */
                       case 8:              /* pron_thr phrase is formed */

                               stc_push(sentence, sentence_len,
                                        &sentence_spoc, FindModifier, 0, 
                                        lklhd, PronounPhrase);
                               break;
 
                       default: return;
                               /*stx_err("unexpected code by do_pronounP", 
                                        __LINE__, __FILE__); */
                     }  /* end of switch(pronounP_code) */
                     break;


    default: stx_err("unexpected problem code", __LINE__, __FILE__);
  }  /* end of switch(who) */


end:  
  if (intrnl_dsply) {
    end_title("pronoun_phrase");
  }
}


/*---------------------------------------------------------------------

  do_pronounP  --  form a pronoun phrase

    input  -- pos: the position where the do_pronounP works on
              lklhd: the likelihood at which do_pronounP works 
    output -- none

  function returns 0: no pronoun phrase can be formed
                   1: pron_sub phrase is formed (I, he, she, we or they)
                   2: pron_obj phrase is formed (me, him, her, us or them)
                   3: pron_sgl phrase is formed (it, this, that, one, 
                              anybody, another, everybody, anyone, nobody, 
                              everyone, someone or somebody)
                   4: pron_pl phrase is formed (these or those)
                   5: pron_you phrase is formed (you)
                   6: pron_pss phrase is formed (mine, ours, yours, his, 
                                                 hers or theirs)
                   7: pron_slf phrase is formed (myself, himself, herself, 
                                             itself, ourselves,  yourself, 
                                                  yourselves or themselves)
                   8: pron_thr phrase is formed (there)

----------------------------------------------------------------------*/
int do_pronounP(pos, lklhd)
int pos;
int lklhd;
{
  int  pron_code, lh;
  char used_ptrn[PTRN_PHRS_MAX+1], phrase_name[PTRN_PHRS_MAX+1] = "",
       clss[MAXPHRASE][PTRN_PHRS_MAX+1];


  if ((pos < 0) OR (pos >= sentence_len)) return 0;

  if ((pron_code = pronoun_info(sentence, sentence_len, pos, used_ptrn,&lh))
      == 0) 
    return 0;

  strcpy(clss[0], used_ptrn);

  switch(pron_code) {
    case 1:                                 /* pron_sub phrase is formed */
            if (intrnl_dsply) {
              printf("%d ==>  phrase pron_sub\n\n", pos);
            } 
            strcpy(phrase_name, "pron_sub");
            break;

    case 2:                                 /* pron_obj phrase is formed */
            if (intrnl_dsply) {
              printf("%d ==>  phrase pron_obj\n\n", pos);
            }    
            strcpy(phrase_name, "pron_obj");
            break;

    case 3:                                 /* pron_sgl phrase is formed */
            if (intrnl_dsply) {
              printf("%d ==>  phrase pron_sgl\n\n", pos);
            }     
            strcpy(phrase_name, "pron_sgl");
            break;

    case 4:                                  /* pron_pl phrase is formed */
            if (intrnl_dsply) {
              printf("%d ==>  phrase pron_pl\n\n", pos);
            }    
            strcpy(phrase_name, "pron_pl");
            break;

    case 5:                                 /* pron_you phrase is formed */
            if (intrnl_dsply) {
              printf("%d ==>  phrase pron_you\n\n", pos);
            }     
            strcpy(phrase_name, "pron_you");
            break;

    case 6:                                 /* pron_pss phrase is formed */
            if (intrnl_dsply) {
              printf("%d ==>  phrase pron_pss\n\n", pos);
            }     
            strcpy(phrase_name, "pron_pss");
            break;

    case 7:                                 /* pron_slf phrase is formed */
            if (intrnl_dsply) {
              printf("%d ==>  phrase pron_slf\n\n", pos);
            } 
            strcpy(phrase_name, "pron_slf");
            break;

    case 8:                                 /* pron_thr phrase is formed */
            if (intrnl_dsply) {
              printf("%d ==>  phrase pron_thr\n\n", pos);
            }     
            strcpy(phrase_name, "pron_thr");
            break;

    default:  stx_err("unexpected code by pronoun_info", 
                       __LINE__, __FILE__);

  }

  cmb_token(sentence, &sentence_len, &sentence_spoc, pos, pos, clss,
            phrase_name, lh);

  return pron_code;
}
