#include "syntx.h"

#define        NumofPvDgns      15
#define        NumofPvNext       8

extern struct Syntx_Term *sentence; 
extern int                sentence_len;
extern struct SPOC_Pos    sentence_spoc;
extern boolean            fatal_error;
extern char               fatalerr_msg[];
extern boolean            intrnl_dsply;




DIAGNOSIS_ARRAY pv_dgns[NumofPvDgns + 1] = { 
                                             "err_be_objpron",
                                             "err_wrong_vbtense",
                                             "next(fnd_adv1)",
                                             "next(fnd_adv2)",
                                             "next(find_be1)",
                                             "next(find_be2)",
                                             "next(find_hv1)",
                                             "next(find_hv2)",
                                             "next(find_obj)",
                                             "next(find_mdf)",
                                             "cmb_rht_one",
                                             "cmb_rht_two",
                                             "pv_phrs(pv_be)",
                                             "pv_phrs(pv_vt)",
                                             "pv_phrs(pv_vi)",
                                             "" 
                                           };   

static int        symptoms_idx;
SYMPTOM_ARRAY     symptoms[mmax(2*MAXPTRNS+2,MAXSYMP)];
  
boolean pronvb_phrase();
int     do_pvP();
/*boolean pvP_cntwd();*/
boolean pvP_rht1wd();
boolean pvP_rht2wd();
boolean pvP_cntptrn();
boolean pvP_rht1ptrn();
boolean pvP_rht2ptrn();
boolean pvP_rht1phrs();
boolean pvP_rht2phrs();
void    create_pv_phrase();
void    issue_right_command();

/*----------------------------------------------------------------------

  pronvb_phrase  --  control the action of do_pvP

    input  -- pos:  the position where the action to be taken
              lklhd: the likelyhood at which the action will take
              who: indicate who invoke the function

    output -- function returns FALSE if some fatal errors occured, 
                               TRUE otherwise
 
-----------------------------------------------------------------------*/
boolean pronvb_phrase(pos, lklhd, who)
int pos;
int lklhd;
int who;
{
  int pvP_code, pv_pos;
  boolean return_val;

  if (intrnl_dsply) {
    begin_title("pronvb_phrase", pos, lklhd);
  }

  load_prb("PronvbPhrase");

  if ((pvP_code = do_pvP(pos, lklhd, who, &pv_pos)) == 0) {
    if (intrnl_dsply) {
      printf("no pronvb phrase can be created \n");
    }
/*    fatal_error = TRUE;
    strcpy(fatalerr_msg, "err_incomplete_pronvb");*/
    return_val = TRUE;
    goto end;
  }

/*  if (fatal_error) { clean_stack(); return_val = FALSE; goto end; } */

  switch (pvP_code) {
    case -1:                 /* case error "err_be_objpron" etc. occures */
            clean_stack();
            return_val = FALSE;
            goto end;

    case 1:                                /* some pv phrase is created */
            break;

    case 2:           /* no phrases are formed, but suggests some action */
            break;

    default: stx_err("unexpected pvP_code", __LINE__, __FILE__);
  }
                    
  return_val = TRUE;


end:
  if (intrnl_dsply) {
    end_title("pronvb_phrase");
  }
  return return_val;
}


/*---------------------------------------------------------------------

  do_pvP  --  execute pronvb_phrase.prg  

    input  -- pos: the position where the do_pvP works on
              lklhd: the likelihood the do_pvP works at
              who:   who issued pronvb_phrase
    output -- pv_pos: the position of the formed phrase

  function returns -1: case error "err_be_objpron" or "err_wrong_vbtense"
                       occures
                    0: no pv phrase can be formed
                    1: a pv phrase is created
                    2: no phrases are formed, but suggests some action

----------------------------------------------------------------------*/
int do_pvP(pos, lklhd, who, pv_pos)
int pos, lklhd, who, *pv_pos;
{
  DIAGNOSIS_ARRAY   dgns[NumofPvDgns + 1];
  char              dgns_var[VAR_MAX + 1],  ptrn[PTRN_PHRS_MAX+1],
                    ptrn_names[MAXPTRNS+1][PTRN_PHRS_MAX+1],
                    word[WORD_MAX+1], phrs[9],
                    used_ptrn[3][PTRN_PHRS_MAX+1] = {"", "", ""};
  int               i, cnt, rht1, rht2, lh, lklhds[MAXPTRNS], return_val;
  boolean           pv_be = FALSE, pv_vt = FALSE, pv_vi = FALSE;
  boolean           next[NumofPvNext];

  symptoms_idx = 0;
  cnt = pos;
  rht1 = pos + 1;
  rht2 = pos + 2;
  return_val = -100;
  for (i = 0; i < NumofPvNext; i++) next[i] = FALSE;

/*
  get_word(sentence, sentence_len, cnt, word);
  if (!EQUAL(word,"")) {
    if (pvP_cntwd(word, used_ptrn[0])) {
      if (word_check(word)==0) {
        sprintf(symptoms[symptoms_idx++], "cnt_wd(%s)", word);
      } else {
        sprintf(symptoms[symptoms_idx++], "cnt_wd(%s_)", word);
      }
    }
  }
*/
  get_word(sentence, sentence_len, rht1, word);
  if (!EQUAL(word,"")) {
    if (pvP_rht1wd(word)) {
      if (word_check(word)==0) {
        sprintf(symptoms[symptoms_idx++], "rht1_wd(%s)", word);
      } else {
        sprintf(symptoms[symptoms_idx++], "rht1_wd(%s_)", word);
      }
    }
  }

  get_word(sentence, sentence_len, rht2, word);
  if (!EQUAL(word,"")) {
    if (pvP_rht2wd(word)) {
      if (word_check(word)==0) {
        sprintf(symptoms[symptoms_idx++], "rht2_wd(%s)", word);
      } else {
        sprintf(symptoms[symptoms_idx++], "rht2_wd(%s_)", word);
      }
    }
  }

  get_ptrn_names(sentence, sentence_len, cnt, ptrn_names, lklhds);
  i = 0;
  strcpy(ptrn, ptrn_names[i++]);
  while (!EQUAL(ptrn,"")) {
    if (pvP_cntptrn(ptrn)) {
      sprintf(symptoms[symptoms_idx++], "cnt_ptn(%s)", ptrn);
      if (EQUAL(used_ptrn[0],"")) strcpy(used_ptrn[0], ptrn);
    }
    strcpy(ptrn, ptrn_names[i++]);
  }

  get_ptrn_names(sentence, sentence_len, rht1, ptrn_names, lklhds);
  i = 0;
  strcpy(ptrn, ptrn_names[i++]);
  while (!EQUAL(ptrn,"")) {
    if (pvP_rht1ptrn(ptrn)) {
      sprintf(symptoms[symptoms_idx++], "rht1_ptn(%s)", ptrn);
      if (EQUAL(used_ptrn[1],"")) strcpy(used_ptrn[1], ptrn);
    }
    strcpy(ptrn, ptrn_names[i++]);
  }
  if (has_pst_only(sentence, sentence_len, rht1)) 
    sprintf(symptoms[symptoms_idx++], "rht1_ptn(vb_past)");

  get_ptrn_names(sentence, sentence_len, rht2, ptrn_names, lklhds);
  i = 0;
  strcpy(ptrn, ptrn_names[i++]);
  while (!EQUAL(ptrn,"")) {
    if (pvP_rht2ptrn(ptrn)) {
      sprintf(symptoms[symptoms_idx++], "rht2_ptn(%s)", ptrn);
      if (EQUAL(used_ptrn[2],"")) strcpy(used_ptrn[2], ptrn);
    }
    strcpy(ptrn, ptrn_names[i++]);
  }
  if (has_pst_only(sentence, sentence_len, rht2)) 
    sprintf(symptoms[symptoms_idx++], "rht2_ptn(vb_past)");


  if (pvP_rht1phrs(sentence, sentence_len, rht1, phrs)) 
    sprintf(symptoms[symptoms_idx++], "rht1_ph(%s)", phrs);

  if (pvP_rht2phrs(sentence, sentence_len, rht2, phrs)) 
    sprintf(symptoms[symptoms_idx++], "rht2_ph(%s)", phrs);

  strcpy(symptoms[symptoms_idx],"");

  mst_smp(symptoms, name, state, type, value, errcde);
  sol_prb(name, state, type, value, errcde);
  if (errcde[0] != 0) {
    stx_err("failed to solve problem", __LINE__, __FILE__);
  }

  if EQUAL(state, "S") { 
    copy_dgns(dgns, pv_dgns);
    get_dgn(dgns,name,state,type,value,errcde);
    if (value[0]) {                      /* has at least one true value */
      boolean cmb_rht_one = FALSE, cmb_rht_two = FALSE, has_next = FALSE;
      char    clss[MAXPHRASE][PTRN_PHRS_MAX+1]; 
      struct  Syntx_Term *stc; 
      int     stc_len;
      struct  SPOC_Pos stc_spoc;


      i = 0;
      while (! EQUAL(dgns[i],"")) {
        strcpy(dgns_var, dgns[i++]);
      
        if (EQUAL(dgns_var,"err_be_objpron")) {
          fatal_error = TRUE;
          strcpy(fatalerr_msg, "err_be_objpron");
          return -1;
        }

        if (EQUAL(dgns_var,"err_wrong_vbtense")) {
          fatal_error = TRUE;
          strcpy(fatalerr_msg, "err_wrong_vbtense");
          return -1;
        }

        if (EQUAL(dgns_var,"next(fnd_adv1)")) { 
          next[0] = TRUE;
          has_next = TRUE;  
          continue; 
        }

        if (EQUAL(dgns_var,"next(fnd_adv2)")) { 
          next[1] = TRUE;
          has_next = TRUE;  
          continue; 
        }

        if (EQUAL(dgns_var,"next(find_be1)")) { 
          next[2] = TRUE;
          has_next = TRUE;  
          continue; 
        }

        if (EQUAL(dgns_var,"next(find_be2)")) { 
          next[3] = TRUE;
          has_next = TRUE;  
          continue; 
        }

        if (EQUAL(dgns_var,"next(find_hv1)")) { 
          next[6] = TRUE;
          has_next = TRUE;  
          continue; 
        }

        if (EQUAL(dgns_var,"next(find_hv2)")) { 
          next[7] = TRUE;
          has_next = TRUE;  
          continue; 
        }

        if (EQUAL(dgns_var,"next(find_obj)")) { 
          next[4] = TRUE;
          has_next = TRUE;  
          continue; 
        }

        if (EQUAL(dgns_var,"next(find_mdf)")) { 
          next[5] = TRUE;
          has_next = TRUE;  
          continue; 
        }

        
        if (EQUAL(dgns_var,"cmb_rht_one")) { 
          cmb_rht_one = TRUE; 
          continue; 
        }

        if (EQUAL(dgns_var,"cmb_rht_two")) { 
          cmb_rht_two = TRUE; 
          continue; 
        }


        if (pv_be = (EQUAL(dgns_var,"pv_phrs(pv_be)"))) { 
          return_val = 1;
          continue;
        }
        if (pv_vt = (EQUAL(dgns_var,"pv_phrs(pv_vt)"))) { 
          return_val = 1;
          continue;
        }
        if (pv_vi = (EQUAL(dgns_var,"pv_phrs(pv_vi)"))) {
          return_val = 1;
          continue;
        }
      }
      if (return_val != 1) { 
        if (has_next) { 
          return_val = 2; 
        }
        else { return_val = 0; goto end_of_dobeP; }
      }

      issue_right_command(sentence, &sentence_len, &sentence_spoc,
                          cnt, next, lklhd);

      *pv_pos = cnt;

      strcpy(clss[0], used_ptrn[0]);
      strcpy(clss[1], used_ptrn[1]);
      strcpy(clss[2], used_ptrn[2]);

      if (pv_be) {
        stc = (struct Syntx_Term *)stc_cpy(&stc_len, &stc_spoc,          
                                   sentence, sentence_len, &sentence_spoc);
        /*if (EQUAL(used_ptrn[0],"pl_be")) { strcpy(clss[0], "pl_be"); }
        else { strcpy(clss[0], "sgl_be"); } */
       
        create_pv_phrase(stc, &stc_len, &stc_spoc, cnt,clss,"pv_be",lklhd, 
                         cmb_rht_one, cmb_rht_two);
        if (next[5]) {                                       /* find_mdf */
          stc_spoc.prd_pos = cnt;
          stc_spoc.sub_pos = cnt;
          stc_push(stc, stc_len, &stc_spoc, FindModifier, 0, lklhd, 
                   PronvbPhrase);
        }
        stc_dsp(stc);
      }

      if (pv_vt) {
        stc = (struct Syntx_Term *)stc_cpy(&stc_len, &stc_spoc,          
                                   sentence, sentence_len, &sentence_spoc);
 /*       if (EQUAL(used_ptrn[0],"pl_be")) { strcpy(clss[0], "pl_be"); }
        else { strcpy(clss[0], "sgl_be"); } */

        create_pv_phrase(stc, &stc_len, &stc_spoc, cnt,clss,"pv_vt",lklhd, 
                         cmb_rht_one, cmb_rht_two);
        if (next[4]) {                                       /* find_obj */
          stc_spoc.prd_pos = cnt;
          stc_spoc.sub_pos = cnt;
          stc_push(stc, stc_len, &stc_spoc, FindObject, cnt+1, lklhd, 
                   PronvbPhrase);
        }
        stc_dsp(stc);
      }

      if (pv_vi) {
        stc = (struct Syntx_Term *)stc_cpy(&stc_len, &stc_spoc,          
                                   sentence, sentence_len, &sentence_spoc);
        /*if (EQUAL(used_ptrn[0],"pl_be")) { strcpy(clss[0], "pl_be"); }
        else { strcpy(clss[0], "sgl_be"); } */

        create_pv_phrase(stc, &stc_len, &stc_spoc, cnt,clss,"pv_vi",lklhd, 
                         cmb_rht_one, cmb_rht_two);
        if (next[5]) {                                       /* find_mdf */
          stc_spoc.prd_pos = cnt;
          stc_spoc.sub_pos = cnt;
          stc_push(stc, stc_len, &stc_spoc, FindModifier, 0, lklhd, 
                   PronvbPhrase);
        }
        stc_dsp(stc);
      }
      goto end_of_dobeP;
    }
  }
  else stx_err("problem is unsatisfied", __LINE__, __FILE__);
  
  return_val = 0;

  
end_of_dobeP:

  strcpy(state, "A");
  rst_prb(name, state, type, value, errcde);
  return return_val;
}


/*---------------------------------------------------------------------

  pvP_cntwd  --  check if current word is considered in the log file

    input  -- word: current word to be checked
    output -- ptrn: the pattern of the word

  function returns TRUE if word is considered in log file, FALSE otherwise

------------------------------------------------------------------------*/
/*
boolean pvP_cntwd(word, ptrn)
char *word, *ptrn;
{
  int  i, len;
  char ch;

  len = strlen(word);
  for (i = 0; i < len; i++) {
    if ((ch = word[i]) == '\'') {
      word[i] = '_';
    }
    else word[i] = tolower(ch);
  }


  if (EQUAL(word,"she_s")  OR
      EQUAL(word,"he_s")   OR
      EQUAL(word,"it_s")   OR
      EQUAL(word,"i_m")    OR
      EQUAL(word,"you_re"))   { strcpy(ptrn, "sgl_be"); return TRUE; }

  if (EQUAL(word,"we_re")  OR
      EQUAL(word,"they_re"))  { strcpy(ptrn, "pl_be"); return TRUE; }

  return FALSE;
}
*/


/*---------------------------------------------------------------------

  pvP_rht1wd  --  check if right1 word is considered in the log file

    input  -- word: right1 word to be checked
    output -- none

  function returns TRUE if word is considered in log file, FALSE otherwise

------------------------------------------------------------------------*/
boolean pvP_rht1wd(word)
char *word;
{
  int  i, len;

  len = strlen(word);
  for (i = 0; i < len; i++) word[i] = tolower(word[i]);

  if (EQUAL(word,"us")   OR
      EQUAL(word,"him")  OR
      EQUAL(word,"her")  OR
      EQUAL(word,"them") OR
      EQUAL(word,"have") OR
      EQUAL(word,"had"))   return TRUE;

  return FALSE;
}


/*---------------------------------------------------------------------

  pvP_rht2wd  --  check if right2 word is considered in the log file

    input  -- word: right2 word to be checked
    output -- none

  function returns TRUE if word is considered in log file, FALSE otherwise

------------------------------------------------------------------------*/
boolean pvP_rht2wd(word)
char *word;
{
  int  i, len;

  len = strlen(word);
  for (i = 0; i < len; i++) word[i] = tolower(word[i]);

  if (EQUAL(word,"us")   OR
      EQUAL(word,"him")  OR
      EQUAL(word,"her")  OR
      EQUAL(word,"them") OR
      EQUAL(word,"have") OR
      EQUAL(word,"had"))   return TRUE;

  return FALSE;
}


/*---------------------------------------------------------------------

  pvP_cntptrn  --  check if current pattern is considered in the log file

    input  -- ptrn: current pattern name to be checked
    output -- none

  function returns TRUE if ptrn is considered in log file, FALSE otherwise

------------------------------------------------------------------------*/
boolean pvP_cntptrn(ptrn)
char *ptrn;
{
  char str[4];

  str[0] = ptrn[0];
  str[1] = ptrn[1];
  str[2] = ptrn[2];
  str[3] = '\0';

  if (EQUAL(str,"sgl")  OR
      EQUAL(str,"pl_"))   return TRUE;

  return FALSE;
}


/*---------------------------------------------------------------------

  pvP_rht1ptrn  --  check if right1 pattern is considered in the log file

    input  -- ptrn: right1 pattern name to be checked
    output -- none

  function returns TRUE if ptrn is considered in log file, FALSE otherwise

------------------------------------------------------------------------*/
boolean pvP_rht1ptrn(ptrn)
char *ptrn;
{
  char str[3];

  str[0] = ptrn[0];
  str[1] = ptrn[1];
  str[2] = '\0';

  if (EQUAL(str,"vt") OR EQUAL(str,"vi")) {
    if ((!EQUAL(ptrn,"vt_pst")) AND
        (!EQUAL(ptrn,"vi_pst")))    return TRUE;
  }

  if (EQUAL(ptrn,"be")       OR
      EQUAL(ptrn,"been")     OR
      EQUAL(ptrn,"lvb_pl")   OR
      EQUAL(ptrn,"lvb_sgl")  OR
      EQUAL(ptrn,"artcl_df") OR
      EQUAL(ptrn,"adv")      OR
      EQUAL(ptrn,"adv_er"))     return TRUE;

  return FALSE;
}

/*---------------------------------------------------------------------

  pvP_rht2ptrn  --  check if right2 pattern is considered in the log file

    input  -- ptrn: right2 pattern name to be checked
    output -- none

  function returns TRUE if ptrn is considered in log file, FALSE otherwise

------------------------------------------------------------------------*/
boolean pvP_rht2ptrn(ptrn)
char *ptrn;
{
  char str[3];

  str[0] = ptrn[0];
  str[1] = ptrn[1];
  str[2] = '\0';

  if (EQUAL(str,"vt") OR EQUAL(str,"vi")) {
    if ((!EQUAL(ptrn,"vt_pst")) AND
        (!EQUAL(ptrn,"vi_pst")))    return TRUE;
  }

  if (EQUAL(ptrn,"be")       OR
      EQUAL(ptrn,"been")     OR
      EQUAL(ptrn,"lvb_pl")   OR
      EQUAL(ptrn,"lvb_sgl")  OR
      EQUAL(ptrn,"adv_est"))     return TRUE;

  return FALSE;
}

/*----------------------------------------------------------------------

  pvP_rht1phrs  --  check if right1 phrase is considered in the log file

    input  -- stc:  a sentence
              slen: sentence length
              idx:  token index
    output -- rht1_phrs: name of right1 phrase

   function returns TRUE if it is considered in log file, FALSE otherwise

-----------------------------------------------------------------------*/
boolean pvP_rht1phrs(stc, slen, idx, phrs)
struct Syntx_Term *stc;
int    slen;
int    idx;
char   *phrs;
{
  int  lh;
  char str[4];

  get_phrs_name(stc, slen, idx, phrs, &lh);

  if (EQUAL(phrs,"")) return FALSE;

  str[0] = phrs[0];
  str[1] = phrs[1];
  str[2] = phrs[2];
  str[3] = '\0';

  if (EQUAL(str,"hv_") OR
      EQUAL(str,"hd_") OR
      EQUAL(str,"bee") OR
      EQUAL(str,"adv"))   return TRUE;

  if ((str[0] == 'b') AND (str[1] == 'e')) {
    if (EQUAL(phrs,"being")) return FALSE;
    return TRUE;
  }

  return FALSE;
}


/*----------------------------------------------------------------------

  pvP_rht2phrs  --  check if right2 phrase is considered in the log file

    input  -- stc:  a sentence
              slen: sentence length
              idx:  token index
    output -- rht2_phrs: name of right2 phrase

   function returns TRUE if it is considered in log file, FALSE otherwise

-----------------------------------------------------------------------*/
boolean pvP_rht2phrs(stc, slen, idx, phrs)
struct Syntx_Term *stc;
int    slen;
int    idx;
char   *phrs;
{
  int  lh;
  char str[4];

  get_phrs_name(stc, slen, idx, phrs, &lh);

  if (EQUAL(phrs,"")) return FALSE;

  str[0] = phrs[0];
  str[1] = phrs[1];
  str[2] = phrs[2];
  str[3] = '\0';

  if (EQUAL(str,"hv_") OR
      EQUAL(str,"hd_") OR
      EQUAL(str,"bee"))   return TRUE;

  if ((str[0] == 'b') AND (str[1] == 'e')) {
    if (EQUAL(phrs,"being")) return FALSE;
    return TRUE;
  }

  return FALSE;
}


/*---------------------------------------------------------------------

  create_pv_phrase  --  combine tokens to create a pv phrase

    input  -- stc:    sentence
              slen:   sentence length
              spoc:   positions of SPOC
              cnt:    current position
              clss:   the classifications of the tokens to be combined
              phrase: the name of the pv phrase
              lh:     likelihood
              cmb1:   indicate combining cnt and right1
              cmb2:   indicate combining cnt, right1 and right2

    output -- none

------------------------------------------------------------------------*/
void create_pv_phrase(stc, slen, spoc, cnt, clss, phrase, lh, cmb1, cmb2)
struct  Syntx_Term *stc; 
int     *slen;
struct  SPOC_Pos *spoc;
int     cnt;
char    clss[MAXPHRASE][PTRN_PHRS_MAX+1]; 
char    *phrase;
int     lh;
boolean cmb1;
boolean cmb2;
{

  if (!cmb1 AND !cmb2) {
    if (intrnl_dsply) {
      printf("%d  ==>  phrase '%s'\n\n", cnt, phrase);
    }

    cmb_token(stc, slen, spoc, cnt, cnt, clss, phrase, lh);
    return;
  }

  if (cmb1) { 
    if (intrnl_dsply) {
      printf("%d + %d  ==>  phrase '%s'\n\n", cnt, cnt+1, phrase);
    }    
   
    cmb_token(stc, slen, spoc, cnt, cnt+1, clss, phrase, lh);
    return;
  }

  if (cmb2) { 
    if (intrnl_dsply) {
      printf("%d + %d + %d  ==>  phrase '%s'\n\n",cnt,cnt+1,cnt+2, phrase);
    }    
  
    cmb_token(stc, slen, spoc, cnt, cnt+2, clss, phrase, lh);
    return;
  }
}



/*---------------------------------------------------------------------

  issue_right_command  --  issue the next right command

    input  -- stc:    sentence
              slen:   sentence length
              spoc:   positions of SPOC
              cnt:    current position
              next:   indicate the next command to be executed
              lh:     likelihood

    output -- none

------------------------------------------------------------------------*/
void issue_right_command(stc, slen, spoc, cnt, next, lh)
struct  Syntx_Term *stc; 
int     *slen;
struct  SPOC_Pos *spoc;
int     cnt;
boolean next[NumofPvNext];
int     lh;
{

  if (next[0]) {                                             /* fnd_adv1 */
    stc_push(stc, *slen, spoc, AdvPhrase, cnt+1, lh, PronvbPhrase);
  }
  if (next[1]) {                                             /* fnd_adv2 */
    stc_push(stc, *slen, spoc, AdvPhrase, cnt+2, lh, PronvbPhrase);
  }
  if (next[2]) {                                             /* find_be1 */
    stc_push(stc, *slen, spoc, BePhrase, cnt+1, lh, PronvbPhrase);
  }
  if (next[3]) {                                             /* find_be2 */
    stc_push(stc, *slen, spoc, BePhrase, cnt+2, lh, PronvbPhrase);
  }
  if (next[6]) {                                             /* find_hv1 */
    stc_push(stc, *slen, spoc, HavePhrase, cnt+1, lh, PronvbPhrase);
  }
  if (next[7]) {                                             /* find_hv2 */
    stc_push(stc, *slen, spoc, HavePhrase, cnt+2, lh, PronvbPhrase);
  }
  
}
