/*  first record of prsprd.c***** */
#include<string.h>
#include<stdio.h>
#include"trparms.h"
#include"trexts.h"
#include"trfdefs.h"
/*
 * ***************************************************
 *  subroutine prsprd
 * 
 *  purpose:  parse predicates section of source
 * 
 *            update the following data structures:
 * 
 *              prdnam :-     stores names of predicates
 *              prdset :-     records index of set(s)
 *                            predicate defined on
 *              sused  :-     records that a set was used
 *                            by a predicate
 *              cnfvar :-     list of all variables
 *                            generated by given
 *                            predicate definitions.
 *              delopt(j) :-  delete option for a
 *                            variable.
 *              cstval(j, ):- cost information for
 *                             a variable.
 *              varopt(j) :-  assigned true/false
 *                            information.
 * 
 *  caution:  the input string is in tmpstr
 * 
 * ****************************************************
 * 
 */
void prsprd() {
/*
 */
  static char ctmp[58+1];
  static char type[1+1];
  static char str[20+1];
  static long indx,l,r,err,tidx,minus,i,ix;
  static long chk,scnt1,vidx,rnam;
  static long n;
  static char tmpnam[58+1],nam[58+1];
  static char msgstr[128+1];
/*
 */
  void chknam();
  void ckeoln();
  void extcmp();
  void fndnam();
  void inperr();
  void maknam();
  void prsopt();
  int  readlogfil();
  void scrwrt();
/*
 * initialize local variables and strings
 */
  ctmp_(58+1) = '\0';
  type_(1+1) = '\0';
  str_(20+1) = '\0';
  tmpnam_(58+1) = '\0';
  nam_(58+1) = '\0';
  msgstr_(128+1) = '\0';
/*
 *  read next record from log file
 */
  zz50:;
  if (readlogfil()==0) {
    return;
  }
/*
 */
  l=0;
  r=0;
/*
 *  get next component of tmpstr
 */
  zz60:;
  extcmp(tmpstr,&l,&r);
  if (lstrncmp(&tmpstr_(l),"ENDATA",r-l+1)==0) {
    inperr(&fatal);
    strcpy(msgstr," LBCC2040: Unexpected ENDATA encountered");
    scrwrt("+W",msgstr);
    return;
  }
/*
 * -------------------------------------------------
 *  state definitions
 * -------------------------------------------------
 * -------------------------------------------------
 *  state q2000 is keyword define
 * -------------------------------------------------
 */
  zz2000:;
  if (strcmp(state ,"Q2000")==0) {
    if (lstrncmp(&tmpstr_(l),"DEFINE",r-l+1)==0) {
/*
 *  check if too many predicates are defined
 */
      if (nprds==prdmax) {
        inperr(&fatal);
        n = prdmax;
        sprintf(msgstr," LBCC2090: Too many predicates;"
            " at most %ld allowed",n);
        scrwrt("+W",msgstr);
        return;
      }
      strcpy(state ,"Q2020");
/*
 *  get next component of current record
 */
      goto zz60;
    } else {
      inperr(&fatal);
      strcpy(msgstr," LBCC2060: Keyword DEFINE expected");
      scrwrt("+W",msgstr);
      return;
    }
  }
/*
 * -----------------------------------
 *  state q2020 is predicate name
 * -----------------------------------
 */
  if (strcmp(state ,"Q2020")==0) {
/*
 *  check if legal name
 */
    n = min(r-l+1,58);
    strncpy(nam,&tmpstr_(l),n);
    nam_(n+1) = '\0';
    chknam(nam,&err,"P");
    if (err!=0) {
      strcpy(state ,"Q2100");
      goto zz50;
    }
/*
 *  check if name already referenced
 */
    fndnam(nam,&indx,type);
    if (indx>0) {
      inperr(&nftl);
      if (strcmp(type  ,"E")==0) {
        strcpy(str   ,"element");
      } else if (strcmp(type  ,"S")==0) {
        strcpy(str   ,"set");
      } else if (strcmp(type  ,"P")==0) {
        strcpy(str   ,"predicate");
      } else if (strcmp(type  ,"Q")==0) {
        error(" prsprd ","  202   ");
      } else if (strcmp(type  ,"V")==0) {
        error(" prsprd ","  204   ");
      } else {
        error(" prsprd ","  206   ");
      }
      strncpy(auxmsg,&tmpstr_(l),r-l+1);
      auxmsg_(r-l+2) = '\0';
      sprintf(msgstr,
          " LBCC1010: %s has already been defined"
          " as a(n) %s",auxmsg,str);
      scrwrt("+W",msgstr);
      strcpy(state ,"Q2100");
      goto zz50;
    }
    nprds=nprds+1;
    strncpy(&prdnam_(1,nprds),&tmpstr_(l),r-l+1);
    prdnam_(r-l+2,nprds) = '\0';
/*
 *  initialize set count = number of sets predicate
 *  defined on.
 */
    prdset_(3,nprds)=0;
    strcpy(state ,"Q2030");
    goto zz60;
  }
/*
 * -----------------------------------
 *  state q2030 is keyword "ON"
 * -----------------------------------
 */
  if (strcmp(state ,"Q2030")==0) {
    if (lstrncmp(&tmpstr_(l),"ON",r-l+1)!=0) {
      inperr(&fatal);
      strcpy(msgstr," LBCC2100: Keyword ON expected");
      scrwrt("+W",msgstr);
      return;
    }
    strcpy(state ,"Q2040");
    goto zz60;
  }
/*
 * ---------------------------------------
 *  state q2040 is <setname> of predicate
 * ---------------------------------------
 */
  if (strcmp(state ,"Q2040")==0) {
    prdset_(3,nprds)=prdset_(3,nprds)+1;
/*
 *  check if legal name
 */
    n = min(r-l+1,58);
    strncpy(nam,&tmpstr_(l),n);
    nam_(n+1) = '\0';
    chknam(nam,&err,"S");
    if (err!=0) {
      strcpy(state ,"Q2100");
      goto zz50;
    }
/*
 *  search for the setname
 */
    n = min(r-l+1,58);
    strncpy(nam,&tmpstr_(l),n);
    nam_(n+1) = '\0';
    fndnam(nam,&indx,type);
    if ((indx>0)&&
        (strcmp(type  ,"S")==0)) {
      prdset_(prdset_(3,nprds),nprds)=indx;
/*
 *  define sused_(index) for this set to be 1
 *  to indicate that the set has been used
 */
      sused_(indx)=1;
      strcpy(state ,"Q2050");
      goto zz60;
    } else {
/*
 *  set name not found; error
 */
      inperr(&nftl);
      strncpy(auxmsg,&tmpstr_(l),r-l+1);
      auxmsg_(r-l+2) = '\0';
      sprintf(msgstr,
          " LBCC1050: Unrecognized set name: %s",
          auxmsg);
      scrwrt("+W",msgstr);
      strcpy(state ,"Q2100");
      goto zz50;
    }
  }
/*
 * ---------------------------------------------------
 *  state q2050 is additional predicate arguments.
 *  allow maximum of two set definitions.
 * ---------------------------------------------------
 */
  if (strcmp(state ,"Q2050")==0) {
/*
 *  if '$' then no more args for this predicate
 */
    if (lstrncmp(&tmpstr_(l),"$",r-l+1)==0) {
/*
 *  have predicate fully defined; generate all possible
 *  variable names record beginning index of current
 *  predicate variables
 */
      tidx=ncnfv+1;
      if (prdset_(3,nprds)==1) {
/*
 *  single argument case
 */
        minus=0;
        for(i=1; i<=setdat_(eltmax+1,prdset_(1,nprds)); i++)  {
          strcpy(nam,&prdnam_(1,nprds));
          maknam(nam,
              &eltnam_(1,setdat_(i,prdset_(1,nprds))),
              "        ",tmpnam,&minus);
          ncnfv=ncnfv+1;
          if (ncnfv>cvmax) {
            inperr(&fatal);
            n = cvmax;
            sprintf(msgstr," LBCC2110: Cannot exceed "
                "%ld total variables",n);
            scrwrt("+W",msgstr);
            return;
          }
          strcpy(&cnfvar_(1,ncnfv),tmpnam);
          strcpy(&cstval_(1,1,ncnfv),truec);
          strcpy(&cstval_(1,2,ncnfv),falsc);
          delopt_(ncnfv)=dopt;
          varopt_(ncnfv)=avopt;
          strcpy(&cnfvarflg_(ncnfv),"");
        }
      } else if (prdset_(3,nprds)==2) {
/*
 * two argument case
 */
        minus=0;
        for(i=1; i<=setdat_(eltmax+1,prdset_(1,nprds)); i++)  {
          for(ix=1; ix<=setdat_(eltmax+1,prdset_(2,nprds)); ix++)  {
            strcpy(nam,&prdnam_(1,nprds));
            maknam(nam,
                &eltnam_(1,setdat_(i, prdset_(1,nprds))),
                &eltnam_(1,setdat_(ix,prdset_(2,nprds))),
                tmpnam,&minus);
            ncnfv=ncnfv+1;
            if (ncnfv>cvmax) {
              inperr(&fatal);
              n = cvmax;
              sprintf(msgstr," LBCC2110: Cannot exceed "
                  "%ld total variables",n);
              scrwrt("+W",msgstr);
              return;
            }
            strcpy(&cnfvar_(1,ncnfv),tmpnam);
            strcpy(&cstval_(1,1,ncnfv),truec);
            strcpy(&cstval_(1,2,ncnfv),falsc);
            delopt_(ncnfv)=dopt;
            varopt_(ncnfv)=avopt;
            strcpy(&cnfvarflg_(ncnfv),"");
          }
        }
      } else {
/*
 *  number of args must be 1 or 2; already checked,
 *  so programming error.
 */
        error(" prsprd ","  2082  ");
      }
/*
 *  initialize variable options
 */
      strcpy(truec ,"      ");
      strcpy(falsc ,"      ");
      dopt=0;
      avopt=0;
      strcpy(state ,"Q2060");
      goto zz50;
    } else if ((lstrncmp(&tmpstr_(l),"X",r-l+1)==0)||
               (lstrncmp(&tmpstr_(l),"x",r-l+1)==0)) {
      if (prdset_(3,nprds)==2) {
        inperr(&nftl);
        strcpy(msgstr,
           " LBCC1060: Too many sets defined on predicate");
        scrwrt("+W",msgstr);
        strcpy(state ,"Q2100");
        goto zz50;
      }
      strcpy(state ,"Q2040");
      goto zz60;
    } else {
/*
 *  retrieve options
 */
      chk=0;
      strcpy(state ,"Q5000");
      r=l-2;
      prsopt(&l,&r,&chk,&err);
      if (err!=0) {
        strcpy(state ,"Q2100");
        goto zz50;
      } else {
        strcpy(state ,"Q2050");
/*
 *  tmpstr(l to r) now contains '$'
 *  (end of string character).
 *  loop around to execute state q2050 again to generate
 *  predicate variables
 */
        goto zz2000;
      }
    }
  }
/*
 * ---------------------------------------------------
 *  state q2060 is appropriate branch.  may get
 *  'DEFINE', 'VARIABLES', 'FACTS' or cost specification.
 * ---------------------------------------------------
 */
  if (strcmp(state ,"Q2060")==0) {
/*
 *  branch on keyword
 */
    if (lstrncmp(&tmpstr_(l),"DEFINE",r-l+1)==0) {
      strcpy(state ,"Q2000");
      goto zz2000;
    }
    if (lstrncmp(&tmpstr_(l),"VARIABLES",r-l+1)==0) {
      ckeoln(tmpstr,&r);
      strcpy(state ,"Q3000");
      return;
    } else if (lstrncmp(&tmpstr_(l),"FACTS",r-l+1)==0) {
      ckeoln(tmpstr,&r);
      strcpy(state ,"Q4000");
      return;
    } else if (lstrncmp(&tmpstr_(l),"PREDICATES",r-l+1)==0) {
      inperr(&fatal);
      strcpy(msgstr," LBCC2130: PREDICATES section "
          "already encountered");
      scrwrt("+W",msgstr);
      return;
    } else {
      if (lstrncmp(&tmpstr_(l),"SETS",r-l+1)==0) {
        inperr(&fatal);
        strcpy(msgstr," LBCC2140: SETS section must come "
            "before PREDICATES");
        scrwrt("+W",msgstr);
        return;
      }
    }
/*
 *  otherwise assume variable option specification.
 *  check that predicate name matches current predicate name.
 */
    if (lstrncmp(&tmpstr_(l),&prdnam_(1,nprds),r-l+1)!=0) {
      inperr(&nftl);
      sprintf(msgstr,
          " LBCC1080: Predicate name mismatch "
          "in variable option specification");
      scrwrt("+W",msgstr);
      strcpy(state ,"Q2060");
      goto zz50;
    }
/*
 *  copy prd name to temp (to build positive literal name)
 */
    rnam=0;
    strncpy(&ctmp_(rnam+1),&tmpstr_(l),r-l+1);
    rnam=rnam+1+r-l;
    strcpy(state ,"Q2070");
    goto zz60;
  }
/*
 * ------------------------------------
 *  state q2070 is "("
 * ------------------------------------
 */
  if (strcmp(state ,"Q2070")==0) {
    if (lstrncmp(&tmpstr_(l),"(",r-l+1)==0) {
/*
 *  initialize running set count index
 */
      scnt1=0;
      rnam=rnam+1;
      ctmp_(rnam) = '(';
      strcpy(state ,"Q2080");
      goto zz60;
    } else {
      inperr(&nftl);
      strncpy(auxmsg,&tmpstr_(l),r-l+1);
      auxmsg_(r-l+2) = '\0';
      sprintf(msgstr,
          " LBCC1090: '(' expected; got: %s",
          auxmsg);
      scrwrt("+W",msgstr);
      strcpy(state ,"Q2060");
      goto zz50;
    }
  }
/*
 * ------------------------------------
 *  state q2080 is <element name>
 * ------------------------------------
 */
  if (strcmp(state ,"Q2080")==0) {
    scnt1=scnt1+1;
/*
 *  check if number of arguments specified exceeds that
 *  of definition
 */
    if (scnt1>prdset_(3,nprds)) {
      inperr(&nftl);
      strcpy(msgstr," LBCC1100: Incorrect number of args "
          "in variable option specification");
      scrwrt("+W",msgstr);
      strcpy(state ,"Q2060");
      goto zz50;
    }
/*
 *  search for matching element name
 */
    n = min(r-l+1,58);
    strncpy(nam,&tmpstr_(l),n);
    nam_(n+1) = '\0';
    fndnam(nam,&indx,type);
    if ((indx>0)&&
        (strcmp(type  ,"E")==0)) {
      rnam=rnam+1;
      strncpy(&ctmp_(rnam),&tmpstr_(l),r-l+1);
      rnam=rnam+r-l;
/*
 *  search for index in set; the element must be a member
 *  of current set.
 */
      if (setdat_(eltmax+1,prdset_(scnt1,nprds))>0) {
        for(ix=1; ix<=setdat_(eltmax+1,prdset_(scnt1,nprds)); ix++)  {
          if (setdat_(ix,prdset_(scnt1,nprds))==indx) {
            strcpy(state ,"Q2090");
            goto zz60;
          }
        }
/*
 *  element not in this set
 */
        inperr(&nftl);
        strncpy(auxmsg,&tmpstr_(l),r-l+1);
        auxmsg_(r-l+2) = '\0';
        sprintf(msgstr,
            "LBCC1110: Element %s is not in set %s",
            auxmsg,&setnam_(1,prdset_(scnt1,nprds)));
        scrwrt("+W",msgstr);
        strcpy(state ,"Q2060");
        goto zz50;
      }
    } else {
      inperr(&nftl);
      strncpy(auxmsg,&tmpstr_(l),r-l+1);
      auxmsg_(r-l+2) = '\0';
      sprintf(msgstr,
          " LBCC1120: Unrecognized element name: %s",
          auxmsg);
      scrwrt("+W",msgstr);
      strcpy(state ,"Q2060");
      goto zz50;
    }
  }
/*
 * ------------------------------------
 *  state q2090 is ")" or ','
 * ------------------------------------
 */
  if (strcmp(state ,"Q2090")==0) {
    if (lstrncmp(&tmpstr_(l),",",r-l+1)==0) {
      rnam=rnam+1;
      ctmp_(rnam) = ',';
      strcpy(state ,"Q2080");
      goto zz60;
    } else if (lstrncmp(&tmpstr_(l),")",r-l+1)==0) {
/*
 *  check if number of arguments is not equal to that
 *  of definition.
 */
      if (scnt1!=prdset_(3,nprds)) {
        inperr(&nftl);
        strcpy(msgstr," LBCC1100: Incorrect number of args "
            "in variable option specification");
        scrwrt("+W",msgstr);
        strcpy(state ,"Q2060");
        goto zz50;
      }
      rnam=rnam+1;
      ctmp_(rnam) = ')';
      rnam=rnam+1;
      ctmp_(rnam) = '\0';
    } else {
      inperr(&nftl);
      strncpy(auxmsg,&tmpstr_(l),r-l+1);
      auxmsg_(r-l+2) = '\0';
      sprintf(msgstr,
          " LBCC1130: ')' or ',' expected; got: %s",
          auxmsg);
      scrwrt("+W",msgstr);
      strcpy(state ,"Q2060");
      goto zz50; 
    }
/*
 *  search for specified predicate variable in cnfvar
 *  and give error if duplicate cost/delete option
 *  specification; or add del. opt. and cost information.
 *  tidx is start index in cnfvar of current predicate
 *  variables.
 *  ncnfv is known to be greater than zero from state q2050.
 */
    for(i=tidx; i<=ncnfv; i++)  {
      if (strcmp(&cnfvar_(1,i),ctmp)==0) {
        if (strcmp(&cnfvarflg_(i),"2")==0) {
          inperr(&warn);
          sprintf(msgstr,
              " LBCC0060: Redefinition of variable options for "
              "%s ignored",ctmp);
          scrwrt("+W",msgstr);
          strcpy(state ,"Q2060");
          goto zz50;
        } else {
/*
 *  variable not redefined with delete option/cost;
 *  store index of var.
 */
          vidx=i;
          goto zz2910;
        }
      }
    }
/*
 *  error: name not found
 */
    error("prsprd","2902");
    zz2910:;
    strcpy(state ,"Q5000");
    chk=1;
    prsopt(&l,&r,&chk,&err);
    if (err!=0) {
      strcpy(state ,"Q2060");
      goto zz50;
    }
/*
 *  store options
 */
    strcpy(&cstval_(1,1,vidx),truec);
    strcpy(&cstval_(1,2,vidx),falsc);
    delopt_(vidx)=dopt;
    varopt_(vidx)=avopt;
    strcpy(&cnfvarflg_(vidx),"2");
/*
 *  initialize variable options
 */
    strcpy(truec ,"      ");
    strcpy(falsc ,"      ");
    dopt=0;
    avopt=0;
    strcpy(state ,"Q2060");
    goto zz50;
  }
/*
 * ------------------------------------
 *  state q2100 is dummy state when bad
 *    set name in predicate definition
 * ------------------------------------
 */
  if (strcmp(state ,"Q2100")==0) {
/*
 *  branch on keyword
 */
    if (lstrncmp(&tmpstr_(l),"DEFINE",r-l+1)==0) {
      strcpy(state ,"Q2000");
      goto zz2000;
    }
    if (lstrncmp(&tmpstr_(l),"VARIABLES",r-l+1)==0) {
      strcpy(state ,"Q3000");
      return;
    } else if (lstrncmp(&tmpstr_(l),"FACTS",r-l+1)==0) {
      strcpy(state ,"Q4000");
      return;
    } else if (lstrncmp(&tmpstr_(l),"SETS",r-l+1)==0) {
      inperr(&fatal);
      strcpy(msgstr,
          " LBCC2140: SETS section must come"
          " before PREDICATES");
      scrwrt("+W",msgstr);
      return;
    } else if (lstrncmp(&tmpstr_(l),"PREDICATES",r-l+1)==0) {
      inperr(&fatal);
      strcpy(msgstr,
          "  LBCC2130: PREDICATES section already encountered");
      scrwrt("+W",msgstr);
      return;
    } else {
/*
 *  must be a cost specification; skip it
 */
      goto zz50;
    }
  }
/*
 * ---------------------------------------------------
 *  error section
 * ---------------------------------------------------
 *  error if state not found
 */
  fprintf(errfil,"STATE = %s", state);
  printf("STATE = %s", state);
  error(" prsprd ","  2802  ");
}
/*  last record of prsprd.c****** */
