#include "syntx.h"

#define        NumSegerrDgns    5


extern struct  Syntx_Term *sentence; 
extern int                 sentence_len;
extern boolean             obvious_error;
extern boolean             intrnl_dsply;

DIAGNOSIS_ARRAY segerr_dgns[NumSegerrDgns + 1] = { 
                                                   "err_double_negative",
                                                   "err_redundancies",
                                                   "err_wrong_comp",
                                                   "err_double_prep",
                                                   "err_misc",
                                                   "" 
                                                 };   


char              obverr_msg[100];
int               obverr_start, obverr_end;
static int        symptoms_idx;
SYMPTOM_ARRAY     symptoms[mmax(4,MAXSYMP)];



boolean find_segment_err();
boolean segment_error();
boolean segerr_cntwd();
boolean segerr_rht1wd();
boolean segerr_rht2wd();



/*----------------------------------------------------------------------

  find_segment_err  --  control the action of segment_error.prg

    input  -- none
    output -- none

  function returns TRUE if some obvious error is found, FALSE otherwise.

------------------------------------------------------------------------*/
boolean find_segment_err()
{
  int   i, len;

  if (intrnl_dsply) {
    begin_title("segment_error", -1, -1);
  }

  load_prb("FindSegmentErr");
  len = sentence_len - 1;
  for (i = 0; i < len; i++) {
    obverr_start = i;
    if (obvious_error=segment_error(sentence, sentence_len, i, obverr_msg)){
      if (intrnl_dsply) {
        end_title("segment_error");
      }
      return obvious_error;
    }
  }
  if (intrnl_dsply) {
    end_title("segment_error");
  }
  return obvious_error;
}



/*---------------------------------------------------------------------

  segment_error  --  execute the segment_error.prg to check if there is 
                     an obvious error around the current, right1 and 
                     right2 words

    input  -- stc:   a sentence
              slen:  sentence length
              idx:   current token index

    output -- msg:   error message if an obvious error occures

  function returns TRUE if there is an obvious error, FALSE otherwise.

----------------------------------------------------------------------*/
boolean segment_error(stc, slen, idx, msg)
struct Syntx_Term *stc;
int    slen;
int    idx;
char   *msg;
{
  int   cnt, rht1, rht2, i;
  char  dgns_var[VAR_MAX + 1], word1[WORD_MAX+1], word2[WORD_MAX+1],
        word3[WORD_MAX+1];
  DIAGNOSIS_ARRAY dgns[NumSegerrDgns + 1];
  boolean has_error = FALSE;


  symptoms_idx = 0;
  cnt = idx;
  rht1 = idx + 1;
  rht2 = idx + 2;

  get_word(sentence, sentence_len, cnt, word1);
  get_word(sentence, sentence_len, rht1, word2);
  get_word(sentence, sentence_len, rht2, word3);
  
  if ((!EQUAL(word1,""))      AND
      (!EQUAL(word1,"UNDEF")))    {
    if (segerr_cntwd(word1)) {
      if (word_check(word1)==0) {
        sprintf(symptoms[symptoms_idx++], "cnt_wd(%s)", word1);
      } else {
        sprintf(symptoms[symptoms_idx++], "cnt_wd(%s_)", word1);
      }
    }
  }

  if ((!EQUAL(word2,""))      AND
      (!EQUAL(word2,"UNDEF")))    {
    if (segerr_rht1wd(word2)) {
      if (word_check(word2)==0) {
        sprintf(symptoms[symptoms_idx++], "rht1_wd(%s)", word2);
      } else {
        sprintf(symptoms[symptoms_idx++], "rht1_wd(%s_)", word2);
      }
    }
  }

  if ((!EQUAL(word3,""))      AND
      (!EQUAL(word3,"UNDEF")))    {
    if (segerr_rht2wd(word3)) {
      if (word_check(word3)==0) {
        sprintf(symptoms[symptoms_idx++], "rht2_wd(%s)", word3);
      } else {
        sprintf(symptoms[symptoms_idx++], "rht2_wd(%s_)", word3);
      }
    }
  }

  strcpy(symptoms[symptoms_idx],"");
  if (symptoms_idx < 2) return FALSE;

  mst_smp(symptoms, name, state, type, value, errcde);
  sol_prb(name, state, type, value, errcde);
  if (errcde[0] != 0) {
    stx_err("failed to solve problem", __LINE__, __FILE__);
  }

  if EQUAL(state, "S") { 
    copy_dgns(dgns, segerr_dgns);
    get_dgn(dgns,name,state,type,value,errcde);
    if (value[0]) {                      /* has at least one true value */
      i = 0;
      while (! EQUAL(dgns[i],"")) {
        strcpy(dgns_var, dgns[i++]);
        
        has_error = TRUE;

        if (EQUAL(dgns_var,"err_double_negative")) {
          strcpy(msg, "err_double_negative");
          obverr_end = rht1;
          break;
        }
        if (EQUAL(dgns_var,"err_redundancies")) {
          strcpy(msg, "err_redundancies");
          obverr_end = rht2;
          break;
        }
        if (EQUAL(dgns_var,"err_wrong_comp")) {
          strcpy(msg, "err_wrong_comp");
          obverr_end = rht1;
          break;
        }
        if (EQUAL(dgns_var,"err_double_prep")) {
          strcpy(msg, "err_double_prep");
          obverr_end = rht1;
          break;
        }
        if (EQUAL(dgns_var,"err_misc")) {
          strcpy(msg, "err_misc");
          obverr_end = rht2;
          break;
        }
      }
    }
  }
  else stx_err("problem is unsatisfied", __LINE__, __FILE__);

  strcpy(state, "A");
  rst_prb(name, state, type, value, errcde);
  if (has_error) return TRUE;
  return FALSE;
}




/*---------------------------------------------------------------------

  segerr_cntwd  --  check if current word is considered in the log file

    input  -- word: current word to be checked
    output -- none

  function returns TRUE if word is considered in log file, FALSE otherwise

------------------------------------------------------------------------*/
boolean segerr_cntwd(word)
char *word;
{
  int  i, len;
  char ch;

  len = strlen(word);

  for (i = 0; i < len; i++) {
    ch = word[i];
    if (ch == '\'') { word[i] = '_'; }
    else { word[i] = tolower(ch); }
  }

  if (len <= 3) {
    if (EQUAL(word,"not") OR
        EQUAL(word,"at")  OR
        EQUAL(word,"off") OR
        EQUAL(word,"in")  OR
        EQUAL(word,"a")   OR
        EQUAL(word,"one"))     return TRUE;

    return FALSE;
  }

  if (len <= 5) {
    if (EQUAL(word,"don_t") OR
        EQUAL(word,"can_t") OR
        EQUAL(word,"won_t") OR
        EQUAL(word,"lapse") OR
        EQUAL(word,"more")  OR
        EQUAL(word,"most")  OR
        EQUAL(word,"less")  OR
        EQUAL(word,"these") OR
        EQUAL(word,"those"))     return TRUE;

    return FALSE;
  }

  if (len <= 8) {
    if (EQUAL(word,"doesn_t")  OR
        EQUAL(word,"couldn_t") OR
        EQUAL(word,"didn_t")   OR
        EQUAL(word,"wouldn_t") OR
        EQUAL(word,"haven_t")  OR
        EQUAL(word,"hasn_t")   OR
        EQUAL(word,"hadn_t")   OR
        EQUAL(word,"period")   OR
        EQUAL(word,"exactly")  OR
        EQUAL(word,"reason")   OR
        EQUAL(word,"inside")   OR
        EQUAL(word,"outside")  OR
        EQUAL(word,"another")  OR
        EQUAL(word,"superior") OR
        EQUAL(word,"seldom"))     return TRUE;

    return FALSE;
  }

  if (EQUAL(word,"individual") OR
      EQUAL(word,"generally") OR
      EQUAL(word,"identical")) { word[8] = '\0'; return TRUE; }

  return FALSE;
}

/*---------------------------------------------------------------------

  segerr_rht1wd  --  check if right1 word is considered in the log file

    input  -- word: right1 word to be checked
    output -- none

  function returns TRUE if word is considered in log file, FALSE otherwise

------------------------------------------------------------------------*/
boolean segerr_rht1wd(word)
char *word;
{
  int  i, len;
  char ch;

  len = strlen(word);

  for (i = 0; i < len; i++) {
    ch = word[i];
    if (ch == '\'') { word[i] = '_'; }
    else { word[i] = tolower(ch); }
  }

  if (len <= 3) {
    if (EQUAL(word,"of") OR
        EQUAL(word,"is") OR
        EQUAL(word,"why"))     return TRUE;

    return FALSE;
  }

  if (len <= 5) {
    if (EQUAL(word,"kind")  OR
        EQUAL(word,"sort")  OR
        EQUAL(word,"never") OR
        EQUAL(word,"same")  OR
        EQUAL(word,"round") OR
        EQUAL(word,"about") OR
        EQUAL(word,"peace") OR
        EQUAL(word,"than")  OR
        EQUAL(word,"ever"))     return TRUE;

    return FALSE;
  }

  if (len <= 8) {
    if (EQUAL(word,"hardly")   OR
        EQUAL(word,"scarcely") OR
        EQUAL(word,"barely")   OR
        EQUAL(word,"person")   OR
        EQUAL(word,"always")   OR
        EQUAL(word,"circular") OR
        EQUAL(word,"square")   OR
        EQUAL(word,"parallel") OR
        EQUAL(word,"unique")   OR
        EQUAL(word,"regards"))     return TRUE;

    return FALSE;
  }

  if (EQUAL(word,"identical") OR
      EQUAL(word,"essential"))  { word[8] = '\0'; return TRUE; }

  return FALSE;
}


/*---------------------------------------------------------------------

  segerr_rht2wd  --  check if right2 word is considered in the log file

    input  -- word: right2 word to be checked
    output -- none

  function returns TRUE if word is considered in log file, FALSE otherwise

------------------------------------------------------------------------*/
boolean segerr_rht2wd(word)
char *word;
{
  int  i, len;
  char ch;

  len = strlen(word);
  for (i = 0; i < len; i++) word[i] = tolower(word[i]);

  if (EQUAL(word,"time")    OR
      EQUAL(word,"to")      OR
      EQUAL(word,"because") OR
      EQUAL(word,"of"))        return TRUE;

  return FALSE;
}

