#include "syntx.h"

#define        NumofSmpDgns    5


DIAGNOSIS_ARRAY smpl_dgns[NumofSmpDgns + 1] = { 
                                                "cnt_noun_adjv_only",
                                                "cnt_article_only",
                                                "cnt_drop_vt",
                                                "cnt_drop_vi",
                                                "cnt_keep_flwto",
                                                "" 
                                              };   



extern struct  User_Word  *user_dict;
extern struct  Syntx_Term *sentence; 
extern int                *original_sentence, sentence_len;
extern struct SPOC_Pos    sentence_spoc;
extern boolean            intrnl_dsply;

boolean no_subjects, has_vbonly;


boolean simplify();
boolean simplify_ptrns();
boolean simplify_token();

boolean smplfy_lftptrn();
boolean smplfy_cntptrn();
boolean smplfy_lftwd();

boolean simplify_again();


/*----------------------------------------------------------------------

  simplify  --  control the action of simplification

    input  -- lvl: at which likelihood level the simplification works on
    output -- none

  function returns TRUE if the simplification is successful and further
  actions can be suggested on the sentence, FALSE otherwise.

------------------------------------------------------------------------*/
boolean simplify(lvl)
int lvl;
{
  boolean simplified, smplf_succ;

  if (intrnl_dsply) {
    printf("\nAt level %s:\n", level[lvl]);
    /*print_dict();*/
    begin_title("simplify_ptrns", -1, -1);
  }

  load_prb("Simplifying");
  if (get_stc_ptrns(lvl)) {
    if (smplf_succ = simplify_ptrns(&simplified)) {
      while (simplified AND smplf_succ) {
        smplf_succ = simplify_ptrns(&simplified);
      }
      if (smplf_succ) {
        stc_push(sentence, sentence_len, &sentence_spoc,
                 FindPredicate, 0, class_likelihood[lvl], Simplify);

        if (intrnl_dsply) {
          end_title("simplify_ptrns");
        }
        return TRUE;
      }
    }
  }
  if (intrnl_dsply) {
    end_title("simplify_ptrns");
  }

  return FALSE;
}

         

/*----------------------------------------------------------------------
 
  simplify_ptrns -- scan the whole sentence to simplify patterns 
    
    input  -- none
    output -- simplified: TRUE -- some patterns were simplified
                          FALSE -- no patterns were simplified

  function returns TRUE if every word in the sentence has at least one
  pattern after simplification, FALSE otherwise.

-----------------------------------------------------------------------*/
boolean simplify_ptrns(simplified)
boolean *simplified;
{
  int     i;

  *simplified = FALSE;

  if (intrnl_dsply) {
    begin_title("simplify", -1, -1);
  }

  no_subjects = TRUE;
  has_vbonly = FALSE;

  strcpy(state, "A");
  rst_prb(name, state, type, value, errcde);

  for (i = 0; i < sentence_len; i++) {
    if (simplify_token(i, simplified)) {
      strcpy(state, "A");
      rst_prb(name, state, type, value, errcde);
    }
    else {
      if (intrnl_dsply) {
        printf("simplify to empty pattern\n");
        end_title("simplify_ptrns");
      }
      return FALSE;
    }
  }

  if (intrnl_dsply) {
    end_title("simplify");
  }
  return TRUE;
}


/*---------------------------------------------------------------------

  simplify_token  --  simplify token by droping off irrelevant patterns

    input  -- idx: the index of a token
    output -- simplified: TRUE -- the patterns were simplified
                          FALSE -- no patterns were simplified

  function returns TRUE if the word contains at least one pattern after
  simplification, FALSE otherwise.

----------------------------------------------------------------------*/
boolean simplify_token(idx, simplified)
int  idx;
boolean *simplified;
{
  int   symptoms_idx = 0, i, lft, current, lklhds[MAXPTRNS], lh, 
        num_dropped;
  char  dgns_var[VAR_MAX + 1], ptrn_names[MAXPTRNS+1][PTRN_PHRS_MAX+1],
        ptrn[PTRN_PHRS_MAX+1], word[WORD_MAX+1];
  SYMPTOM_ARRAY   symptoms[2*MAXPTRNS + 6];
  DIAGNOSIS_ARRAY dgns[NumofSmpDgns + 1];


  if (idx == (sentence_len - 1)) 
    strcpy(symptoms[symptoms_idx++], "last_word");

  lft = idx - 1;
  get_ptrn_names(sentence, sentence_len, lft, ptrn_names, lklhds);
  i = 0;
  strcpy(ptrn, ptrn_names[i++]);
  while (!EQUAL(ptrn,"")) {
    if (smplfy_lftptrn(ptrn)) {
      sprintf(symptoms[symptoms_idx++], "lft_ptn(%s)", ptrn);
    }
    strcpy(ptrn, ptrn_names[i++]);
  }
  if (i == 2) strcpy(symptoms[symptoms_idx++], "lft_onlyone");
  if (has_adjpron_only(sentence, sentence_len, lft, &lh))   
    strcpy(symptoms[symptoms_idx++], "lft_ptn(pron_adj)");

  get_word(sentence, sentence_len, lft, word);
  if (!EQUAL(word,"")) {
    if (smplfy_lftwd(word)) {
      sprintf(symptoms[symptoms_idx++], "lftwd_is_to");
    }
  }
  if ((!has_vbonly) AND (!EQUAL(word,"to")))
    has_vbonly = has_verb_only(sentence, sentence_len, lft);
 



  current = idx;
  get_ptrn_names(sentence, sentence_len, current, ptrn_names, lklhds);
  i = 0;
  strcpy(ptrn, ptrn_names[i++]);
  while (!EQUAL(ptrn,"")) {
    if (smplfy_cntptrn(ptrn)) {
      sprintf(symptoms[symptoms_idx++], "cnt_ptn(%s)", ptrn);
    }
    strcpy(ptrn, ptrn_names[i++]);
  }
  if (has_vbonly)
    strcpy(symptoms[symptoms_idx++], "vbonly_occurred");


  strcpy(symptoms[symptoms_idx],"");

  mst_smp(symptoms, name, state, type, value, errcde);

  sol_prb(name, state, type, value, errcde);
  if (errcde[0] != 0) {
    stx_err("failed to solve problem", __LINE__, __FILE__);
  }

  if EQUAL(state, "S") { 
    copy_dgns(dgns, smpl_dgns);
    get_dgn(dgns,name,state,type,value,errcde);
    if (value[0]) {                    /* has at least one true value */
      i = 0;
      while (! EQUAL(dgns[i],"")) {
        strcpy(dgns_var, dgns[i++]);


        if EQUAL(dgns_var,"cnt_noun_adjv_only") {
          if (intrnl_dsply) {
            printf("%s:  ",
               user_dict[original_sentence[sentence[current].begin]].word);
            printf("keep noun, adj and adv patterns, delete all others\n\n");
          }

          if ((num_dropped = keep_noun_adjv(sentence, sentence_len, 
                                            current, FALSE)) == -1)
            return FALSE;
          if (num_dropped > 0) *simplified = TRUE;
          continue;
        }

        if EQUAL(dgns_var,"cnt_article_only") {
          if (intrnl_dsply) {
printf("%s:  ",user_dict[original_sentence[sentence[current].begin]].word);
printf("keep article, delete all others\n\n");
          }

          if ((num_dropped = keep_article(sentence, sentence_len,
                                          current, FALSE)) == -1)
            return FALSE;
          if (num_dropped > 0) *simplified = TRUE;
          continue;
        }

        if EQUAL(dgns_var,"cnt_drop_vt") {
          if (intrnl_dsply) {
printf("%s:  ",user_dict[original_sentence[sentence[current].begin]].word);
printf("drop off vt pattern\n\n");
          }

          if ((num_dropped = drop_vt(sentence, sentence_len, 
                                     current, FALSE)) == -1)
            return FALSE;
          if (num_dropped > 0) *simplified = TRUE;
          continue;
        }

        if EQUAL(dgns_var,"cnt_drop_vi") {
          if (intrnl_dsply) {
printf("%s:  ",user_dict[original_sentence[sentence[current].begin]].word);
printf("drop off vi pattern\n\n");
          }

          if ((num_dropped = drop_vi(sentence, sentence_len,
                                     current, FALSE)) == -1)
            return FALSE;
          if (num_dropped > 0) *simplified = TRUE;
          continue;
        }

        if EQUAL(dgns_var,"cnt_keep_flwto") {
          if (intrnl_dsply) {
printf("%s:  ",user_dict[original_sentence[sentence[current].begin]].word);
printf("drop patterns which are not allowed to follow to\n\n");
          }

          if ((num_dropped = drop_non_flwto(sentence, sentence_len,
                                            current, FALSE)) == -1)
            return FALSE;
          if (num_dropped > 0) *simplified = TRUE;
          continue;
        }
      }  
    }
  }
  else stx_err("problem is unsatisfied", __LINE__, __FILE__);
  
  return TRUE;

}
  

/*---------------------------------------------------------------------

  smplfy_lftptrn  --  check if the pattern is considered in the log file

    input  -- ptrn: left pattern name to be checked
    output -- none

  function returns TRUE if ptrn is considered in log file, FALSE otherwise

------------------------------------------------------------------------*/
boolean smplfy_lftptrn(ptrn)
char *ptrn;
{

  if (EQUAL(ptrn,"prep")) return TRUE;
  return FALSE;
}



/*---------------------------------------------------------------------

  smplfy_cntptrn  --  check if the pattern is considered in the log file

    input  -- ptrn: current pattern name to be checked
    output -- none

  function returns TRUE if ptrn is considered in log file, FALSE otherwise

------------------------------------------------------------------------*/
boolean smplfy_cntptrn(ptrn)
char *ptrn;
{
  char str[4];

  str[0] = ptrn[0];
  str[1] = ptrn[1];
  str[2] = ptrn[2];
  str[3] = '\0';

  if (EQUAL(str,"art")) return TRUE;
  if (EQUAL(ptrn,"vt_sgl")) return TRUE;
  if (EQUAL(ptrn,"vt_pl")) return TRUE;
  if (EQUAL(ptrn,"vt_pst")) return TRUE;
  if (EQUAL(ptrn,"vi_sgl")) return TRUE;
  if (EQUAL(ptrn,"vi_pl")) return TRUE;
  if (EQUAL(ptrn,"vi_pst")) return TRUE;
  if (EQUAL(ptrn,"lvb_pl")) return TRUE;
  if (EQUAL(ptrn,"lvb_sgl")) return TRUE;
  if (EQUAL(ptrn,"lvb_pst")) return TRUE;

  return FALSE;
}



/*----------------------------------------------------------------------

  simplify_again  --  control the action of simplifying sentence again

    input  -- none
    output -- none

  function returns TRUE if the simplification is successful and further
  actions can be suggested on the sentence, FALSE otherwise.

------------------------------------------------------------------------*/
boolean simplify_again(stc, len)
struct Syntx_Term *stc;
int len;
{
  boolean simplified, smplf_succ, swapped = FALSE;
  struct Syntx_Term *stc_tmp;
  int  len_tmp;

  if (intrnl_dsply) {
    printf("\nSimplify Again!!\n");
  }

  load_prb("Simplifying");
  if (stc != sentence) {
    swapped = TRUE;
    stc_tmp = sentence;
    len_tmp = sentence_len;
    sentence = stc;
    sentence_len = len;
  }
  if (smplf_succ = simplify_ptrns(&simplified)) {
    while (simplified AND smplf_succ) 
      smplf_succ = simplify_ptrns(&simplified);
  }
  if (swapped) {
    sentence = stc_tmp;
    sentence_len = len_tmp;
  }
  if (smplf_succ) return TRUE;
  return FALSE;
}




/*---------------------------------------------------------------------

  smplfy_lftwd  --  check if left word is considered in the log file

    input  -- word: left word to be checked

  function returns TRUE if word is considered in log file, FALSE otherwise

------------------------------------------------------------------------*/
boolean smplfy_lftwd(word)
char *word;
{
  int  i, len;
  char ch;

  len = strlen(word);
  for (i = 0; i < len; i++) {
    if ((ch = word[i]) == '\'') {
      word[i] = '_';
    }
    else word[i] = tolower(ch);
  }

  if (EQUAL(word,"to")) return TRUE; 

  return FALSE;
}






    

