/*-----------------------------------------------------
* Include files
-----------------------------------------------------*/
# include <stdio.h>
# include <string.h>
# include <fcntl.h>
/*# include <malloc.h> removed 10/24/2005 since now in stdlib.h */
# include <stdlib.h>
# include "macro.h"


/*-----------------------------------------------------
* get_dgn()
*    Get the diagnosis variables having value True.
*
* INPUT
* The set of diagnosis variables must be specified in  
* array diagnoses[]. The last record in array diagnoses  
* has the string of length zero (""). Other parameters
* are not used.
*
* OUTPUT
* value[0]:  Number of diagnosis variables with value True
*
* diagnoses[]:  Contains the diagnosis variables with value True.
*               The last record must be the empty string "".
* 
* errcde[0]:  = 0 : No errors, continue
*             = 1 : Warning, may continue
*             = 2 : Non-fatal error, may continue
*             = 3 : Fatal error, stop program
*             = - : System error, program stops. 
*
* Only level 3 error is monitored.
*
* CAUTION: Problem must have been loaded with inz_prb()
*          so that mke_hst() has been executed.
-----------------------------------------------------*/
void get_dgn(diagnoses,name,state,type,value,errcde)
DIAGNOSIS_ARRAY diagnoses[];
char name[], state[], type[];
long value[];
short errcde[];
{
    short i = 0, maxerr =0;
    long  num_of_true = 0;


    while (strcmp(diagnoses[i], "") != 0) 
      {
	strcpy(name, diagnoses[i]);
        var_idx(name, state, type, value, errcde);
        if (errcde[0] > maxerr)
          {
            maxerr = errcde[0];
          };

        get_val(name, state, type, value, errcde);
        if (errcde[0] > maxerr)
          {
            maxerr = errcde[0];
          };

        if (state[0] == 'T') 
          { 
            strcpy(diagnoses[num_of_true++], diagnoses[i++]);
          }
        else 
          {
            i++;
          } 
        
      }

    strcpy(diagnoses[num_of_true], "");

    value[0] = num_of_true;
    errcde[0] = maxerr;
}

/*-----------------------------------------------------
* mst_smp()
*    Change values of a specified set of symptom 
*    variables to True.
*
* INPUT
* The set of symptom variables must be specified in  
* array symptoms[]. The last record in array symptoms  
* has the string of length zero (""). Other parameters
* are not used.
*
* OUTPUT
* errcde[0]:  = 0 : No errors, continue
*             = 1 : Warning, may continue
*             = 2 : Non-fatal error, may continue
*             = 3 : Fatal error, stop program
*             = - : System error, program stops. 
*
* Only level 3 error is monitored.
*
* CAUTION: Problem must have been loaded with inz_prb() 
*          so that mke_hst() has been executed.
-----------------------------------------------------*/
void mst_smp(symptoms,name,state,type,value,errcde)
SYMPTOM_ARRAY symptoms[];
char name[], state[], type[];
long value[];
short errcde[];
{
    short i, maxerr =0;

    i = 0;
    while (strcmp(symptoms[i], "") != 0) 
      {
	strcpy(name, symptoms[i++]);
        var_idx(name, state, type, value, errcde);
        if (errcde[0] > maxerr)
          {
            maxerr = errcde[0];
          };

        strcpy(state, "T");

        mst_var(name, state, type, value, errcde);
        if (errcde[0] > maxerr)
          {
            maxerr = errcde[0];
          };

      }

    errcde[0] = maxerr;

}

/*-----------------------------------------------------
* var_idx
*    Return index for specified variable name.
*
* INPUT
* name     : name of variable
* state    : not used
* type     : not used
* errcde   : not used
* value    : not used
*
* OUTPUT
* value[0] : index of specified variable;
*            = 0 if variable not found. 
-----------------------------------------------------*/
/*-----------------------------------------------------
* var_idx
*    Return index for specified variable name.
*
* INPUT
* name     : name of variable
* state    : not used
* type     : not used
* errcde   : not used
* value    : not used
*
* OUTPUT
* value[0] : index of specified variable;
*            = 0 if variable not found. 
-----------------------------------------------------*/
void var_idx(name, state, type, value, errcde)
char name[], state[], type[];
long value[];
short errcde[];
{
    struct nlist *tp;

    tp = lookup(name);
    if (tp == NULL) 
        value[0] = 0;
    else 
	value[0] = tp->index;
}


/*-----------------------------------------------------
* FUNCTIONS BELOW FOR INTERNAL USE ONLY.
-----------------------------------------------------*/

/*-----------------------------------------------------
* Hash table declaration.
-----------------------------------------------------*/
static struct nlist *hashtab[HASHSIZE];

/*-----------------------------------------------------
* prg_hash_table declaration.
------------------------------------------------------*/
static struct prg_hash *prg_hash_table = NULL;



/*---------------------------------------------------------------------
  dup_hshtbl

  check if the .prg name is already in prg_hash_table.
-----------------------------------------------------------------------*/
int  dup_hshtbl(name)

char  name[];

{
  struct prg_hash *ptr;

  ptr = prg_hash_table;
  while (ptr != NULL) {
    if (strcmp(ptr->prg_name, name) == 0) return 1;
    ptr = ptr->next;
  }

  return 0;
}


/*-----------------------------------------------------
* hash
*
* Returns hash value of specified string
-----------------------------------------------------*/
unsigned hash(s)
char *s;
{
    unsigned hashval;

    for(hashval = 0; *s != '\0'; s++)
	hashval = *s + 31 * hashval;

    return hashval % HASHSIZE;
}


/*-----------------------------------------------------
* has_wildcard
*
* check if a name contains a wildcard '?'
------------------------------------------------------*/
int  has_wildcard(name)
char  name[];
{
  int  i, len;

  len = strlen(name);

  for (i = 0; i < len; i++)  {
    if (name[i] == '?') return 1;
  }
  return 0;
}

/*-----------------------------------------------------
* install
*
* puts (name, index) in hashtab;
* returns NULL if no room for new entry
-----------------------------------------------------*/
struct nlist*install(name, index)
char *name;
int index;
{
    struct nlist *np;
    unsigned hashval;

    if ((np = lookup(name)) == NULL) {
        np = (struct nlist *)malloc(sizeof(*np));
	if ((np == NULL) ||
	    (np->name = strdup(name)) == NULL)
		return NULL;
        np->index = index;
        hashval = hash(name);
	np->next = hashtab[hashval];
	hashtab[hashval] = np;
    } else
        mac_err("Duplicate entry", __LINE__, __FILE__);
    return np;
}

/*--------------------------------------------------------------
* load_hshtbl
*
* load a hash table corresponding to the .prg name
* into current hash table.
---------------------------------------------------------------*/
void  load_hshtbl(name)

char  name[];

{
  struct prg_hash *ptr;
  int              i;

  ptr = prg_hash_table;
  while (ptr != NULL) {
    if (strcmp(ptr->prg_name, name) == 0) {
      for (i = 0; i < HASHSIZE; i++) {
        hashtab[i] = ptr->hash_table[i];
      }
      return;
    }
    ptr = ptr->next;
  }
  mac_err("No such hash table", __LINE__, __FILE__);
}

/*-----------------------------------------------------
* lookup
*
* look for specified string in hashtab;
* returns NULL if string is not found
-----------------------------------------------------*/
struct nlist *lookup(s)
char *s;
{
    struct nlist *np;

    for(np = hashtab[hash(s)]; 
            np != NULL; np = np->next) {
        if(strcmp(s, np->name) == 0)
	    return np;
    }
    return NULL;
}


/*-----------------------------------------------------
* mac_err
*
* error handling routine.
-----------------------------------------------------*/
void mac_err(message, line_no, filename)
char *message;
int line_no;
char *filename;
{
    printf("***Leibniz macro.c Error*************\n");
    printf("Err Line %d, File %s: %s\n", 
        line_no, filename, message);
    printf("*************************************\n");
    exit(1);
}


/*-----------------------------------------------------
* mke_hst
*
* create  variable  name/index   hash table 
*
* INPUT
* name     : not used
* state    : not used
* type     : not used
* errcde   : not used
* value    : not used
*
* OUTPUT
* value[0] : number of variables
-----------------------------------------------------*/
void mke_hst(name, state, type, value, errcde)
char name[], state[], type[];
long value[];
short errcde[];
{
    long index, no_of_var;

    num_var(name, state, type, value, errcde);
    no_of_var = value[0];

#ifdef DEBUG
    printf("no_of_var %ld\n", no_of_var);
#endif 

    for(index=1; index <= no_of_var; index++) {
	value[0] = index;
        sts_var(name, state, type, value, errcde);

#ifdef DEBUG
        printf("index %ld name %s\n", index, name);
#endif
	install(name, index);
    }

    value[0] = no_of_var;
}


/*-----------------------------------------------------
* num_var
*    Finds  number  of  variables  in currently  loaded
*    problem
*
* INPUT
* name     : not used
* state    : not used
* type     : not used
* errcde   : not used
* value    : not used
*
* OUTPUT
* value[0] : number  of  variables in  currently loaded 
*            problem.
* errcde[0]:  = 0 : No errors, continue
*             = 1 : Warning, may continue
*             = 2 : Non-fatal error, may continue
*             = 3 : Fatal error, stop program
*             = - : System error, program stops. 
*
* Only level 3 error is monitored.
-----------------------------------------------------*/
void num_var(name, state, type, value, errcde)
char name[], state[], type[];
long value[];
short errcde[];
{
    char buf[100];
    sts_prb(name,state,type,value,errcde);
    if (errcde[0] != 0) {
        sprintf(buf, 
        "Error received from STS PRB: %d\n", 
	errcde[0]);
        mac_err(buf, __LINE__, __FILE__);
    }
}

  
/*-------------------------------------------------------------
*  store_hshtbl
*
*  puts (prgname, hashtable) into prg_hash_table.
* -------------------------------------------------------------*/
void  store_hshtbl(name)

char  name[];

{
  struct prg_hash *entry;
  int              i;

  if (dup_hshtbl(name)) return;

  entry = (struct prg_hash *)malloc(sizeof(*entry));
  if (entry == NULL) mac_err("Out of memory", __LINE__, __FILE__);
  
  if ((entry->prg_name = strdup(name)) == NULL) 
    mac_err("Out of memory", __LINE__, __FILE__);

  for (i = 0; i < HASHSIZE; i++) {
    entry->hash_table[i] = hashtab[i];
  }

  entry->next = prg_hash_table;
  prg_hash_table = entry;
}


/*---------------------------------------------------------------
*  store_reset_hshtbl
*
*  put (prgname, hashtable) into prg_hash_table 
*  reset the current hash table.
* ---------------------------------------------------------------*/
void  store_reset_hshtbl(name)

char  name[];

{
  struct prg_hash *entry;
  int              i;

  if (dup_hshtbl(name)) {
    for (i = 0; i < HASHSIZE; i++) {
      hashtab[i] = NULL;
    }
    return;
  }

  entry = (struct prg_hash *)malloc(sizeof(*entry));
  if (entry == NULL) mac_err("Out of memory", __LINE__, __FILE__);
  
  if ((entry->prg_name = strdup(name)) == NULL) 
    mac_err("Out of memory", __LINE__, __FILE__);

  for (i = 0; i < HASHSIZE; i++) {
    entry->hash_table[i] = hashtab[i];
    hashtab[i] = NULL;
  }

  entry->next = prg_hash_table;
  prg_hash_table = entry;
}

/****end of special_macro.c*******************/


