#include "spell_cnst.h"

#include "spellroutines.h"

extern int                   onewddat_ptr, usrdctdat_ptr, innertext_ptr, 
                             innertxtout_ptr, wrongwd_ptr, errhstry_ptr, 
                             errhstry_head, usrdct_ptr, errrpt_ptr, 
                             innertext[], innertxtout[], maxfrq, dcmdct_ptr,
                             err_hstry[RCNTERR_MAX + 1], usrdct_head[],
                             dcmdct_head[];
extern float                 alpha;
extern struct  OneWord       onewddat[];
extern struct  UserWd        usrdctdat[];
extern struct  DictIndex     usr_dct[], dcm_dct[];
extern struct  WrongWord     wrongwd[WRONGWD_MAX];
extern char                  err_report[2 * WRONGWD_MAX][MAXWORDLEN + 1];




int    total, repeat, transp, spc_trsp, neighb, insert, omit, de_di, di_de, 
       ei_ie, ie_ei, retain_e, spelling, lower_ltr;



/***************************************************************************/
/*  Function to print the head of the Laempel System                       */
/***************************************************************************/
void  print_laempel_title()
{
  printf("                   Laempel System -- Spell Checking\n");
  printf("                              Version 7.0\n\n");
  printf(" Copyright: Leibniz, 1990, 1999,  University of Texas at Dallas, 1999\n");
  printf("                    Richardson,  Texas 75083,  USA\n\n\n");
  printf(" The expert systems contained in this software have been created with\n");
  printf(" the Leibniz System for logic programming.\n\n");
  printf(" Implementation  of this  software has been  supported in part by the\n");
  printf(" Office of Naval Research under grant N00014-93-1-0096. This software\n");
  printf(" is licensed free of charge in source code form.\n\n\n");
}

/************************************************************************/
/*  Function to count the errors user made.                             */
/************************************************************************/
void  error_count()
{
  int    ptr;

  total = 0;
  repeat = 0;
  transp = 0;
  spc_trsp = 0;
  neighb = 0;
  insert = 0;
  omit = 0;
  de_di = 0;
  di_de = 0;
  ei_ie = 0;
  ie_ei = 0;
  retain_e = 0;
  spelling = 0;
  lower_ltr = 0;

  ptr = errhstry_head;
  while (ptr != errhstry_ptr)
  {
    switch (err_hstry[ptr])
    {
      case REPEAT:    repeat++;
                      break;
      case TRANSP:    transp++;
                      break;
      case SPC_TRSP:  spc_trsp++;
                      break;
      case NEIGHB:    neighb++;
                      break;
      case INSERT:    insert++;
                      break;
      case OMIT:      omit++;
                      break;
      case DE_DI:     de_di++;
                      break;
      case DI_DE:     di_de++;
                      break;
      case EI_IE:     ei_ie++;
                      break;
      case IE_EI:     ie_ei++;
                      break;
      case RETAIN_E:  retain_e++;
                      break;
      case SPELLING:  spelling++;
                      break;
      case LOWER_LTR: lower_ltr++;
                      break;
      default:        /*printf("wrong error number %d\n", err_hstry[ptr]);
                      exit(0);    */
                      system_error("wrong error type.");  
    };
    total ++;
    ptr++;
    ptr = ptr % (RCNTERR_MAX + 1);
  };    
}


/***************************************************************************/
/*     Function to print information                                       */
/***************************************************************************/
void  info_print()
{  
  int   i;

printf("$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$\n\n");

    printf("inner text\n");
    for (i = 0; i < innertext_ptr; i++)
    {
      printf("%d   ", innertext[i]);
    };
    printf("\n\n");
    
    printf("working dictionary(data)\n");
    for (i = 0; i < onewddat_ptr; i++)
    {
      printf("%s,%s,%s,%s,%c,%c,%d,%d,%d,%d,%d,%d\n", onewddat[i].word, 
                                                      onewddat[i].root, 
                                                      onewddat[i].suffix, 
                                                      onewddat[i].pttrn,
                                                      onewddat[i].Txt_Alt, 
                                                      onewddat[i].Fnd_Not, 
                                                      onewddat[i].is_wf, 
                                                      onewddat[i].not_wf,
                                                      onewddat[i].errtype,
                                                      onewddat[i].ptr1, 
                                                      onewddat[i].ptr2, 
                                                      onewddat[i].ptr3);
    };

    printf("\n\n");

  printf("working dictionary(index, dcm_dct)\n\n");

  for (i = 1; i <= MAXWORDLEN; i++)
  {
    if (dcmdct_head[i] != -1)
    {
      printf("dcmdct_head%d = %d\n", i, dcmdct_head[i]);
    };
  };

  for (i = 0; i < dcmdct_ptr; i++)
  {
    printf("%s, %d, %d, %d, %d, %d\n", dcm_dct[i].word,
                                       dcm_dct[i].count,
                                       dcm_dct[i].code,
                                       dcm_dct[i].red,
                                       dcm_dct[i].left,
                                       dcm_dct[i].right);
  };



    printf("updated innertext\n");
    for (i = 0; i < innertxtout_ptr; i++)
    {
      printf("%d   ", innertxtout[i]);
    };
    printf("\n\n");
    
    printf("user words(data)\n");
    for (i = 0; i < usrdctdat_ptr; i++)
    {
      printf("%s,%s,%s,%s,%d,%d\n", usrdctdat[i].word, 
                                    usrdctdat[i].root, 
                                    usrdctdat[i].suffix, 
                                    usrdctdat[i].pttrns,
                                    usrdctdat[i].is_wf, 
                                    usrdctdat[i].not_wf);
    };

    printf("\n\n");

  printf("user words(idex, usr_dct) \n\n");

  for (i = 1; i <= MAXWORDLEN; i++)
  {
    if (usrdct_head[i] != -1)
    {
      printf("usrdct_head%d = %d\n", i, usrdct_head[i]);
    };
  };

  for (i = 0; i < usrdct_ptr; i++)
  {
    printf("%s, %d, %d, %d, %d, %d\n", usr_dct[i].word,
                                       usr_dct[i].count,
                                       usr_dct[i].code,
                                       usr_dct[i].red,
                                       usr_dct[i].left,
                                       usr_dct[i].right);
  };



    printf("wrong word\n");
    for (i= 0; i < wrongwd_ptr; i++)
    {
      printf("code: %d, pos: %d\n", wrongwd[i].code, wrongwd[i].pos);
    };
    printf("\n\n");

      
  printf("\nalpha = %f\n", alpha);

  printf("\nUser History: \n");
  i = errhstry_head;
  while (i != errhstry_ptr)
  {
    printf("%d  ", err_hstry[i]);
    i ++;
    i = i % (RCNTERR_MAX + 1);
  };

  printf("\n\nMax frequency:  %d\n\n", maxfrq);


  error_count();

  spc_trsp = spc_trsp / 2;
  total = total - spc_trsp;
  printf("\n\n\n Total Errors: %d\n\n", total);
  printf("     spelling: %d,    %6.2f%%\n", spelling, spelling * 100.0 / total);
  printf("     repeat: %d,    %6.2f%%\n", repeat, repeat * 100.0 / total);
  printf("     transp: %d,    %6.2f%%\n", transp, transp * 100.0 / total);
  printf("     spc_trsp: %d,    %6.2f%%\n", spc_trsp, spc_trsp * 100.0 / total);
  printf("     neighb: %d,    %6.2f%%\n", neighb, neighb * 100.0 / total);
  printf("     insert: %d,    %6.2f%%\n", insert, insert * 100.0 / total);
  printf("     omit: %d,    %6.2f%%\n", omit, omit * 100.0 / total);
  printf("     de_di: %d,    %6.2f%%\n", de_di, de_di * 100.0 / total);
  printf("     di_de: %d,    %6.2f%%\n", di_de, di_de * 100.0 / total);
  printf("     ei_ie: %d,    %6.2f%%\n", ei_ie, ei_ie * 100.0 / total);
  printf("     ie_ei: %d,    %6.2f%%\n", ie_ei, ie_ei * 100.0 / total);
  printf("     retain_e: %d,    %6.2f%%\n", retain_e, retain_e * 100.0 / total);
  printf("     lower_ltr: %d,    %6.2f%%\n", lower_ltr, lower_ltr * 100.0 / total);

/*
  printf("\n\nerror file \n\n");
  printf("wrong word                correction\n\n");
  for (i = 1; i < errrpt_ptr; i = i + 2)
  {
    if (strcmp(err_report[i - 1], err_report[i]) == 0)  continue;
    printf("%s                        %s\n", err_report[i - 1], err_report[i]);
  }*/

printf("$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$\n\n");
}



