#include "syntx.h"


struct Stack_Elmnt {                       /* structure of stack element */
         struct Syntx_Term  *sentence;     /* pointer to a sentence      */
         int    length;                    /* the length of a sentence   */
         int    prbcode; /* the code of the action (problem) to be taken */
         int    pos;        /* the position where the action to be taken */
         int    likelihood;    /* the likelyhood the action is taking at */
         int    who;       /* the code of the block who issue the action */
         int    sub_pos;                  /* subject_pos of the sentence */
         int    prdct_pos;              /* predicate_pos of the sentence */
         int    obj_pos;                   /* object_pos of the sentence */
         int    cpl_pos;                 /* complement pos of a sentence */
         struct Stack_Elmnt  *next;        /* pointer to the next element*/
       };      



struct Stack_Elmnt *stack_head = NULL;         /* the stack head pointer */
int    stack_size = 0;                          /* the size of the stack */



struct Syntx_Term *stc_cpy();
void               stc_push();
struct Syntx_Term *stc_pop();
void               stc_dsp();
void               check_stack();
void               VerbPhrase_stc_push();
void               PronvbPhrase_stc_push();
void               clean_stack();

boolean empty_stack();






/*-----------------------------------------------------------------------

  stc_push  --  to push the a sentence into stack

    input  -- stc       : a sentence
              len       : sentence length
              spoc      : positions of SPOC
              prbcode   : the code of the problem (suggested action) 
              pos       : the position where the action to be taken 
              likelihood: the likelyhood at which the action is taking
              who       : the code of the block who issue the action
    output -- none

-----------------------------------------------------------------------*/
void stc_push(stc, len, spoc, prbcode, pos, likelihood, who) 
struct Syntx_Term *stc;
int     len;
struct  SPOC_Pos *spoc;
int     prbcode;
int     pos; 
int     likelihood; 
int     who;
{
  struct Stack_Elmnt  *ptr;
  struct Syntx_Term   *stc1;
  int    len1;
  struct SPOC_Pos spoc1;
  
  if (!(ptr = (struct Stack_Elmnt *) malloc(sizeof(*ptr))))
    stx_err("memory request failed", __LINE__, __FILE__);

  stc1 = stc_cpy(&len1, &spoc1, stc, len, spoc);
  ptr->sentence = stc1;
  ptr->length = len1;
  ptr->prbcode = prbcode;
  ptr->pos = pos;
  ptr->likelihood = likelihood;
  ptr->who = who;
  ptr->sub_pos = spoc1.sub_pos;
  ptr->prdct_pos = spoc1.prd_pos;
  ptr->obj_pos = spoc1.obj_pos;
  ptr->cpl_pos = spoc1.cpl_pos;
  ptr->next = stack_head;
  stack_head = ptr;
  stack_size++;
}



/*-----------------------------------------------------------------------

  stc_pop  --  pop the top sentence out of the working stack

    input  --  none
    output --  len       : sentence length
               prbcode   : the ocde of the problem(action)
               pos       : the position where the action to be taken 
               likelihood: the likelyhood at which the action is taking
               who       : the code of the block who issue the action
               spoc      : positions of SPOC
               
               function will return a pointer to the popped sentence or
               NULL if the stack is empty
 
-----------------------------------------------------------------------*/
struct Syntx_Term *stc_pop(len, prbcode, pos, likelihood, who, spoc)
int    *len;
int    *prbcode; 
int    *pos;
int    *likelihood; 
int    *who;
struct SPOC_Pos *spoc;
{
  struct Stack_Elmnt *ptr;
  struct Syntx_Term  *stc;

  if (empty_stack()) return NULL;
  ptr = stack_head;
  stc = ptr->sentence;
  *len = ptr->length;
  *prbcode = ptr->prbcode;
  *pos =  ptr->pos;
  *likelihood = ptr->likelihood;
  *who = ptr->who;
  spoc->sub_pos = ptr->sub_pos;
  spoc->prd_pos = ptr->prdct_pos;
  spoc->obj_pos = ptr->obj_pos;
  spoc->cpl_pos = ptr->cpl_pos;
  stack_head = ptr->next;
  free(ptr);
  stack_size--;
  return stc;
}
 

/*-----------------------------------------------------------------------

  stc_cpy  --  make a copy of a sentence

    input  -- stc2: the sentece to be copied
              len2: its length
              spoc2: its positions of SPOC
    output -- len1: the length of copied sentence
              spoc1: the SPOC positions of copied sentence
    function returns a pointer pointing to the copy of the sentence

------------------------------------------------------------------------*/
struct Syntx_Term *stc_cpy(len1, spoc1, stc2, len2, spoc2)
int               *len1;
struct SPOC_Pos   *spoc1;
struct Syntx_Term *stc2;
int                len2;
struct SPOC_Pos    *spoc2;
{
  int  i, j, len;
  char pts[MAXPTRNS+1];
  struct Syntx_Term  *stc;

  if (!(stc = (struct Syntx_Term *)malloc(len2 * sizeof(*stc)+5)))
    stx_err("memory request failed", __LINE__, __FILE__);
  
  for (i = 0; i < len2; i++) {
    stc[i].begin = stc2[i].begin;
    stc[i].end = stc2[i].end;

    strcpy(stc[i].ptrns, stc2[i].ptrns);
    len = strlen(stc2[i].ptrns);
    for (j = 0; j < len; j++) stc[i].ptrns_lh[j] = stc2[i].ptrns_lh[j];

    strcpy(stc[i].phrase, stc2[i].phrase);

    strcpy(stc[i].usedptrns, stc2[i].usedptrns);
    len = strlen(stc2[i].usedptrns);
    for (j = 0; j < len; j++) 
      stc[i].usedptrns_lh[j] = stc2[i].usedptrns_lh[j];
  }
  *len1 = len2; 
  spoc1->sub_pos = spoc2->sub_pos;
  spoc1->prd_pos = spoc2->prd_pos;
  spoc1->obj_pos = spoc2->obj_pos;
  spoc1->cpl_pos = spoc2->cpl_pos;

  return stc;
}


/*----------------------------------------------------------------------

  stc_dsp  --  dispose a sentence

    input  -- stc: sentence to be disposed
    output -- none

----------------------------------------------------------------------*/
void stc_dsp(stc)
struct Syntx_Term  *stc;
{
  free(stc);
}



/*---------------------------------------------------------------------

  empty_stack  -- check if the stack is empty or not

    input  -- none
    output -- TRUE if the stack is empty
              FALSE otherwise

----------------------------------------------------------------------*/
boolean empty_stack()
{
  return(stack_head == NULL);
}


/*---------------------------------------------------------------------

  check_stack  --  check the containts of the stack

    input  -- none
    output -- none

-----------------------------------------------------------------------*/
void check_stack()
{
  struct Syntx_Term  *stc;
  struct Stack_Elmnt *ptr;
  int    stc_len, pos, lhd, who, i;
  char   name[PRBNAME_MAX + 1], whoname[PRBNAME_MAX + 1], word[WORD_MAX+1];

  printf("\n<<------------------- Current Stack ------------------->>\n\n");

  ptr = stack_head;
  while (ptr != NULL) {
    stc = ptr->sentence;
    stc_len = ptr->length;
    block_name(ptr->prbcode, name);
    pos = ptr->pos;
    lhd = ptr->likelihood;
    who = ptr->who;
    block_name(who, whoname);

    for (i = 0; i < stc_len; i++) {
      if (stc[i].phrase[0] == '\0') {
        get_word(stc, stc_len, i, word);
        printf(" %s", word);
        printf("(%s)", stc[i].ptrns);
      }
      else {
        char words[MAXPHRASE+1][WORD_MAX+1];
        int  j = 0;

        words_in_phrase(stc, stc_len, i, words);
        while (!EQUAL(words[j],"")) printf(" %s", words[j++]);
        printf("[%s(", stc[i].phrase);
        printf("%s)]", stc[i].usedptrns);
      }
    }
    printf("\n \"%s\" at position %d with likelihood %d issued by %s\n", name, pos, lhd, whoname);
    printf("subject at %d;  predicate at %d;  object at %d; complement at %d\n\n", ptr->sub_pos, ptr->prdct_pos, ptr->obj_pos, ptr->cpl_pos);
    ptr = ptr->next;
  }
  printf("<<------------------- Current Stack ------------------->>\n\n");
}



    
/*-----------------------------------------------------------------------

  VerbPhrase_stc_push  --  to push sentence(s) w/ problemcode VerPhrase
                           into stack

    input  -- stc       : a sentence
              len       : sentence length
              spoc      : positions of SPOC
              pos       : the position where the action to be taken 
              who       : the code of the block who issue the action
              lhd       : likelihood at which simplify suggests 
                          FindPredicate
    output -- none

-----------------------------------------------------------------------*/
void VerbPhrase_stc_push(stc, len, spoc, pos, who, lhd) 
struct Syntx_Term *stc;
int     len;
struct  SPOC_Pos *spoc;
int     pos; 
int     who;
int     lhd;
{
  int  pt_len, len_tmp, i, lh;
  struct Syntx_Term *stc_tmp;
  struct SPOC_Pos    spoc_tmp;

  pt_len = ptrn_length(stc, len, pos);
  for (i = (pt_len - 1); i >= 0; i--) {
    stc_tmp = stc_cpy(&len_tmp, &spoc_tmp, stc, len, spoc);
    keep_ith_class(stc_tmp, len_tmp, pos, i, &lh);

    if (lh > lhd) {
      int j;

      for (j = 0; j < len_tmp; j++) {
        if (lhd_cmp(stc_tmp, len_tmp, j, lhd) != 1) {
          stc_push(stc_tmp, len_tmp, &spoc_tmp, VerbPhrase, pos, lh, who);
          break;
        }
      }
      stc_dsp(stc_tmp);
      continue;
    }

    stc_push(stc_tmp, len_tmp, &spoc_tmp, VerbPhrase, pos, lh, who);
    stc_dsp(stc_tmp);
  }
}

/*-----------------------------------------------------------------------

  PronvbPhrase_stc_push  --  to push sentence(s) w/ problemcode 
                             PronvbPhrase into stack

    input  -- stc       : a sentence
              len       : sentence length
              spoc      : positions of SPOC
              pos       : the position where the action to be taken 
              who       : the code of the block who issue the action
              lhd       : likelihood at which simplify suggests 
                          FindPredicate
    output -- none

-----------------------------------------------------------------------*/
void PronvbPhrase_stc_push(stc, len, spoc, pos, who, lhd) 
struct Syntx_Term *stc;
int     len;
struct SPOC_Pos *spoc;
int     pos; 
int     who;
int     lhd;
{
  int  pt_len, len_tmp, i, lh;
  struct Syntx_Term *stc_tmp;
  struct SPOC_Pos    spoc_tmp;

  pt_len = ptrn_length(stc, len, pos);
  for (i = (pt_len - 1); i >= 0; i--) {
    stc_tmp = stc_cpy(&len_tmp, &spoc_tmp, stc, len, spoc);
    keep_ith_class(stc_tmp, len_tmp, pos, i, &lh);

    if (lh > lhd) {
      int j;

      for (j = 0; j < len_tmp; j++) {
        if (lhd_cmp(stc_tmp, len_tmp, j, lhd) != 1) {
          stc_push(stc_tmp, len_tmp, &spoc_tmp,PronvbPhrase, pos, lh, who);
          break;
        }
      }
      stc_dsp(stc_tmp);
      continue;
    }

    stc_push(stc_tmp, len_tmp, &spoc_tmp, PronvbPhrase, pos, lh, who);
    stc_dsp(stc_tmp);
  }
}


/*-------------------------------------------------------------------

  clean_stack  --  clean away all sentences in the stack

    input  -- none
    output -- none

--------------------------------------------------------------------*/
void clean_stack()
{
  int len, prbcode, pos, likelihood, who, sub, prdct, obj;
  struct Syntx_Term  *stc;
  struct SPOC_Pos spoc;

  while ((stc = stc_pop(&len,&prbcode,&pos,
                        &likelihood,&who,&spoc)) != NULL) stc_dsp(stc);
}
 

