#include "syntx.h"

#define        NumofSPOChkDgns    3

extern struct Syntx_Term *sentence; 
extern int                sentence_len;
extern struct SPOC_Pos    sentence_spoc;

DIAGNOSIS_ARRAY spochk_dgns[NumofSPOChkDgns + 1] = { 
                                                 "err_subject_missing",
                                                 "err_object_missing",
                                                 "err_sub_disagree_prdct",
                                                 ""
                                                   };

static int        symptoms_idx;
SYMPTOM_ARRAY     symptoms[mmax(4,MAXSYMP)];

void    subprdctobj_checker();
int     do_spocheck();
  
boolean has_subject();
boolean has_object();
boolean has_predicate();



/*----------------------------------------------------------------------

  subprdctobj_checker  --  control the action of do_spocheck

    input  -- pos:  the position where the action to be taken
              lklhd: the likelyhood at which the action will take
              who: indicate who invoke the function    
    output -- none
 
-----------------------------------------------------------------------*/
void subprdctobj_checker(pos, lklhd, who)
int pos, lklhd, who;
{
  int  err_code;
  char syntx_err[80];

# ifdef  TRACK
begin_title("subprdctobj_checker", pos, lklhd);
# endif

  load_prb("SubPrdctObjChecker");
  
  if ((err_code = do_spocheck()) == 0) {
    stc_push(sentence, sentence_len, &sentence_spoc, MdfChecker, 
             0, lklhd, SPOChecker);
    goto end_subprdctobj_checker;
  }

  switch(err_code) {
    case 1:                                         /* err_prdct_missing */
            strcpy(syntx_err, "err_prdct_missing");
            break;

    case 2:                                       /* err_subject_missing */
            strcpy(syntx_err, "err_subject_missing");
            break;

    case 3:                                        /* err_object_missing */
            strcpy(syntx_err, "err_object_missing");
            break;

    case 4:                                    /* err_sub_disagree_prdct */
            strcpy(syntx_err, "err_sub_disagree_prdct");
            break;

    case 6:                                /* err_sub_prdct_obj_disorder */
            strcpy(syntx_err, "err_sub_prdct_obj_disorder");
            break;

    default: stx_err("unexpected code by do_spocheck",__LINE__,__FILE__);
  }
  save_stxerr(syntx_err, lklhd, sentence, sentence_len, &sentence_spoc);


end_subprdctobj_checker:  ;
# ifdef  TRACK
end_title("subprdctobj_checker");
# endif
}

/*---------------------------------------------------------------------

  do_spocheck  --  execute subprdctobj_checker.prg  

    input  -- none
    output -- none

  function returns 0: no syntax error can be found
                   1: err_prdct_missing
                   2: err_subject_missing
                   3: err_object_missing
                   4: err_sub_disagree_prdct
                   6: err_sub_prdct_obj_disorder

----------------------------------------------------------------------*/
int do_spocheck()
{
  DIAGNOSIS_ARRAY   dgns[NumofSPOChkDgns + 1];
  char              dgns_var[VAR_MAX + 1], phrase[PTRN_PHRS_MAX+1];
  int               sub, prdct, obj, return_val;

  symptoms_idx = 0;
  sub = sentence_spoc.sub_pos;
  prdct = sentence_spoc.prd_pos;
  obj = sentence_spoc.obj_pos;

/*  if (sub == prdct) { return_val = 0; goto end_do_spocheck; } */
  if (prdct == -1) { return_val = 1; goto end_do_spocheck; }

  if (((sub != -1) AND (obj != -1) AND (sub >= obj))     OR
      ((sub != -1) AND (prdct != -1) AND (sub > prdct))  OR
      ((obj != -1) AND (prdct != -1) AND (obj <= prdct)))   {
    return_val = 6; goto end_do_spocheck;
  }

  if (has_subject(sentence, sentence_len, sub, phrase))
    sprintf(symptoms[symptoms_idx++], "subject(%s)", phrase);

  if (has_object(sentence, sentence_len, obj, phrase))
    sprintf(symptoms[symptoms_idx++], "object(%s)", phrase);

  if (has_predicate(sentence, sentence_len, prdct, phrase))
    sprintf(symptoms[symptoms_idx++], "prdct(%s)", phrase);

  strcpy(symptoms[symptoms_idx],"");


  mst_smp(symptoms, name, state, type, value, errcde);
  sol_prb(name, state, type, value, errcde);
  if (errcde[0] != 0) {
    stx_err("failed to solve problem", __LINE__, __FILE__);
  }

  if EQUAL(state, "S") { 
    copy_dgns(dgns, spochk_dgns);
    get_dgn(dgns,name,state,type,value,errcde);
    if (value[0]) {                      /* has at least one true value */
      int  i;

      i = 0;
      while (! EQUAL(dgns[i],"")) {
        strcpy(dgns_var, dgns[i++]);

        if (EQUAL(dgns_var,"err_subject_missing")) {
          return_val = 2; goto end_do_spocheck;
        }

        if (EQUAL(dgns_var,"err_object_missing")) {
          return_val = 3; goto end_do_spocheck;
        }

        if (EQUAL(dgns_var,"err_sub_disagree_prdct")) {
          return_val = 4; goto end_do_spocheck;
        }
      }
    }
  }
  else stx_err("problem is unsatisfied", __LINE__, __FILE__);
  
  return_val = 0;

end_do_spocheck:

  strcpy(state, "A");
  rst_prb(name, state, type, value, errcde);
  return return_val;
}



  
/*---------------------------------------------------------------------

  has_subject  --  check if there is a subject

    input  -- stc:  a sentence
              len:  sentence length
              idx:  subject index
    output -- sub_name: subject name
              function returns TRUE  if there is a subject, 
                               FALSE otherwise

-----------------------------------------------------------------------*/
boolean has_subject(stc, len, idx, sub_name)
struct Syntx_Term *stc;
int    len;
int    idx;
char   *sub_name;
{
  char word[WORD_MAX+1];
  int  i, l;
  
  if ((idx < 0) OR (idx >= len)) return FALSE;
  if (EQUAL(stc[idx].phrase,"")) return FALSE;

  strcpy(sub_name, stc[idx].phrase);
  if (EQUAL(sub_name,"pron_sub")) {
    get_1stwd_of_phrs(stc, len, idx, word);
    l = strlen(word);
    for (i = 0; i < l; i++) word[i] = tolower(word[i]);
    if (EQUAL(word,"i")) { strcpy(sub_name, "pron_I"); return TRUE; }
    if (EQUAL(word,"he")) { strcpy(sub_name, "pron_he"); return TRUE; }
    if (EQUAL(word,"she")) { strcpy(sub_name, "pron_she"); return TRUE; }
    if (EQUAL(word,"we")) { strcpy(sub_name, "pron_we"); return TRUE; }
    if (EQUAL(word,"they")) { strcpy(sub_name, "prn_they"); return TRUE; }
  }

  if (EQUAL(sub_name,"pron_sgl") OR
      EQUAL(sub_name,"pron_pl")  OR
      EQUAL(sub_name,"pron_you") OR
      EQUAL(sub_name,"pron_pss") OR
      EQUAL(sub_name,"pron_slf") OR
      EQUAL(sub_name,"pron_thr") OR
      EQUAL(sub_name,"noun_sgl") OR
      EQUAL(sub_name,"noun_pl")  OR
      EQUAL(sub_name,"vt_ing")   OR
      EQUAL(sub_name,"vi_ing")   OR
      EQUAL(sub_name,"being")    OR
      EQUAL(sub_name,"pv_be")    OR
      EQUAL(sub_name,"pv_vt")    OR
      EQUAL(sub_name,"pv_vi")    OR
      EQUAL(sub_name,"to_vt")    OR
      EQUAL(sub_name,"to_vi")    OR
      EQUAL(sub_name,"to_be")    OR
      EQUAL(sub_name,"to_lvb")) return TRUE;

  return FALSE;
}


/*---------------------------------------------------------------------

  has_object  --  check if there is a object

    input  -- stc:  a sentence
              len:  sentence length
              idx:  object index
    output -- obj_name: object name
              function returns TRUE  if there is an object, 
                               FALSE otherwise

-----------------------------------------------------------------------*/
boolean has_object(stc, len, idx, obj_name)
struct Syntx_Term *stc;
int    len;
int    idx;
char   *obj_name;
{
 
  if ((idx < 0) OR (idx >= len)) return FALSE;
  if (EQUAL(stc[idx].phrase,"")) return FALSE;

  strcpy(obj_name, stc[idx].phrase);

  if (EQUAL(obj_name,"pron_obj") OR
      EQUAL(obj_name,"pron_sgl") OR
      EQUAL(obj_name,"pron_pl")  OR
      EQUAL(obj_name,"pron_you") OR
      EQUAL(obj_name,"pron_pss") OR
      EQUAL(obj_name,"pron_slf") OR
      EQUAL(obj_name,"noun_sgl") OR
      EQUAL(obj_name,"noun_pl")  OR
      EQUAL(obj_name,"vt_ing")   OR
      EQUAL(obj_name,"vi_ing")   OR
      EQUAL(obj_name,"being")    OR
      EQUAL(obj_name,"to_vt")    OR
      EQUAL(obj_name,"to_vi")    OR
      EQUAL(obj_name,"to_be")    OR
      EQUAL(obj_name,"to_lvb")) return TRUE;

  return FALSE;
}


/*---------------------------------------------------------------------

  has_predicate  --  check if there is a predicate

    input  -- stc:  a sentence
              len:  sentence length
              idx:  predicate index
    output -- prdct_name: predicate name
              function returns TRUE  if there is a predicate, 
                               FALSE otherwise

-----------------------------------------------------------------------*/
boolean has_predicate(stc, len, idx, prdct_name)
struct Syntx_Term *stc;
int    len;
int    idx;
char   *prdct_name;
{
  int  i, l, lh;
  char ch, str[4];

  get_phrs_name(stc, len, idx, prdct_name, &lh);

  if (EQUAL(prdct_name,"")) return FALSE;

  str[0] = prdct_name[0];
  str[1] = prdct_name[1];
  str[2] = prdct_name[2];
  str[3] = '\0';

  if (EQUAL(str,"doe") OR 
      EQUAL(str,"do_") OR
      EQUAL(str,"did"))   return TRUE;

  if (EQUAL(str,"hs_") OR 
      EQUAL(str,"hv_") OR
      EQUAL(str,"hd_"))   {

    if (EQUAL(prdct_name,"hd_be")) return FALSE;
    return TRUE;
  }

  if (EQUAL(str,"aux") OR 
      EQUAL(str,"auX"))   {

    if (EQUAL(prdct_name,"aux_be") OR 
        EQUAL(prdct_name,"auX_be"))   return FALSE;
    return TRUE;
  }

  if (EQUAL(prdct_name,"vt_pl")   OR
      EQUAL(prdct_name,"vi_pl")   OR
      EQUAL(prdct_name,"vt_sgl")  OR
      EQUAL(prdct_name,"vi_sgl")  OR
      EQUAL(prdct_name,"vt_pst")  OR
      EQUAL(prdct_name,"vi_pst")  OR
      EQUAL(prdct_name,"lvb_pl")  OR
      EQUAL(prdct_name,"lvb_sgl") OR
      EQUAL(prdct_name,"pv_vt")   OR
      EQUAL(prdct_name,"pv_vi"))     return TRUE;

  l = strlen(prdct_name);
  ch = prdct_name[l-1];
  if ((ch == 'P') OR
      (ch == 'T') OR
      (ch == 'I') OR
      (ch == 'B'))   prdct_name[l-1] = '\0';

  if (EQUAL(prdct_name,"am")) return TRUE;

  if (EQUAL(prdct_name,"is")    OR
      EQUAL(prdct_name,"isn_t")) { 
    strcpy(prdct_name, "is"); 
    return TRUE; 
  }

  if (EQUAL(str,"are")) { 
    strcpy(prdct_name, "are"); 
    return TRUE; 
  }

  if (EQUAL(str,"was")) { 
    strcpy(prdct_name, "was"); 
    return TRUE; 
  }

  if (EQUAL(str,"were")) { 
    strcpy(prdct_name, "were"); 
    return TRUE; 
  }

  return FALSE;
}

