#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/time.h>
#include "macro.h"



#define  TRUE           1              /* Boolean value TRUE             */
#define  FALSE          0              /* Boolean value FALSE            */
#define  AND            &&
#define  OR             ||
#define  EQUAL(x,y)     (strcmp(x,y) == 0)
#define  READ           "r"
#define  WRITE          "w"
#define  READWRITE      "r+"


#define  PTRN_PHRS_MAX  8          /* Max length of pattern(phrase) name */
#define  VAR_MAX        27          /* Max length of variable            */
#define  WORD_MAX       80          /* Maximum length of a word          */
#define  VERB_MAX       5           /* Max number of verbs in a sentence */
#define  OBJ_MAX        5         /* Max number of objects in a sentence */
#define  SUB_MAX        5        /* Max number of subjects in a sentence */
#define  NOUN_MAX       20          /* Max number of nouns in a sentence */
#define  PRBNAME_MAX    20          /* Max length of problem's name      */
#define  MAXPTRNS       10          /* Max number of patterns a word has */
#define  MAXPHRASE      10          /* Max length of a phrase            */
#define  STC_END     -1999         /* a mark to indicate the end of a 
                                      sentence in learn_utl.c */
#define  TRP_END     -1999         /* a mark to indicate the end of a 
                                      triple in learn_utl.c */


#define  ALWAYS         0
#define  VERYFREQLY     1
#define  FREQLY         2
#define  HALF           3
#define  SOMETIME       4
#define  RARELY         5

#define  Simplify         1
#define  FindPredicate    2
#define  FindSubject      3
#define  FindObject       4
#define  FindModifier     5

#define  VerbPhrase      10
#define  LnkvbPhrase     11
#define  BePhrase        12
#define  DoPhrase        13
#define  HavePhrase      14
#define  AuxPhrase       15
#define  PronvbPhrase    16
#define  PronvbBe        17
#define  HaveBe          18
#define  AuxHave          9
#define  AuxHaveVerb      8
#define  AuxHaveBe        7
#define  AuxBe            6
#define  PvHave          90
#define  PvHaveVerb      91
#define  PvHaveBe        92

#define  NounPhrase      20
#define  SubNoun         21
#define  ObjNoun         22
#define  MdfNoun         23
#define  PrepNoun        24
#define  SubNounAdjNoun  25
#define  ObjNounAdjNoun  26
#define  MdfNounAdjNoun  27
#define  PrepNounAdjNoun 28

#define  PronounPhrase   30

#define  AdjPhrase       40
#define  PrepAdj         41
#define  SubNounAdj      42
#define  ObjNounAdj      43
#define  MdfNounAdj      44
#define  PrepNounAdj     45

#define  AdvPhrase       50
#define  PrepAdjAdv      51
#define  SubNounAdjAdv   52
#define  ObjNounAdjAdv   53
#define  MdfNounAdjAdv   54
#define  PrepNounAdjAdv  55
#define  PronvbBeAdv     56
#define  HaveBeAdv       57
#define  AuxHaveAdv      58
#define  AuxHaveBeAdv    59
#define  AuxBeAdv        49
#define  PvHaveAdv       48
#define  PvHaveBeAdv     47

#define  PrepPhrase      60
#define  ToPhrase        65

#define  SPOChecker      70
#define  MdfChecker      71

#define  SimpleSentence  1
#define  TitleSentence   2
#define  ComplexSentence 3

#define  MAXTIME         10


#define  FindIntprt       1 /* indicate that the system is in a status of 
                               finding an interpretation of a sentence   */
#define  FindError        2 /* indicate that the system is in a status of 
                               finding syntax errors of a sentence       */
/*--------- 042797 ----------*/
#define  VERSION21       "<Version 2.1>"
/*---------------------------*/

/*--------- 110398 ----------*/
#define  VERSION22       "<Version 2.2>"
/*---------------------------*/

/*-----------ver3.0------------*/
#define  VERSION30       "<Version 3.0>"
/*---------end-ver3.0----------*/

/*-----------ver7.0------------*/
#define  VERSION70       "<Version 7.0>"
/*---------end-ver7.0----------*/


typedef  unsigned short boolean;
typedef        char POSITION[7];

struct Syntx_Term {                         /* structure of a syntx term */
         int   begin;         /* begining position of a term in sentence */
         int   end;                /* end position of a term in sentence */
         char  ptrns[MAXPTRNS+1];  /* patterns that the current word has */
         int   ptrns_lh[MAXPTRNS+1];                /* ptrns' likelihood */
         char  phrase[PTRN_PHRS_MAX+1];                   /* phrase name */
         int   phrase_lh;                    /* likelihood of the phrase */
         char  usedptrns[MAXPHRASE+1];
                                 /* a pattern or patterns currently used */
         int   usedptrns_lh[MAXPHRASE];
                                /* the likelihood of the pattern(s) used */
       };



struct User_Word {             /* structure of a word in user dictionary */
         char  word[WORD_MAX+1];                                 /* word */
         char  root[WORD_MAX+1];                     /* root of the word */
         int   word_freq;         /* number of the times the word occured*/
         char  ptrns[MAXPTRNS+1];                /* patterns of the word */
         int   ptrns_likelihood[MAXPTRNS];    /* likelihoods of patterns */
         int   ptrns_freq[MAXPTRNS]; /* the ussage frequency of patterns */
         char  ptrns_window[5][MAXPTRNS+1];
                                         /* patterns distribution window */
       };


struct VerbPos {           /* structure of a verb position in a sentence */
         int     index;               /* index of the verb in a sentence */
         boolean vt;                              /* indicate it is a vt */
         int     vt_lhd;                     /* the likelihood of the vt */
         boolean vi;                              /* indicate it is a vi */
         int     vi_lhd;                     /* the likelihood of the vi */
       };
           

struct SPOC_Pos {         /* record positions of the subject, predicate, 
                             object and complement of a sentence         */
         int     sub_pos;
         int     prd_pos;
         int     obj_pos;
         int     cpl_pos;
       };


char             name[129];           /* parameters used to call macro.c */
char             state[4];
char             type[5];
long             value[8];
short            errcde[2];

char             err_msg[120];

static char      level[6][16] = {
                                  "ALWAYS",
                                  "VERY FREQUENTLY",
                                  "FREQUENTLY",
                                  "HALF THE TIME",
                                  "SOMETIMES",
                                  "RARELY"
                                };


static int       class_likelihood[6] = {100, 85, 65, 50, 25, 10};


/**%%%default_text_name%%%%%%%**/
  
  char     filename_storage[120], default_text_name[120];

/**%%%end_default_text_name%%%**/

/*  
 *  mmax function definition and MAXSYMP
 *  for elimination of warning messages 
 *  concerning allocation for string symptoms
 */
#define mmax(x,y) ((x<y) ? y:x)
#define MAXSYMP  100

