#include <sys/time.h>
#include "syntx.h"

extern int  sentence_type;
extern char sentence_punc;
extern int  system_status;
extern int  bottom_likelihood;
/*
extern int  outoftime_num;
extern boolean  outoftime;
extern int simple_total;*/


int     sentence_len;                  /* the length of current sentence */
struct  Syntx_Term *sentence;    /* an array containing current sentence */
struct  SPOC_Pos    sentence_spoc;                  /* positions of SPOC */
boolean syntx_intprt;       /* indicate a sentence has an interpretation */
boolean obvious_error;    /* indicate an obvious error in error checking */

/**################**/
boolean   quant_error;    /* indicate an obvious error in error checking */
boolean   sucprn_error;   /* indicate an obvious error in error checking */
/**################**/

/**@@@@@@@@@7/99@@@@@@@@**/
boolean   pronoun_noun_error;  /* indicate an obvious error in error checking */
/**@@@@@@end7/99@@@@@@@@**/

boolean fatal_error;        /* indicate a fatal error in syntax analysis */
boolean syntx_error;       /* indicate a syntax error in syntax analysis */
boolean syntx_giveup;    /* indicate giving up finding an interpretation */
 

boolean syntax_checking();
boolean stc_has_conj();
boolean clause_conj();

  
/*--------------------------------------------------------------------

  syntax_checking -- read a sentence from text, do the syntax checking
                     on it.

    input  -- none
    output -- none

  function returns FALSE if no more sentence to be checked in text,
                   TRUE otherwise.

---------------------------------------------------------------------*/
boolean syntax_checking()
{
  int  pos, lklhd, cmd_code, i, stc_len, who;
  struct Syntx_Term *stc; 
  struct SPOC_Pos    stc_spoc;
  boolean            has_conj;
  struct timeval     before, after;

/*if (outoftime) { outoftime_num++; outoftime = FALSE; }*/

  if (!get_raw_sentence()) return FALSE;

  if (!cleanup_raw_sentence()) {
    free_complex_stc();
    return TRUE;
  }

  while (get_simple_sentence()) {
/*simple_total++;*/
    syntx_intprt = FALSE;
    obvious_error = FALSE;
    fatal_error = FALSE;
    syntx_error = FALSE;  
    syntx_giveup = FALSE;

    if (find_stc()) {
# ifdef  TRACK
print_orgstc();
printf("-- I've already known the sentence!!\n\n");
# endif
      free_simple_stc();
      continue;
    }

    if (find_obvious_err()) {
      talk_to_user();
      free_simple_stc();
      free_complex_stc();
      return TRUE;
    }
  /*########## added 3-16-99 ######## **/
   if (find_succ_quant()) {
      talk_to_user();
      free_simple_stc();
      free_complex_stc();
      return TRUE;
    }
    
    if (find_succ_pronoun()) {
      talk_to_user();
      free_simple_stc();
      free_complex_stc();
      return TRUE;
    }
    
  /*########## end added 3-16-99 ########**/   
    
    /**@@@@@@@@@7/99@@@@@@@@**/
    if (find_pron_noun()) {
      talk_to_user();
      free_simple_stc();
      free_complex_stc();
      return TRUE;
    }
   /**@@@@@@end7/99@@@@@@@@**/
    
    if (find_segment_err()) {
      talk_to_user();
      free_simple_stc();
      free_complex_stc();
      return TRUE;
    }

    if (system_status == FindError) {
      talk_to_user();
      free_simple_stc();
      continue;
    }

    if ((system_status == FindIntprt)  AND
        ((sentence_type != SimpleSentence) OR (sentence_punc != '.'))) {
      syntx_giveup = TRUE;
      talk_to_user();
      free_simple_stc();
      continue;
    }

    has_conj = stc_has_conj(sentence, sentence_len);

    gettimeofday(&before, NULL);
    for (i = VERYFREQLY; i <= RARELY; i++) {

      if (syntx_intprt) break;

      gettimeofday(&after, NULL);
      if ((after.tv_sec - before.tv_sec) > MAXTIME) {
/*outoftime = TRUE;*/
        syntx_giveup = TRUE;
        clean_stack();
        break;
      }

      /*if (has_conj AND (i > bottom_likelihood)) {*/
      if (i > bottom_likelihood) {
/*outoftime = TRUE;*/
        syntx_giveup = TRUE;
        clean_stack();
        break;
      }

      if (has_two_fixed_prdcts(sentence, sentence_len)) {
/*outoftime = TRUE;*/
        syntx_giveup = TRUE;
        clean_stack();
        break;
      }

      sentence_spoc.sub_pos = -1;
      sentence_spoc.prd_pos = -1;
      sentence_spoc.obj_pos = -1;
      sentence_spoc.cpl_pos = -1;

      if (simplify(i)) {

        stc = (struct Syntx_Term *)stc_cpy(&stc_len, &stc_spoc,
                                   sentence, sentence_len, &sentence_spoc);

        while (cmd_code = next_move(&pos, &lklhd, &who)) {

          if (!syntax_analysis(cmd_code, pos, lklhd, who)) { /* fatal err*/
            talk_to_user();
            free_simple_stc();
            free_complex_stc();
            return TRUE;
          }

/*--------- 070697 ----------*/
          gettimeofday(&after, NULL);
          if ((after.tv_sec - before.tv_sec) > MAXTIME) {
            syntx_giveup = TRUE;
            clean_stack();
            goto exit_for;
          }
/*---------------------------*/

        }
        sentence = (struct Syntx_Term *)stc_cpy(&sentence_len, 
                                  &sentence_spoc, stc, stc_len, &stc_spoc);
      } /* (simplify(i)) */
    } /* for */

exit_for:

    if ((!syntx_intprt) AND (!syntx_giveup)) syntx_error = TRUE;

    talk_to_user();
    free_simple_stc();

  } /* while */

  free_complex_stc();
  return TRUE;
}


/*--------------------------------------------------------------------

  stc_has_conj -- check if a sentence has a clause conj

    input  -- stc:  a sentence
              slen: sentence length
    output -- none

  function returns TRUE if the sentence has a conj, FALSE otherwise.

---------------------------------------------------------------------*/
boolean stc_has_conj(stc, slen)
struct Syntx_Term *stc;
int    slen;
{
  int  i;
  char word[WORD_MAX+1];

  for (i = 0; i < slen; i++) {
    get_word(stc, slen, i, word);
    if (clause_conj(word)) return TRUE;
  }
  return FALSE;
}


/*---------------------------------------------------------------------

  clause_conj  --  check if a word is a clause conjunction

    input  -- word: the word to be checked
    output -- none

  function returns TRUE if word is a clause conjunction, FALSE otherwise

------------------------------------------------------------------------*/
boolean clause_conj(word)
char *word;
{
  int  i, len;
 
  len = strlen(word);

  for (i = 0; i < len; i++) word[i] = tolower(word[i]); 

  if (len == 2) {
    if (EQUAL(word,"or") OR
        EQUAL(word,"so") OR
        EQUAL(word,"as") OR
        EQUAL(word,"if"))    return TRUE;
  }

  if (len == 3) {
    if (EQUAL(word,"and") OR
        EQUAL(word,"but") OR
        EQUAL(word,"for") OR
        EQUAL(word,"nor") OR
        EQUAL(word,"yet") OR
        EQUAL(word,"who"))   return TRUE;
  }

  if ((len > 3) AND (len < 6)) {
    if (EQUAL(word,"after") OR
        EQUAL(word,"since") OR
        EQUAL(word,"until") OR
        EQUAL(word,"when")  OR
        EQUAL(word,"than")  OR
        EQUAL(word,"which") OR
        EQUAL(word,"while") OR
        EQUAL(word,"where"))   return TRUE;
  }

  if ((len > 5) AND (len < 9)) {
    if (EQUAL(word,"because")  OR
        EQUAL(word,"although") OR
        EQUAL(word,"though")   OR
        EQUAL(word,"before")   OR
        EQUAL(word,"unless")   OR
        EQUAL(word,"whereas"))    return TRUE;
  }

  return FALSE;
}
