#include "syntx.h"


typedef char PTRNS[MAXPTRNS+1];

extern struct  Syntx_Term  *sentence; 
extern int                 sentence_len;
extern struct  SPOC_Pos    sentence_spoc;
extern int                 system_status;
extern char                text_name[];
extern char                old_text_name[];
extern long                text_line;


struct User_Word  *user_dict;     /* an array containing user dictionary */

struct free_sentence {
         struct  Syntx_Term   *snt;
         struct free_sentence *next;
       };

struct free_pattern {
         struct Pttrn_Phrs   *ptrn;
         struct free_pattern *next;
       };



struct free_sentence  *free_snts = NULL;
struct free_pattern   *free_ptrns = NULL;

int    user_dict_size;  /* the size (number of words) of user dictionary */
int    *text;                 /* array containing user text (inner form) */
long   text_len;           /* the length (number of tokens) of user text */
long   text_inner_ptr;       /* point at start pos of a sentence in text */
int    *original_sentence; /* array containing current original sentence */
int    original_sentence_len;     /* the length of the original sentence */
PTRNS  *old_stc_ptrns;
int    *raw_sentence;           /* array containing current raw sentence */
int    raw_sentence_len;               /* the length of the raw sentence */
int    *clean_sentence;       /* array containing current clean sentence */
int    clean_sentence_len;           /* the length of the clean sentence */
int    UNDEF_code;                    /* the code corresponding to UNDEF */
int    simple_sentence_ptr;      /* point to the start position of a simple
                                    sentence in the clean_sentence       */
int    sentence_type;           /* indicate the type of a clean_sentence */
char   sentence_punc;  /* indicate the punctuation of a chopped sentence */
boolean has_front_undef;       /* indicate there are undef's before the 1st
                                           token of the cleaned sentence */
char   version[100];          /* save the version of the user dictionary */
char   data_path[120];                           /* path to Laempel/Data */
char   wds_usr[120];                                  /* user dictionary */
char   wds_bak[120];                           /* user dictionary backup */
char   triple_usr[120];                     /* user special pattern file */
char   triple_bak[120];              /* user special pattern file backup */
boolean intrnl_dsply;                   /* indicate the internal display */
boolean display_syntx_msg;      /* indicate displaying message on screen */
int    stxerr_count;          /* counting the number of syntactic errors */
char   *sentence_intprt;       /* a string pointing to an interpretation */

FILE   *syntx_msg;    /* pointer to the file storing syntax checking msg */



void stx_err();
void inzprb();
void read_usr_dict();
boolean read_text_inner();
boolean get_sentence();
void sentence_check();
void init_ptrns_window();
void class_name();
void name_class();
void name_class_lhd();
void copy_dgns();
void load_prb();
void begin_title();
void end_title();
void window_drop();
void get_numofwords();

void field_seperate();
int  next_move();
void intlhd_to_strlhd();
void get_word();
void get_1stwd_of_phrs();
void get_lastwd_of_phrs();
void words_in_phrase();
int  lklhd_level();

boolean get_word_ptrns();
boolean get_stc_ptrns();

void block_name();

void update_classlh();
void update_usrdict();

boolean syntax_analysis();

void free_simple_stc();
void free_complex_stc();
void write_usr_dict();

boolean get_raw_sentence();
boolean cleanup_raw_sentence();
boolean get_a_token();
boolean is_space();
boolean is_punctuation();
boolean get_simple_sentence();

void    print_rawstc();
void    print_cleansentence();

void read_params();
void get_lhs_rhs();

boolean input_text_name();

void print_title();

void system_error();
void running_error();

int word_check();




/*----------------------------------------------------------------------

  stx_err -- syntax checking error handling routine
    
    input  -- message:  error message
              line_no:  the line where error occures
              filename: the file name where error occures
    output -- none

----------------------------------------------------------------------*/
void stx_err(msg, line_no, fname)
char *msg;
int  line_no;
char *fname;
{
  char errmsg[100];

  sprintf(errmsg, "laempel.stx Err Line %d, File %s: %s\n", line_no, fname, msg);
  system_error(errmsg);
}


/*----------------------------------------------------------------------

  inzprb -- call macro inz_prb to initialize problems

    input  -- none
    output -- none

----------------------------------------------------------------------*/
void inzprb()
{
  PRGNAME_ARRAY  prgname[21];

  printf("\n\nReading system files, user dictionary ...\n\n");

  sprintf(prgname[0], "%s/%s", data_path, "obvious_error.prg");
  sprintf(prgname[1], "%s/%s", data_path, "segment_error.prg");
  sprintf(prgname[2], "%s/%s", data_path, "simplify_ptrns.prg");
  sprintf(prgname[3], "%s/%s", data_path, "find_predicate.prg");
  sprintf(prgname[4], "%s/%s", data_path, "find_subject.prg");
  sprintf(prgname[5], "%s/%s", data_path, "find_object.prg");
  sprintf(prgname[6], "%s/%s", data_path, "find_modifier.prg");
  sprintf(prgname[7], "%s/%s", data_path, "to_phrase.prg");
  sprintf(prgname[8], "%s/%s", data_path, "verb_phrase.prg");
  sprintf(prgname[9], "%s/%s", data_path, "be_phrase.prg");
  sprintf(prgname[10], "%s/%s", data_path, "have_phrase.prg");
  sprintf(prgname[11], "%s/%s", data_path, "do_phrase.prg");
  sprintf(prgname[12], "%s/%s", data_path, "aux_phrase.prg");
  sprintf(prgname[13], "%s/%s", data_path, "pronvb_phrase.prg");
  sprintf(prgname[14], "%s/%s", data_path, "noun_phrase.prg");
  sprintf(prgname[15], "%s/%s", data_path, "adv_phrase.prg");
  sprintf(prgname[16], "%s/%s", data_path, "prep_phrase.prg");
  sprintf(prgname[17], "%s/%s", data_path, "adj_phrase.prg");
  sprintf(prgname[18], "%s/%s", data_path, "subprdctobj_checker.prg");
  sprintf(prgname[19], "%s/%s", data_path, "modifier_checker.prg");
  strcpy(prgname[20], "");

  value[0] = 1;
  inz_prb(prgname, name, state, type, value, errcde);
  if (errcde[0] != 0) 
    stx_err("failed to initialize problems", __LINE__, __FILE__);
}


/*----------------------------------------------------------------------

  read_usr_dict -- copy the user dictionary from disk to memory 
       
    input  -- none
    output -- none

----------------------------------------------------------------------*/   
void read_usr_dict()
{
  int    i, j, k, num;
  float  base, diff;
  char   line[120], fields[MAXPTRNS+7][80], str[120];
  struct User_Word  usr_word;

  FILE   *usrdict;

  while ((usrdict = fopen(wds_usr, READ)) == NULL) {
    printf("\nCannot find file = %s\n", wds_usr);
    printf("Type the file name of your dictionary.\n");
    printf("\n>> ");
    gets(str);
    if (!EQUAL(str,"")) strcpy(wds_usr, str);
    continue;
  }

  fgets(line, 120, usrdict);
  get_numofwords(line, version, &user_dict_size);

  user_dict = (struct User_Word *) malloc((user_dict_size+1) * sizeof(usr_word));
  if (! user_dict) {
    stx_err("memory request failed", __LINE__, __FILE__);
  }

  for (i = 0; i < user_dict_size; i++) {
    fgets(line, 120, usrdict);
    field_seperate(line, fields);
    strcpy(user_dict[i].word, fields[0]);
    strcpy(user_dict[i].root, fields[1]);
    user_dict[i].word_freq = atoi(fields[3]);
    strcpy(user_dict[i].ptrns, fields[4]);

    num = strlen(user_dict[i].ptrns);
    k = 5;
    for (j = 0;  j < num; j++) {
      user_dict[i].ptrns_likelihood[j] = atoi(fields[k++]);
      user_dict[i].ptrns_freq[j] = 0;
    }

    init_ptrns_window(user_dict[i].ptrns, 
                     user_dict[i].ptrns_likelihood,
                     user_dict[i].ptrns_window);
  }

  i = user_dict_size;
  UNDEF_code = i;
  strcpy(user_dict[i].word, "UNDEF");
  strcpy(user_dict[i].root, " ");
  user_dict[i].word_freq = 0;
  strcpy(user_dict[i].ptrns, "nN");
  user_dict[i].ptrns_likelihood[0] = 67;
  user_dict[i].ptrns_freq[0] = 0;
  user_dict[i].ptrns_likelihood[1] = 33;
  user_dict[i].ptrns_freq[1] = 0;
  init_ptrns_window(user_dict[i].ptrns, 
                    user_dict[i].ptrns_likelihood,
                    user_dict[i].ptrns_window);

  fclose(usrdict);

}
        

/*----------------------------------------------------------------------
  read_text_inner -- copy the user text (inner form) into memory.

    input  -- none
    output -- function returns TRUE if an inner text file has been opened,
              FALSE if there is no such file.

----------------------------------------------------------------------*/ 
boolean read_text_inner()
{
  long         i;
  char         token[7], text_inner[120];
  FILE         *usrtext, *n;

  void save_default_text_name();

  stxerr_count = 0;
  if (!display_syntx_msg) {
    char text_msg[120];
  
    sprintf(text_msg, "%s.err", text_name); 
    syntx_msg = fopen(text_msg, WRITE);
  }
      
  sprintf(text_inner, "%s.stx", text_name);
  if ((usrtext = fopen(text_inner, READ)) == NULL) {
    if ((n = fopen(text_name, READ)) == NULL) {
      printf("\nFile %s does not exist\n", text_name);
      printf("Check the file name and try again\n\n");
      return FALSE;
    }
    printf("\nCannot process file %s since it has not been checked for spelling errors.\n", text_name);
 
    printf("Do the Spell Checking, or type another file name.\n\n");
    fclose(n);
    return FALSE;
  }

  fscanf(usrtext, "Number of Tokens = %ld\n", &text_len);
  text = (int*) malloc(text_len * sizeof(int));

  for (i = 0; i < text_len; i++) {
    fscanf(usrtext, "%s", token);
    text[i] = atoi(token);
  }

  fclose(usrtext);
  text_inner_ptr = 0;
  /**%%%default_text_name%%%%%%%**/
   save_default_text_name();
  /**%%%end_default_text_name%%%**/
  return TRUE;
}
    

/*---------------------------------------------------------------------

  field_seperate -- seperate fields in a line of user dict.

    input  -- line: the line to be processed
    output -- fields: the seperated fields  

---------------------------------------------------------------------*/
void field_seperate(line, fields)
char line[120];
char fields[MAXPTRNS+7][80];
{
  int  j, l, k, len;
  char ch;
      
  k = 0;
  l = 0;

  len = strlen(line); 
  for (j = 0; j < len; j++) {
    ch = line[j];

    if ((ch != ',') && (ch != '\n')) {
      fields[k][l++] = ch;
    }
    else {
      fields[k++][l] = '\0';
      l = 0;
    }
  }
}


/*-----------------------------------------------------------------------
  
  get_sentence  --  read a sentence from text, transform it to a sequence
                    of syntax token which is headed by globle variable
                    'sentence', and an original sentence headed by globle
                    veriable 'original_sentence'.         

    input  -- none
    output -- none

  function returns TRUE if such a sentence is read, FALSE otherwise.

----------------------------------------------------------------------*/
boolean get_sentence()
{
  int    i, j;
  int    code, k;
  struct Syntx_Term  term;


  sentence_len = 0;

  i = text_inner_ptr;
  while (i < text_len) {
    if (text[i++] >= 0) break;
  }

  if (i == text_len) return FALSE;

  j = --i;
  while (j < text_len) { 
    code = text[j++];
    if (code == -32) continue;
    if (code == -46) break;
    sentence_len++;
  }
  text_inner_ptr = j;

  sentence = (struct Syntx_Term *)malloc(sentence_len * sizeof(term));
  if (! sentence) {
    stx_err("memory request failed", __LINE__, __FILE__);
  }
  original_sentence = (int *)malloc(sentence_len * sizeof(int));
  original_sentence_len = sentence_len;
  if (! original_sentence) {
    stx_err("memory request failed", __LINE__, __FILE__);
  }

  for (k = 0; k < sentence_len; k++) {
    code = text[i++];
    while (code < 0) code = text[i++];
    original_sentence[k] = code;
    sentence[k].begin = k;
    sentence[k].end = k;

    get_word_ptrns(code, RARELY, sentence[k].ptrns, sentence[k].ptrns_lh);
    sentence[k].phrase[0] = '\0';
    strcpy(sentence[k].usedptrns, "");
  }    

  old_stc_ptrns = (PTRNS *)malloc(sentence_len * sizeof(PTRNS));
  for (k = 0; k < sentence_len; k++) 
    strcpy(old_stc_ptrns[k], "");

  init_stxerrlist();
  return TRUE;
}


/*------------------------------------------------------------------------
  load_prb -- load a problem

    input  -- prb_name: the name of the problem to be loaded
    output -- none

------------------------------------------------------------------------*/
void load_prb(prb_name)
char  *prb_name;
{
  strcpy(name, prb_name);
  ret_prb(name, state, type, value, errcde);
  if (errcde[0] != 0) stx_err("failed to retrive a problem",
                                 __LINE__, __FILE__);

}


/*----------------------------------------------------------------------
  
  copy_dgns -- copy a set of diagnostic variables, dgns2, to dgns1

    input  -- dgns2: a set of diagnostic variables to be copied
    output -- dgns1: copied set of diagnostic variables

----------------------------------------------------------------------*/
void copy_dgns(dgns1, dgns2)
DIAGNOSIS_ARRAY dgns1[], dgns2[];
{
  int  i = 0;

  while (! EQUAL(dgns2[i], "")) {
    strcpy(dgns1[i], dgns2[i]);
    i++;
  }

  strcpy(dgns1[i], dgns2[i]);
}


/*----------------------------------------------------------------------
  
  next_move  --  pop a sentence from stack, return the corresponding 
                 problem code

    input  -- none
    output -- pos:   the position where the action to be taken
              lklhd: the likelyhood at which the action will take
              who:   the block code which issued this action
 
    function returns the corresponding problem code, or 0 if stack is empty

-----------------------------------------------------------------------*/
int next_move(pos, lklhd, who)
int *pos;
int *lklhd;
int *who;
{
  int  prbcode;

  stc_dsp(sentence);
  if ((sentence = (struct Syntx_Term *)stc_pop(&sentence_len, &prbcode, 
       pos, lklhd, who, &sentence_spoc)) != NULL) {
    return prbcode;
  }
  return 0;
}


  
/*---------------------------------------------------------------------

  intlhd_to_strlhd  --  convert an integer likelihood to a string 
                        likelihood used in log files

    input  -- intlhd: integer likelihood
    output -- strlhd: string likelihood

----------------------------------------------------------------------*/
void intlhd_to_strlhd(intlhd, strlhd)
int  intlhd;
char *strlhd;
{
  if ((intlhd <= 100) AND (intlhd > 92)) {
    strcpy(strlhd, "100");
  }

  if ((intlhd <= 92) AND (intlhd > 75)) {
    strcpy(strlhd, "085");
  }

  if ((intlhd <= 75) AND (intlhd > 57)) {
    strcpy(strlhd, "065");
  }

  if ((intlhd <= 57) AND (intlhd > 37)) {
    strcpy(strlhd, "050");
  }

  if ((intlhd <= 37) AND (intlhd > 17)) {
    strcpy(strlhd, "025");
  }

  if ((intlhd <= 17) AND (intlhd > 0)) {
    strcpy(strlhd, "010");
  }

}


/*-------------------------------------------------------------------
 
  dgns_val  --  return the value (TRUE or FALSE) of a diagnostic variable

    input  -- var:  a diagnostic variable
              dgns: a list of diagnostic variable with TRUE value
    output -- TRUE if the var has TRUE value
              FALSE otherwise

---------------------------------------------------------------------*/
/*boolean dgns_val(var, dgns)
char *var;
DIAGNOSIS_ARRAY dgns[];
{
  int  j = 0;

  while (! EQUAL(dgns[j],"")) { 
    if EQUAL(dgns[j++],var) return TRUE;
  }
  return FALSE;
}
*/


/*--------------------------------------------------------------------
  
  get_word  --  get a word from a sentence

    input  -- stc:  a sentence
              len:  sentence length
              idx:  the index of the word in  the sentence
    output -- word: the word

---------------------------------------------------------------------*/
void get_word(stc, len, idx, word)
struct Syntx_Term *stc;
int    len;
int    idx;
char   *word;
{
  strcpy(word, ""); 
  if ((idx < 0) OR (idx >= len)) return;
  if (EQUAL(stc[idx].phrase,""))   
    strcpy(word, user_dict[original_sentence[stc[idx].begin]].word);
}


/*--------------------------------------------------------------------
  
  get_1stwd_of_phrs  --  get the 1st word of a phrase

    input  -- stc:  a sentence
              len:  sentence length
              idx:  the index of the word in  the sentence
    output -- word: the word

---------------------------------------------------------------------*/
void get_1stwd_of_phrs(stc, len, idx, word)
struct Syntx_Term *stc;
int    len;
int    idx;
char   *word;
{
  strcpy(word, ""); 
  if ((idx < 0) OR (idx >= len)) return;
  if (!EQUAL(stc[idx].phrase,""))   
    strcpy(word, user_dict[original_sentence[stc[idx].begin]].word);
}


/*--------------------------------------------------------------------
  
  get_lastwd_of_phrs  --  get the last word of a phrase

    input  -- stc:  a sentence
              len:  sentence length
              idx:  the index of a token in  the sentence
    output -- word: the word

---------------------------------------------------------------------*/
void get_lastwd_of_phrs(stc, len, idx, word)
struct Syntx_Term *stc;
int    len;
int    idx;
char   *word;
{
  strcpy(word, ""); 
  if ((idx < 0) OR (idx >= len)) return;
  if (!EQUAL(stc[idx].phrase,""))   
    strcpy(word, user_dict[original_sentence[stc[idx].end]].word);
}


/*--------------------------------------------------------------------
  
  words_in_phrase  --  get corresponding words for a phrase

    input  -- stc:  a sentence
              len:  sentence length
              idx: the index of the word in  the sentence
    output -- words: returned words

---------------------------------------------------------------------*/
void words_in_phrase(stc, len, idx, words)
struct Syntx_Term *stc;
int    len;
int    idx;
char   words[MAXPHRASE+1][WORD_MAX+1]; 
{
  int i, j;

  if ((idx < 0) OR (idx >= len)) {
    strcpy(words[0], "");
    return;
  }
  j = 0;
  for (i = stc[idx].begin; i <= stc[idx].end; i++) 
    strcpy(words[j++], user_dict[original_sentence[i]].word);
  strcpy(words[j], "");
}

/*--------------------------------------------------------------------

  begin_title  --  print the title at the begining of executing a model

    input  -- ttl: name of the model
              pos: the position where a model work on
              lklhd: the likelihood which a model work at
    output -- none

----------------------------------------------------------------------*/
void begin_title(ttl, pos, lklhd)
char *ttl;
int  pos, lklhd;
{
  char tk[WORD_MAX + 1];

  if (pos == -1) {
    printf("\n\n<<<----------------- '%s' begin ---------------->>> \n\n", ttl);
    sentence_check(sentence, sentence_len);
    return;
  }
  get_word(sentence, sentence_len, pos, tk);
  printf("\n\n<<<------- '%s' begins on word '%s' at %d%% ------->>> \n\n", ttl, tk, lklhd);
  sentence_check(sentence, sentence_len);
}

/*--------------------------------------------------------------------

  end_title  --  print the title at the end of executing a model

    input  -- ttl: name of the model
    output -- none

----------------------------------------------------------------------*/
void end_title(ttl)
char *ttl;
{
/*  sentence_check(sentence, sentence_len);*/
  check_stack();
  printf("\n<<<-------------- '%s' end -------------->>> \n\n\n", ttl);
}



/*--------------------------------------------------------------------

  init_ptrns_window  --  initialize patterns window based on the 
                         likelihood

    input  -- ptrns: patterns
              lklhd: patterns' likelihood
    output -- window: patterns distribution window

---------------------------------------------------------------------*/
void init_ptrns_window(ptrns, lklhd, window)
char *ptrns;
int  lklhd[MAXPTRNS];
char  window[5][MAXPTRNS+1];
{
  int  len, i, j, lh[MAXPTRNS], pre_lh, win_idx;
  char pt[MAXPTRNS+1], class[2];

  len = strlen(ptrns);
  strcpy(pt, ptrns);
  for (i = 0; i < len; i++) lh[i] = lklhd[i];

/* sort the patterns in decreasing order */
  for (i = 0; i < (len - 1); i++) {
    for (j = i + 1; j < len; j++) {
      int  val;
      char ch;

      if (lh[i] < lh[j]) {
        val = lh[i];
        ch = pt[i];
        lh[i] = lh[j];
        pt[i] = pt[j];
        lh[j] = val;
        pt[j] = ch;
      }
    }
  }

/* assign the patterns into window */
  for (i = 0; i < 5; i++) strcpy(window[i], "");
  win_idx = 0;
  class[0] = pt[0];
  class[1] = '\0';
  strcat(window[win_idx], class);
  pre_lh = lh[0];
  for (i = 1; i < len; i++) {
    int  val;

    class[0] = pt[i];
    class[1] = '\0';
    if (win_idx < 4) {
      val = ((pre_lh - lh[i]) * 100) / pre_lh;
      if (val >= 10) pre_lh = lh[i];
      while (val >= 10) {
        if (win_idx < 4) {
          win_idx++;
          val = val - 20;
        }
        else break;
      }
    }
    strcat(window[win_idx], class);
  }
}

      
/*--------------------------------------------------------------------

  class_name  -- return the full name of a classification

    input  -- stc:  a sentence
              slen: sentence length
              idx:  token index
              pt: a classification
    output -- name: the full name of pt

-----------------------------------------------------------------------*/
void class_name(stc, slen, idx, pt, name)
struct Syntx_Term *stc;
int    slen;
int    idx;
char pt, *name;
{
  char word[WORD_MAX+1];
  int  i, l;

  strcpy(name, "");

  switch (pt) {
    case 'n':
    case 'o':  strcpy(name, "noun_sgl");
               return;

    case 'N':  strcpy(name, "noun_pl");
               return;

    case 't':  strcpy(name, "vt_pl");
               return;

    case 'u':  get_word(stc, slen, idx, word);
               l = strlen(word);
               for (i = 0; i < l; i++) word[i] = tolower(word[i]);
               if (EQUAL(word,"has")  OR
                   EQUAL(word,"does"))   { strcpy(name, "vt_sgl"); }
               else { strcpy(name, "vt_pl"); }
               return;

    case 'T':  strcpy(name, "vt_sgl");
               return;

    case 'i':  strcpy(name, "vi_pl");
               return;

    case 'j':  get_word(stc, slen, idx, word);
               l = strlen(word);
               for (i = 0; i < l; i++) word[i] = tolower(word[i]);
               if (EQUAL(word,"has")  OR
                   EQUAL(word,"does"))   { strcpy(name, "vi_sgl"); }
               else { strcpy(name, "vi_pl"); }
               return;

    case 'I':  strcpy(name, "vi_sgl");
               return;

    case 'v':  strcpy(name, "vt_pst");
               return;

    case 'V':  strcpy(name, "vt_pprt");
               return;

    case 'g':  strcpy(name, "vt_ing");
               return;

    case 'k':  strcpy(name, "vi_pst");
               return;

    case 'K':  strcpy(name, "vi_pprt");
               return;

    case 'G':  strcpy(name, "vi_ing");
               return;

    case 'a':  strcpy(name, "adj");
               return;
  
    case 'f':  strcpy(name, "adj_er");
               return;

    case 'F':  strcpy(name, "adj_est");
               return;

    case 'y':  strcpy(name, "adv");
               return;
              
    case 'm':  strcpy(name, "adv_er");
               return;
              
    case 'M':  strcpy(name, "adv_est");
               return;

    case 'p':  strcpy(name, "prep");
               return;
  
    case 's':  strcpy(name, "pron_sgl");
               return;

    case 'S':  strcpy(name, "pron_pl");
               return;

    case 'r':  strcpy(name, "pron");
               return;

    case 'c':  strcpy(name, "conj");
               return;
  
    case 'q':  strcpy(name, "intj");
               return;

    case 'E':  strcpy(name, "artcl_df");
               return;

    case 'e':  strcpy(name, "artcl_id");
               return;
 
    case 'b':  strcpy(name, "be_prst");
               return;

    case 'B':  strcpy(name, "be_past");
               return;

    case 'w':  strcpy(name, "be");
               return;

    case 'W':  strcpy(name, "been");
               return;

    case 'z':  strcpy(name, "being");
               return;

    case 'd':  strcpy(name, "do_prst");
               return;

    case 'D':  strcpy(name, "do_past");
               return;

    case 'h':  strcpy(name, "hv_prst");
               return;

    case 'H':  strcpy(name, "hv_past");
               return;

    case 'x':  strcpy(name, "aux_prst");
               return;

    case 'X':  strcpy(name, "aux_past");
               return;

    case 'A':  strcpy(name, "abrv");
               return;

    case 'l':  strcpy(name, "lvb_pl");
               return;

    case 'L':  strcpy(name, "lvb_sgl");
               return;

    case 'O':  strcpy(name, "lvb_pst");
               return;

    case 'Q':  strcpy(name, "lvb_pprt");
               return;

    case 'R':  strcpy(name, "lvb_ing");
               return;

    case 'U':  strcpy(name, "undef");
               return;

    case 'Y':  strcpy(name, "dgt_sgl");
               return;

    case 'Z':  strcpy(name, "dgt_pl");
               return;

    case '0':  strcpy(name, "sgl_hv");
               return;

    case '1':  strcpy(name, "sgl_hd");
               return;

    case '2':  strcpy(name, "sgl_be");
               return;

    case '3':  strcpy(name, "sgl_ll");
               return;

    case '4':  strcpy(name, "sgl_ld");
               return;

    case '5':  strcpy(name, "pl_hv");
               return;

    case '6':  strcpy(name, "pl_hd");
               return;

    case '7':  strcpy(name, "pl_be");
               return;

    case '8':  strcpy(name, "pl_ll");
               return;

    case '9':  strcpy(name, "pl_ld");
               return;

    /* original code commented out below, replaced by next line */
    /* change made 2/20/2007 to supress error action */
    /* this is a temporary fix that does not eliminate the error*/
    /* but masks the problem by substiting the "undef" case */
    /* default:   sprintf(err_msg, "unrecognized pattern: %c", pt); */
    /* stx_err(err_msg, __LINE__, __FILE__); */
       default: strcpy(name, "undef");
                return;
  }
}


/*--------------------------------------------------------------------

  name_class  --  given a fullname, return its classification

    input  --  name: fullname
    output --  cls:  classification(s)

---------------------------------------------------------------------*/
void name_class(name, cls)
char *name, *cls;
{
  char str[3];

  str[0] = name[0];
  str[1] = name[1];
  str[2] = '\0';

  if (EQUAL(str,"no")) {
    if (EQUAL(name,"noun_sgl")) { strcpy(cls, "no"); return; }
    if (EQUAL(name,"noun_pl")) { strcpy(cls, "N"); return; }
    goto default_name;
  }
  if (EQUAL(str,"vt")) {
    if (EQUAL(name,"vt_pl")) { strcpy(cls, "tu"); return; }
    if (EQUAL(name,"vt_sgl")) { strcpy(cls, "Tu"); return; }
    if (EQUAL(name,"vt_pst")) { strcpy(cls, "v"); return; }
    if (EQUAL(name,"vt_pprt")) { strcpy(cls, "V"); return; }
    if (EQUAL(name,"vt_ing")) { strcpy(cls, "g"); return; }
    goto default_name;
  }
  if (EQUAL(str,"vi")) {
    if (EQUAL(name,"vi_pl")) { strcpy(cls, "ij"); return; }
    if (EQUAL(name,"vi_sgl")) { strcpy(cls, "Ij"); return; }
    if (EQUAL(name,"vi_pst")) { strcpy(cls, "k"); return; }
    if (EQUAL(name,"vi_pprt")) { strcpy(cls, "K"); return; }
    if (EQUAL(name,"vi_ing")) { strcpy(cls, "G"); return; }
    goto default_name;
  }
  if (EQUAL(str,"ad")) {
    if (EQUAL(name,"adj")) { strcpy(cls, "a"); return; }
    if (EQUAL(name,"adj_er")) { strcpy(cls, "f"); return; }
    if (EQUAL(name,"adj_est")) { strcpy(cls, "F"); return; }
    if (EQUAL(name,"adv")) { strcpy(cls, "y"); return; }
    if (EQUAL(name,"adv_er")) { strcpy(cls, "m"); return; }
    if (EQUAL(name,"adv_est")) { strcpy(cls, "M"); return; }
    goto default_name;
  }
  if (EQUAL(str,"pr")) {
    if (EQUAL(name,"pron_sgl")) { strcpy(cls, "s"); return; }
    if (EQUAL(name,"pron_pl")) { strcpy(cls, "S"); return; }
    if (EQUAL(name,"pron")) { strcpy(cls, "r"); return; }
    if (EQUAL(name,"prep")) { strcpy(cls, "p"); return; }
    if (EQUAL(name,"pron_adj")) { strcpy(cls, "sSra"); return; }
    goto default_name;
  }
  if (EQUAL(str,"ar")) {
    if (EQUAL(name,"artcl_df")) { strcpy(cls, "E"); return; }
    if (EQUAL(name,"artcl_id")) { strcpy(cls, "e"); return; }
  }
  if (EQUAL(str,"be")) {
    if (EQUAL(name,"be_prst")) { strcpy(cls, "b"); return; }
    if (EQUAL(name,"be_past")) { strcpy(cls, "B"); return; }
    if (EQUAL(name,"be")) { strcpy(cls, "w"); return; }
    if (EQUAL(name,"been")) { strcpy(cls, "W"); return; }
    if (EQUAL(name,"being")) { strcpy(cls, "z"); return; }
  }
  if (EQUAL(str,"do")) {
    if (EQUAL(name,"do_prst")) { strcpy(cls, "d"); return; }
    if (EQUAL(name,"do_past")) { strcpy(cls, "D"); return; }
  }
  if (EQUAL(str,"hv")) {
    if (EQUAL(name,"hv_prst")) { strcpy(cls, "h"); return; }
    if (EQUAL(name,"hv_past")) { strcpy(cls, "H"); return; }
  }
  if (EQUAL(str,"au")) {
    if (EQUAL(name,"aux_prst")) { strcpy(cls, "x"); return; }
    if (EQUAL(name,"aux_past")) { strcpy(cls, "X"); return; }
  }
  if (EQUAL(str,"lv")) {
    if (EQUAL(name,"lvb_pl")) { strcpy(cls, "l"); return; }
    if (EQUAL(name,"lvb_sgl")) { strcpy(cls, "L"); return; }
    if (EQUAL(name,"lvb_pst")) { strcpy(cls, "O"); return; }
    if (EQUAL(name,"lvb_pprt")) { strcpy(cls, "Q"); return; }
    if (EQUAL(name,"lvb_ing")) { strcpy(cls, "R"); return; }
  }
  if (EQUAL(str,"sg")) {
    if (EQUAL(name,"sgl_hv")) { strcpy(cls, "0"); return; }
    if (EQUAL(name,"sgl_hd")) { strcpy(cls, "1"); return; }
    if (EQUAL(name,"sgl_be")) { strcpy(cls, "2"); return; }
    if (EQUAL(name,"sgl_ll")) { strcpy(cls, "3"); return; }
    if (EQUAL(name,"sgl_ld")) { strcpy(cls, "4"); return; }
  }
  if (EQUAL(str,"pl")) {
    if (EQUAL(name,"pl_hv")) { strcpy(cls, "5"); return; }
    if (EQUAL(name,"pl_hd")) { strcpy(cls, "6"); return; }
    if (EQUAL(name,"pl_be")) { strcpy(cls, "7"); return; }
    if (EQUAL(name,"pl_ll")) { strcpy(cls, "8"); return; }
    if (EQUAL(name,"pl_ld")) { strcpy(cls, "9"); return; }
  }
  if (EQUAL(name,"conj")) { strcpy(cls, "c"); return; }
  if (EQUAL(name,"intj")) { strcpy(cls, "q"); return; }
  if (EQUAL(name,"abrv")) { strcpy(cls, "A"); return; }
  if (EQUAL(name,"lnkvb")) { strcpy(cls, "l"); return; }
  if (EQUAL(name,"undef")) { strcpy(cls, "U"); return; }

default_name:
  sprintf(err_msg, "unrecognized pattern name: %s", name);
  stx_err(err_msg, __LINE__, __FILE__);
}



/*--------------------------------------------------------------------

  name_class_lhd  -- given a fullname, return its classification and 
                     likelihood in the specified token

    input  -- stc:   a sentence
              slen:  sentence length
              idx:   token index
              name:  the fullname
    output -- cls: the classification
              lhd: likelihood of the classification

-----------------------------------------------------------------------*/
void name_class_lhd(stc, slen, idx, name, cls, lhd)
struct Syntx_Term *stc;
int    slen, idx;
char   *name, *cls;
int    *lhd;
{
  char clss[5];
  int  len, i;

  name_class(name, clss);
  len = strlen(clss);
  for (i = 0; i < len; i++) {
    cls[0] = clss[i];
    cls[1] = '\0';
    if (has_class(stc, slen, idx, cls[0], lhd)) return;
  }
  strcpy(cls,"");
  *lhd = 0;
}



/*----------------------------------------------------------------------

  get_word_ptrns  --  check the patterns window in user_dict, return the 
                      patterns corresponding to the specified likelihood.

    input  -- code: index of the word in the user_dict
              lklhd: likelihood
    output -- ptrns: returned patterns
              lh:    retruned patterns' likelihood

  function returns TRUE if it find some patterns, FALSE otherwise

-----------------------------------------------------------------------*/
boolean get_word_ptrns(code, lklhd, ptrns, lh)
int  code, lklhd;
char *ptrns;
int  lh[MAXPTRNS];
{
  int  i, j, k, len;
  static int class_lh[5] = {85, 65, 50, 25, 10};

  strcpy(ptrns, "");
  k = 0;
  for (i = 0; i < lklhd; i++) {
    strcat(ptrns, user_dict[code].ptrns_window[i]);
    len = strlen(user_dict[code].ptrns_window[i]);
    for (j = 0; j < len; j++) lh[k++] = class_lh[i];
  }
  if (EQUAL(ptrns,"")) return FALSE;
  return TRUE;
}


/*---------------------------------------------------------------------

  get_stc_ptrns  -- check the patterns window in user_dict, set the 
                    patterns for each word in the sentence based on the
                    specified likelihood.

    input  -- lklhd: likelihood
    output -- none

  function returns FALSE if it can not find a pattern for some word or all
  patterns matched w/ last patterns sequence,  TRUE otherwise.

-----------------------------------------------------------------------*/
boolean get_stc_ptrns(lklhd)
int  lklhd;
{
  int i, code;
  char ptrns[MAXPTRNS+1];
  boolean not_same = FALSE;

  for (i = 0; i < sentence_len; i++) {
    code = original_sentence[sentence[i].begin];
    if (!get_word_ptrns(code, lklhd, ptrns, sentence[i].ptrns_lh)) 
      return FALSE;
    if (!EQUAL(old_stc_ptrns[i],ptrns)) {
      strcpy(old_stc_ptrns[i], ptrns);
      not_same = TRUE;
    }
    strcpy(sentence[i].ptrns, ptrns);
    sentence[i].phrase[0] = '\0';
    strcpy(sentence[i].usedptrns, "");
  }    
  return not_same;
}


/*---------------------------------------------------------------------

  window_drop  --  drop a pattern from the patterns' window

    input  -- stc: a sentence
              idx: token index
              pt:  the pattern to be dropped
    output -- none

-----------------------------------------------------------------------*/
void window_drop(stc, idx, pt)
struct Syntx_Term *stc;
int    idx;
char   pt;
{
  int  code, i, j, k, len;
  char ptrns[MAXPTRNS+1], newpts[MAXPTRNS+1];

  code = original_sentence[stc[idx].begin];
  for (i = 0; i < 5; i++) {
    strcpy(ptrns, user_dict[code].ptrns_window[i]);
    len = strlen(ptrns);
    k = 0;
    for (j = 0; j < len; j++) {
      if (ptrns[j] != pt) newpts[k++] = ptrns[j];
    }
    newpts[k] = '\0';
    strcpy(user_dict[code].ptrns_window[i], newpts);
  }
}


/*---------------------------------------------------------------------
  
  get_numofwords  --  return the number of words in the user dictionary.

    input  -- title: the first line in the user dictionary
    output -- num: the number of words in user dictionary

-----------------------------------------------------------------------*/
void get_numofwords(title, version, num)
char *title;
char *version;
int  *num;
{
  char  number[120];
  int   len, i, k = 0;

  len = strlen(title);
  for (i = 0; i < len; i++) 
    if (title[i] == '=') break;
  i++;
  while (i < len) number[k++] = title[i++];
  number[k] = '\0';
  *num = atoi(number);
} 

  
/*----------------------------------------------------------------------

  lklhd_level  --  get the level of a likelihood

    input  -- lklhd: the likelihood
    output -- none

  function returns the level of lklhd

----------------------------------------------------------------------*/
int lklhd_level(lklhd)
int lklhd;
{
  switch (lklhd) {
    case 100: return ALWAYS;
    case  85: return VERYFREQLY;
    case  65: return FREQLY;
    case  50: return HALF;
    case  25: return SOMETIME;
    case  10: return RARELY;
    default: stx_err("unexpected likelihood", __LINE__, __FILE__);
  }
}


/*---------------------------------------------------------------------

  block_name  --  return the block name of the given block code

    input  -- code: code of the block
    output -- name: name of the block

----------------------------------------------------------------------*/
void block_name(code, name)
int  code;
char *name;
{
  switch (code) {
    case Simplify:     
                            strcpy(name, "Simplify");
                            break;
    case FindPredicate: 
                            strcpy(name, "FindPredicate");
                            break;
    case FindSubject:   
                            strcpy(name, "FindSubject");
                            break;
    case FindObject:    
                            strcpy(name, "FindObject");
                            break;
    case FindModifier:  
                            strcpy(name, "FindModifier");
                            break;
    case ToPhrase:    
                            strcpy(name, "ToPhrase");
                            break;
    case VerbPhrase:    
                            strcpy(name, "VerbPhrase");
                            break;
    case LnkvbPhrase:   
                            strcpy(name, "LnkvbPhrase");
                            break;
    case BePhrase:      
                            strcpy(name, "BePhrase");
                            break;
    case DoPhrase:      
                            strcpy(name, "DoPhrase");
                            break;
    case HavePhrase:    
                            strcpy(name, "HavePhrase");
                            break;
    case AuxPhrase:    
                            strcpy(name, "AuxPhrase");
                            break;
    case PronvbPhrase:  
                            strcpy(name, "PronvbPhrase");
                            break;
    case PronvbBe:  
                            strcpy(name, "PronvbBe");
                            break;
    case HaveBe:  
                            strcpy(name, "HaveBe");
                            break;
    case AuxHave:  
                            strcpy(name, "AuxHave");
                            break;
    case AuxHaveVerb:  
                            strcpy(name, "AuxHaveVerb");
                            break;
    case AuxHaveBe:  
                            strcpy(name, "AuxHaveBe");
                            break;
    case PvHave:  
                            strcpy(name, "PvHave");
                            break;
    case PvHaveVerb:  
                            strcpy(name, "PvHaveVerb");
                            break;
    case PvHaveBe:  
                            strcpy(name, "PvHaveBe");
                            break;
    case AuxBe:  
                            strcpy(name, "AuxBe");
                            break;
    case NounPhrase:   
                            strcpy(name, "NounPhrase");
                            break;
    case SubNoun:       
                            strcpy(name, "SubNoun");
                            break;
    case ObjNoun:       
                            strcpy(name, "ObjNoun");
                            break;
    case MdfNoun:       
                            strcpy(name, "MdfNoun");
                            break;
    case PrepNoun:      
                            strcpy(name, "PrepNoun");
                            break;
    case SubNounAdjNoun:      
                            strcpy(name, "SubNounAdjNoun");
                            break;
    case ObjNounAdjNoun:      
                            strcpy(name, "ObjNounAdjNoun");
                            break;
    case MdfNounAdjNoun:      
                            strcpy(name, "MdfNounAdjNoun");
                            break;
    case PrepNounAdjNoun:      
                            strcpy(name, "PrepNounAdjNoun");
                            break;
    case PronounPhrase: 
                            strcpy(name, "PronounPhrase");
                            break;
    case AdjPhrase:    
                            strcpy(name, "AdjPhrase");
                            break;
    case PrepAdj:           
                            strcpy(name, "PrepAdj");
                            break;
    case SubNounAdj:      
                            strcpy(name, "SubNounAdj");
                            break;
    case ObjNounAdj:      
                            strcpy(name, "ObjNounAdj");
                            break;
    case MdfNounAdj:      
                            strcpy(name, "MdfNounAdj");
                            break;
    case PrepNounAdj:      
                            strcpy(name, "PrepNounAdj");
                            break;
    case AdvPhrase:    
                            strcpy(name, "AdvPhrase");
                            break;
    case PrepAdjAdv:           
                            strcpy(name, "PrepAdjAdv");
                            break;
    case SubNounAdjAdv:      
                            strcpy(name, "SubNounAdjAdv");
                            break;
    case ObjNounAdjAdv:      
                            strcpy(name, "ObjNounAdjAdv");
                            break;
    case MdfNounAdjAdv:      
                            strcpy(name, "MdfNounAdjAdv");
                            break;
    case PrepNounAdjAdv:      
                            strcpy(name, "PrepNounAdjAdv");
                            break;
    case PronvbBeAdv:
                            strcpy(name, "PronvbBeAdv");
                            break;
    case HaveBeAdv:
                            strcpy(name, "HaveBeAdv");
                            break;
    case AuxHaveAdv:
                            strcpy(name, "AuxHaveAdv");
                            break;
    case PvHaveAdv:
                            strcpy(name, "PvHaveAdv");
                            break;
    case AuxHaveBeAdv:
                            strcpy(name, "AuxHaveBeAdv");
                            break;
    case PvHaveBeAdv:
                            strcpy(name, "PvHaveBeAdv");
                            break;
    case AuxBeAdv:
                            strcpy(name, "AuxBeAdv");
                            break;
    case PrepPhrase:    
                            strcpy(name, "PrepPhrase");
                            break;
    case SPOChecker:
                            strcpy(name, "SPOChecker");
                            break;

    case MdfChecker:  
                            strcpy(name, "MdfChecker");
                            break;

    default: stx_err("unexpected problem code", __LINE__, __FILE__);
  }
}


/*-------------------------------------------------------------------

  sentence_check  --  print out the current sentence

    input  -- none
    output -- none

--------------------------------------------------------------------*/
void  sentence_check(stc, slen)
struct  Syntx_Term  *stc;
int     slen;
{
  int  i, j, code, b, e, len;

  printf("<<------------------ Current Sentence ------------------>>\n");
  for (i = 0; i < slen; i++) {
    e = stc[i].end;
    for (b = stc[i].begin; b <= e; b++) {
      code = original_sentence[b];
      printf(" %s", user_dict[code].word);
    }
    printf("[");
    if (EQUAL(stc[i].phrase,"")) {
      len = strlen(stc[i].ptrns);
      for (j = 0; j < len; j++) {
        printf("%c(%d)", stc[i].ptrns[j], stc[i].ptrns_lh[j]);
      }
    }
    else {
      printf("%s(",stc[i].phrase);
      len = strlen(stc[i].usedptrns);
      for (j = 0; j < len; j++) {
        printf("%c(%d)", stc[i].usedptrns[j],
                          stc[i].usedptrns_lh[j]);
      }
      printf(")");
    }
    printf("]   ");
  }
  printf("\n<<------------------ Current Sentence ------------------>>\n\n");
}



/*---------------------------------------------------------------------

  update_classlh  --  update the classification likelihoods after a good
                      interpretation of a sentence has been found.

    input  -- stc:  a sentence
              slen:  sentence length
    output -- none

-----------------------------------------------------------------------*/
void update_classlh(stc, slen)
struct Syntx_Term *stc;
int    slen;
{
  int  i;

  for (i = 0; i < slen; i++) {
    int  begin, end, j, k;

    if (EQUAL(stc[i].phrase,"")) continue;
    begin = stc[i].begin;
    end = stc[i].end;
    k = 0;
    for (j = begin; j <= end; j++) {
      char pt;

      pt = stc[i].usedptrns[k++];
      update_usrdict(original_sentence[j], pt);
    }
  }
}


/*----------------------------------------------------------------------

  update_usrdict  --  update the likelihoods of classifications of a word
                      , specified by idx, in user_dict based on the fact
                      that the frequency of pattern pt should be increased
                      by one.

    input  -- idx: index to the word in user_dict
              pt:  the pattern whose frequency should be increased by one
    output -- none

-----------------------------------------------------------------------*/
void update_usrdict(idx, pt)
int  idx;
char pt;
{
  int  freq_sum = 0, lh_sum = 0, max = 0, max_idx = 0, len, i;
  float flh;
  char ptrns[MAXPTRNS+1];

  len = strlen(user_dict[idx].ptrns);
  strcpy(ptrns, user_dict[idx].ptrns);
  for (i = 0; i < len; i++) {
    if (ptrns[i] == pt) user_dict[idx].ptrns_freq[i]++;
    freq_sum = freq_sum + user_dict[idx].ptrns_freq[i];
  }
  if (freq_sum == 0) return;
  for (i = 0; i < len; i++) {
    int  lh, freq, new_lh;

    lh = user_dict[idx].ptrns_likelihood[i];
    freq = user_dict[idx].ptrns_freq[i];
    flh = (float) (lh + (100 * freq) / freq_sum) / 2;
    new_lh = (int) flh;
    if (new_lh == 0) new_lh = 1;
    if (max < new_lh) { max = new_lh; max_idx = i; }
    lh_sum = lh_sum + new_lh;
    user_dict[idx].ptrns_likelihood[i] = new_lh;
  }
  user_dict[idx].ptrns_likelihood[max_idx] = 
                   user_dict[idx].ptrns_likelihood[max_idx] + 100 - lh_sum;

  init_ptrns_window(user_dict[idx].ptrns, 
                    user_dict[idx].ptrns_likelihood,
                    user_dict[idx].ptrns_window);
}

      
      

  
/*---------------------------------------------------------------------
  
  write_usr_dict -- make the backup copy of user dictionary, copy the
                     updated user dictionary back to disk.

    input  -- none
    output -- none

----------------------------------------------------------------------*/
void write_usr_dict()
{
  int          i, j, num;
  char         line[120], title[120]; 
  FILE         *usrdict, *usrbak;

  if (!display_syntx_msg) {
    fclose(syntx_msg);
    if (stxerr_count == 0) {
      char cmd[130];

      sprintf(cmd, "rm -f %s.err", text_name);
      system(cmd);
    }
  }

  if ((usrdict = fopen(wds_usr, READWRITE)) == NULL) {
    sprintf(err_msg, "failed to open %s", wds_usr);
    stx_err(err_msg, __LINE__, __FILE__);
  }

  if ((usrbak = fopen(wds_bak, WRITE)) == NULL) {
    sprintf(err_msg, "failed to open %s", wds_bak);
    stx_err(err_msg, __LINE__, __FILE__);
  }

  fgets(title, 120, usrdict);
  fprintf(usrbak, "%s", title);
  /* make backup copy */
  for (i = 1; i <= user_dict_size; i++) {
    fgets(line, 120, usrdict);
    fprintf(usrbak, "%s", line);
  }
  fclose(usrbak);
  fclose(usrdict);

  if ((usrdict = fopen(wds_usr, WRITE)) == NULL) {
    sprintf(err_msg, "failed to open %s", wds_usr);
    stx_err(err_msg, __LINE__, __FILE__);
  }
  fprintf(usrdict, "%s", title);
  for (i = 0; i < user_dict_size; i++) {
    fprintf(usrdict, "%s,%s,%s,%d,%s,", user_dict[i].word,
                                        user_dict[i].root,
                                        " ",
                                        user_dict[i].word_freq,
                                        user_dict[i].ptrns);
    num = strlen(user_dict[i].ptrns) - 1;
    for (j = 0; j < num; j++) {
      fprintf(usrdict, "%d,", user_dict[i].ptrns_likelihood[j]);
    }
    fprintf(usrdict, "%d\n", user_dict[i].ptrns_likelihood[j]);
  }

  fclose(usrdict);

  free(user_dict);
}


/*--------------------------------------------------------------------

  syntax_analysis  --  does proper syntax analysis

    input  -- cmd_code: indicate what kind of analysis should be done
              pos:      position the analysis will start at
              lklhd:    likelihood the analysis will work at
              who:      indicate who issue the analysis

    output -- function returns FALSE if some fatal errors occured, 
                               TRUE otherwise.

---------------------------------------------------------------------*/
boolean syntax_analysis(cmd_code, pos, lklhd, who)
int cmd_code, pos, lklhd, who;
{
   switch(cmd_code) {
     case FindPredicate: sentence_spoc.sub_pos = -1;
                         sentence_spoc.prd_pos = -1;
                         sentence_spoc.obj_pos = -1;
                         sentence_spoc.cpl_pos = -1;
                         find_predicate(pos, lklhd, who);
                         break;

     case FindSubject:   find_subject(pos, lklhd, who);
                         break;

     case FindObject:    find_object(pos, lklhd, who);
                         break;

     case FindModifier:  find_modifier(pos, lklhd, who);
                         break;

     case ToPhrase:      to_phrase(pos, lklhd, who);
                         break;

     case VerbPhrase:    verb_phrase(pos, lklhd, who);
                         break;

     case LnkvbPhrase:   printf("\n\nLnkvbPhrase processing\n\n");
                         break;

     case BePhrase:      if (be_phrase(pos, lklhd, who)) break;
                         /*printf("\n\nBePhrase processing\n\n");*/
                         return FALSE;

     case DoPhrase:      do_phrase(pos, lklhd, who);
                         /*printf("\n\nDoPhrase processing\n\n");*/
                         break;

     case HavePhrase:    have_phrase(pos, lklhd, who);
                         /* printf("\n\nHavePhrase processing\n\n"); */
                         break;

     case AuxPhrase:     if (aux_phrase(pos, lklhd, who)) break;
                         return FALSE;
                         /*printf("\n\nAuxPhrase processing\n\n");*/

     case PronvbPhrase:  if (pronvb_phrase(pos, lklhd, who)) break;
                         /*printf("\n\nPronvbPhrase processing\n\n");*/
                         return FALSE;

     case NounPhrase:    noun_phrase(pos, lklhd, who);
                         break;
         
     case PronounPhrase: pronoun_phrase(pos, lklhd, who);
                         break;

     case AdjPhrase:     adj_phrase(pos, lklhd, who);
                         break;

     case AdvPhrase:     adv_phrase(pos, lklhd, who);
                         break;

     case PrepPhrase:    prep_phrase(pos, lklhd, who);
                         break;

     case SPOChecker:    subprdctobj_checker(pos, lklhd, who);
                         break;

     case MdfChecker:    modifier_checker(pos, lklhd, who);
                         break;

     default: stx_err("unexpected command code", __LINE__, __FILE__);
   }
   return TRUE;
}



/*???????????????????????????????????????????????????????????????????*/



void print_dict()
{
  int  i, len, j;

  for (i = 0; i < user_dict_size; i++) {
    printf("%s,%s,%d,%s,", user_dict[i].word, user_dict[i].root,
          user_dict[i].word_freq, user_dict[i].ptrns);
    len = strlen(user_dict[i].ptrns);
    for (j = 0; j < len; j++) {
      printf("%d,", user_dict[i].ptrns_likelihood[j]);
    }
    for (j = 0; j < len; j++) {
      printf("%d,", user_dict[i].ptrns_freq[j]);
    }
    for (j = 0; j < 4; j++) {
      printf("%s,", user_dict[i].ptrns_window[j]);
    }
    printf("%s\n", user_dict[i].ptrns_window[j]);
  }
}





/*--------------------------------------------------------------------

  free_simple_stc  -- free memory occupied by global variables "sentence"
                      and "original_sentence"

    input  -- none
    output -- none

----------------------------------------------------------------------*/
void free_simple_stc()
{
  free(sentence);
  free(original_sentence);
  free(old_stc_ptrns);
  free(sentence_intprt);
}


/*--------------------------------------------------------------------

  free_complex_stc  -- free memory occupied by global variables 
                       "raw_sentence" and "clean_sentence"

    input  -- none
    output -- none

----------------------------------------------------------------------*/
void free_complex_stc()
{
  free(raw_sentence);
  free(clean_sentence);
}
    





/*-----------------------------------------------------------------------
  
  get_raw_sentence  --  read a raw sentence, terminated by a '.' and a
                        space. save the sentence (a sequence of codes) in
                        global variables raw_sentence and raw_sentence_len         
  function returns TRUE if such a sentence is read, FALSE otherwise.

----------------------------------------------------------------------*/
boolean get_raw_sentence()
{
  long  i, j;
  int   lft_code = -999, cnt_code;

  raw_sentence_len = 0;

  i = text_inner_ptr;
  while (i < text_len) {
    cnt_code = text[i];
    if (is_space(cnt_code)) {
      if (cnt_code == -10) text_line++;
      lft_code = cnt_code;
      i++;
    }
    else break;
  }

  if (i >= text_len) return FALSE;

  j = i;
  while (j < text_len) { 
    cnt_code = text[j++];
    if (cnt_code == -10) text_line++;

    if ((lft_code == -10) AND (cnt_code == -10)) break;

    if ((lft_code == -46) OR   /* token == '.' */
        (lft_code == -59) OR   /* token == ';' */
        (lft_code == -63))  {  /* token == '?' */

      if (is_space(cnt_code)) {
        raw_sentence_len++;
        break;
      }
    }

    lft_code = cnt_code;
    raw_sentence_len++;
  }

  if (j > text_len) return FALSE;

  text_inner_ptr = j;

  raw_sentence = (int *)malloc(raw_sentence_len * sizeof(int));
  if (! raw_sentence) {
    stx_err("memory request failed", __LINE__, __FILE__);
  }

  for (j = 0; j < raw_sentence_len; j++) {
    raw_sentence[j] = text[i++];
  }

 /* if (system_status == FindIntprt) print_rawstc(); */

  return TRUE;
}
  
  

/*-----------------------------------------------------------------------
  
  cleanup_raw_sentence  --  clean up a raw sentence by eliminating 
                            undefined tokens. The cleaned sentence is 
                            saved in clean_sentence.

  function returns TRUE if a cleaned sentence can be found, 
                   FALSE otherwise.

----------------------------------------------------------------------*/
boolean cleanup_raw_sentence()
{
  int  raw_stc_idx, clean_stc_idx, lft_code, cnt_code, front_undef,
       difference, num_of_def = 0, num_of_undef = 0, i, j;
  boolean stc_start = FALSE;

  clean_sentence = (int *)malloc(raw_sentence_len * sizeof(int));
  if (! clean_sentence) {
    stx_err("memory request failed", __LINE__, __FILE__);
  }

  raw_stc_idx = 0;
  clean_stc_idx = 0;
  lft_code = UNDEF_code;

/* the following block is used when the clean sentence is defined as a 
   sentence starting with two defined tokens                             */
/*
  while (raw_stc_idx < raw_sentence_len) {
    if (!get_a_token(raw_stc_idx, &cnt_code, &difference)) break;
    if ((lft_code >= 0) AND (lft_code < UNDEF_code)) {  
      if (cnt_code < UNDEF_code) { 
        raw_stc_idx = raw_stc_idx + difference;
        stc_start = TRUE;
        if (cnt_code < 0) { num_of_def = 1; }
        else { num_of_def = 2; }
        break;
      }
    }
    raw_stc_idx = raw_stc_idx + difference;
    lft_code = cnt_code;
  }
*/

/* the following block is used when the clean sentence is defined as a 
   sentence allowing starting with one UNDEF token                       */

  front_undef = -1;
  while (raw_stc_idx < raw_sentence_len) {
    if (!get_a_token(raw_stc_idx, &cnt_code, &difference)) break;
    if ((lft_code >= 0) AND (lft_code < UNDEF_code)) {  
      raw_stc_idx = raw_stc_idx + difference;
      stc_start = TRUE;
      num_of_def = 1;
      if (cnt_code < 0) break; 
      if (cnt_code == UNDEF_code) { num_of_undef = 1; break; }
      num_of_def++; 
      break;
    }
    raw_stc_idx = raw_stc_idx + difference;
    lft_code = cnt_code;
    front_undef++;
  }

  if (!stc_start) return FALSE;
  
  if (front_undef > 0) { has_front_undef = TRUE; }
  else { has_front_undef = FALSE; }
  
  clean_sentence[0] = lft_code;
  clean_sentence[1] = cnt_code;
  clean_stc_idx = 2;

  lft_code = cnt_code;
  
  while (raw_stc_idx < raw_sentence_len) {
    if (!get_a_token(raw_stc_idx, &cnt_code, &difference)) break;
    clean_sentence[clean_stc_idx++] = cnt_code;
    raw_stc_idx = raw_stc_idx + difference;
    if (cnt_code == UNDEF_code) { num_of_undef++; continue; }
    if (cnt_code >= 0) num_of_def++;
  }

  if (num_of_undef >= num_of_def) return FALSE;
  
  clean_sentence_len = clean_stc_idx;
  if (clean_sentence_len > 1) {
    lft_code = clean_sentence[0];
    j = 0;
    for (i = 1; i < clean_sentence_len; i++) {
      cnt_code = clean_sentence[i];
      if ((lft_code == UNDEF_code)  AND
          (cnt_code == UNDEF_code))     continue;
      clean_sentence[j++] = lft_code;
      lft_code = cnt_code;
    }
    clean_sentence[j++] = lft_code;
    clean_sentence_len = j;
  }

  simple_sentence_ptr = 0;

  if (intrnl_dsply AND (system_status==FindIntprt)) print_cleansentence();

  for (i = 0; i < (clean_sentence_len - 1); i++) {
    if (clean_sentence[i] < 0) {
      sentence_type = ComplexSentence;
      return TRUE;
    }
  }
  if (clean_sentence[clean_sentence_len-1] < 0) {
    sentence_type = SimpleSentence;
  }
  else { sentence_type = TitleSentence; }

  return TRUE;
}

        
/*-----------------------------------------------------------------------
  
  get_a_token  --  scan the raw_sentence starting from idx, find a 
                   subsequence codes surrended by space, and convert the 
                   subsequence into a token.

    input  -- idx:  start position in raw_sentence
    output -- code: < 0              --> returned token is a punctuation
                    = UNDEF_code     --> returned token is UNDEF
                    otherwise        --> returned token is defined
              len:  the length scaned

  function returns TRUE if successfully get a token, FALSE otherwise

----------------------------------------------------------------------*/
boolean get_a_token(idx, code, len)
int idx, *code, *len;
{
  int *sub_seq, raw_idx, sub_idx, cnt_code, i;

  sub_seq = (int *)malloc(raw_sentence_len * sizeof(int));
  if (! sub_seq) {
    stx_err("memory request failed", __LINE__, __FILE__);
  }
  raw_idx = idx;
  sub_idx = 0;

  while (raw_idx < raw_sentence_len) {
    cnt_code = raw_sentence[raw_idx];
    if (is_space(cnt_code)) {
      raw_idx++;
      continue;
    }
    break;
  }

  if (raw_idx >= raw_sentence_len) return FALSE;

  if (is_punctuation(cnt_code)) {
    if (raw_idx < raw_sentence_len) { 
      if (!is_space(raw_sentence[raw_idx+1])) {

        /* ?????? */
    
      }
      *code = cnt_code;
      *len = raw_idx - idx + 2;
      free(sub_seq);
      return TRUE;
    }
    *code = cnt_code;
    *len = raw_idx - idx + 1;
    free(sub_seq);
    return TRUE;
  }

  sub_seq[sub_idx++] = cnt_code;
  raw_idx++;
  while (raw_idx < raw_sentence_len) {
    cnt_code = raw_sentence[raw_idx];
    if (is_space(cnt_code)) { raw_idx++; break; }
    if (is_punctuation(cnt_code)) {
      int rht_code;
      
/* ====== added on 10/31/96 ====== */
      if ((raw_idx+1) >= raw_sentence_len) break;
/* =============================== */

      rht_code = raw_sentence[raw_idx+1];
      if (is_space(rht_code)) break;
      if ((rht_code >= 0)                          AND
          (!EQUAL(user_dict[rht_code].ptrns,"U")))      {
        
        /* ????  error processing   ??? */
      }
    }
    sub_seq[sub_idx++] = cnt_code;
    raw_idx++;
  }

  *len = raw_idx - idx;
  if (sub_idx > 1) { *code = UNDEF_code; free(sub_seq); return TRUE; }
  cnt_code = sub_seq[0];
  if (cnt_code < 0) { *code = UNDEF_code; free(sub_seq); return TRUE; }
  /*if (EQUAL(user_dict[cnt_code].ptrns,"U"))
    { *code = UNDEF_code; free(sub_seq); return TRUE; }*/
  *code = cnt_code;
  free(sub_seq);
  return TRUE;
}


/*-----------------------------------------------------------------------

  is_space  --  check if a code is representing a space

    input  -- code: the code to be checked
    output -- none

  function returns TRUE if the code is the space, FALSE otherwise

------------------------------------------------------------------------*/
boolean is_space(code)
int code;
{
  if ((code == -32) OR   /*  ' ' */
      (code == -13) OR   /* '\r' */
      (code == -9)  OR   /* '\t' */
      (code == -10) OR   /* '\n' */
      (code == -8))      /* '\b' */ return TRUE;
  return FALSE;
}

/*-----------------------------------------------------------------------

  is_punctuation  --  check if a code is representing a punctuation

    input  -- code: the code to be checked
    output -- none

  function returns TRUE if the code is a punctuation, FALSE otherwise

------------------------------------------------------------------------*/
boolean is_punctuation(code)
int code;
{
  if ((code == -44) OR   /* , */
      (code == -59) OR   /* ; */
      (code == -58) OR   /* : */
      (code == -46) OR   /* . */
      (code == -63))     /* ? */ return TRUE;
  return FALSE;
}


/*----------------------------------------------------------------------

  get_simple_sentence  --  get a simple sentence from clean_sentence, 
                           save it in global variables 'sentence' and 
                           'orignal_sentence'

    input  -- none;
    output -- none;

  function returns TRUE if a simple sentence is found, FALSE otherwise

-----------------------------------------------------------------------*/
boolean get_simple_sentence()
{
  int  i, j, code;
  struct Syntx_Term  term;

find_simple_stc:

  sentence_len = 0;
  sentence_punc = ' ';
  for (i = simple_sentence_ptr; i < clean_sentence_len; i++) {
    if (clean_sentence[i] < 0) {
      sentence_punc = -clean_sentence[i];
      break;
    }
    sentence_len++;
  }
  if ((i >= clean_sentence_len) AND (sentence_len <=1)) return FALSE;
  if (sentence_len <= 1) {
    simple_sentence_ptr = i + 1;
    goto find_simple_stc;
  }

  sentence = (struct Syntx_Term *)malloc(sentence_len * sizeof(struct Syntx_Term));    
  /* replaced "sizeof(term)" by "sizeof(struct Syntx_Term)" 9/1/2005 */

  if (! sentence) {
    stx_err("memory request failed", __LINE__, __FILE__);
  }
  original_sentence = (int *)malloc(sentence_len * sizeof(int));
  original_sentence_len = sentence_len;
  if (! original_sentence) {
    stx_err("memory request failed", __LINE__, __FILE__);
  }

  /* changed allocation line below, replacing "sentence_len" */
  /* by "sentence_len+1", Sept 9, 2005 */
  /*  sentence_intprt = (char *)malloc(sentence_len * sizeof(char)); */
  sentence_intprt = (char *)malloc((sentence_len+1) * sizeof(char));

  if (! sentence_intprt) {
    stx_err("memory request failed", __LINE__, __FILE__);
  }

  i = simple_sentence_ptr; 
  for (j = 0; j < sentence_len; j++) {
    code = clean_sentence[i++];
    original_sentence[j] = code;
    sentence[j].begin = j;
    sentence[j].end = j;

    get_word_ptrns(code, RARELY, sentence[j].ptrns, sentence[j].ptrns_lh);
    sentence[j].phrase[0] = '\0';
    strcpy(sentence[j].usedptrns, "");

    sentence_intprt[j] = '-';
  }    
  sentence_intprt[j] = '\0';
  simple_sentence_ptr = i + 1;

  old_stc_ptrns = (PTRNS *)malloc(sentence_len * sizeof(PTRNS));
  for (i = 0; i < sentence_len; i++) 
    strcpy(old_stc_ptrns[i], "");

  init_stxerrlist();
  return TRUE;
}


/*---------------------------------------------------------------------

  print_rawstc  --  print raw sentence

    input  -- none
    output -- none

----------------------------------------------------------------------*/
void print_rawstc()
{
  int  i, code;

  printf("\n\n-------------------------------------------------------\n");
  for (i = 0; i < (raw_sentence_len-1); i++) {
    code = raw_sentence[i];
    if (i >= 0) {
      printf("%s ", user_dict[code].word);
    }
    else printf("%c", -code);
  }
  printf("\n");
  printf("-------------------------------------------------------\n\n");
}



void chop_sentence()
{
  int num_smpl_stc = 0, i, code;

  
  for (i = 0; i < clean_sentence_len; i++) 
    if (clean_sentence[i] < 0) num_smpl_stc++;

  if (num_smpl_stc < 2) {
    printf("   THIS IS A SIMPLE SENTENCE \n");
    return;
  }

  printf("   This SENTENCE CONSISTS OF %d SUBSENTENCES:\n\n",num_smpl_stc);
  for (i = 0; i < clean_sentence_len; i++) {
    code = clean_sentence[i];
    if (code < 0) { printf("\n"); }
    else {
      printf("%s ", user_dict[code].word);
    }
  }
}

void get_sentence_test()
{
  int code, i;

  while (get_raw_sentence()) {
    if (cleanup_raw_sentence()) {
      printf("\n");
      for (i = 0; i < clean_sentence_len; i++) {
        code = clean_sentence[i];
        if (code < 0) { printf("%c", -code); }
        else {
          printf(" %s", user_dict[code].word);
        }
      }
      printf("\n");
/*      chop_sentence();*/
    }
  }
}


void print_cleansentence()
{
  int code, i;

  printf("\n---------------------------------------------------------\n");
      for (i = 0; i < clean_sentence_len; i++) {
        code = clean_sentence[i];
        if (code < 0) { printf("%c", -code); }
        else {
          printf(" %s", user_dict[code].word);
        }
      }
  printf("\n-------------------------------------------------------\n\n");
}


  



/*----------------------------------------------------------------------
  read_params  --  read information from params.dat

    input  -- display: indicate if the information to be displayed
    output -- none

-----------------------------------------------------------------------*/
void read_params(display)
boolean display;
{
  char lhs[120], rhs[120], line[120], ch;
  FILE *params;

  if ((params = fopen("params.dat", READ)) == NULL) {
    running_error("failed to open file params.dat.");
  }

  data_path[0] = '\0';
  wds_usr[0] = '\0';
  wds_bak[0] = '\0';
  triple_usr[0] = '\0';
  triple_bak[0] = '\0';

  fgets(line, 120, params);
  while (!feof(params)) {
    if ((line[0] == '*') AND (line[1] == '*') AND (line[2] == '*')) {
      fgets(line, 120, params);
      continue;
    }
 
    get_lhs_rhs(line, lhs, rhs);

    if EQUAL(lhs,"displayallfilesduringlaempel(forinternaluseonly)") {
      intrnl_dsply = TRUE;
      fgets(line, 120, params);
      continue;
    };

    if EQUAL(lhs,"directoryofLaempel/Data") {
      strcpy(data_path, rhs);
      fgets(line, 120, params);
      continue;
    };

    if EQUAL(lhs,"userdictionary") {
      strcpy(wds_usr, rhs);
      fgets(line, 120, params);
      continue;
    };

    if EQUAL(lhs,"userdictionarybackup") {
      strcpy(wds_bak, rhs);
      fgets(line, 120, params);
      continue;
    };
 
    if EQUAL(lhs,"userphrasepattern") {
      strcpy(triple_usr, rhs);
      fgets(line, 120, params);
      continue;
    };

    if EQUAL(lhs,"userphrasepatternbackup") {
      strcpy(triple_bak, rhs);
      fgets(line, 120, params);
      continue;
    };

    if EQUAL(lhs,"findsyntacticalinterpretations(yes,no)"){
      if EQUAL(rhs,"yes") { system_status = FindIntprt; }
      else { system_status = FindError; }
      fgets(line, 120, params);
      continue;
    }
  
    if EQUAL(lhs,"interactiveorbatchsyntaxchecking(int,bat)"){
      if EQUAL(rhs,"int") { display_syntx_msg = TRUE; }
      else { display_syntx_msg = FALSE; }
      fgets(line, 120, params);
      continue;
    }

  /**%%%%%%%%8/99%%%%%%%%%**/

  /***parameters for semantic checking***/

   if( EQUAL(lhs,"directoryofWorkarea") OR
       EQUAL(lhs,"historyreferencefile") OR
       EQUAL(lhs,"separationsfile") OR
       EQUAL(lhs,"directoryoflsquare.sepprogram") OR
       EQUAL(lhs,"directoryoflsquare.votprogram") OR
       EQUAL(lhs,"directoryofLsquare/Data") OR
       /**EQUAL(lhs,"directoryofLeibniztranslatprogram") OR**/
       EQUAL(lhs,"usersuggestedreplacements") /**OR
       EQUAL(lhs,"directoryofLeibnizgenerateprogram")**/    ) 
     {
          fgets(line, 120, params);
          continue;
      }

  /**%%%%%%end8/99%%%%%%%%**/
  
  /**%%%default_text_name%%%%%%%**/
  
  if (strcmp(lhs, "filenamestorage") == 0)
    {
      strcpy(filename_storage, rhs);
      fgets(line, 120, params);
      continue;
    };
  
  /**%%%end_default_text_name%%%**/

    if (strcmp(lhs, "ENDATA") == 0)
    {
      break;
    };

    if (EQUAL(lhs,"acceptallunknownwordsascorrect(yes,no)") OR
         EQUAL(lhs,"fileofwordsexcludedbyuser")              OR
/*----- added on 04/06/97 -----*/
/*                             */ 
        EQUAL(lhs,"maxnumberofalternativewords(1,2,3)")  OR
/*                             */
/*-----------------------------*/
        EQUAL(lhs,"userhistoryfile"))                               {
      fgets(line, 120, params);
      continue;
    }
    
    printf("\nRecord in file params.dat cannot be interpreted:\n\n");
    printf("    %s\n\n", line);
    printf("Please check the params.dat file and run Laempel again.\n\n");
    
    printf("Press return key to quit Laempel\n");
    ch = getchar();
    exit(0);
  } /* end of while */

  if (display) {
    printf("\nParameters used (in file params.dat):\n");
    printf("directory of Laempel/Data  = %s\n", data_path);
    printf("user dictionary            = %s\n", wds_usr);
    printf("user dictionary backup     = %s\n", wds_bak);
    printf("user phrase pattern        = %s\n", triple_usr);
    printf("user phrase pattern backup = %s\n", triple_bak);
    if (system_status == FindIntprt) {
      printf("system will find syntactical interpretation.\n");
    }
    else {
      printf("system will find syntactical errors only.\n");
    }
    if (display_syntx_msg) {
      printf("system will do interactive syntax checking.\n");
    }
    else {
      printf("system will do batch syntax checking.\n");
    }
  }
  fclose(params);
}

  
/*----------------------------------------------------------------------
  get_lhs_rhs  --  get lhs and rhs of a line in params.dat
 
    input  -- line: a line in params.dat
    output -- lhs: compressed lhs
              rhs: rhs

-----------------------------------------------------------------------*/
void get_lhs_rhs(line, lhs, rhs)
char *line, *lhs, *rhs;
{
  char    ch;
  int     i, l = 0, r = 0;
  boolean right = FALSE, done = FALSE;

  for (i = 0; i < 120; i++) {
    ch = line[i];
    switch (ch)
    {
      case ' ':
                break;
  
      case '=':
                right = TRUE;
                break;

      case '\n':
                 done = TRUE;
                 break;

      default:
               if (right) { rhs[r++] = ch; }
               else { lhs[l++] = ch; }
    }

    if (done) break; 
  }

  lhs[l] = '\0';
  rhs[r] = '\0';
}


/*-----------------------------------------------------------------------

  input_text_name  --  ask user to input the name of the text file

    input  -- none;
    output -- function returns TRUE if user type a file name, FALSE if 
              user type 'q'.

-----------------------------------------------------------------------*/
boolean input_text_name()
{
  char name[120];
  
  /**%%%default_text_name%%%%%%%**/
  char    ch, temp_name[120];
  boolean get_default_text_name();
  /**%%%end_default_text_name%%%**/
  
  /**%%%default_text_name%%%* 
  if (EQUAL(old_text_name,"")) {
    printf("\nName of text file ('q' to quit).\n");
  }
  else {
    printf("\nName of text file ('!!' for most recent name,\n");
    printf("                   'q' to quit).\n");
  }
  printf("\nSYNTAX >> ");
   *%%%end_default_text_name%%%**/
  
  /**%%%default_text_name%%%%%%%**/ 
   if ( get_default_text_name() ) 
     {   printf(
        "\nName of text file:\n");
        printf("   Press return for '%s'"
              " (=most recent name)\n",
                default_text_name);
        printf("   'q' to quit\n");
      }
      else
      {
         printf("\nName of text file ('q' to quit)\n");
      };
      
    printf("\nSYNTAX >> ");
    fflush(stdin);
    ch = getchar();
    if (ch == '\n') {  /*** use the default name ***/
                       strcpy(text_name,default_text_name);
                       return TRUE;
                     }
      else {  int   i;
               
              text_name[0] = ch;     
              i=1;
              do{
                  ch=getchar();
                 text_name[i++] = ch;
                } while( ch != '\n' );
              text_name[--i] = '\0';
              if( strcmp(text_name,"q") == 0 ) return FALSE;
              return TRUE;
            }
   /**%%%end_default_text_name%%%**/
  
  scanf("%s", name);
  getchar();

  if EQUAL(name,"q") return FALSE;

  if EQUAL(name,"!!") {
    strcpy(text_name, old_text_name);
    return TRUE;
  }
  strcpy(old_text_name, name);
  strcpy(text_name, name);
  return TRUE;
}



/*----------------------------------------------------------------------

  print_title  --  print the title of the laempel system

    input  -- none
    output -- none

-----------------------------------------------------------------------*/
void print_title()
{
  printf("                   Laempel System -- Syntax Checking\n");
  printf("                              Version 7.0\n\n");
  printf(" Copyright: Leibniz, 1990, 1999,  University of Texas at Dallas, 1999\n");
  printf("                    Richardson,  Texas 75083,  USA\n\n\n");
  printf(" The expert systems contained in this software have been created with\n");
  printf(" the Leibniz System for logic programming.\n\n");
  printf(" Implementation  of this  software has been  supported in part by the\n");
  printf(" Office of Naval Research under grant N00014-93-1-0096. This software\n");
  printf(" is licensed free of charge in source code form.\n\n\n");
}


/*---------------------------------------------------------------------

  system_error  --  report system error

    input  -- msg: error message
    output -- none

----------------------------------------------------------------------*/
void system_error(msg)
char *msg;
{
  char ch;

  printf("------------------------------------------------------------\n");
  printf("\nLaempel System Error: ");
  printf("%s\n\n", msg);
  printf("------------------------------------------------------------\n");
  printf("Press return key to quit Laempel\n");

  ch = getchar();
  exit(0);
}


/*---------------------------------------------------------------------

  running_error  --  report running error

    input  -- msg: error message
    output -- none

----------------------------------------------------------------------*/
void running_error(msg)
char *msg;
{
  char ch;

  printf("------------------------------------------------------------\n");
  printf("\nError: ");
  printf("%s\n\n", msg);
  printf("------------------------------------------------------------\n");
  printf("Press return key to quit Laempel\n");

  ch = getchar();
  exit(0);
}



/**%%%default_text_name%%%%%%%**/

/*---------------------------------------------------------------------

  get_default_text_name  --  read default text name from disk file

    input  -- none
    output -- none

----------------------------------------------------------------------*/
boolean  get_default_text_name()
{
 FILE     *dfp;
 
 
 if( (dfp=fopen(filename_storage,"r")) == NULL )
     {
        return FALSE;
      
       }
 
 fscanf(dfp,"%s",default_text_name);
 fclose(dfp);
 if( strcmp(default_text_name,"") == 0 ) return FALSE;
 return TRUE;

}


/*---------------------------------------------------------------------

  save_default_text_name()  --  save text name in disk file to be used 
                                 as default text name.

    input  -- none
    output -- none

----------------------------------------------------------------------*/
void  save_default_text_name()
{
 FILE     *dfp;
 
 
 if( (dfp=fopen(filename_storage,"w")) == NULL )
     {
        printf("\n Error writing file %s.\n",filename_storage);
        return;
      
       }
 
 fprintf(dfp,"%s",text_name);
 fclose(dfp);
 return;

}
/*----------------------------------------------------------------------*/

/**%%%end_default_text_name%%%**/



/**@@@@@@@7/99@@@@@@@@@@@@@@**/

/*--------------------------------------------------------------------
  
  get_word_pos  --  get patterns(pos) of a word.

    input  -- stc:  a sentence
              len:  sentence length
              idx:  the index of the word in  the sentence
    output -- word: word's pattern.

---------------------------------------------------------------------*/
void get_word_pos(stc, len, idx, wordpos)
struct Syntx_Term *stc;
int    len;
int    idx;
char   *wordpos;
{
  int    i, ptrn_len, lkhd[MAXPTRNS];
  
  strcpy(wordpos, ""); 
  if ((idx < 0) OR (idx >= len)) return;
  if (EQUAL(stc[idx].phrase,""))   
    strcpy(wordpos, user_dict[original_sentence[stc[idx].begin]].ptrns);

   ptrn_len = strlen(wordpos);
    
  /*** compute patterns(pos) likelihoods ***/
  for(i=0; i<ptrn_len; i++)
     lkhd[i] = user_dict[original_sentence[stc[idx].begin]].ptrns_likelihood[i];

  for(i=0; i<ptrn_len; i++)
    if( lkhd[i] >  99 ) 
          {  wordpos[0] = user_dict[original_sentence[stc[idx].begin]].ptrns[i];
             return;
           }
   wordpos[0] = '/';
}

/*--------------------------------------------------------------------
  
  adj_adv_noun  --  checks if a word can be at least two of 
                     adjective, adverb, or noun.

    input  -- stc:  a sentence
              len:  sentence length
              idx:  the index of the word in  the sentence
    output -- word: TRUE/FALSE accordingly.

---------------------------------------------------------------------*/
boolean adj_adv_noun(stc, len, idx)
struct Syntx_Term *stc;
int    len;
int    idx;

{
  int    i, ptrn_len, lkhd[MAXPTRNS];
  char   wordpos[4];
  
  strcpy(wordpos, ""); 
  if ((idx < 0) OR (idx >= len)) return(FALSE);
  if (EQUAL(stc[idx].phrase,""))   
    strcpy(wordpos, user_dict[original_sentence[stc[idx].begin]].ptrns);

   ptrn_len = strlen(wordpos);
   
   if( ptrn_len > 3 )  return(FALSE);
   
   for(i=0; i<ptrn_len; i++)
     if( (wordpos[i] != 'a') &&
         (wordpos[i] != 'y') &&
         (wordpos[i] != 'n') &&
         (wordpos[i] != 'N')     )  return(FALSE);
   
   return(TRUE);
     
}

/*******************************************************
 *
 * word_check(): checks if word is a special string that
 * has been used with underscore in .log files. The underscore
 * is needed for words that considered reserved in .log files
 * return: 1 if word is special
 *         0 otherwise
 *******************************************************/
int word_check(char* word) {

  if ((EQUAL(word,"not")) ||
      (EQUAL(word,"at")) ||
      (EQUAL(word,"in")) ||
      (EQUAL(word,"always")) ||
      (EQUAL(word,"time")) ||
      (EQUAL(word,"the")) ||
      (EQUAL(word,"and")) ||
      (EQUAL(word,"or")) ||
      (EQUAL(word,"there")) ||
      (EQUAL(word,"very"))) {
    return 1;
  } else {
    return 0;
  }
}

/**@@@@@@@end 7/99@@@@@@@@@@@@@@**/
