#include "syntx.h"

#define        NumofToDgns     9
#define        NumofToNext     3

extern struct Syntx_Term *sentence; 
extern int                sentence_len;
extern struct SPOC_Pos    sentence_spoc;
extern boolean            intrnl_dsply;



DIAGNOSIS_ARRAY to_dgns[NumofToDgns + 1] = { 
                                             "cmb_rht_one",
                                             "cmb_rht_two",
                                             "next(find_sub)",
                                             "next(sub_mdf)",
                                             "next(find_mdf)",
                                             "to_phrs(to_be)",
                                             "to_phrs(to_lvb)",
                                             "to_phrs(to_vi)",
                                             "to_phrs(to_vt)",
                                             "" 
                                           };   

static int        symptoms_idx;
SYMPTOM_ARRAY     symptoms[mmax(3*MAXPTRNS+2,MAXSYMP)];
  
void    to_phrase();
int     do_toP();
boolean toP_cntwd();
boolean toP_rht1ptrn();
boolean toP_rht2ptrn();
boolean toP_cmb_and_next();


/*----------------------------------------------------------------------

  to_phrase  --  control the action of do_toP

    input  -- pos:  the position where the action to be taken
              lklhd: the likelyhood at which the action will take
              who: indicate who invoke the function
    output -- none
 
-----------------------------------------------------------------------*/
void to_phrase(pos, lklhd, who)
int pos;
int lklhd;
int who;
{
  int toP_code, to_pos;

  if (intrnl_dsply) {
    begin_title("to_phrase", pos, lklhd);
  }

  load_prb("ToPhrase");

  if ((toP_code = do_toP(pos, lklhd, who, &to_pos)) == 0) {
    if (intrnl_dsply) {
      printf("no to phrases can be formed\n");
    }
    goto end;
  }

  switch (toP_code) {
    case 1:                                                     /* to_be */
    case 2:                                                    /* to_lvb */
    case 3:                                                     /* to_vi */
    case 4:                                                     /* to_vt */
            break;

    default: stx_err("unexpected toP_code", __LINE__, __FILE__);
  }
  
end: 
  if (intrnl_dsply) {
    end_title("to_phrase");
  }
}


/*---------------------------------------------------------------------

  do_toP  --  execute do_toP.prg  

    input  -- pos: the position where the do_toP works on
              lklhd: the likelihood the do_toP works at
              who:   who issued to_phrase
    output -- to_pos: the position of the formed phrase
 
  function returns  0: no to phrase can be formed
                    1: to_be is formed
                    2: to_lvb is formed
                    3: to_vi is formed 
                    4: to_vt is formed

----------------------------------------------------------------------*/
int do_toP(pos, lklhd, who, to_pos)
int pos, lklhd, who, *to_pos;
{
  DIAGNOSIS_ARRAY   dgns[NumofToDgns + 1];
  char              dgns_var[VAR_MAX + 1],  ptrn[PTRN_PHRS_MAX+1],
                    ptrn_names[MAXPTRNS+1][PTRN_PHRS_MAX+1],
                    used_ptrn[3][PTRN_PHRS_MAX+1] = {"", "", ""},
                    word[WORD_MAX+1];
  int               i, cnt, rht1, rht2, lh, lklhds[MAXPTRNS], return_val=0;
  boolean           next[NumofToNext];

  symptoms_idx = 0;
  cnt = pos;
  rht1 = pos + 1;
  rht2 = pos + 2;
  for (i = 0; i < NumofToNext; i++) next[i] = FALSE;

  switch(who) {
    case FindSubject: 
                        strcpy(symptoms[symptoms_idx++], "who(find_sub)");
                        break;

    case FindObject:
                        strcpy(symptoms[symptoms_idx++], "who(find_obj)");
                        break;
  
    case FindModifier:
                        strcpy(symptoms[symptoms_idx++], "who(find_mdf)");
                        break;

    default: stx_err("unexpected problem code", __LINE__, __FILE__);
  }
    

  get_word(sentence, sentence_len, cnt, word);
  if (!EQUAL(word,"")) {
    if (toP_cntwd(word)) {
      if (word_check(word)==0) {
        sprintf(symptoms[symptoms_idx++], "cnt_wd(%s)", word);
      } else {
        sprintf(symptoms[symptoms_idx++], "cnt_wd(%s_)", word);
      }
      strcpy(used_ptrn[0], "prep");
    }
  }

  get_ptrn_names(sentence, sentence_len, rht1, ptrn_names, lklhds);
  i = 0;
  strcpy(ptrn, ptrn_names[i++]);
  while (!EQUAL(ptrn,"")) {
    if (toP_rht1ptrn(ptrn)) {
      sprintf(symptoms[symptoms_idx++], "rht1_ptn(%s)", ptrn);
      if (EQUAL(used_ptrn[1],"")) strcpy(used_ptrn[1], ptrn);
    }
    strcpy(ptrn, ptrn_names[i++]);
  }

  get_ptrn_names(sentence, sentence_len, rht2, ptrn_names, lklhds);
  i = 0;
  strcpy(ptrn, ptrn_names[i++]);
  while (!EQUAL(ptrn,"")) {
    if (toP_rht2ptrn(ptrn)) {
      sprintf(symptoms[symptoms_idx++], "rht2_ptn(%s)", ptrn);
      if (EQUAL(used_ptrn[2],"")) strcpy(used_ptrn[2], ptrn);
    }
    strcpy(ptrn, ptrn_names[i++]);
  }

  strcpy(symptoms[symptoms_idx],"");

  mst_smp(symptoms, name, state, type, value, errcde);
  sol_prb(name, state, type, value, errcde);
  if (errcde[0] != 0) {
    stx_err("failed to solve problem", __LINE__, __FILE__);
  }

  if EQUAL(state, "S") { 
    copy_dgns(dgns, to_dgns);
    get_dgn(dgns,name,state,type,value,errcde);
    if (value[0]) {                      /* has at least one true value */
      boolean cmb_rht_one = FALSE, cmb_rht_two = FALSE;
      char    clss[MAXPHRASE][PTRN_PHRS_MAX+1]; 
      struct  Syntx_Term *stc; 
      int     stc_len;
      struct  SPOC_Pos stc_spoc;

      i = 0;
      strcpy(clss[0], used_ptrn[0]);
      strcpy(clss[1], used_ptrn[1]);
      strcpy(clss[2], used_ptrn[2]);

      while (! EQUAL(dgns[i],"")) {
        strcpy(dgns_var, dgns[i++]);

        if (EQUAL(dgns_var,"cmb_rht_one")) { 
          cmb_rht_one = TRUE; 
          continue; 
        }
        if (EQUAL(dgns_var,"cmb_rht_two")) { 
          cmb_rht_two = TRUE; 
          continue; 
        }

        if (EQUAL(dgns_var,"next(find_sub)")) { 
          next[0] = TRUE; 
          continue; 
        }
        if (EQUAL(dgns_var,"next(sub_mdf)")) { 
          next[1] = TRUE; 
          continue; 
        }
        if (EQUAL(dgns_var,"next(find_mdf)")) { 
          next[2] = TRUE; 
          continue; 
        }

        
        if (EQUAL(dgns_var,"to_phrs(to_be)")) { 
          stc = (struct Syntx_Term *)stc_cpy(&stc_len, &stc_spoc,
                                   sentence, sentence_len, &sentence_spoc);
          if (toP_cmb_and_next(stc, &stc_len,&stc_spoc,cnt,clss,"to_be",
                               lklhd, cmb_rht_one, cmb_rht_two, next)) 
            return_val = 1;
          stc_dsp(stc);
          continue;
        }
        if (EQUAL(dgns_var,"to_phrs(to_lvb)")) { 
          stc = (struct Syntx_Term *)stc_cpy(&stc_len, &stc_spoc,
                                   sentence, sentence_len, &sentence_spoc);
          if (toP_cmb_and_next(stc, &stc_len,&stc_spoc,cnt, clss,"to_lvb",
                               lklhd, cmb_rht_one, cmb_rht_two, next)) 
            return_val = 2;
          stc_dsp(stc);
          continue;
        }
        if (EQUAL(dgns_var,"to_phrs(to_vi)")) { 
          stc = (struct Syntx_Term *)stc_cpy(&stc_len, &stc_spoc,
                                   sentence, sentence_len, &sentence_spoc);
          if (toP_cmb_and_next(stc, &stc_len, &stc_spoc,cnt, clss,"to_vi",
                               lklhd, cmb_rht_one, cmb_rht_two, next)) 
            return_val = 3;
          stc_dsp(stc);
          continue;
        }
        if (EQUAL(dgns_var,"to_phrs(to_vt)")) { 
          stc = (struct Syntx_Term *)stc_cpy(&stc_len, &stc_spoc,
                                   sentence, sentence_len, &sentence_spoc);
          if (toP_cmb_and_next(stc, &stc_len, &stc_spoc,cnt, clss,"to_vt",
                               lklhd, cmb_rht_one, cmb_rht_two, next)) 
            return_val = 4;
          stc_dsp(stc);
          continue;
        }
      }
  
      *to_pos = cnt;
    }
  }
  else stx_err("problem is unsatisfied", __LINE__, __FILE__);
  

  strcpy(state, "A");
  rst_prb(name, state, type, value, errcde);
  return return_val;
}


/*---------------------------------------------------------------------

  toP_cntwd  --  check if current word is considered in the log file

    input  -- word: current word to be checked

  function returns TRUE if word is considered in log file, FALSE otherwise

------------------------------------------------------------------------*/
boolean toP_cntwd(word)
char *word;
{
  int  i, len;
  char ch;

  len = strlen(word);
  for (i = 0; i < len; i++) {
    if ((ch = word[i]) == '\'') {
      word[i] = '_';
    }
    else word[i] = tolower(ch);
  }

  if (EQUAL(word,"to"))   return TRUE; 

  return FALSE;
}

/*---------------------------------------------------------------------

  toP_rht1ptrn  --  check if right1 pattern is considered in the log file

    input  -- ptrn: right1 pattern name to be checked
    output -- none

  function returns TRUE if ptrn is considered in log file, FALSE otherwise

------------------------------------------------------------------------*/
boolean toP_rht1ptrn(ptrn)
char *ptrn;
{
  if (EQUAL(ptrn,"vt_pl")  OR
      EQUAL(ptrn,"vi_pl")  OR
      EQUAL(ptrn,"be")     OR
      EQUAL(ptrn,"lvb_pl"))     return TRUE;

  return FALSE;
}

/*---------------------------------------------------------------------

  toP_rht2ptrn  --  check if right2 pattern is considered in the log file

    input  -- ptrn: right2 pattern name to be checked
    output -- none

  function returns TRUE if ptrn is considered in log file, FALSE otherwise

------------------------------------------------------------------------*/
boolean toP_rht2ptrn(ptrn)
char *ptrn;
{
  if (EQUAL(ptrn,"vt_pprt") OR
      EQUAL(ptrn,"vt_ing")  OR
      EQUAL(ptrn,"vi_ing")  OR
      EQUAL(ptrn,"lvb_ing"))       return TRUE;

  return FALSE;
}

/*---------------------------------------------------------------------

  toP_cmb_and_next  --  combine tokens and issue next command

    input  -- stc:    sentence
              slen:   sentence length
              spoc:   positions of SPOC;
              cnt:    current position
              clss:   the classifications of the tokens to be combined
              phrase: the name of the combined phrase
              lh:     likelihood
              cmb1:   indicate combining cnt and right1
              cmb2:   indicate combining cnt, right1 and right2
              next:   indicate the next command to be executed

    output -- function return TRUE if some to phrase can be formed,
                              FALSE otherwise.

------------------------------------------------------------------------*/
boolean toP_cmb_and_next(stc,slen,spoc,cnt,clss,phrase,lh,cmb1,cmb2,next)
struct  Syntx_Term *stc; 
int     *slen;
struct  SPOC_Pos *spoc;
int     cnt;
char    clss[MAXPHRASE][PTRN_PHRS_MAX+1]; 
char    *phrase;
int     lh;
boolean cmb1;
boolean cmb2;
boolean next[NumofToNext];
{
  
  if (!cmb1 AND !cmb2) return FALSE;

  if (cmb1) { 
    if (intrnl_dsply) {
      printf("%d + %d  ==>  phrase '%s'\n\n", cnt, cnt+1, phrase);
    }    
   
    cmb_token(stc, slen, spoc, cnt, cnt+1, clss, phrase, lh);
  }

  if (cmb2) { 
    if (intrnl_dsply) {
      printf("%d + %d + %d  ==>  phrase '%s'\n\n",cnt,cnt+1,cnt+2, phrase);
    }
  
    cmb_token(stc, slen, spoc, cnt, cnt+2, clss, phrase, lh);
  }

  if (next[0]) {                                  /* next[0] = find_sub */
    spoc->obj_pos = cnt;
    stc_push(stc, *slen, spoc, FindSubject, spoc->prd_pos-1, lh, ToPhrase);
    return TRUE;
  }

  if (next[1]) {                                    /* next[1] = sub_mdf */
    spoc->sub_pos = cnt;
    stc_push(stc, *slen, spoc, FindModifier, 0, lh, ToPhrase);
    return TRUE;
  }
  
  if (next[2]) {                                   /* next[2] = find_mdf */
    stc_push(stc, *slen, spoc, FindModifier, cnt, lh, ToPhrase);
    return TRUE;
  }

  return FALSE;
}

