#include "syntx.h"


void    get_ptrn_names();
void    get_phrs_name();
int     pronoun_info();
boolean has_adjpron();
boolean has_adjpron_only();
boolean has_subpron();
boolean has_objpron();
boolean has_subjects();
boolean has_objects();
boolean has_subobjects();
boolean has_verb_only();
boolean has_vi_only();
boolean has_vt();
boolean has_vi();
boolean has_vb_flwto();
boolean has_past_vt();
boolean has_past_vi();
boolean has_be();
boolean has_do();
boolean has_have();
boolean has_aux();
boolean has_pronvb();
boolean has_lnkvb();
boolean has_adv_advP_only();
boolean has_article();
boolean has_prep();
boolean has_class();
boolean has_pattern();
boolean has_superative();
boolean has_cmpsup_only();
boolean has_prep_only();
boolean has_sgl3rdvb_only();
boolean has_pst_only();
boolean has_pst_pprt_only();
boolean has_adjadv_only();
boolean has_prdct();
boolean is_sub_only();


int     find_verb_pos();
void    find_noun_pos();
void    find_sub_pos();
void    find_obj_pos();


int     keep_noun_adjv();
int     keep_article();
int     keep_all_vtvi();
int     keep_be();
int     keep_do();
int     keep_have();
int     keep_aux();
int     keep_lnkvb();
int     keep_ith_class();
int     keep_adjlike();
int     keep_adv();
int     keep_prep();

int     drop_vt();
int     drop_vi();
int     drop_verb();
int     drop_be();
int     drop_do();
int     drop_have();
int     drop_aux();
int     drop_all_vb();
int     drop_noun();
int     drop_plural();
int     drop_noun_plural();
int     drop_adj();
int     drop_lnkvb();
int     drop_mdfr();
int     drop_non_flwto();

int     ptrn_length();

void    cmb_token();

int     lhd_cmp();

boolean is_word();
boolean has_phrase();
boolean left_stop_noun();

boolean has_two_fixed_prdcts();

/*boolean left_stop_adj();*/




/*----------------------------------------------------------------------

  get_ptrn_names  --  get fullnames of patterns for a syntax token

    input  -- stc:  a sentence
              slen: sentence length
              idx:  token index
    output -- names: fullnames
              lklhd: likelihoods of patterns

-----------------------------------------------------------------------*/
void get_ptrn_names(stc, slen, idx, names, lklhd)
struct Syntx_Term *stc;
int    slen;
int    idx;
char   names[MAXPTRNS+1][PTRN_PHRS_MAX+1];
int    lklhd[MAXPTRNS];
{ 
  int  i, len;
  char pt[MAXPTRNS+1];
  
  if ((idx < 0) OR (idx >= slen)) {
    strcpy(names[0], "");
    return;
  }
  if (stc[idx].phrase[0] != '\0') { strcpy(names[0], ""); return; }

  strcpy(pt, stc[idx].ptrns);
  len = strlen(pt);
  for (i = 0; i < len; i++) {
    class_name(stc, slen, idx, pt[i], names[i]);
    lklhd[i] = stc[idx].ptrns_lh[i];
  }
  strcpy(names[i], "");
}

/*----------------------------------------------------------------------

  get_phrs_name  --  get phrase name of a syntax token

    input  -- stc:  a sentence
              slen: sentence length
              idx:  token index
    output -- name: phrase name
              lklhd: likelihoods of phrase

-----------------------------------------------------------------------*/
void get_phrs_name(stc, slen, idx, name, lklhd)
struct Syntx_Term *stc;
int    slen;
int    idx;
char   *name;
int    *lklhd;
{ 
  int  i, len;
  char pt[MAXPTRNS+1];
  
  if ((idx < 0) OR (idx >= slen)) {
    strcpy(name, "");
    return;
  }

  strcpy(name, stc[idx].phrase);
  *lklhd = stc[idx].phrase_lh;
}


/*----------------------------------------------------------------------

  pronoun_info -- return corresponding information about a pronoun

    input  -- stc:  a sentence
              slen:  sentence length
              idx:  token index
    output -- used_ptrn: the name of the pattern;
              lklhd: the likelihood of the pattern

  function returns 0: no pronoun patterns
                   1: pronoun I, he, she, we or they
                   2: pronoun me, him, her, us or them
                   3: pronoun it, this, that, one, anybody, another, 
                              everybody, anyone, nobody, everyone,someone, 
                              somebody, something, anything, nothing, or
                              everything
                   4: pronoun these or those
                   5: pronoun you
                   6: pronoun mine, ours, yours, his, hers or theirs
                   7: pronoun myself, himself, herself, itself, ourselves,
                              yourself, yourselves or themselves
                   8: pronoun there

-----------------------------------------------------------------------*/
int pronoun_info(stc, slen, idx, used_ptrn, lklhd)
struct Syntx_Term *stc;
int    slen;
int    idx;
char   *used_ptrn;
int    *lklhd;
{
  char word[WORD_MAX + 1], ptrns[MAXPTRNS+1], pt; 
  int  len, i;

  if ((idx < 0) OR (idx >= slen)) return 0;
  if (stc[idx].phrase[0] != '\0') return 0;

  strcpy(ptrns, stc[idx].ptrns);
  len = strlen(ptrns);
  for (i = 0; i < len; i++) {
    pt = ptrns[i];
    if (pt == 's') { 
      *lklhd = stc[idx].ptrns_lh[i]; 
      strcpy(used_ptrn, "pron_sgl");
      break;
    }
    if (pt == 'S') { 
      *lklhd = stc[idx].ptrns_lh[i]; 
      strcpy(used_ptrn, "pron_pl");
      break;
    }
    if (pt == 'r') { 
      *lklhd = stc[idx].ptrns_lh[i]; 
      strcpy(used_ptrn, "pron");
      break;
    }
  }
  if (i == len) return 0;

  get_word(stc, slen, idx, word);
  len = strlen(word);
  for (i = 0; i < len; i++) word[i] = tolower(word[i]);


  if (EQUAL(word,"you")) return 5;

  if (EQUAL(word,"there")) return 8;

  if (EQUAL(word,"these") OR
      EQUAL(word,"those"))   return 4;

  if (EQUAL(word,"i")     OR
      EQUAL(word,"he")    OR
      EQUAL(word,"she")   OR
      EQUAL(word,"we")    OR
      EQUAL(word,"they"))    return 1;

  if (EQUAL(word,"me")    OR
      EQUAL(word,"him")   OR
      EQUAL(word,"her")   OR
      EQUAL(word,"us")    OR
      EQUAL(word,"them"))    return 2;

  if (EQUAL(word,"mine")   OR
      EQUAL(word,"ours")   OR
      EQUAL(word,"yours")  OR
      EQUAL(word,"his")    OR
      EQUAL(word,"hers")   OR
      EQUAL(word,"theirs"))   return 6;

  if (EQUAL(word,"myself")      OR
      EQUAL(word,"himself")     OR
      EQUAL(word,"herself")     OR
      EQUAL(word,"itself")      OR
      EQUAL(word,"yourself")    OR
      EQUAL(word,"ourselves")   OR
      EQUAL(word,"yourselves")  OR
      EQUAL(word,"themselves"))    return 7;

  if (EQUAL(word,"it")        OR
      EQUAL(word,"this")      OR
      EQUAL(word,"that")      OR
      EQUAL(word,"one")       OR
      EQUAL(word,"anyone")    OR
      EQUAL(word,"someone")   OR
      EQUAL(word,"everyone")  OR
      EQUAL(word,"anybody")   OR
      EQUAL(word,"everybody") OR
      EQUAL(word,"nobody")    OR
      EQUAL(word,"somebody")  OR
      EQUAL(word,"another")   OR
      EQUAL(word,"something") OR
      EQUAL(word,"anything")  OR
      EQUAL(word,"nothing")   OR
      EQUAL(word,"everything"))     return 3;

  return 0;
}


/*----------------------------------------------------------------------
 
  has_adjpron -- check if a syntax token has relative prnoun or adjective
                 form of possessive pronoun

    input  -- stc:  a sentence
              slen:  sentence length
              idx:  token index
    output -- lklhd: the likelihood of the pattern
              function returns,
                TRUE:  has such patterns
                FALSE: otherwise

-----------------------------------------------------------------------*/
boolean has_adjpron(stc, slen, idx, lklhd)
struct Syntx_Term *stc;
int    slen;
int    idx;
int    *lklhd;
{
  char word[WORD_MAX + 1]; 
  int  len, i;

  if ((idx < 0) OR (idx >= slen)) return FALSE;
  if (stc[idx].phrase[0] != '\0') return FALSE;

  get_word(stc, slen, idx, word);
  len = strlen(word);
  for (i = 0; i < len; i++) word[i] = tolower(word[i]);
  if (EQUAL(word,"this")  OR
      EQUAL(word,"that")  OR
      EQUAL(word,"these") OR
      EQUAL(word,"those") OR
      EQUAL(word,"its")   OR
      EQUAL(word,"his")   OR
      EQUAL(word,"her")   OR
      EQUAL(word,"their") OR
      EQUAL(word,"my")    OR
      EQUAL(word,"our")   OR
      EQUAL(word,"your"))    return TRUE;

  return FALSE;
}

/*----------------------------------------------------------------------
 
  has_adjpron-only -- check if a syntax token has only adjective form of
                      possessive pronoun 

    input  -- stc:  a sentence
              slen:  sentence length
              idx:  token index
    output -- lklhd: the likelihood of the pattern
              function returns,
                TRUE:  has such patterns
                FALSE: otherwise

-----------------------------------------------------------------------*/
boolean has_adjpron_only(stc, slen, idx, lklhd)
struct Syntx_Term *stc;
int    slen;
int    idx;
int    *lklhd;
{
  char word[WORD_MAX + 1]; 
  int  len, i;

  if ((idx < 0) OR (idx >= slen)) return FALSE;
  if (stc[idx].phrase[0] != '\0') return FALSE;

  get_word(stc, slen, idx, word);
  len = strlen(word);
  for (i = 0; i < len; i++) word[i] = tolower(word[i]);
  if (EQUAL(word,"her")   OR
      EQUAL(word,"their") OR
      EQUAL(word,"my")    OR
      EQUAL(word,"our")   OR
      EQUAL(word,"your"))    return TRUE;

  return FALSE;
}

/*----------------------------------------------------------------------
 
  has_subpron -- check if a syntax token has pronoun which can be used as
                 a subject

    input  -- stc:  a sentence
              slen: sentence length
              idx:  token index
    output -- none
              function returns,
                TRUE:  has such patterns
                FALSE: otherwise

-----------------------------------------------------------------------*/
boolean has_subpron(stc, slen, idx)
struct Syntx_Term *stc;
int    slen;
int    idx;
{
  char ptrns[MAXPTRNS+1], pt; 
  int  len, i;

  if ((idx < 0) OR (idx >= slen)) return FALSE;
  if (stc[idx].phrase[0] != '\0') return FALSE;

  strcpy(ptrns, stc[idx].ptrns);
  len = strlen(ptrns);
  for (i = 0; i < len; i++) {
    pt = ptrns[i];
    if (pt == 'r') {              /* pron     */
      char word[WORD_MAX+1];
      int  l, j;
  
      get_word(stc, slen, idx, word);
      l = strlen(word);
      for (j = 0; j < l; j++) word[j] = tolower(word[j]);
      if (!(EQUAL(word,"my"))     AND
          !(EQUAL(word,"our"))    AND
          !(EQUAL(word,"your"))   AND
          !(EQUAL(word,"its"))    AND
          !(EQUAL(word,"their")))    return TRUE;
    }

    if ((pt == 's') OR            /* pron_sgl */
        (pt == 'S')) {            /* pron_pl  */
      char word[WORD_MAX+1];
      int  l, j;
  
      get_word(stc, slen, idx, word);
      l = strlen(word);
      for (j = 0; j < l; j++) word[j] = tolower(word[j]);
      if (!(EQUAL(word,"me"))    AND
          !(EQUAL(word,"us"))    AND
          !(EQUAL(word,"him"))   AND
          !(EQUAL(word,"her"))   AND
          !(EQUAL(word,"them")))    return TRUE;
    }
  }
  return FALSE;
}




/*----------------------------------------------------------------------
 
  has_objpron -- check if a syntax token has a pronoun which can be used 
                 as a object 

    input  -- stc:  a sentence
              slen:  sentence length
              idx:  token index
    output -- none
              function returns,
                TRUE:  has such patterns
                FALSE: otherwise

-----------------------------------------------------------------------*/
boolean has_objpron(stc, slen, idx)
struct Syntx_Term *stc;
int    slen;
int    idx;
{
  char ptrns[MAXPTRNS+1], pt; 
  int  len, i;

  if ((idx < 0) OR (idx >= slen)) return FALSE;
  if (stc[idx].phrase[0] != '\0') return FALSE;

  strcpy(ptrns, stc[idx].ptrns);
  len = strlen(ptrns);
  for (i = 0; i < len; i++) {
    pt = ptrns[i];
    if (pt == 'r') {              /* pron     */
      char word[WORD_MAX+1];
      int  l, j;
  
      get_word(stc, slen, idx, word);
      l = strlen(word);
      for (j = 0; j < l; j++) word[j] = tolower(word[j]);
      if (!(EQUAL(word,"my"))     AND
          !(EQUAL(word,"our"))    AND
          !(EQUAL(word,"your"))   AND
          !(EQUAL(word,"its"))    AND
          !(EQUAL(word,"their")))    return TRUE;
    }

    if ((pt == 's') OR            /* pron_sgl */
        (pt == 'S')) {            /* pron_pl  */
      char word[WORD_MAX+1];
      int  l, j;
  
      get_word(stc, slen, idx, word);
      l = strlen(word);
      for (j = 0; j < l; j++) word[j] = tolower(word[j]);
      if (!(EQUAL(word,"i"))    AND
          !(EQUAL(word,"we"))   AND
          !(EQUAL(word,"he"))   AND
          !(EQUAL(word,"she"))  AND
          !(EQUAL(word,"they")))    return TRUE;
    }
  }
  return FALSE;
}



/*----------------------------------------------------------------------
 
  has_subjects -- check if a syntax token has patterns which can be used 
                  as subjects

    input  -- stc:  a sentence
              slen:  sentence length
              idx:  token index
    output -- none
              function returns,
                TRUE:  has such patterns
                FALSE: otherwise

-----------------------------------------------------------------------*/
boolean has_subjects(stc, slen, idx)
struct Syntx_Term *stc;
int    slen;
int    idx;
{
  char ptrns[MAXPTRNS+1], pt; 
  int  len, i;

  if ((idx < 0) OR (idx >= slen)) return FALSE;
  if (stc[idx].phrase[0] != '\0') return FALSE;
  if (has_prep_only(stc, slen, idx-1)) return FALSE;

  strcpy(ptrns, stc[idx].ptrns);
  len = strlen(ptrns);
  for (i = 0; i < len; i++) {
    pt = ptrns[i];
    if ((pt == 'n') OR            /* noun_sgl */
        (pt == 'o') OR            /* noun_sgl */
        (pt == 'N') OR            /* noun_pl  */
        (pt == 'g') OR            /* vt_ing   */
        (pt == 'G') OR            /* vi_ing   */
        (pt == 'r') OR            /* pron     */
        (pt == 'z'))   {          /* being    */
      return TRUE;
    }
    if ((pt == 's') OR            /* pron_sgl */
        (pt == 'S')) {            /* pron_pl  */
      char word[WORD_MAX+1];
      int  l, j;
  
      get_word(stc, slen, idx, word);
      l = strlen(word);
      for (j = 0; j < l; j++) word[j] = tolower(word[j]);
      if (!(EQUAL(word,"me"))   AND
          !(EQUAL(word,"us"))   AND
          !(EQUAL(word,"him"))  AND
          !(EQUAL(word,"her"))  AND
          !(EQUAL(word,"them")))    return TRUE;
    }
  }
  return FALSE;
}


/*----------------------------------------------------------------------
 
  has_objects -- check if a syntax token has patterns which can be used 
                 as objects

    input  -- stc:  a sentence
              slen:  sentence length
              idx:  token index
    output -- none
              function returns,
                TRUE:  has such patterns
                FALSE: otherwise

-----------------------------------------------------------------------*/
boolean has_objects(stc, slen, idx)
struct Syntx_Term *stc;
int    slen;
int    idx;
{
  char ptrns[MAXPTRNS+1], pt; 
  int  len, i;

  if ((idx < 0) OR (idx >= slen)) return FALSE;
  if (stc[idx].phrase[0] != '\0') return FALSE;
  if (has_prep_only(stc, slen, idx-1)) return FALSE;

  strcpy(ptrns, stc[idx].ptrns);
  len = strlen(ptrns);
  for (i = 0; i < len; i++) {
    pt = ptrns[i];
    if ((pt == 'n') OR            /* noun_sgl */
        (pt == 'o') OR            /* noun_sgl */
        (pt == 'N') OR            /* noun_pl  */
        (pt == 'g') OR            /* vt_ing   */
        (pt == 'G') OR            /* vi_ing   */
        (pt == 'r') OR            /* pron     */
        (pt == 'z'))   {          /* being    */
      return TRUE;
    }
    if ((pt == 's') OR            /* pron_sgl */
        (pt == 'S')) {            /* pron_pl  */
      char word[WORD_MAX+1];
      int  l, j;
  
      get_word(stc, slen, idx, word);
      l = strlen(word);
      for (j = 0; j < l; j++) word[j] = tolower(word[j]);
      if (!(EQUAL(word,"i"))    AND
          !(EQUAL(word,"we"))   AND
          !(EQUAL(word,"he"))   AND
          !(EQUAL(word,"she"))  AND
          !(EQUAL(word,"they")))    return TRUE;
    }
  }
  return FALSE;
}


/*----------------------------------------------------------------------
 
  has_subobjects -- check if a syntax token has patterns which can be used 
                    as subjects or objects

    input  -- stc:  a sentence
              slen:  sentence length
              idx:  token index
    output -- none
              function returns,
                TRUE:  has such patterns
                FALSE: otherwise

-----------------------------------------------------------------------*/
boolean has_subobjects(stc, slen, idx)
struct Syntx_Term *stc;
int    slen;
int    idx;
{
  char ptrns[MAXPTRNS+1], pt; 
  int  len, i;

  if ((idx < 0) OR (idx >= slen)) return FALSE;
  if (stc[idx].phrase[0] != '\0') return FALSE;

  strcpy(ptrns, stc[idx].ptrns);
  len = strlen(ptrns);
  for (i = 0; i < len; i++) {
    pt = ptrns[i];
    if ((pt == 'n') OR            /* noun_sgl */
        (pt == 'o') OR            /* noun_sgl */
        (pt == 'N') OR            /* noun_pl  */
        (pt == 'g') OR            /* vt_ing   */
        (pt == 'G') OR            /* vi_ing   */
        (pt == 's') OR            /* pron_sgl */
        (pt == 'S') OR            /* pron_pl  */
        (pt == 'r') OR            /* pron     */
        (pt == 'z'))   {          /* being    */
      return TRUE;
    }
  }
  return FALSE;
}


/*----------------------------------------------------------------------
 
  has_vt -- check if a syntax token has transitive verb patterns 

    input  -- stc:  a sentence
              slen:  sentence length
              idx:  token index
    output -- lklhd: the likelihood of the pattern
              function returns,
                TRUE:  has such patterns
                FALSE: otherwise

-----------------------------------------------------------------------*/
boolean has_vt(stc, slen, idx, lklhd)
struct Syntx_Term *stc;
int    slen;
int    idx;
int    *lklhd;
{
  char ptrns[MAXPTRNS+1], pt; 
  int  len, i;

  if ((idx < 0) OR (idx >= slen)) return FALSE;
  if (stc[idx].phrase[0] != '\0') return FALSE;

  strcpy(ptrns, stc[idx].ptrns);
  len = strlen(ptrns);
  for (i = 0; i < len; i++) {
    pt = ptrns[i];
    if ((pt == 't') OR            /* vt_pl  */
        (pt == 'u') OR            /* vt_pl  */
        (pt == 'T'))   {          /* vt_sgl */
      *lklhd = stc[idx].ptrns_lh[i];
      return TRUE;
    }
  }
  return FALSE;
}


/*----------------------------------------------------------------------
 
  has_vi -- check if a syntax token has intransitive verb patterns 

    input  -- stc:  a sentence
              slen:  sentence length
              idx:  token index
    output -- lklhd: the likelihood of the pattern
              function returns,
                TRUE:  has such patterns
                FALSE: otherwise

-----------------------------------------------------------------------*/
boolean has_vi(stc, slen, idx, lklhd)
struct Syntx_Term *stc;
int    slen;
int    idx;
int    *lklhd;
{
  char ptrns[MAXPTRNS+1], pt; 
  int  len, i;

  if ((idx < 0) OR (idx >= slen)) return FALSE;
  if (stc[idx].phrase[0] != '\0') return FALSE;

  strcpy(ptrns, stc[idx].ptrns);
  len = strlen(ptrns);
  for (i = 0; i < len; i++) {
    pt = ptrns[i];
    if ((pt == 'i') OR                                         /* vi_pl  */
        (pt == 'j') OR                                         /* vi_pl  */
        (pt == 'I') OR                                         /* vi_sgl */
        (pt == 'l') OR                                         /* lvb_pl */
        (pt == 'L'))   {                                      /* lvb_sgl */
      *lklhd = stc[idx].ptrns_lh[i];
      return TRUE;
    }
  }
  return FALSE;
}


/*----------------------------------------------------------------------
 
  has_vb_flwto -- check if a syntax token has verb patterns which are 
                  allowed to follow the to 

    input  -- stc:  a sentence
              slen:  sentence length
              idx:  token index
    output -- lklhd: the likelihood of the pattern
              function returns,
                TRUE:  has such patterns
                FALSE: otherwise

-----------------------------------------------------------------------*/
boolean has_vb_flwto(stc, slen, idx, lklhd)
struct Syntx_Term *stc;
int    slen;
int    idx;
int    *lklhd;
{
  char ptrns[MAXPTRNS+1], pt; 
  int  len, i;

  if ((idx < 0) OR (idx >= slen)) return FALSE;
  if (stc[idx].phrase[0] != '\0') return FALSE;

  strcpy(ptrns, stc[idx].ptrns);
  len = strlen(ptrns);
  for (i = 0; i < len; i++) {
    pt = ptrns[i];
    if ((pt == 'i') OR                                         /* vi_pl  */
        (pt == 'j') OR                                         /* vi_pl  */
        (pt == 't') OR                                         /* vt_pl  */
        (pt == 'u') OR                                         /* vt_pl  */
        (pt == 'w') OR                                         /* be     */
        (pt == 'l'))   {                                       /* lvb_pl */
      *lklhd = stc[idx].ptrns_lh[i];
      return TRUE;
    }
  }
  return FALSE;
}


/*----------------------------------------------------------------------
 
  has_verb_only -- check if a syntax token has verb patterns (vt or vi)
                   only 

    input  -- stc:  a sentence
              slen:  sentence length
              idx:  token index
    output -- none
              function returns,
                TRUE:  has such patterns
                FALSE: otherwise

-----------------------------------------------------------------------*/
boolean has_verb_only(stc, slen, idx)
struct Syntx_Term *stc;
int    slen;
int    idx;
{
  char ptrns[MAXPTRNS+1], pt; 
  int  len, i;

  if ((idx < 0) OR (idx >= slen)) return FALSE;
  if (stc[idx].phrase[0] != '\0') return FALSE;

  strcpy(ptrns, stc[idx].ptrns);
  len = strlen(ptrns);
  for (i = 0; i < len; i++) {
    pt = ptrns[i];
    if ((pt != 't') AND                                        /* vt_pl  */
        (pt != 'u') AND                                        /* vt_pl  */
        (pt != 'T') AND                                        /* vt_sgl */
        (pt != 'v') AND                                        /* vt_pst */
        (pt != 'i') AND                                        /* vi_pl  */
        (pt != 'j') AND                                        /* vi_pl  */
        (pt != 'I') AND                                        /* vi_sgl */
        (pt != 'k') AND                                        /* vi_pst */
        (pt != 'O') AND                                       /* lvb_pst */
        (pt != 'l') AND                                        /* lvb_pl */
        (pt != 'L'))   {                                      /* lvb_sgl */
      return FALSE;
    }
  }
  if (len == 0) return FALSE;
  return TRUE;
}


/*----------------------------------------------------------------------
 
  has_vi_only -- check if a syntax token has predicate vi patterns only

    input  -- stc:  a sentence
              slen:  sentence length
              idx:  token index
    output -- none
              function returns,
                TRUE:  has such patterns
                FALSE: otherwise

-----------------------------------------------------------------------*/
boolean has_vi_only(stc, slen, idx)
struct Syntx_Term *stc;
int    slen;
int    idx;
{
  char ptrns[MAXPTRNS+1], pt; 
  int  len, i;

  if ((idx < 0) OR (idx >= slen)) return FALSE;
  if (stc[idx].phrase[0] != '\0') return FALSE;

  strcpy(ptrns, stc[idx].ptrns);
  len = strlen(ptrns);
  for (i = 0; i < len; i++) {
    pt = ptrns[i];
    if ((pt != 'i') AND                                        /* vi_pl  */
        (pt != 'j') AND                                        /* vi_pl  */
        (pt != 'I') AND                                        /* vi_sgl */
        (pt != 'k') AND                                        /* vi_pst */
        (pt != 'O') AND                                       /* lvb_pst */
        (pt != 'l') AND                                        /* lvb_pl */
        (pt != 'L'))   {                                      /* lvb_sgl */
      return FALSE;
    }
  }
  if (len == 0) return FALSE;
  return TRUE;
}


/*----------------------------------------------------------------------
 
  has_past_vt -- check if a syntax token has past transitive verb patterns

    input  -- stc:  a sentence
              slen: sentence length
              idx:  token index
    output -- lklhd: the likelihood of the pattern
              function returns,
                TRUE:  has such patterns
                FALSE: otherwise

-----------------------------------------------------------------------*/
boolean has_past_vt(stc, slen, idx, lklhd)
struct Syntx_Term *stc;
int    slen;
int    idx;
int    *lklhd;
{
  char ptrns[MAXPTRNS+1], pt; 
  int  len, i;

  if ((idx < 0) OR (idx >= slen)) return FALSE;
  if (stc[idx].phrase[0] != '\0') return FALSE;

  strcpy(ptrns, stc[idx].ptrns);
  len = strlen(ptrns);
  for (i = 0; i < len; i++) {
    pt = ptrns[i];
    if (pt == 'v') {           /* vt_pst */
      *lklhd = stc[idx].ptrns_lh[i];
      return TRUE;
    }
  }
  return FALSE;
}

/*----------------------------------------------------------------------
 
  has_past_vi -- check if a syntax token has past intransitive verb 
                 patterns

    input  -- stc:  a sentence
              slen: sentence length
              idx:  token index
    output -- lklhd: the likelihood of the pattern
              function returns,
                TRUE:  has such patterns
                FALSE: otherwise

-----------------------------------------------------------------------*/
boolean has_past_vi(stc, slen, idx, lklhd)
struct Syntx_Term *stc;
int    slen;
int    idx;
int    *lklhd;
{
  char ptrns[MAXPTRNS+1], pt; 
  int  len, i;

  if ((idx < 0) OR (idx >= slen)) return FALSE;
  if (stc[idx].phrase[0] != '\0') return FALSE;

  strcpy(ptrns, stc[idx].ptrns);
  len = strlen(ptrns);
  for (i = 0; i < len; i++) {
    pt = ptrns[i];
    if ((pt == 'k') OR                                         /* vi_pst */
        (pt == 'O'))  {                                       /* lvb_pst */
      *lklhd = stc[idx].ptrns_lh[i];
      return TRUE;
    }
  }
  return FALSE;
}



/*----------------------------------------------------------------------
 
  has_be -- check if a syntax token has verb be patterns

    input  -- stc:  a sentence
              slen: sentence length
              idx:  token index
    output -- lklhd: the likelihood of the pattern
              function returns,
                TRUE:  has such patterns
                FALSE: otherwise

-----------------------------------------------------------------------*/
boolean has_be(stc, slen, idx, lklhd)
struct Syntx_Term *stc;
int    slen;
int    idx;
int    *lklhd;
{
  char ptrns[MAXPTRNS+1], pt; 
  int  len, i;

  if ((idx < 0) OR (idx >= slen)) return FALSE;
  if (stc[idx].phrase[0] != '\0') return FALSE;

  strcpy(ptrns, stc[idx].ptrns);
  len = strlen(ptrns);
  for (i = 0; i < len; i++) {
    pt = ptrns[i];
    if ((pt == 'b') OR            /* be_prst */
        (pt == 'B'))   {          /* be_past */
      *lklhd = stc[idx].ptrns_lh[i];
      return TRUE;
    }
  }
  return FALSE;
}

/*----------------------------------------------------------------------
 
  has_do -- check if a syntax token has verb do patterns

    input  -- stc:  a sentence
              slen: sentence length
              idx:  token index
    output -- lklhd: the likelihood of the pattern
              function returns,
                TRUE:  has such patterns
                FALSE: otherwise

-----------------------------------------------------------------------*/
boolean has_do(stc, slen, idx, lklhd)
struct Syntx_Term *stc;
int    slen;
int    idx;
int    *lklhd;
{
  char ptrns[MAXPTRNS+1], pt; 
  int  len, i;

  if ((idx < 0) OR (idx >= slen)) return FALSE;
  if (stc[idx].phrase[0] != '\0') return FALSE;

  strcpy(ptrns, stc[idx].ptrns);
  len = strlen(ptrns);
  for (i = 0; i < len; i++) {
    pt = ptrns[i];
    if ((pt == 'd') OR            /* do_prst */
        (pt == 'D'))   {          /* do_past */
      *lklhd = stc[idx].ptrns_lh[i];
      return TRUE;
    }
  }
  return FALSE;
}

/*----------------------------------------------------------------------
 
  has_have -- check if a syntax token has verb have patterns

    input  -- stc:  a sentence
              slen: sentence length
              idx:  token index
    output -- lklhd: the likelihood of the pattern
              function returns,
                TRUE:  has such patterns
                FALSE: otherwise

-----------------------------------------------------------------------*/
boolean has_have(stc, slen, idx, lklhd)
struct Syntx_Term *stc;
int    slen;
int    idx;
int    *lklhd;
{
  char ptrns[MAXPTRNS+1], pt; 
  int  len, i;

  if ((idx < 0) OR (idx >= slen)) return FALSE;
  if (stc[idx].phrase[0] != '\0') return FALSE;

  strcpy(ptrns, stc[idx].ptrns);
  len = strlen(ptrns);
  for (i = 0; i < len; i++) {
    pt = ptrns[i];
    if ((pt == 'h') OR            /* hv_prst */
        (pt == 'H'))   {          /* hv_past */
      *lklhd = stc[idx].ptrns_lh[i];
      return TRUE;
    }
  }
  return FALSE;
}


/*----------------------------------------------------------------------
 
  has_aux -- check if a syntax token has aux verb patterns

    input  -- stc:  a sentence
              slen: sentence length
              idx:  token index
    output -- lklhd: the likelihood of the pattern
              function returns,
                TRUE:  has such patterns
                FALSE: otherwise

-----------------------------------------------------------------------*/
boolean has_aux(stc, slen, idx, lklhd)
struct Syntx_Term *stc;
int    slen;
int    idx;
int    *lklhd;
{
  char ptrns[MAXPTRNS+1], pt; 
  int  len, i;

  if ((idx < 0) OR (idx >= slen)) return FALSE;
  if (stc[idx].phrase[0] != '\0') return FALSE;

  strcpy(ptrns, stc[idx].ptrns);
  len = strlen(ptrns);
  for (i = 0; i < len; i++) {
    pt = ptrns[i];
    if ((pt == 'x') OR            /* aux_prst */
        (pt == 'X'))   {          /* aux_past */
      *lklhd = stc[idx].ptrns_lh[i];
      return TRUE;
    }
  }
  return FALSE;
}


/*----------------------------------------------------------------------
 
  has_pronvb -- check if a syntax token has pronoun'verb patterns

    input  -- stc:  a sentence
              slen: sentence length
              idx:  token index
    output -- lklhd: the likelihood of the pattern
              function returns,
                TRUE:  has such patterns
                FALSE: otherwise

-----------------------------------------------------------------------*/
boolean has_pronvb(stc, slen, idx, lklhd)
struct Syntx_Term *stc;
int    slen;
int    idx;
int    *lklhd;
{
  char ptrns[MAXPTRNS+1], pt; 
  int  len, i;

  if ((idx < 0) OR (idx >= slen)) return FALSE;
  if (stc[idx].phrase[0] != '\0') return FALSE;

  strcpy(ptrns, stc[idx].ptrns);
  len = strlen(ptrns);
  for (i = 0; i < len; i++) {
    pt = ptrns[i];
    if ((pt >= '0') AND (pt <= '9')) {
      *lklhd = stc[idx].ptrns_lh[i];
      return TRUE;
    }
  }
  return FALSE;
}

/*----------------------------------------------------------------------
 
  has_lnkvb -- check if a syntax token has linking verb patterns

    input  -- stc:  a sentence
              slen: sentence length
              idx:  token index
    output -- lklhd: the likelihood of the pattern
              function returns,
                TRUE:  has such patterns
                FALSE: otherwise

-----------------------------------------------------------------------*/
boolean has_lnkvb(stc, slen, idx, lklhd)
struct Syntx_Term *stc;
int    slen;
int    idx;
int    *lklhd;
{
  char ptrns[MAXPTRNS+1], ch; 
  int  len, i;

  if ((idx < 0) OR (idx >= slen)) return FALSE;
  if (stc[idx].phrase[0] != '\0') return FALSE;

  strcpy(ptrns, stc[idx].ptrns);
  len = strlen(ptrns);
  for (i = 0; i < len; i++) {
    ch = ptrns[i];
    if ((ch == 'l') OR                                         /* lvb_pl */
        (ch == 'L') OR                                        /* lvb_sgl */
        (ch == 'O') OR                                        /* lvb_pst */
        (ch == 'Q') OR                                       /* lvb_pprt */
        (ch == 'R'))   {                                      /* lvb_ing */
      *lklhd = stc[idx].ptrns_lh[i];
      return TRUE;
    }
  }
  return FALSE;
}


/*----------------------------------------------------------------------
 
  has_adv_advP_only -- check if a syntax token is an adverb phrase or 
                       has adverb pattern only

    input  -- stc:  a sentence
              slen: sentence length
              idx:  token index
    output -- lklhd: the likelihood of the pattern
              function returns,
                TRUE:  has such patterns
                FALSE: otherwise

-----------------------------------------------------------------------*/
boolean has_adv_advP_only(stc, slen, idx, lklhd)
struct Syntx_Term *stc;
int    slen;
int    idx;
int    *lklhd;
{
  char ptrns[MAXPTRNS+1]; 
  int  len;

  if ((idx < 0) OR (idx >= slen)) return FALSE;

  if (EQUAL(stc[idx].phrase,"adv")) {
    *lklhd = stc[idx].phrase_lh;
    return TRUE;
  }
  if (stc[idx].phrase[0] != '\0') return FALSE;

  strcpy(ptrns, stc[idx].ptrns);
  len = strlen(ptrns);
  if ((len == 1) AND (ptrns[0] == 'y')) {
    *lklhd = stc[idx].ptrns_lh[0];
    return TRUE;
  }
  return FALSE;
}


/*----------------------------------------------------------------------
 
  has_article -- check if a syntax token has article patterns

    input  -- stc:  a sentence
              slen: sentence length
              idx:  token index
    output -- lklhd: the likelihood of the pattern
              function returns,
                TRUE:  has such patterns
                FALSE: otherwise

-----------------------------------------------------------------------*/
boolean has_article(stc, slen, idx, lklhd)
struct Syntx_Term *stc;
int    slen;
int    idx;
int    *lklhd;
{
  char ptrns[MAXPTRNS+1], pt; 
  int  len, i;

  if ((idx < 0) OR (idx >= slen)) return FALSE;
  if (stc[idx].phrase[0] != '\0') return FALSE;

  strcpy(ptrns, stc[idx].ptrns);
  len = strlen(ptrns);
  for (i = 0; i < len; i++) {
    pt = ptrns[i];
    if ((pt == 'E') OR            /* artcl_df */
        (pt == 'e'))   {          /* artcl_id */
      *lklhd = stc[idx].ptrns_lh[i];
      return TRUE;
    }
  }
  return FALSE;
}

/*----------------------------------------------------------------------
 
  has_prep -- check if a syntax token has prep patterns

    input  -- stc:  a sentence
              slen: sentence length
              idx:  token index
    output -- lklhd: the likelihood of the pattern
              function returns,
                TRUE:  has such patterns
                FALSE: otherwise

-----------------------------------------------------------------------*/
boolean has_prep(stc, slen, idx, lklhd)
struct Syntx_Term *stc;
int    slen;
int    idx;
int    *lklhd;
{
  char ptrns[MAXPTRNS+1], pt; 
  int  len, i;

  if ((idx < 0) OR (idx >= slen)) return FALSE;
  if (stc[idx].phrase[0] != '\0') return FALSE;

  strcpy(ptrns, stc[idx].ptrns);
  len = strlen(ptrns);
  for (i = 0; i < len; i++) {
    pt = ptrns[i];
    if (pt == 'p') {          /* prep */
      *lklhd = stc[idx].ptrns_lh[i];
      return TRUE;
    }
  }
  return FALSE;
}

/*----------------------------------------------------------------------
 
  has_prdct -- check if a syntax token has predicate patterns 

    input  -- stc:  a sentence
              slen:  sentence length
              idx:  token index
    output -- none
              function returns,
                TRUE:  has such patterns
                FALSE: otherwise

-----------------------------------------------------------------------*/
boolean has_prdct(stc, slen, idx)
struct Syntx_Term *stc;
int    slen;
int    idx;
{
  char ptrns[MAXPTRNS+1], pt; 
  int  len, i;

  if ((idx < 0) OR (idx >= slen)) return FALSE;
  if (stc[idx].phrase[0] != '\0') return FALSE;

  strcpy(ptrns, stc[idx].ptrns);
  len = strlen(ptrns);
  for (i = 0; i < len; i++) {
    pt = ptrns[i];
    if ((pt >= '0') AND (pt <= '9')) return TRUE;
  
    if ((pt == 't') OR            /* vt_pl  */
        (pt == 'u') OR            /* vt_pl  */
        (pt == 'T') OR            /* vt_sgl */
        (pt == 'i') OR            /* vi_pl  */
        (pt == 'j') OR            /* vi_pl  */
        (pt == 'I') OR            /* vi_sgl */
        (pt == 'v') OR            /* vt_pst */
        (pt == 'k') OR            /* vi_pst */
        (pt == 'b') OR            /* be_prst */
        (pt == 'B') OR            /* be_past */
        (pt == 'w') OR            /* be      */
        (pt == 'd') OR            /* do_prst */
        (pt == 'D') OR            /* do_past */
        (pt == 'h') OR            /* hv_prst */
        (pt == 'H'))   {          /* hv_past */
      return TRUE;
    }
  }
  return FALSE;
}


/*----------------------------------------------------------------------
 
  has_class -- check if a syntax token has a specified classification

    input  -- stc:  a sentence
              slen: sentence length
              idx:  token index
              cls:  the classification to be checked
    output -- lklhd: the likelihood of the classification
              function returns,
                TRUE:  has such classification
                FALSE: otherwise

-----------------------------------------------------------------------*/
boolean has_class(stc, slen, idx, cls, lklhd)
struct Syntx_Term *stc;
int    slen;
int    idx;
char   cls;
int    *lklhd;
{
  char ptrns[MAXPTRNS+1]; 
  int  len, i;

  if ((idx < 0) OR (idx >= slen)) return FALSE;
  if (stc[idx].phrase[0] != '\0') return FALSE;

  strcpy(ptrns, stc[idx].ptrns);
  len = strlen(ptrns);
  for (i = 0; i < len; i++) {
    if (ptrns[i] == cls)  {  
      *lklhd = stc[idx].ptrns_lh[i];
      return TRUE;
    }
  }
  return FALSE;
}


/*----------------------------------------------------------------------
 
  has_pattern -- check if a syntax token has a specified pattern

    input  -- stc:  a sentence
              slen: sentence length
              idx:  token index
              ptrn: the pattern to be checked
    output -- lklhd: the likelihood of the pattern
              function returns,
                TRUE:  has such classification
                FALSE: otherwise

-----------------------------------------------------------------------*/
boolean has_pattern(stc, slen, idx, ptrn, lklhd)
struct Syntx_Term *stc;
int    slen;
int    idx;
char   *ptrn;
int    *lklhd;
{
  char ptrn_names[MAXPTRNS+1][PTRN_PHRS_MAX+1], pt[PTRN_PHRS_MAX+1];
  int  i, lklhds[MAXPTRNS];

  if ((idx < 0) OR (idx >= slen)) return FALSE;
  if (stc[idx].phrase[0] != '\0') return FALSE;

  get_ptrn_names(stc, slen, idx, ptrn_names, lklhds);
  i = 0;
  strcpy(pt, ptrn_names[i++]);
  while (!EQUAL(pt,"")) {
    if (EQUAL(pt,ptrn)) { *lklhd = lklhds[i-1]; return TRUE; }
    strcpy(pt, ptrn_names[i++]);
  }
  return FALSE;
}

/*----------------------------------------------------------------------
 
  has_superative -- check if a syntax token adj(adv) superative patterns

    input  -- stc:  a sentence
              slen: sentence length
              idx:  token index
    output -- lklhd: the likelihood of the pattern
              function returns,
                TRUE:  has such patterns
                FALSE: otherwise

-----------------------------------------------------------------------*/
boolean has_superative(stc, slen, idx, lklhd)
struct Syntx_Term *stc;
int    slen;
int    idx;
int    *lklhd;
{
  char ptrns[MAXPTRNS+1], pt; 
  int  len, i;

  if ((idx < 0) OR (idx >= slen)) return FALSE;
  if (stc[idx].phrase[0] != '\0') return FALSE;

  strcpy(ptrns, stc[idx].ptrns);
  len = strlen(ptrns);
  for (i = 0; i < len; i++) {
    pt = ptrns[i];
    if ((pt == 'F') OR            /* adj_est */
        (pt == 'M'))   {          /* adv_est */
      *lklhd = stc[idx].ptrns_lh[i];
      return TRUE;
    }
  }
  return FALSE;
}



/*----------------------------------------------------------------------
 
  has_cmpsup_only -- check if a syntax token has adj(adv) comparative
                     or superative only 

    input  -- stc:  a sentence
              slen:  sentence length
              idx:  token index
    output -- none
              function returns,
                TRUE:  has such patterns
                FALSE: otherwise

-----------------------------------------------------------------------*/
boolean has_cmpsup_only(stc, slen, idx)
struct Syntx_Term *stc;
int    slen;
int    idx;
{
  char ptrns[MAXPTRNS+1], pt; 
  int  len, i;

  if ((idx < 0) OR (idx >= slen)) return FALSE;
  if (stc[idx].phrase[0] != '\0') return FALSE;

  strcpy(ptrns, stc[idx].ptrns);
  len = strlen(ptrns);
  for (i = 0; i < len; i++) {
    pt = ptrns[i];
    if ((pt != 'f') AND            /* adj_er  */
        (pt != 'F') AND            /* adj_est */
        (pt != 'm') AND            /* adv_er  */
        (pt != 'M'))    {          /* adv_est */
      return FALSE;
    }
  }
  if (len == 0) return FALSE;
  return TRUE;
}


/*----------------------------------------------------------------------
 
  has_prep_only -- check if a syntax token has preposition pattern only 

    input  -- stc:  a sentence
              slen:  sentence length
              idx:  token index
    output -- none
              function returns,
                TRUE:  has such patterns
                FALSE: otherwise

-----------------------------------------------------------------------*/
boolean has_prep_only(stc, slen, idx)
struct Syntx_Term *stc;
int    slen;
int    idx;
{
  char ptrns[MAXPTRNS+1], pt; 
  int  len, i;

  if ((idx < 0) OR (idx >= slen)) return FALSE;
  if (stc[idx].phrase[0] != '\0') return FALSE;

  strcpy(ptrns, stc[idx].ptrns);
  len = strlen(ptrns);
  pt = ptrns[0];
  if ((len == 1) AND (pt == 'p')) return TRUE;
  return FALSE;
}


/*----------------------------------------------------------------------
 
  has_sgl3rdvb_only -- check if a syntax token has singular verb in 3rd
                       person.

    input  -- stc:  a sentence
              slen: sentence length
              idx:  token index
    output -- none
              function returns,
                TRUE:  has such patterns
                FALSE: otherwise

-----------------------------------------------------------------------*/
boolean has_sgl3rdvb_only(stc, slen, idx)
struct Syntx_Term *stc;
int    slen;
int    idx;
{
  char ptrns[MAXPTRNS+1], pt; 
  int  len, i;

  if ((idx < 0) OR (idx >= slen)) return FALSE;
  if (stc[idx].phrase[0] != '\0') return FALSE;

  strcpy(ptrns, stc[idx].ptrns);
  len = strlen(ptrns);
  for (i = 0; i < len; i++) {
    pt = ptrns[i];
    if ((pt != 'T') AND            /* vt_sgl */
        (pt != 'I'))    {          /* vi_sgl */
      return FALSE;
    }
  }
  if (len == 0) return FALSE;
  return TRUE;
}


/*----------------------------------------------------------------------
 
  has_pst_only -- check if a syntax token has vt_pst (vi_pst) only

    input  -- stc:  a sentence
              slen: sentence length
              idx:  token index
    output -- none
              function returns,
                TRUE:  has such patterns
                FALSE: otherwise

-----------------------------------------------------------------------*/
boolean has_pst_only(stc, slen, idx)
struct Syntx_Term *stc;
int    slen;
int    idx;
{
  char ptrns[MAXPTRNS+1], pt; 
  int  len, i;

  if ((idx < 0) OR (idx >= slen)) return FALSE;
  if (stc[idx].phrase[0] != '\0') return FALSE;

  strcpy(ptrns, stc[idx].ptrns);
  len = strlen(ptrns);
  for (i = 0; i < len; i++) {
    pt = ptrns[i];
    if ((pt != 'v') AND            /* vt_pst */
        (pt != 'k'))    {          /* vi_pst */
      return FALSE;
    }
  }
  if (len == 0) return FALSE;
  return TRUE;
}


/*----------------------------------------------------------------------
 
  has_pst_pprt_only -- check if a syntax token has vt_pst (vi_pst) or 
                       vt_pprt (vi_pprt) only

    input  -- stc:  a sentence
              slen: sentence length
              idx:  token index
    output -- none
              function returns,
                TRUE:  has such patterns
                FALSE: otherwise

-----------------------------------------------------------------------*/
boolean has_pst_pprt_only(stc, slen, idx)
struct Syntx_Term *stc;
int    slen;
int    idx;
{
  char ptrns[MAXPTRNS+1], pt; 
  int  len, i;

  if ((idx < 0) OR (idx >= slen)) return FALSE;
  if (stc[idx].phrase[0] != '\0') return FALSE;

  strcpy(ptrns, stc[idx].ptrns);
  len = strlen(ptrns);
  for (i = 0; i < len; i++) {
    pt = ptrns[i];
    if ((pt != 'v') AND            /* vt_pst */
        (pt != 'V') AND            /* vt_pprt */
        (pt != 'k') AND            /* vi_pst */
        (pt != 'K'))    {          /* vi_pprt */
      return FALSE;
    }
  }
  if (len == 0) return FALSE;
  return TRUE;
}


/*----------------------------------------------------------------------
 
  has_adjadv_only -- check if a syntax token has adjective or adverb
                     patterns only

    input  -- stc:  a sentence
              slen: sentence length
              idx:  token index
    output -- none
              function returns,
                TRUE:  has such patterns
                FALSE: otherwise

-----------------------------------------------------------------------*/
boolean has_adjadv_only(stc, slen, idx)
struct Syntx_Term *stc;
int    slen;
int    idx;
{
  char ptrns[MAXPTRNS+1], pt; 
  int  len, i;

  if ((idx < 0) OR (idx >= slen)) return FALSE;
  if (stc[idx].phrase[0] != '\0') return FALSE;

  strcpy(ptrns, stc[idx].ptrns);
  len = strlen(ptrns);
  for (i = 0; i < len; i++) {
    pt = ptrns[i];
    if ((pt != 'a') AND            /* adj */
        (pt != 'y'))    {          /* adv */
      return FALSE;
    }
  }
  if (len == 0) return FALSE;
  return TRUE;
}

/*----------------------------------------------------------------------

  keep_noun_adjv -- keep noun, adjective, and adverb patterns of a
                    token, eliminate all the others

    input  -- stc: a sentence
              slen: sentence length
              idx: token index
              windrop: indicate if the patterns in the window are dropped
    output -- none

  function returns   -1    if the pattern is simplified to empty, 
                   n >= 0  the number of classifications dropped

-----------------------------------------------------------------------*/
int keep_noun_adjv(stc, slen, idx, windrop)
struct Syntx_Term *stc;
int    slen;
int    idx;
boolean windrop;
{
  int  i, j, len;
  char ptrns[MAXPTRNS+1], pt;

  if ((idx < 0) OR (idx >= slen)) return;

  j = 0;
  strcpy(ptrns, stc[idx].ptrns);
  len = strlen(ptrns);
  for (i = 0; i < len; i++) {
    pt = ptrns[i];
    if ((pt == 'n') OR            /* noun_sgl */
        (pt == 'o') OR            /* noun_sgl */
        (pt == 'N') OR            /* noun_pl  */
        (pt == 'V') OR            /* vt_pprt  */
        (pt == 'g') OR            /* vt_ing   */
        (pt == 'K') OR            /* vi_pprt  */
        (pt == 'G') OR            /* vi_ing   */
        (pt == 'a') OR            /* adj      */
        (pt == 'f') OR            /* adj_er   */
        (pt == 'F') OR            /* adj_est  */
        (pt == 'y') OR            /* adv      */
        (pt == 'm') OR            /* adv_er   */
        (pt == 'M') OR            /* adv_est  */
        (pt == 'z'))   {          /* being    */

      stc[idx].ptrns[j] = pt;
      stc[idx].ptrns_lh[j++] = stc[idx].ptrns_lh[i];
    }
    else {
      if (windrop) window_drop(stc, idx, pt);
    }
  }
  stc[idx].ptrns[j] = '\0';
  if (j == 0) return -1;
  return(len - j);
}



/*----------------------------------------------------------------------

  keep_all_vtvi -- keep all vt, vi (present or past) patterns of a token,  
                   eliminate all the others

    input  -- stc: a sentence
              slen: sentence length
              idx: token index
              windrop: indicate if the patterns in the window are dropped
    output -- none

  function returns   -1    if the pattern is simplified to empty, 
                   n >= 0  the number of classifications dropped

-----------------------------------------------------------------------*/
int keep_all_vtvi(stc, slen, idx, windrop)
struct Syntx_Term *stc;
int    slen;
int    idx;
boolean windrop;
{
  int  i, j, len;
  char ptrns[MAXPTRNS+1], pt;

  if ((idx < 0) OR (idx >= slen)) return;

  j = 0;
  strcpy(ptrns, stc[idx].ptrns);
  len = strlen(ptrns);
  for (i = 0; i < len; i++) {
    pt = ptrns[i];
    if ((pt == 'v') OR                                         /* vt_pst */
        (pt == 'k') OR                                         /* vi_pst */
        (pt == 'I') OR                                         /* vi_sgl */
        (pt == 'i') OR                                         /* vi_pl  */
        (pt == 'j') OR                                         /* vi_pl  */
        (pt == 'T') OR                                         /* vt_sgl */
        (pt == 't') OR                                         /* vt_pl  */
        (pt == 'u') OR                                         /* vt_pl  */
        (pt == 'l') OR                                         /* lvb_pl */
        (pt == 'L') OR                                        /* lvb_sgl */
        (pt == 'O'))   {                                      /* lvb_pst */
      stc[idx].ptrns[j] = pt;
      stc[idx].ptrns_lh[j++] = stc[idx].ptrns_lh[i];
    }
    else {
      if (windrop) window_drop(stc, idx, pt);
    }
  }
  stc[idx].ptrns[j] = '\0';
  if (j == 0) return -1;
  return(len - j);
}


/*----------------------------------------------------------------------

  keep_article -- keep article patterns of a token, eliminate all the 
                  others

    input  -- stc: a sentence
              slen: sentence length
              idx: token index
              windrop: indicate if the patterns in the window are dropped
    output -- none

  function returns   -1    if the pattern is simplified to empty, 
                   n >= 0  the number of classifications dropped

-----------------------------------------------------------------------*/
int keep_article(stc, slen, idx, windrop)
struct Syntx_Term *stc;
int    slen;
int    idx;
boolean windrop;
{
  int  i, j, len;
  char ptrns[MAXPTRNS+1], pt;

  if ((idx < 0) OR (idx >= slen)) return;

  j = 0;
  strcpy(ptrns, stc[idx].ptrns);
  len = strlen(ptrns);
  for (i = 0; i < len; i++) {
    pt = ptrns[i];
    if ((pt == 'E') OR            /* artcl_df */
        (pt == 'e'))   {          /* artcl_id */
 
      stc[idx].ptrns[j] = pt;
      stc[idx].ptrns_lh[j++] = stc[idx].ptrns_lh[i];
    }
    else {    
      if (windrop) window_drop(stc, idx, pt);
    }
  }
  stc[idx].ptrns[j] = '\0';
  if (j == 0) return -1;
  return(len - j);
}


/*----------------------------------------------------------------------

  keep_be -- keep be patterns of a token, eliminate all the others

    input  -- stc: a sentence
              slen: sentence length
              idx: token index
              windrop: indicate if the patterns in the window are dropped
    output -- none

  function returns   -1    if the pattern is simplified to empty, 
                   n >= 0  the number of classifications dropped

-----------------------------------------------------------------------*/
int keep_be(stc, slen, idx, windrop)
struct Syntx_Term *stc;
int    slen;
int    idx;
boolean windrop;
{
  int  i, j, len;
  char ptrns[MAXPTRNS+1], pt;

  if ((idx < 0) OR (idx >= slen)) return;

  j = 0;
  strcpy(ptrns, stc[idx].ptrns);
  len = strlen(ptrns);
  for (i = 0; i < len; i++) {
    pt = ptrns[i];
    if ((pt == 'b') OR            /* be_prst */
        (pt == 'B'))   {          /* be_past */
 
      stc[idx].ptrns[j] = pt;
      stc[idx].ptrns_lh[j++] = stc[idx].ptrns_lh[i];
    }
    else {    
      if (windrop) window_drop(stc, idx, pt);
    }
  }
  stc[idx].ptrns[j] = '\0';
  if (j == 0) return -1;
  return(len - j);
}


/*----------------------------------------------------------------------

  keep_do -- keep do patterns of a token, eliminate all the others

    input  -- stc: a sentence
              slen: sentence length
              idx: token index
              windrop: indicate if the patterns in the window are dropped
    output -- none

  function returns   -1    if the pattern is simplified to empty, 
                   n >= 0  the number of classifications dropped

-----------------------------------------------------------------------*/
int keep_do(stc, slen, idx, windrop)
struct Syntx_Term *stc;
int    slen;
int    idx;
boolean windrop;
{
  int  i, j, len;
  char ptrns[MAXPTRNS+1], pt;

  if ((idx < 0) OR (idx >= slen)) return;

  j = 0;
  strcpy(ptrns, stc[idx].ptrns);
  len = strlen(ptrns);
  for (i = 0; i < len; i++) {
    pt = ptrns[i];
    if ((pt == 'd') OR            /* do_prst */
        (pt == 'D'))   {          /* do_past */
 
      stc[idx].ptrns[j] = pt;
      stc[idx].ptrns_lh[j++] = stc[idx].ptrns_lh[i];
    }
    else {    
      if (windrop) window_drop(stc, idx, pt);
    }
  }
  stc[idx].ptrns[j] = '\0';
  if (j == 0) return -1;
  return(len - j);
}


/*----------------------------------------------------------------------

  keep_have -- keep have patterns of a token, eliminate all the others

    input  -- stc: a sentence
              slen: sentence length
              idx: token index
              windrop: indicate if the patterns in the window are dropped
    output -- none

  function returns   -1    if the pattern is simplified to empty, 
                   n >= 0  the number of classifications dropped

-----------------------------------------------------------------------*/
int keep_have(stc, slen, idx, windrop)
struct Syntx_Term *stc;
int    slen;
int    idx;
boolean windrop;
{
  int  i, j, len;
  char ptrns[MAXPTRNS+1], pt;

  if ((idx < 0) OR (idx >= slen)) return;

  j = 0;
  strcpy(ptrns, stc[idx].ptrns);
  len = strlen(ptrns);
  for (i = 0; i < len; i++) {
    pt = ptrns[i];
    if ((pt == 'h') OR            /* hv_prst */
        (pt == 'H'))   {          /* hv_past */
 
      stc[idx].ptrns[j] = pt;
      stc[idx].ptrns_lh[j++] = stc[idx].ptrns_lh[i];
    }
    else {    
      if (windrop) window_drop(stc, idx, pt);
    }
  }
  stc[idx].ptrns[j] = '\0';
  if (j == 0) return -1;
  return(len - j);
}


/*----------------------------------------------------------------------

  keep_aux -- keep aux patterns of a token, eliminate all the others

    input  -- stc: a sentence
              slen: sentence length
              idx: token index
              windrop: indicate if the patterns in the window are dropped
    output -- none

  function returns   -1    if the pattern is simplified to empty, 
                   n >= 0  the number of classifications dropped

-----------------------------------------------------------------------*/
int keep_aux(stc, slen, idx, windrop)
struct Syntx_Term *stc;
int    slen;
int    idx;
boolean windrop;
{
  int  i, j, len;
  char ptrns[MAXPTRNS+1], pt;

  if ((idx < 0) OR (idx >= slen)) return;

  j = 0;
  strcpy(ptrns, stc[idx].ptrns);
  len = strlen(ptrns);
  for (i = 0; i < len; i++) {
    pt = ptrns[i];
    if ((pt == 'x') OR            /* aux_prst */
        (pt == 'X'))   {          /* aux_past */
 
      stc[idx].ptrns[j] = pt;
      stc[idx].ptrns_lh[j++] = stc[idx].ptrns_lh[i];
    }
    else {    
      if (windrop) window_drop(stc, idx, pt);
    }
  }
  stc[idx].ptrns[j] = '\0';
  if (j == 0) return -1;
  return(len - j);
}


/*----------------------------------------------------------------------

  keep_lnkvb -- keep lnkvb patterns of a token, eliminate all the others

    input  -- stc: a sentence
              slen: sentence length
              idx: token index
              windrop: indicate if the patterns in the window are dropped
    output -- none

  function returns   -1    if the pattern is simplified to empty, 
                   n >= 0  the number of classifications dropped

-----------------------------------------------------------------------*/
int keep_lnkvb(stc, slen, idx, windrop)
struct Syntx_Term *stc;
int    slen;
int    idx;
boolean windrop;
{
  int  i, j, len;
  char ptrns[MAXPTRNS+1], pt;

  if ((idx < 0) OR (idx >= slen)) return;

  j = 0;
  strcpy(ptrns, stc[idx].ptrns);
  len = strlen(ptrns);
  for (i = 0; i < len; i++) {
    pt = ptrns[i];
    if ((pt == 'l') OR                                         /* lvb_pl */
        (pt == 'L') OR                                        /* lvb_sgl */
        (pt == 'O'))   {                                      /* lvb_pst */
      stc[idx].ptrns[j] = pt;
      stc[idx].ptrns_lh[j++] = stc[idx].ptrns_lh[i];
    }
    else {    
      if (windrop) window_drop(stc, idx, pt);
    }
  }
  stc[idx].ptrns[j] = '\0';
  if (j == 0) return -1;
  return(len - j);
}

/*----------------------------------------------------------------------

  keep_adjlike -- keep adjective like patterns of a token, eliminate all
                  the others

    input  -- stc: a sentence
              slen: sentence length
              idx: token index
    output -- lh: the largest likelihood of such patterns

  function returns   -1    if the pattern is simplified to empty, 
                   n >= 0  the number of classifications dropped

-----------------------------------------------------------------------*/
int keep_adjlike(stc, slen, idx, lh)
struct Syntx_Term *stc;
int    slen;
int    idx;
int    *lh;
{
  int  i, j, len, max = 0;
  char ptrns[MAXPTRNS+1], pt;

  if ((idx < 0) OR (idx >= slen)) return -1;

  j = 0;
  strcpy(ptrns, stc[idx].ptrns);
  len = strlen(ptrns);
  for (i = 0; i < len; i++) {
    int  val;

    pt = ptrns[i];
    if ((pt == 'a') OR            /* adj     */
        (pt == 'f') OR            /* adj_er  */
        (pt == 'F') OR            /* adj_est */
        (pt == 'g') OR            /* vt_ing  */
        (pt == 'G') OR            /* vi_ing  */
        (pt == 'V') OR            /* vt_pprt */
        (pt == 'K'))   {          /* vi_pprt */

      stc[idx].ptrns[j] = pt;
      val = stc[idx].ptrns_lh[i];
      stc[idx].ptrns_lh[j++] = val;
      if (val > max) max = val;
    }
  }
  *lh = max;
  stc[idx].ptrns[j] = '\0';
  if (j == 0) return -1;
  return(len - j);
}


/*----------------------------------------------------------------------

  keep_adv -- keep adverb patterns of a token, eliminate all the others

    input  -- stc: a sentence
              slen: sentence length
              idx: token index
    output -- lh: the largest likelihood of such patterns

  function returns   -1    if the pattern is simplified to empty, 
                   n >= 0  the number of classifications dropped

-----------------------------------------------------------------------*/
int keep_adv(stc, slen, idx, lh)
struct Syntx_Term *stc;
int    slen;
int    idx;
int    *lh;
{
  int  i, j, len, max = 0;
  char ptrns[MAXPTRNS+1], pt;

  if ((idx < 0) OR (idx >= slen)) return -1;

  j = 0;
  strcpy(ptrns, stc[idx].ptrns);
  len = strlen(ptrns);
  for (i = 0; i < len; i++) {
    int  val;

    pt = ptrns[i];
    if ((pt == 'y') OR            /* adv     */
        (pt == 'm') OR            /* adv_er  */
        (pt == 'M'))   {          /* adv_est */

      stc[idx].ptrns[j] = pt;
      val = stc[idx].ptrns_lh[i];
      stc[idx].ptrns_lh[j++] = val;
      if (val > max) max = val;
    }
  }
  *lh = max;
  stc[idx].ptrns[j] = '\0';
  if (j == 0) return -1;
  return(len - j);
}

/*----------------------------------------------------------------------

  keep_prep -- keep preposition pattern of a token,eliminate all the others

    input  -- stc: a sentence
              slen: sentence length
              idx: token index
    output -- lh: the largest likelihood of such patterns

  function returns   -1    if the pattern is simplified to empty, 
                   n >= 0  the number of classifications dropped

-----------------------------------------------------------------------*/
int keep_prep(stc, slen, idx, lh)
struct Syntx_Term *stc;
int    slen;
int    idx;
int    *lh;
{
  int  i, j, len;
  char ptrns[MAXPTRNS+1], pt;

  if ((idx < 0) OR (idx >= slen)) return -1;

  j = 0;
  strcpy(ptrns, stc[idx].ptrns);
  len = strlen(ptrns);
  for (i = 0; i < len; i++) {
    int  val;

    pt = ptrns[i];
    if (pt == 'p') {          /* prep */
      stc[idx].ptrns[j] = pt;
      val = stc[idx].ptrns_lh[i];
      stc[idx].ptrns_lh[j++] = val;
      *lh = val;
    }
  }
  stc[idx].ptrns[j] = '\0';
  if (j == 0) return -1;
  return(len - j);
}

/*----------------------------------------------------------------------

  keep_ith_class -- keep the ith class in a pattern of a token, eliminate 
                    all the others

    input  -- stc: a sentence
              slen: sentence length
              idx: token index
              ith: the index of the ith class
    output -- lh: the likelihood of the ith pattern

  function returns   -1    if the pattern is simplified to empty, 
                   n >= 0  the number of classifications dropped

-----------------------------------------------------------------------*/
int keep_ith_class(stc, slen, idx, ith, lh)
struct Syntx_Term *stc;
int    slen;
int    idx;
int    ith;
int    *lh;
{
  int  i, j, len;
  char ptrns[MAXPTRNS+1], pt;

  if ((idx < 0) OR (idx >= slen)) return 0;

  stc[idx].ptrns[0] = stc[idx].ptrns[ith];
  stc[idx].ptrns[1] = '\0';
  *lh = stc[idx].ptrns_lh[ith];
  return (strlen(stc[idx].ptrns) - 1);
}


/*----------------------------------------------------------------------

  drop_noun -- drop off noun patterns from a token

    input  -- stc: a sentence
              slen: sentence length
              idx: token index
              windrop: indicate if the patterns in the window are dropped
    output -- none

  function returns   -1    if the pattern is simplified to empty, 
                   n >= 0  the number of classifications dropped

-----------------------------------------------------------------------*/
int drop_noun(stc, slen, idx, windrop)
struct Syntx_Term *stc;
int    slen;
int    idx;
boolean windrop;
{
  int  i, j, len;
  char ptrns[MAXPTRNS+1], pt;

  if ((idx < 0) OR (idx >= slen)) return;

  j = 0;
  strcpy(ptrns, stc[idx].ptrns);
  len = strlen(ptrns);
  for (i = 0; i < len; i++) {
    pt = ptrns[i];
    if ((pt != 'n') AND           /* noun_sgl */
        (pt != 'o'))   {          /* noun_sgl */
 
      stc[idx].ptrns[j] = pt;
      stc[idx].ptrns_lh[j++] = stc[idx].ptrns_lh[i];
    }
    else {
      if (windrop) window_drop(stc, idx, pt);
    }
  }
  stc[idx].ptrns[j] = '\0';
  if (j == 0) return -1;
  return(len - j);
}


/*----------------------------------------------------------------------

  drop_plural -- drop off plural patterns from a token

    input  -- stc: a sentence
              slen: sentence length
              idx: token index
              windrop: indicate if the patterns in the window are dropped
    output -- none

  function returns   -1    if the pattern is simplified to empty, 
                   n >= 0  the number of classifications dropped

-----------------------------------------------------------------------*/
int drop_plural(stc, slen, idx, windrop)
struct Syntx_Term *stc;
int    slen;
int    idx;
boolean windrop;
{
  int  i, j, len;
  char ptrns[MAXPTRNS+1], pt;

  if ((idx < 0) OR (idx >= slen)) return;

  j = 0;
  strcpy(ptrns, stc[idx].ptrns);
  len = strlen(ptrns);
  for (i = 0; i < len; i++) {
    pt = ptrns[i];
    if (pt != 'N')   {    /* noun_pl */
      stc[idx].ptrns[j] = pt;
      stc[idx].ptrns_lh[j++] = stc[idx].ptrns_lh[i];
    }
    else {
      if (windrop) window_drop(stc, idx, pt);
    }
  }
  stc[idx].ptrns[j] = '\0';
  if (j == 0) return -1;
  return(len - j);
}



/*----------------------------------------------------------------------

  drop_noun_plural -- drop off noun and plural patterns from a token

    input  -- stc: a sentence
              slen: sentence length
              idx: token index
              windrop: indicate if the patterns in the window are dropped
    output -- none

  function returns   -1    if the pattern is simplified to empty, 
                   n >= 0  the number of classifications dropped

-----------------------------------------------------------------------*/
int drop_noun_plural(stc, slen, idx, windrop)
struct Syntx_Term *stc;
int    slen;
int    idx;
boolean windrop;
{
  int  i, j, len;
  char ptrns[MAXPTRNS+1], pt;

  if ((idx < 0) OR (idx >= slen)) return;

  j = 0;
  strcpy(ptrns, stc[idx].ptrns);
  len = strlen(ptrns);
  for (i = 0; i < len; i++) {
    pt = ptrns[i];
    if ((pt != 'n') AND           /* noun_sgl */
        (pt != 'o') AND           /* noun_sgl */
        (pt != 'N'))   {          /* noun_pl */
 
      stc[idx].ptrns[j] = pt;
      stc[idx].ptrns_lh[j++] = stc[idx].ptrns_lh[i];
    }
    else {
      if (windrop) window_drop(stc, idx, pt);
    }
  }
  stc[idx].ptrns[j] = '\0';
  if (j == 0) return -1;
  return(len - j);

}


/*----------------------------------------------------------------------

  drop_non_flwto -- drop off patterns which are not allowed to follow "to"

    input  -- stc: a sentence
              slen: sentence length
              idx: token index
              windrop: indicate if the patterns in the window are dropped
    output -- none

  function returns   -1    if the pattern is simplified to empty, 
                   n >= 0  the number of classifications dropped

-----------------------------------------------------------------------*/
int drop_non_flwto(stc, slen, idx, windrop)
struct Syntx_Term *stc;
int    slen;
int    idx;
boolean windrop;
{
  int  i, j, len;
  char ptrns[MAXPTRNS+1], pt;

  if ((idx < 0) OR (idx >= slen)) return;

  j = 0;
  strcpy(ptrns, stc[idx].ptrns);
  len = strlen(ptrns);
  for (i = 0; i < len; i++) {
    pt = ptrns[i];
    if ((pt != 'd') AND           /* do_prst */
        (pt != 'D') AND           /* do_past */
        (pt != 'T') AND           /* vt_sgl */
        (pt != 'I') AND           /* vi_sgl */
        (pt != 'L') AND           /* lvb_sgl */
        (pt != 'v') AND           /* vt_pst */
        (pt != 'k') AND           /* vi_pst */
        (pt != 'O'))   {          /* lvb_pst */
 
      stc[idx].ptrns[j] = pt;
      stc[idx].ptrns_lh[j++] = stc[idx].ptrns_lh[i];
    }
    else {
      if (windrop) window_drop(stc, idx, pt);
    }
  }
  stc[idx].ptrns[j] = '\0';
  if (j == 0) return -1;
  return(len - j);

}


/*----------------------------------------------------------------------

  drop_adj -- drop off adj patterns from a token

    input  -- stc: a sentence
              slen: sentence length
              idx: token index
              windrop: indicate if the patterns in the window are dropped
    output -- none

  function returns   -1    if the pattern is simplified to empty, 
                   n >= 0  the number of classifications dropped

-----------------------------------------------------------------------*/
int drop_adj(stc, slen, idx, windrop)
struct Syntx_Term *stc;
int    slen;
int    idx;
boolean windrop;
{
  int  i, j, len;
  char ptrns[MAXPTRNS+1], pt;

  if ((idx < 0) OR (idx >= slen)) return;

  j = 0;
  strcpy(ptrns, stc[idx].ptrns);
  len = strlen(ptrns);
  for (i = 0; i < len; i++) {
    pt = ptrns[i];
    if (pt != 'a') {          /* adj */
      stc[idx].ptrns[j] = pt;
      stc[idx].ptrns_lh[j++] = stc[idx].ptrns_lh[i];
    }
    else {
      if (windrop) window_drop(stc, idx, pt);
    }
  }
  stc[idx].ptrns[j] = '\0';
  if (j == 0) return -1;
  return(len - j);

}



/*----------------------------------------------------------------------

  drop_vt -- drop off vt patterns from a token

    input  -- stc: a sentence
              slen: sentence length
              idx: token index
              windrop: indicate if the patterns in the window are dropped
    output -- none

  function returns   -1    if the pattern is simplified to empty, 
                   n >= 0  the number of classifications dropped

-----------------------------------------------------------------------*/
int drop_vt(stc, slen, idx, windrop)
struct Syntx_Term *stc;
int    slen;
int    idx;
boolean windrop;
{
  int  i, j, len;
  char ptrns[MAXPTRNS+1], pt;

  if ((idx < 0) OR (idx >= slen)) return;

  j = 0;
  strcpy(ptrns, stc[idx].ptrns);
  len = strlen(ptrns);
  for (i = 0; i < len; i++) {
    pt = ptrns[i];
    if ((pt != 'T') AND           /* vt_sgl */
        (pt != 't') AND           /* vt_pl  */
        (pt != 'u'))   {          /* vt_pl  */
 
      stc[idx].ptrns[j] = pt;
      stc[idx].ptrns_lh[j++] = stc[idx].ptrns_lh[i];
    }
    else {
      if (windrop) window_drop(stc, idx, pt);
    }
  }
  stc[idx].ptrns[j] = '\0';
  if (j == 0) return -1;
  return(len - j);
}



/*----------------------------------------------------------------------

  drop_vi -- drop off vi patterns from a token

    input  -- stc: a sentence
              slen: sentence length
              idx: token index
              windrop: indicate if the patterns in the window are dropped
    output -- none

  function returns   -1    if the pattern is simplified to empty, 
                   n >= 0  the number of classifications dropped

-----------------------------------------------------------------------*/
int drop_vi(stc, slen, idx, windrop)
struct Syntx_Term *stc;
int    slen;
int    idx;
boolean windrop;
{
  int  i, j, len;
  char ptrns[MAXPTRNS+1], pt;

  if ((idx < 0) OR (idx >= slen)) return;

  j = 0;
  strcpy(ptrns, stc[idx].ptrns);
  len = strlen(ptrns);
  for (i = 0; i < len; i++) {
    pt = ptrns[i];
    if ((pt != 'I') AND                                        /* vi_sgl */
        (pt != 'i') AND                                        /* vi_pl  */
        (pt != 'j') AND                                        /* vi_pl  */
        (pt != 'l') AND                                        /* lvb_pl */
        (pt != 'L'))    {                                     /* lvb_sgl */
      stc[idx].ptrns[j] = pt;
      stc[idx].ptrns_lh[j++] = stc[idx].ptrns_lh[i];
    }
    else {
      if (windrop) window_drop(stc, idx, pt);
    }
  }
  stc[idx].ptrns[j] = '\0';
  if (j == 0) return -1;
  return(len - j);
}

/*----------------------------------------------------------------------

  drop_verb -- drop off verb patterns from a token

    input  -- stc: a sentence
              slen: sentence length
              idx: token index
              windrop: indicate if the patterns in the window are dropped
    output -- none

  function returns   -1    if the pattern is simplified to empty, 
                   n >= 0  the number of classifications dropped

-----------------------------------------------------------------------*/
int drop_verb(stc, slen, idx, windrop)
struct Syntx_Term *stc;
int    slen;
int    idx;
boolean windrop;
{
  int  i, j, len;
  char ptrns[MAXPTRNS+1], pt;

  if ((idx < 0) OR (idx >= slen)) return;

  j = 0;
  strcpy(ptrns, stc[idx].ptrns);
  len = strlen(ptrns);
  for (i = 0; i < len; i++) {
    pt = ptrns[i];
    if ((pt != 'I') AND                                        /* vi_sgl */
        (pt != 'i') AND                                        /* vi_pl  */
        (pt != 'j') AND                                        /* vi_pl  */
        (pt != 'T') AND                                        /* vt_sgl */
        (pt != 't') AND                                        /* vt_pl  */
        (pt != 'u') AND                                        /* vt_pl  */
        (pt != 'l') AND                                        /* lvb_pl */
        (pt != 'L'))    {                                     /* lvb_sgl */
      stc[idx].ptrns[j] = pt;
      stc[idx].ptrns_lh[j++] = stc[idx].ptrns_lh[i];
    }
    else {
      if (windrop) window_drop(stc, idx, pt);
    }
  }
  stc[idx].ptrns[j] = '\0';
  if (j == 0) return -1;
  return(len - j);
}


/*----------------------------------------------------------------------

  drop_all_vb -- drop off vt, vi patterns (present and past) from a token

    input  -- stc: a sentence
              slen: sentence length
              idx: token index
              windrop: indicate if the patterns in the window are dropped
    output -- none

  function returns   -1    if the pattern is simplified to empty, 
                   n >= 0  the number of classifications dropped

-----------------------------------------------------------------------*/
int drop_all_vb(stc, slen, idx, windrop)
struct Syntx_Term *stc;
int    slen;
int    idx;
boolean windrop;
{
  int  i, j, len;
  char ptrns[MAXPTRNS+1], pt;
  char word[WORD_MAX+1];

  if ((idx < 0) OR (idx >= slen)) return;
  get_word(stc, slen, idx-1, word);
  len = strlen(word);
  for (i = 0; i < len; i++) word[i] = tolower(word[i]);
  if (EQUAL(word,"to")) return 0;

  j = 0;
  strcpy(ptrns, stc[idx].ptrns);
  len = strlen(ptrns);
  for (i = 0; i < len; i++) {
    pt = ptrns[i];
    if ((pt != 'v') AND                                        /* vt_pst */
        (pt != 'k') AND                                        /* vi_pst */
        (pt != 'I') AND                                        /* vi_sgl */
        (pt != 'i') AND                                        /* vi_pl  */
        (pt != 'j') AND                                        /* vi_pl  */
        (pt != 'T') AND                                        /* vt_sgl */
        (pt != 't') AND                                        /* vt_pl  */
        (pt != 'u') AND                                        /* vt_pl  */
        (pt != 'l') AND                                        /* lvb_pl */
        (pt != 'L') AND                                       /* lvb_sgl */
        (pt != 'O'))    {                                     /* lvb_pst */
      stc[idx].ptrns[j] = pt;
      stc[idx].ptrns_lh[j++] = stc[idx].ptrns_lh[i];
    }
    else {
      if (windrop) window_drop(stc, idx, pt);
    }
  }
  stc[idx].ptrns[j] = '\0';
  if (j == 0) return -1;
  return(len - j);
}

/*----------------------------------------------------------------------

  drop_be -- drop off be patterns from a token

    input  -- stc: a sentence
              slen: sentence length
              idx: token index
              windrop: indicate if the patterns in the window are dropped
    output -- none

  function returns   -1    if the pattern is simplified to empty, 
                   n >= 0  the number of classifications dropped

-----------------------------------------------------------------------*/
int drop_be(stc, slen, idx, windrop)
struct Syntx_Term *stc;
int    slen;
int    idx;
boolean windrop;
{
  int  i, j, len;
  char ptrns[MAXPTRNS+1], pt;

  if ((idx < 0) OR (idx >= slen)) return;

  j = 0;
  strcpy(ptrns, stc[idx].ptrns);
  len = strlen(ptrns);
  for (i = 0; i < len; i++) {
    pt = ptrns[i];
    if ((pt != 'b') AND           /* be_prst */
        (pt != 'B'))   {          /* be_past  */
 
      stc[idx].ptrns[j] = pt;
      stc[idx].ptrns_lh[j++] = stc[idx].ptrns_lh[i];
    }
    else {
      if (windrop) window_drop(stc, idx, pt);
    }
  }
  stc[idx].ptrns[j] = '\0';
  if (j == 0) return -1;
  return(len - j);
}


/*----------------------------------------------------------------------

  drop_do -- drop off do patterns from a token

    input  -- stc: a sentence
              slen: sentence length
              idx: token index
              windrop: indicate if the patterns in the window are dropped
    output -- none

  function returns   -1    if the pattern is simplified to empty, 
                   n >= 0  the number of classifications dropped

-----------------------------------------------------------------------*/
int drop_do(stc, slen, idx, windrop)
struct Syntx_Term *stc;
int    slen;
int    idx;
boolean windrop;
{
  int  i, j, len;
  char ptrns[MAXPTRNS+1], pt;

  if ((idx < 0) OR (idx >= slen)) return;

  j = 0;
  strcpy(ptrns, stc[idx].ptrns);
  len = strlen(ptrns);
  for (i = 0; i < len; i++) {
    pt = ptrns[i];
    if ((pt != 'd') AND           /* do_prst */
        (pt != 'D'))   {          /* do_past  */
 
      stc[idx].ptrns[j] = pt;
      stc[idx].ptrns_lh[j++] = stc[idx].ptrns_lh[i];
    }
    else {
      if (windrop) window_drop(stc, idx, pt);
    }
  }
  stc[idx].ptrns[j] = '\0';
  if (j == 0) return -1;
  return(len - j);
}


/*----------------------------------------------------------------------

  drop_have -- drop off have patterns from a token

    input  -- stc: a sentence
              slen: sentence length
              idx: token index
              windrop: indicate if the patterns in the window are dropped
    output -- none

  function returns   -1    if the pattern is simplified to empty, 
                   n >= 0  the number of classifications dropped

-----------------------------------------------------------------------*/
int drop_have(stc, slen, idx, windrop)
struct Syntx_Term *stc;
int    slen;
int    idx;
boolean windrop;
{
  int  i, j, len;
  char ptrns[MAXPTRNS+1], pt;

  if ((idx < 0) OR (idx >= slen)) return;

  j = 0;
  strcpy(ptrns, stc[idx].ptrns);
  len = strlen(ptrns);
  for (i = 0; i < len; i++) {
    pt = ptrns[i];
    if ((pt != 'h') AND           /* hv_prst */
        (pt != 'H'))   {          /* hv_past  */
 
      stc[idx].ptrns[j] = pt;
      stc[idx].ptrns_lh[j++] = stc[idx].ptrns_lh[i];
    }
    else {
      if (windrop) window_drop(stc, idx, pt);
    }
  }
  stc[idx].ptrns[j] = '\0';
  if (j == 0) return -1;
  return(len - j);
}


/*----------------------------------------------------------------------

  drop_aux -- drop off aux patterns from a token

    input  -- stc: a sentence
              slen: sentence length
              idx: token index
              windrop: indicate if the patterns in the window are dropped
    output -- none

  function returns   -1    if the pattern is simplified to empty, 
                   n >= 0  the number of classifications dropped

-----------------------------------------------------------------------*/
int drop_aux(stc, slen, idx, windrop)
struct Syntx_Term *stc;
int    slen;
int    idx;
boolean windrop;
{
  int  i, j, len;
  char ptrns[MAXPTRNS+1], pt;

  if ((idx < 0) OR (idx >= slen)) return;

  j = 0;
  strcpy(ptrns, stc[idx].ptrns);
  len = strlen(ptrns);
  for (i = 0; i < len; i++) {
    pt = ptrns[i];
    if ((pt != 'x') AND           /* aux_prst */
        (pt != 'X'))   {          /* aux_past  */
 
      stc[idx].ptrns[j] = pt;
      stc[idx].ptrns_lh[j++] = stc[idx].ptrns_lh[i];
    }
    else {
      if (windrop) window_drop(stc, idx, pt);
    }
  }
  stc[idx].ptrns[j] = '\0';
  if (j == 0) return -1;
  return(len - j);
}

/*----------------------------------------------------------------------

  drop_lnkvb -- drop off lnkvb patterns from a token

    input  -- stc: a sentence
              slen: sentence length
              idx: token index
              windrop: indicate if the patterns in the window are dropped
    output -- none

  function returns   -1    if the pattern is simplified to empty, 
                   n >= 0  the number of classifications dropped

-----------------------------------------------------------------------*/
int drop_lnkvb(stc, slen, idx, windrop)
struct Syntx_Term *stc;
int    slen;
int    idx;
boolean windrop;
{
  int  i, j, len;
  char ptrns[MAXPTRNS+1], pt;

  if ((idx < 0) OR (idx >= slen)) return;

  j = 0;
  strcpy(ptrns, stc[idx].ptrns);
  len = strlen(ptrns);
  for (i = 0; i < len; i++) {
    pt = ptrns[i];
    if ((pt != 'l') AND                                        /* lvb_pl */
        (pt != 'L') AND                                       /* lvb_sgl */
        (pt != 'O'))    {                                     /* lvb_pst */
      stc[idx].ptrns[j] = pt;
      stc[idx].ptrns_lh[j++] = stc[idx].ptrns_lh[i];
    }
    else {
      if (windrop) window_drop(stc, idx, pt);
    }
  }
  stc[idx].ptrns[j] = '\0';
  if (j == 0) return -1;
  return(len - j);

}


/*----------------------------------------------------------------------

  drop_mdfr -- drop off modifier and noun patterns from a token

    input  -- stc: a sentence
              slen: sentence length
              idx: token index
    output -- lh: the largest likelihood of remaining patterns

  function returns   -1    if the pattern is simplified to empty, 
                   n >= 0  the number of classifications dropped

-----------------------------------------------------------------------*/
int drop_mdfr(stc, slen, idx, lh)
struct Syntx_Term *stc;
int    slen;
int    idx;
int    *lh;
{
  int  i, j, len, val, max = 0;
  char ptrns[MAXPTRNS+1], pt;

  if ((idx < 0) OR (idx >= slen)) return;

  j = 0;
  strcpy(ptrns, stc[idx].ptrns);
  len = strlen(ptrns);
  for (i = 0; i < len; i++) {
    pt = ptrns[i];
    if ((pt != 'a') AND           /* adj      */
        (pt != 'f') AND           /* adj_er   */
        (pt != 'F') AND           /* adj_est  */
        (pt != 'g') AND           /* vt_ing   */
        (pt != 'G') AND           /* vi_ing   */
        (pt != 'V') AND           /* vt_pprt  */
        (pt != 'K') AND           /* vi_pprt  */
        (pt != 'y') AND           /* adv      */
        (pt != 'm') AND           /* adv_er   */
        (pt != 'M') AND           /* adv_est  */
        (pt != 'n') AND           /* noun_sgl */
        (pt != 'o') AND           /* noun_sgl */
        (pt != 'N') AND           /* noun_pl  */
        (pt != 'E') AND           /* artcl_df */
        (pt != 'e') AND           /* artcl_id */
        (pt != 's') AND           /* pron_sgl */
        (pt != 'S') AND           /* pron_pl  */
        (pt != 'r') AND           /* pron     */
        (pt != 'p'))    {         /* prep     */
 
      stc[idx].ptrns[j] = pt;
      val = stc[idx].ptrns_lh[i];
      stc[idx].ptrns_lh[j++] = val;
      if (val > max) max = val;
    }
  }
  stc[idx].ptrns[j] = '\0';
  *lh = max;
  if (j == 0) return -1;
  return(len - j);
}


/*----------------------------------------------------------------------

  find_noun_pos  --  find all nouns in a sentence

    input  -- stc:   a sentence
              len:   its length
    output -- num:   number of nouns in a sentence
              nounpos: a list of index corresponding to those nouns

-----------------------------------------------------------------------*/
void find_noun_pos(stc, len, num, nounpos)
struct Syntx_Term *stc;
int    len;
int    *num;
int    nounpos[];
{
  int  i;

  *num = 0;
  for (i = 0; i < len; i++) {
    if (has_subobjects(stc, len, i)) {
      if ((*num) >= NOUN_MAX) return;
      nounpos[(*num)++] = i;
    }
  }
}

/*----------------------------------------------------------------------

  find_sub_pos  --  find all subjects in a sentence

    input  -- stc:   a sentence
              slen:  its length
              idx:   search for subjects from 0 to idx
    output -- num:   number of subjects in a sentence
              subpos: a list of index corresponding to those subjects

-----------------------------------------------------------------------*/
void find_sub_pos(stc, slen, idx, num, subpos)
struct Syntx_Term *stc;
int    slen;
int    idx;
int    *num;
int    subpos[];
{
  int  i, lh1, lh2, lh3;

  *num = 0;
  for (i = 0; i <= idx; i++) {

    if (is_sub_only(stc, slen, i)) {
      subpos[0] = i;
      *num = 1;
      return;
    }
      
    if (((has_article(stc, slen, i, &lh1))     AND 
         (!has_superative(stc, slen, i+1, &lh2)))    OR
        (has_adjpron(stc, slen, i, &lh3)))   {
      if ((*num) >= SUB_MAX) return;
      subpos[(*num)++] = i++;
      while (i <= idx) {
        if (has_subjects(stc, slen, i)) goto loop_bottom;
        i++;
      }
      if (!has_subjects(stc, slen, subpos[(*num)-1]))
        subpos[--(*num)] = 0;
      continue;
    }
    if (has_subjects(stc, slen, i)) {
      if ((*num) >= SUB_MAX) return;
      subpos[(*num)++] = i;
    }

loop_bottom: ;

  }
  
  for (i = 0; i < idx; i++) {
    char word[WORD_MAX+1];
    int  l, j;
  
    get_word(stc, slen, i, word);
    l = strlen(word);
    for (j = 0; j < l; j++) word[j] = tolower(word[j]);
    if (EQUAL(word,"to")) {
      if (has_vb_flwto(stc, slen, i+1, &lh1)) {
        if ((*num) >= SUB_MAX) return;
        subpos[(*num)++] = i;
      }
    }
  }    
}



/*----------------------------------------------------------------------

  find_obj_pos  --  find all objects in a sentence

    input  -- stc:   a sentence
              slen:  its length
              idx:   search for objects from idx to left
    output -- num:   number of objects in a sentence
              objpos: a list of index corresponding to those objects

-----------------------------------------------------------------------*/
void find_obj_pos(stc, slen, idx, num, objpos)
struct Syntx_Term *stc;
int    slen;
int    idx;
int    *num;
int    objpos[];
{
  int  i, lh1, lh2, lh3;

  *num = 0;
  for (i = idx; i < slen; i++) {
    if (((has_article(stc, slen, i, &lh1))     AND 
         (!has_superative(stc, slen, i+1, &lh2)))    OR
        (has_adjpron(stc, slen, i, &lh3)))   {
      if ((*num) >= OBJ_MAX) return;
      objpos[(*num)++] = i++;
      while (i < slen) {
        if (has_objects(stc, slen, i)) goto end_loop;
        i++;
      }
      objpos[--(*num)] = 0;
      continue;
    }
    if (has_objects(stc, slen, i)) {
      if ((*num) >= OBJ_MAX) return;
      objpos[(*num)++] = i;
    }

end_loop: ;

  }

  for (i = idx; i < (slen-1); i++) {
    char word[WORD_MAX+1];
    int  l, j;
  
    get_word(stc, slen, i, word);
    l = strlen(word);
    for (j = 0; j < l; j++) word[j] = tolower(word[j]);
    if (EQUAL(word,"to")) {
      if (has_vb_flwto(stc, slen, i+1, &lh1)) {
        if ((*num) >= OBJ_MAX) return;
        objpos[(*num)++] = i;
      }
    }
  }    
}


/*----------------------------------------------------------------------

  find_verb_pos  --  find all verbs' positions in a sentence

    input  -- stc:   a sentence
              len:   its length
    output -- num:   number of such positions in a sentence
              vbpos: a list of those positions

  function returns 0: no such positions are found
                   1: such positions are found
                   2: verb be is found at the position vbpos[0]
                   3: verb do is found at the position vbpos[0]
                   4: verb have is found at the position vbpos[0]
                   5. aux verb is found at the position vbpos[0]
                   6. pronoun'verb is found at the position vbpos[0]
                   7. verb-only at the position vbpos[0]
                   8. only one verb position (at vbpos[0])

-----------------------------------------------------------------------*/
int find_verb_pos(stc, len, num, vbpos)
struct Syntx_Term *stc;
int    len;
int    *num;
struct VerbPos vbpos[];
{
  int  i, lklhd, numofvb = 0;
  char word[WORD_MAX+1];

  numofvb = 0;
  vbpos[0].vt = vbpos[0].vi = FALSE;
  for (i = 0; i < len; i++) {

    get_word(stc, len, i-1, word);
    word[0] = tolower(word[0]);
    word[1] = tolower(word[1]);
    if (EQUAL(word,"to")) continue;

    if (has_be(stc, len, i, &lklhd)) {
      vbpos[0].index = i;
      *num = 1;
      return 2;
    }
    if (has_do(stc, len, i, &lklhd)) {
      vbpos[0].index = i;
      *num = 1;
      return 3;
    }
    if (has_have(stc, len, i, &lklhd)) {
      vbpos[0].index = i;
      *num = 1;
      return 4;
    }
    if (has_aux(stc, len, i, &lklhd)) {
      vbpos[0].index = i;
      *num = 1;
      return 5;
    }
    if (has_pronvb(stc, len, i, &lklhd)) {
      vbpos[0].index = i;
      *num = 1;
      return 6;
    }
    if (has_verb_only(stc, len, i)) {
      vbpos[0].index = i;
      *num = 1;
      return 7;
    }
    if ((has_vt(stc, len, i, &lklhd))     OR
        (has_past_vt(stc, len, i, &lklhd)))  {
      vbpos[numofvb].index = i;
      vbpos[numofvb].vt = TRUE;
      vbpos[numofvb].vt_lhd = lklhd;
    }
    if ((has_vi(stc, len, i, &lklhd))     OR
        (has_past_vi(stc, len, i, &lklhd)))  {
      vbpos[numofvb].index = i;
      vbpos[numofvb].vi = TRUE;
      vbpos[numofvb].vi_lhd = lklhd;
    }
      
    if (vbpos[numofvb].vt OR vbpos[numofvb].vi) {
      if ((++numofvb) >= VERB_MAX) { *num = numofvb; return 1; }  
      vbpos[numofvb].vt = vbpos[numofvb].vi = FALSE;
    }
  }
  if (numofvb == 0) return 0;
  if (numofvb == 1) return 8;
  *num = numofvb;
  return 1;
}


/*---------------------------------------------------------------------

  ptrn_length  --  return the length of the pattern of a token

    input  -- stc: a sentence
              slen: sentence length
              idx: token index
    output -- none

-----------------------------------------------------------------------*/
int ptrn_length(stc, slen, idx)
struct Syntx_Term *stc;
int    slen;
int    idx;
{
  if ((idx < 0) OR (idx >= slen)) return 0;
  return (strlen(stc[idx].ptrns));
}




/*----------------------------------------------------------------------
 
  cmb_token -- combine token(s) into a phrase

    input  -- stc:   a sentence
              slen:  sentence length
              spoc:  positions of SPOC
              idx1:  first token index
              idx2:  last token index
              clss:  the classifications of combined tokens
              phrs:  phrase name
              lklhd: suggested likelihood of the phrase
    output -- slen: the length of the updated sentence

-----------------------------------------------------------------------*/
void cmb_token(stc, slen, spoc, idx1, idx2, clss, phrs, lklhd)
struct Syntx_Term *stc;
int    *slen;
struct SPOC_Pos *spoc;
int    idx1, idx2;
char   clss[MAXPHRASE][PTRN_PHRS_MAX+1], *phrs;
int    lklhd;
{
  int  i, j, k, lh, len, sub, prd, obj, cpl, usedptrns_lh[MAXPHRASE];
  char usedptrns[MAXPHRASE+1], pt[2];

  if ((idx1 < 0) OR (idx1 >= *slen)) return;
  if ((idx2 < 0) OR (idx2 >= *slen)) return;

  sub = spoc->sub_pos;
  prd = spoc->prd_pos;
  obj = spoc->obj_pos;
  cpl = spoc->cpl_pos;

  if (idx1 == idx2) {
    if (EQUAL(stc[idx1].phrase,"")) {
      name_class_lhd(stc, *slen, idx1, clss[0], 
                     stc[idx1].usedptrns, &stc[idx1].usedptrns_lh[0]);
      stc[idx1].ptrns[0] = '\0';
    }
    strcpy(stc[idx1].phrase, phrs);
    stc[idx1].phrase_lh = lklhd;
    return;

  }

  stc[idx1].end = stc[idx2].end;
  j = 0;
  usedptrns[0] = '\0';
  for (i = idx1; i <= idx2; i++) {
    if (EQUAL(stc[i].phrase,"")) {    /* the current token is a word */
      name_class_lhd(stc, *slen, i, clss[i-idx1], pt, &lh);
      strcat(usedptrns, pt);
      usedptrns_lh[j++] = lh;
      continue;
    }

/* the current token is a phrase */
    len = strlen(stc[i].usedptrns);
    strcat(usedptrns, stc[i].usedptrns);
    for (k = 0; k < len; k++) usedptrns_lh[j++] = stc[i].usedptrns_lh[k];
  }

  strcpy(stc[idx1].usedptrns, usedptrns);
  len = j;
  for (i = 0; i < len; i++) stc[idx1].usedptrns_lh[i] = usedptrns_lh[i];
  strcpy(stc[idx1].phrase, phrs);
  stc[idx1].phrase_lh = lklhd;
  stc[idx1].ptrns[0] = '\0';

  j = idx1 + 1;
  for (i = idx2+1; i < *slen; i++) {
    stc[j].begin = stc[i].begin;
    stc[j].end = stc[i].end;
    strcpy(stc[j].ptrns, stc[i].ptrns);
    len = strlen(stc[i].ptrns);
    for (k = 0; k < len; k++) stc[j].ptrns_lh[k] = stc[i].ptrns_lh[k];
    strcpy(stc[j].phrase, stc[i].phrase);
    stc[j].phrase_lh = stc[i].phrase_lh;
    strcpy(stc[j].usedptrns, stc[i].usedptrns);
    len = strlen(stc[i].usedptrns);
    for (k = 0; k < len; k++) 
      stc[j].usedptrns_lh[k] = stc[i].usedptrns_lh[k];
    j++;
  }
  *slen = j;
  if ((sub != -1) AND (sub > idx1)) {
    if ((sub > idx1) AND (sub <= idx2)) {
      sub = idx1; }
    else {
      if (sub > idx2) sub = sub - (idx2 - idx1);
    }
  }
  if ((prd != -1) AND (prd > idx1)) {
    if ((prd > idx1) AND (prd <= idx2)) {
      prd = idx1; }
    else {
      if (prd > idx2)
        prd = prd - (idx2 - idx1);
    }
  }
  if ((obj != -1) AND (obj > idx1)) {
    if ((obj > idx1) AND (obj <= idx2)) {
      obj = idx1; }
    else {
      if (obj > idx2) obj = obj - (idx2 - idx1);
    }
  }
  if ((cpl != -1) AND (cpl > idx1)) {
    if ((cpl > idx1) AND (cpl <= idx2)) {
      cpl = idx1; }
    else {
      if (cpl > idx2) cpl = cpl - (idx2 - idx1);
    }
  }

  spoc->sub_pos = sub;
  spoc->prd_pos = prd;
  spoc->obj_pos = obj;
  spoc->cpl_pos = cpl;
      
}


/*-----------------------------------------------------------------------

  lhd_cmp  --  compare the likelihoods of a token w/ a specified likelihood

    input  --  stc: a sentence
               slen: the length of the sentence
               idx: token index
               lhd: the likelihood to be compared

    output -- none

  function returns -1: one of token likelihoods is less than lhd
                    0: one of token likelihoods is equal to lhd
                    1: all token likelihoods are greater than lhd

------------------------------------------------------------------------*/
int lhd_cmp(stc, slen, idx, lhd)
struct Syntx_Term *stc;
int slen, idx, lhd;
{
  int len, i;

  if ((idx < 0) OR (idx >= slen)) return -1;
  
  len = strlen(stc[idx].ptrns);
  for (i = 0; i < len; i++) {
    if (stc[idx].ptrns_lh[i] == lhd) return 0;
    if (stc[idx].ptrns_lh[i] < lhd) return -1;
  }
  return 1;
}


/*---------------------------------------------------------------------

  is_word  --  check if a token is a word

    input  -- stc:  a sentence
              len:  sentence length
              idx:  token index
    output -- function returns TRUE if the token is a word, FALSE otherwise

------------------------------------------------------------------------*/
boolean is_word(stc, len, idx)
struct Syntx_Term *stc;
int    len;
int    idx;
{
  if ((idx < 0) OR (idx >= len)) return FALSE;
  return(EQUAL(stc[idx].phrase,""));
}


/*-----------------------------------------------------------------------

  has_phrase  --  check if a token is some phrase

    input  -- stc:  a sentence
              len:  sentence length
              idx:  token index
              phrase: phrase name
    output -- lh: the likelihood of the phrase

  function return TRUE if the token is the specified phrase, 
                  FALSE otherwise

-----------------------------------------------------------------------*/
boolean has_phrase(stc, len, idx, phrase, lh)
struct Syntx_Term *stc;
int    len;
int    idx;
char   *phrase;
int    *lh;
{
  if ((idx < 0) OR (idx >= len)) return FALSE;
  *lh = stc[idx].phrase_lh;
  return(EQUAL(stc[idx].phrase,phrase));
}



/*---------------------------------------------------------------------

  left_stop_noun  --  check if a token is a left stop noun phrase  

    input  -- stc:  a sentence
              len:  sentence length
              idx:  token index
    output -- function returns TRUE  if the token is left stop, 
                               FALSE otherwise

-----------------------------------------------------------------------*/
boolean left_stop_noun(stc, len, idx)
struct Syntx_Term *stc;
int    len;
int    idx;
{
  char pt, word[WORD_MAX+1];
  int  i, l;

  if ((idx < 0) OR (idx >= len)) return FALSE;
  if (EQUAL(stc[idx].phrase,"")) return FALSE;
  pt = stc[idx].usedptrns[0];
  if ((pt == 'E') OR (pt == 'e')) return TRUE;
  get_1stwd_of_phrs(stc, len, idx, word);
  l = strlen(word);
  for (i = 0; i < l; i++) word[i] = tolower(word[i]);
  if (EQUAL(word,"this")  OR
      EQUAL(word,"that")  OR
      EQUAL(word,"these") OR
      EQUAL(word,"those") OR
      EQUAL(word,"its")   OR
      EQUAL(word,"his")   OR
      EQUAL(word,"her")   OR
      EQUAL(word,"their") OR
      EQUAL(word,"my")    OR
      EQUAL(word,"our")   OR
      EQUAL(word,"your"))    return TRUE;
  return FALSE;
}
  

/*----------------------------------------------------------------------
 
  is_sub_only -- check if a syntax token can be used as subject only

    input  -- stc:  a sentence
              slen:  sentence length
              idx:  token index
    output -- none
              function returns,
                TRUE:  is such token
                FALSE: otherwise

-----------------------------------------------------------------------*/
boolean is_sub_only(stc, slen, idx)
struct Syntx_Term *stc;
int    slen;
int    idx;
{
  char word[WORD_MAX+1]; 
  int  len, i;

  if ((idx < 0) OR (idx >= slen)) return FALSE;
  if (stc[idx].phrase[0] != '\0') return FALSE;

  get_word(stc, slen, idx, word);
  len = strlen(word);
  for (i = 0; i < len; i++) word[i] = tolower(word[i]);
  if (EQUAL(word,"i")   OR
      EQUAL(word,"we")  OR
      EQUAL(word,"he")  OR
      EQUAL(word,"she") OR
      EQUAL(word,"they"))   return TRUE;
  
  if (EQUAL(word,"it") OR 
      EQUAL(word,"you"))  {
    if (has_prep(stc, slen, idx-1, &idx)) return FALSE;
    return TRUE;
  }

  return FALSE;
}

/*----------------------------------------------------------------------

  has_two_fixed_prdcts -- check if there are more than one fixed predicates
                          in the sentence.

    input  -- stc:  a sentence
              slen:  sentence length
    output -- none
              funtion returns TRUE if there are such predicates
                              FALSE otherwise

----------------------------------------------------------------------*/
boolean has_two_fixed_prdcts(stc, slen)
struct Syntx_Term *stc;
int    slen;
{
  int  i, lklhd;
  char word[WORD_MAX+1];
  boolean had_one = FALSE;

  for (i = 0; i < slen; i++) {
    get_word(stc, slen, i-1, word);
    word[0] = tolower(word[0]);
    word[1] = tolower(word[1]);
    if (EQUAL(word,"to")) continue;

    if (has_be(stc, slen, i, &lklhd)) {
      if (had_one) return TRUE;
      had_one = TRUE;
    }
 
    if (has_pronvb(stc, slen, i, &lklhd)) {
      if (had_one) return TRUE;
      had_one = TRUE;
    }

    if (has_verb_only(stc, slen, i)) {
      if (had_one) return TRUE;
      had_one = TRUE;
    }
  }

  return FALSE;
}


/*---------------------------------------------------------------------

  left_stop_adj  --  check if a token is a left stop adjective phrase  

    input  -- stc:  a sentence
              len:  sentence length
              idx:  token index
    output -- function returns TRUE  if the token is left stop, 
                               FALSE otherwise

-----------------------------------------------------------------------*/
/*boolean left_stop_adj(stc, len, idx)
struct Syntx_Term *stc;
int    len;
int    idx;
{
  char pt;
  int  i, l;

  if ((idx < 0) OR (idx >= len)) return FALSE;
  if (EQUAL(stc[idx].phrase,"")) return FALSE;
  pt = stc[idx].usedptrns[0];
  if (pt == 'a') return TRUE;
  return FALSE;
}
*/
/*oldoldoldoldoldoldoldoldoldoldoldoldoldoldoldoldoldoldoldoldoldoldold*/









/*------------------------------------------------------------------------

  set_cnt_ptrn  --  set the current pattern of a token to be a specified one

    input  -- stc:   a sentence
              len:   sentence length
              idx:   token index
              ptrn:  pattern to be specified
              lklhd: the likelihood of the pattern
    output -- none

-------------------------------------------------------------------------*/
/*
void set_cnt_ptrn(stc, len, idx, ptrn, lklhd)
struct Syntx_Term *stc;
int    len;
int    idx;
char   *ptrn;
int    lklhd;
{
  if ((idx < 0) OR (idx >= len)) return;

  strcpy(stc[idx].cnt_ptrns[0].ptn_phr, ptrn);
  stc[idx].cnt_ptrns[0].likelihood = lklhd;
  strcpy(stc[idx].cnt_ptrns[1].ptn_phr, "");
}
*/





