#include "syntx.h"

#define        NumofVerbDgns    22
#define        NumofVerbNext     7

extern struct Syntx_Term *sentence; 
extern int                sentence_len;
extern struct SPOC_Pos    sentence_spoc;
extern boolean            intrnl_dsply;




DIAGNOSIS_ARRAY verb_dgns[NumofVerbDgns + 1] = { 
                                                  "rht_cmb",
                                                  "lft_cmb",
                                                  "next(find_sub)",
                                                  "next(find_obj)",
                                                  "next(hv_phrs)",
                                                  "next(do_phrs)",
                                                  "next(aux_phrs)",
                                                  "next(X_H_VB_H)",
                                                  "next(PVHVBH)",
                                                  "is_phrs(vt_pl)",
                                                  "is_phrs(vt_sgl)",
                                                  "is_phrs(vt_pst)",
                                                  "is_phrs(vi_pl)",
                                                  "is_phrs(vi_sgl)",
                                                  "is_phrs(vi_pst)",
                                                  "is_phrs(vt_pprt)",
                                                  "is_phrs(vi_pprt)",
                                                  "is_phrs(vt_ing)",
                                                  "is_phrs(vi_ing)",
                                                  "is_phrs(lvb_pl)",
                                                  "is_phrs(lvb_sgl)",
                                                  "is_phrs(lvb_pst)",
                                                  "" 
                                               };   

static int        symptoms_idx;
SYMPTOM_ARRAY     symptoms[mmax(3*MAXPTRNS+2,MAXSYMP)];
  
void    verb_phrase();
int     do_verbP();
boolean verbP_cntptrn();
boolean verbP_cmb_and_next();


/*----------------------------------------------------------------------

  verb_phrase  --  control the action of do_verbP

    input  -- pos:  the position where the action to be taken
              lklhd: the likelyhood at which the action will take
              who: indicate who invoke the function
    output -- none
 
-----------------------------------------------------------------------*/
void verb_phrase(pos, lklhd, who)
int pos;
int lklhd;
int who;
{
  int vbP_code, verb_pos;

  if (intrnl_dsply) {
    begin_title("verb_phrase", pos, lklhd);
  }

  load_prb("VerbPhrase");

  if ((vbP_code = do_verbP(pos, lklhd, who, &verb_pos)) == 0) {
    if (intrnl_dsply) {
      printf("no verb phrases can be formed\n");
    }
    goto end;
  }

  switch (vbP_code) {
    case 1:                                           /* transitive verb */
    case 2:                                         /* intransitive verb */
    case 3:                                        /* vt_pprt or vi_pprt */
    case 4:                                          /* vt_ing or vi_ing */
            break;

    default: stx_err("unexpected vbP_code", __LINE__, __FILE__);
  }
  
end:  
  if (intrnl_dsply) {
    end_title("verb_phrase");
  }
}


/*---------------------------------------------------------------------

  do_verbP  --  execute do_verbP.prg  

    input  -- pos: the position where the do_verbP works on
              lklhd: the likelihood the do_verbP works at
              who:   who issued verb_phrase
    output -- verb_pos: the position of the formed phrase
 
  function returns  0: no verb phrase can be formed
                    1: vt_pl, vt_sgl or vt_pst phrase is formed
                    2: vi_pl, vi_sgl or vi_pst phrase is formed 
                    3: vt_pprt or vi_pprt is formed
                    4: vt_ing or vi_ing is formed

----------------------------------------------------------------------*/
int do_verbP(pos, lklhd, who, verb_pos)
int pos, lklhd, who, *verb_pos;
{
  DIAGNOSIS_ARRAY   dgns[NumofVerbDgns + 1];
  char              dgns_var[VAR_MAX + 1],  ptrn[PTRN_PHRS_MAX+1],
                    ptrn_names[MAXPTRNS+1][PTRN_PHRS_MAX+1];
  int               i, lft, rht, cnt, lh, lklhds[MAXPTRNS], return_val = 0;
  boolean           next[NumofVerbNext];

  symptoms_idx = 0;
  cnt = pos;
  lft = pos - 1;
  rht = pos + 1;
  for (i = 0; i < NumofVerbNext; i++) next[i] = FALSE;

  switch(who) {
    case FindPredicate: 
                        strcpy(symptoms[symptoms_idx++], "who(find_prd)");
                        break;

    case HavePhrase:
                        strcpy(symptoms[symptoms_idx++], "who(hv_phrs)");
                        break;
  
    case DoPhrase:
                        strcpy(symptoms[symptoms_idx++], "who(do_phrs)");
                        break;
  
    case AuxPhrase:
                        strcpy(symptoms[symptoms_idx++], "who(aux_phrs)");
                        break;

    case AuxHave:
                        strcpy(symptoms[symptoms_idx++], "who(aux_hv)");
                        break;

    case PvHave:
                        strcpy(symptoms[symptoms_idx++], "who(pv_hv)");
                        break;

    default: stx_err("unexpected problem code", __LINE__, __FILE__);
  }
    
  if (has_adv_advP_only(sentence, sentence_len, lft, &lh))
    strcpy(symptoms[symptoms_idx++], "lft_adv_only");
  if (has_adv_advP_only(sentence, sentence_len, rht, &lh))
    strcpy(symptoms[symptoms_idx++], "rht_adv_only");

  get_ptrn_names(sentence, sentence_len, cnt, ptrn_names, lklhds);
  i = 0;
  strcpy(ptrn, ptrn_names[i++]);
  while (!EQUAL(ptrn,"")) {
    if (verbP_cntptrn(ptrn)) {
      sprintf(symptoms[symptoms_idx++], "cnt_ptrn(%s)", ptrn);
    }
    strcpy(ptrn, ptrn_names[i++]);
  }
  strcpy(symptoms[symptoms_idx],"");

  mst_smp(symptoms, name, state, type, value, errcde);
  sol_prb(name, state, type, value, errcde);
  if (errcde[0] != 0) {
    stx_err("failed to solve problem", __LINE__, __FILE__);
  }

  if EQUAL(state, "S") { 
    copy_dgns(dgns, verb_dgns);
    get_dgn(dgns,name,state,type,value,errcde);
    if (value[0]) {                      /* has at least one true value */
      boolean rht_cmb = FALSE, lft_cmb = FALSE;
      struct  Syntx_Term *stc; 
      int     stc_len;
      struct  SPOC_Pos stc_spoc;

      i = 0;
      while (! EQUAL(dgns[i],"")) {
        strcpy(dgns_var, dgns[i++]);
        if (EQUAL(dgns_var,"rht_cmb")) { rht_cmb = TRUE; continue; }
        if (EQUAL(dgns_var,"lft_cmb")) { lft_cmb = TRUE; continue; }

        if (EQUAL(dgns_var,"next(find_sub)")) { 
          next[0] = TRUE; 
          continue; 
        }
        if (EQUAL(dgns_var,"next(find_obj)")) { 
          next[1] = TRUE; 
          continue; 
        }
        if (EQUAL(dgns_var,"next(hv_phrs)")) { 
          next[2] = TRUE; 
          continue; 
        }
        if (EQUAL(dgns_var,"next(do_phrs)")) { 
          next[3] = TRUE; 
          continue; 
        }
        if (EQUAL(dgns_var,"next(aux_phrs)")) { 
          next[4] = TRUE; 
          continue; 
        }
        if (EQUAL(dgns_var,"next(X_H_VB_H)")) { 
          next[5] = TRUE; 
          continue; 
        }
        if (EQUAL(dgns_var,"next(PVHVBH)")) { 
          next[6] = TRUE; 
          continue; 
        }

        
        if (EQUAL(dgns_var,"is_phrs(vt_pl)")) { 
          stc = (struct Syntx_Term *)stc_cpy(&stc_len, &stc_spoc,
                                   sentence, sentence_len, &sentence_spoc);
          if (verbP_cmb_and_next(stc, &stc_len, &stc_spoc, cnt, 
              "vt_pl",lklhd,lft_cmb, rht_cmb, next, TRUE)) return_val = 1;
          stc_dsp(stc);
          continue;
        }
        if (EQUAL(dgns_var,"is_phrs(vt_sgl)")) { 
          stc = (struct Syntx_Term *)stc_cpy(&stc_len, &stc_spoc,
                                   sentence, sentence_len, &sentence_spoc);
          if (verbP_cmb_and_next(stc,&stc_len,&stc_spoc,cnt,"vt_sgl",
                      lklhd, lft_cmb, rht_cmb, next, TRUE)) return_val = 1;
          stc_dsp(stc);
          continue;
        }
        if (EQUAL(dgns_var,"is_phrs(vt_pst)")) { 
          stc = (struct Syntx_Term *)stc_cpy(&stc_len, &stc_spoc,
                                   sentence, sentence_len, &sentence_spoc);
          if (verbP_cmb_and_next(stc, &stc_len, &stc_spoc, cnt, 
              "vt_pst",lklhd,lft_cmb, rht_cmb, next, TRUE)) return_val = 1;
          stc_dsp(stc);
          continue;
        }
        if (EQUAL(dgns_var,"is_phrs(vi_pl)")) { 
          stc = (struct Syntx_Term *)stc_cpy(&stc_len, &stc_spoc,
                                   sentence, sentence_len, &sentence_spoc);
          if (verbP_cmb_and_next(stc, &stc_len, &stc_spoc, cnt,  
              "vi_pl",lklhd, lft_cmb, rht_cmb, next, FALSE)) return_val =2;
          stc_dsp(stc);
          continue;
        }
        if (EQUAL(dgns_var,"is_phrs(vi_sgl)")) { 
          stc = (struct Syntx_Term *)stc_cpy(&stc_len, &stc_spoc,
                                   sentence, sentence_len, &sentence_spoc);
          if (verbP_cmb_and_next(stc, &stc_len, &stc_spoc, cnt,
              "vi_sgl",lklhd,lft_cmb, rht_cmb, next, FALSE)) return_val =2;
          stc_dsp(stc);
          continue;
        }
        if (EQUAL(dgns_var,"is_phrs(vi_pst)")) { 
          stc = (struct Syntx_Term *)stc_cpy(&stc_len, &stc_spoc,
                                   sentence, sentence_len, &sentence_spoc);
          if (verbP_cmb_and_next(stc, &stc_len, &stc_spoc, cnt, 
              "vi_pst",lklhd,lft_cmb, rht_cmb, next, FALSE)) return_val =2;
          stc_dsp(stc);
          continue;
        }
        if (EQUAL(dgns_var,"is_phrs(lvb_pl)")) { 
          stc = (struct Syntx_Term *)stc_cpy(&stc_len, &stc_spoc,
                                   sentence, sentence_len, &sentence_spoc);
          if (verbP_cmb_and_next(stc, &stc_len, &stc_spoc, cnt, 
              "lvb_pl",lklhd,lft_cmb, rht_cmb, next, FALSE)) return_val =2;
          stc_dsp(stc);
          continue;
        }
        if (EQUAL(dgns_var,"is_phrs(lvb_sgl)")) { 
          stc = (struct Syntx_Term *)stc_cpy(&stc_len, &stc_spoc,
                                   sentence, sentence_len, &sentence_spoc);
          if (verbP_cmb_and_next(stc, &stc_len, &stc_spoc, cnt, 
             "lvb_sgl",lklhd,lft_cmb, rht_cmb, next, FALSE)) return_val =2;
          stc_dsp(stc);
          continue;
        }
        if (EQUAL(dgns_var,"is_phrs(lvb_pst)")) { 
          stc = (struct Syntx_Term *)stc_cpy(&stc_len, &stc_spoc,
                                   sentence, sentence_len, &sentence_spoc);
          if (verbP_cmb_and_next(stc, &stc_len, &stc_spoc, cnt, 
             "lvb_pst",lklhd,lft_cmb, rht_cmb, next, FALSE)) return_val =2;
          stc_dsp(stc);
          continue;
        }
        if (EQUAL(dgns_var,"is_phrs(vt_pprt)")) { 
          stc = (struct Syntx_Term *)stc_cpy(&stc_len, &stc_spoc,
                                   sentence, sentence_len, &sentence_spoc);
          if (verbP_cmb_and_next(stc, &stc_len, &stc_spoc, cnt, 
             "vt_pprt",lklhd,lft_cmb, rht_cmb, next, TRUE)) return_val = 3;
          stc_dsp(stc);
          continue;
        }
        if (EQUAL(dgns_var,"is_phrs(vi_pprt)")) { 
          stc = (struct Syntx_Term *)stc_cpy(&stc_len, &stc_spoc,
                                   sentence, sentence_len, &sentence_spoc);
          if (verbP_cmb_and_next(stc, &stc_len, &stc_spoc, cnt, 
             "vi_pprt",lklhd,lft_cmb, rht_cmb, next, FALSE)) return_val =3;
          stc_dsp(stc);
          continue;
        }
        if (EQUAL(dgns_var,"is_phrs(vt_ing)")) { 
          stc = (struct Syntx_Term *)stc_cpy(&stc_len, &stc_spoc,
                                   sentence, sentence_len, &sentence_spoc);
          if (verbP_cmb_and_next(stc, &stc_len, &stc_spoc, cnt, 
              "vt_ing",lklhd,lft_cmb, rht_cmb, next, TRUE)) return_val = 4;
          stc_dsp(stc);
          continue;
        }
        if (EQUAL(dgns_var,"is_phrs(vi_ing)")) { 
          stc = (struct Syntx_Term *)stc_cpy(&stc_len, &stc_spoc,
                                   sentence, sentence_len, &sentence_spoc);
          if (verbP_cmb_and_next(stc, &stc_len, &stc_spoc, cnt, 
             "vi_ing",lklhd, lft_cmb, rht_cmb, next, FALSE)) return_val =4;
          stc_dsp(stc);
          continue;
        }
      }
  
      if (lft_cmb) { *verb_pos = cnt-1; }
      else *verb_pos = cnt;
    }
  }
  else stx_err("problem is unsatisfied", __LINE__, __FILE__);
  

  strcpy(state, "A");
  rst_prb(name, state, type, value, errcde);
  return return_val;
}


/*---------------------------------------------------------------------

  verbP_cntptrn  --  check if the pattern is considered in the log file

    input  -- ptrn: current pattern name to be checked
    output -- none

  function returns TRUE if ptrn is considered in log file, FALSE otherwise

------------------------------------------------------------------------*/
boolean verbP_cntptrn(ptrn)
char *ptrn;
{
  char str[3];

  str[0] = ptrn[0];
  str[1] = ptrn[1];
  str[2] = '\0';

  if ((EQUAL(str,"vt")) OR
      (EQUAL(str,"vi")))   return TRUE;

  if (EQUAL(ptrn,"lvb_pl")  OR
      EQUAL(ptrn,"lvb_sgl") OR
      EQUAL(ptrn,"lvb_pst"))   return TRUE;

  return FALSE;
}



/*---------------------------------------------------------------------

  verbP_cmb_and_next  --  combine tokens and issue next command

    input  -- stc:    sentence
              slen:   sentence length
              spoc:   positions of SPOC
              cnt:    current position
              phrase: the name of the combined phrase
              lh:     likelihood
              cmbl:   indicate combining cnt and left
              cmbr:   indicate combining cnt and right
              next:   indicate the next command to be executed
              vt:     indicate the phrase is a vt

    output -- function return TRUE if some verb phrase can be formed,
                              FALSE otherwise.

------------------------------------------------------------------------*/
boolean verbP_cmb_and_next(stc,slen,spoc,cnt,phrase,lh,cmbl,cmbr,next,vt)
struct  Syntx_Term *stc; 
int     *slen;
struct  SPOC_Pos *spoc;
int     cnt;
char    *phrase;
int     lh;
boolean cmbl;
boolean cmbr;
boolean next[NumofVerbNext];
boolean vt;
{
  int  vb_pos = cnt;
  char clss[MAXPHRASE][PTRN_PHRS_MAX+1]; 

  if ((!cmbl) AND (!cmbr)) {
    if (intrnl_dsply) {
      printf("%d ==>  phrase '%s'\n\n", cnt, phrase);
    }    
    strcpy(clss[0], phrase);
    cmb_token(stc, slen, spoc, cnt, cnt, clss, phrase, lh);
    goto issue_next;
  }
  
  if ((cmbl) AND (cmbr)) {
    if (intrnl_dsply) {
      printf("%d + %d + %d ==>  phrase '%s'\n\n",cnt-1,cnt, cnt+1, phrase);
    }
    strcpy(clss[0], "adv");
    strcpy(clss[1], phrase);
    strcpy(clss[2], "adv");
    cmb_token(stc, slen, spoc, cnt-1, cnt+1, clss, phrase, lh);
    vb_pos = cnt-1;
    goto issue_next;
  }

  if (cmbl) { 
    if (intrnl_dsply) {
      printf("%d + %d  ==>  phrase '%s'\n\n", cnt-1, cnt, phrase);
    }     
    strcpy(clss[0], "adv");
    strcpy(clss[1], phrase);
    cmb_token(stc, slen, spoc, cnt-1, cnt, clss, phrase, lh);
    vb_pos = cnt-1;
    goto issue_next;
  }

  if (cmbr) { 
    if (intrnl_dsply) {
      printf("%d + %d  ==>  phrase '%s'\n\n", cnt, cnt+1, phrase);
    }   
    strcpy(clss[0], phrase);
    strcpy(clss[1], "adv");
    cmb_token(stc, slen, spoc, cnt, cnt+1, clss, phrase, lh);
    goto issue_next;
  }
  return FALSE;

issue_next:


  if ((next[0]) AND !vt) {                         /* next[0] = find_sub */
    spoc->prd_pos = vb_pos;
    stc_push(stc, *slen, spoc, FindSubject, vb_pos-1, lh, VerbPhrase);
  }

  if ((next[1]) AND vt) {                          /* next[1] = find_obj */
    spoc->prd_pos = vb_pos;
    stc_push(stc, *slen, spoc, FindObject, vb_pos+1, lh, VerbPhrase);
  }
  
  if (next[2]) {                                    /* next[2] = hv_phrs */
    stc_push(stc, *slen, spoc, HavePhrase, vb_pos-1, lh, VerbPhrase);
  }

  if (next[3]) {                                    /* next[3] = do_phrs */
    stc_push(stc, *slen, spoc, DoPhrase, vb_pos-1, lh, VerbPhrase);
  }

  if (next[4]) {                                   /* next[4] = aux_phrs */
    int l;
    
    if (has_aux(stc, *slen, vb_pos-1, &l)) {
      stc_push(stc, *slen, spoc, AuxPhrase, vb_pos-1, lh, VerbPhrase);
    }
    else stc_push(stc, *slen, spoc, AuxPhrase, vb_pos-2, lh, VerbPhrase);
  }
  
  if (next[5]) {                                   /* next[5] = X_H_VB_H */
    stc_push(stc, *slen, spoc, HavePhrase, vb_pos-1, lh, AuxHaveVerb);
  }

  if (next[6]) {                                     /* next[6] = PVHVBH */
    stc_push(stc, *slen, spoc, HavePhrase, vb_pos-1, lh, PvHaveVerb);
  }
  return TRUE;

}

    


